@@ -266,7 +266,7 @@ do_iconview (GtkWidget *do_widget)
       gtk_container_add (GTK_CONTAINER (window), vbox);
 
       tool_bar = gtk_toolbar_new ();
-      gtk_box_pack_start (GTK_BOX (vbox), tool_bar, FALSE, FALSE);
+      gtk_box_pack_start (GTK_BOX (vbox), tool_bar);
 
       up_button = gtk_tool_button_new (NULL, NULL);
       gtk_tool_button_set_label (GTK_TOOL_BUTTON (up_button), _("_Up"));
@@ -290,8 +290,9 @@ do_iconview (GtkWidget *do_widget)
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                       GTK_POLICY_AUTOMATIC,
                                       GTK_POLICY_AUTOMATIC);
+      gtk_widget_set_vexpand (sw, TRUE);
 
-      gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE);
+      gtk_box_pack_start (GTK_BOX (vbox), sw);
 
       /* Create the store and fill it with the contents of '/' */
       parent = g_strdup ("/");
@@ -326,7 +327,7 @@ do_iconview (GtkWidget *do_widget)
     }
 
   if (!gtk_widget_get_visible (window))
-    gtk_widget_show_all (window);
+    gtk_widget_show (window);
   else
     gtk_widget_destroy (window);
 
