@@ -148,7 +148,7 @@ do_panes (GtkWidget *do_widget)
 
       vpaned = gtk_paned_new (GTK_ORIENTATION_VERTICAL);
       g_object_set (vpaned, "margin", 5, NULL);
-      gtk_box_pack_start (GTK_BOX (vbox), vpaned, TRUE, TRUE);
+      gtk_box_pack_start (GTK_BOX (vbox), vpaned);
 
       hpaned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
       gtk_paned_add1 (GTK_PANED (vpaned), hpaned);
@@ -177,15 +177,13 @@ do_panes (GtkWidget *do_widget)
                           create_pane_options (GTK_PANED (hpaned),
                                                "Horizontal",
                                                "Left",
-                                               "Right"),
-                          FALSE, FALSE);
+                                               "Right"));
 
       gtk_box_pack_start (GTK_BOX (vbox),
                           create_pane_options (GTK_PANED (vpaned),
                                                "Vertical",
                                                "Top",
-                                               "Bottom"),
-                          FALSE, FALSE);
+                                               "Bottom"));
     }
 
   if (!gtk_widget_get_visible (window))
