@@ -402,8 +402,7 @@ do_tree_store (GtkWidget *do_widget)
       gtk_container_add (GTK_CONTAINER (window), vbox);
 
       gtk_box_pack_start (GTK_BOX (vbox),
-                          gtk_label_new ("Jonathan's Holiday Card Planning Sheet"),
-                          FALSE, FALSE);
+                          gtk_label_new ("Jonathan's Holiday Card Planning Sheet"));
 
       sw = gtk_scrolled_window_new (NULL, NULL);
       gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
@@ -411,7 +410,7 @@ do_tree_store (GtkWidget *do_widget)
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                       GTK_POLICY_AUTOMATIC,
                                       GTK_POLICY_AUTOMATIC);
-      gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE);
+      gtk_box_pack_start (GTK_BOX (vbox), sw);
 
       /* create model */
       model = create_model ();
@@ -433,7 +432,7 @@ do_tree_store (GtkWidget *do_widget)
     }
 
   if (!gtk_widget_get_visible (window))
-    gtk_widget_show_all (window);
+    gtk_widget_show (window);
   else
     gtk_widget_destroy (window);
 
