@@ -1,4 +1,5 @@
 #include <gtk/gtk.h>
+#include <glib/gstdio.h>
 
 static void
 print_hello (GtkWidget *widget,
@@ -15,6 +16,10 @@ main (int   argc,
   GObject *window;
   GObject *button;
 
+#ifdef GTK_SRCDIR
+  g_chdir (GTK_SRCDIR);
+#endif
+
   gtk_init ();
 
   /* Construct a GtkBuilder instance and load our UI description */
