@@ -16,16 +16,13 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GDK_BROADWAY_SCREEN_H__
 #define __GDK_BROADWAY_SCREEN_H__
 
 #include <gdk/gdkscreenprivate.h>
-#include <gdk/gdkvisual.h>
 #include "gdkprivate-broadway.h"
 
 G_BEGIN_DECLS
@@ -48,19 +45,6 @@ struct _GdkBroadwayScreen
 
   GdkDisplay *display;
   GdkWindow *root_window;
-
-  int width;
-  int height;
-
-  /* Visual Part */
-  GdkVisual **visuals;
-  gint nvisuals;
-  GdkVisual *system_visual;
-  GdkVisual *rgba_visual;
-  gint available_depths[7];
-  gint navailable_depths;
-  GdkVisualType available_types[6];
-  gint navailable_types;
 };
 
 struct _GdkBroadwayScreenClass
@@ -73,7 +57,6 @@ struct _GdkBroadwayScreenClass
 GType       gdk_broadway_screen_get_type (void);
 GdkScreen * _gdk_broadway_screen_new      (GdkDisplay *display,
 					   gint	  screen_number);
-void _gdk_broadway_screen_setup           (GdkScreen *screen);
 
 G_END_DECLS
 
