@@ -349,7 +349,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                   G_TYPE_FROM_CLASS (object_class),
                   G_SIGNAL_RUN_LAST,
                   0, NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -367,7 +367,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                   G_TYPE_FROM_CLASS (object_class),
                   G_SIGNAL_RUN_LAST,
                   0, NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1, GDK_TYPE_DEVICE_TOOL);
 }
 
@@ -1456,18 +1456,6 @@ gdk_device_grab (GdkDevice        *device,
 
   native = gdk_window_get_toplevel (window);
 
-  while (native->window_type == GDK_WINDOW_OFFSCREEN)
-    {
-      native = gdk_offscreen_window_get_embedder (native);
-
-      if (native == NULL ||
-          (!_gdk_window_has_impl (native) &&
-           !gdk_window_is_viewable (native)))
-        return GDK_GRAB_NOT_VIEWABLE;
-
-      native = gdk_window_get_toplevel (native);
-    }
-
   if (native == NULL || GDK_WINDOW_DESTROYED (native))
     return GDK_GRAB_NOT_VIEWABLE;
 
@@ -1708,23 +1696,8 @@ _gdk_device_translate_window_coord (GdkDevice *device,
   device_width = axis_info_x->max_value - axis_info_x->min_value;
   device_height = axis_info_y->max_value - axis_info_y->min_value;
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  if (device_width > 0)
-    x_min = axis_info_x->min_value;
-  else
-    {
-      device_width = gdk_screen_get_width (gdk_window_get_screen (window));
-      x_min = 0;
-    }
-
-  if (device_height > 0)
-    y_min = axis_info_y->min_value;
-  else
-    {
-      device_height = gdk_screen_get_height (gdk_window_get_screen (window));
-      y_min = 0;
-    }
-G_GNUC_END_IGNORE_DEPRECATIONS
+  x_min = axis_info_x->min_value;
+  y_min = axis_info_y->min_value;
 
   window_width = gdk_window_get_width (window);
   window_height = gdk_window_get_height (window);
@@ -1786,6 +1759,8 @@ _gdk_device_translate_screen_coord (GdkDevice *device,
                                     GdkWindow *window,
                                     gdouble    window_root_x,
                                     gdouble    window_root_y,
+                                    gdouble    screen_width,
+                                    gdouble    screen_height,
                                     guint      index_,
                                     gdouble    value,
                                     gdouble   *axis_value)
@@ -1807,11 +1782,10 @@ _gdk_device_translate_screen_coord (GdkDevice *device,
 
   axis_width = axis_info.max_value - axis_info.min_value;
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   if (axis_info.use == GDK_AXIS_X)
     {
       if (axis_width > 0)
-        scale = gdk_screen_get_width (gdk_window_get_screen (window)) / axis_width;
+        scale = screen_width / axis_width;
       else
         scale = 1;
 
@@ -1820,13 +1794,12 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   else
     {
       if (axis_width > 0)
-        scale = gdk_screen_get_height (gdk_window_get_screen (window)) / axis_width;
+        scale = screen_height / axis_width;
       else
         scale = 1;
 
       offset = - window_root_y - window->abs_y;
     }
-G_GNUC_END_IGNORE_DEPRECATIONS
 
   if (axis_value)
     *axis_value = offset + scale * (value - axis_info.min_value);
