@@ -24,6 +24,7 @@
 #include "gdkdisplay.h"
 #include "gdkdisplayprivate.h"
 
+#include "gdkintl.h"
 #include "gdk-private.h"
 
 #include "gdkdeviceprivate.h"
@@ -66,6 +67,15 @@
  * gdk_display_get_device_manager().
  */
 
+enum
+{
+  PROP_0,
+  PROP_COMPOSITED,
+  PROP_RGBA,
+  LAST_PROP
+};
+
+static GParamSpec *props[LAST_PROP] = { NULL, };
 
 enum {
   OPENED,
@@ -89,6 +99,29 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GdkDisplay, gdk_display, G_TYPE_OBJECT)
 
+static void
+gdk_display_get_property (GObject    *object,
+                          guint       prop_id,
+                          GValue     *value,
+                          GParamSpec *pspec)
+{
+  GdkDisplay *display = GDK_DISPLAY (object);
+
+  switch (prop_id)
+    {
+    case PROP_COMPOSITED:
+      g_value_set_boolean (value, gdk_display_is_composited (display));
+      break;
+
+    case PROP_RGBA:
+      g_value_set_boolean (value, gdk_display_is_rgba (display));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    }
+}
+
 static void
 gdk_display_real_make_default (GdkDisplay *display)
 {
@@ -143,6 +176,12 @@ gdk_display_real_get_default_seat (GdkDisplay *display)
   return display->seats->data;
 }
 
+static GdkMonitor *
+gdk_display_real_get_primary_monitor (GdkDisplay *display)
+{
+  return gdk_display_get_monitor (display, 0);
+}
+
 static void
 gdk_display_class_init (GdkDisplayClass *class)
 {
@@ -150,6 +189,7 @@ gdk_display_class_init (GdkDisplayClass *class)
 
   object_class->finalize = gdk_display_finalize;
   object_class->dispose = gdk_display_dispose;
+  object_class->get_property = gdk_display_get_property;
 
   class->get_app_launch_context = gdk_display_real_get_app_launch_context;
   class->window_type = GDK_TYPE_WINDOW;
@@ -160,6 +200,40 @@ gdk_display_class_init (GdkDisplayClass *class)
   class->event_data_free = gdk_display_real_event_data_free;
   class->get_default_seat = gdk_display_real_get_default_seat;
 
+  class->get_primary_monitor = gdk_display_real_get_primary_monitor;
+
+  /**
+   * GdkDisplay:composited:
+   *
+   * %TRUE if the display properly composits the alpha channel.
+   * See gdk_display_is_composited() for details.
+   *
+   * Since: 3.90
+   */
+  props[PROP_COMPOSITED] =
+    g_param_spec_boolean ("composited",
+                          P_("Composited"),
+                          P_("Composited"),
+                          TRUE,
+                          G_PARAM_READABLE);
+
+  /**
+   * GdkDisplay:rgba:
+   *
+   * %TRUE if the display supports an alpha channel. See gdk_display_is_rgba()
+   * for details.
+   *
+   * Since: 3.90
+   */
+  props[PROP_RGBA] =
+    g_param_spec_boolean ("rgba",
+                          P_("RGBA"),
+                          P_("RGBA"),
+                          TRUE,
+                          G_PARAM_READABLE);
+
+  g_object_class_install_properties (object_class, LAST_PROP, props);
+
   /**
    * GdkDisplay::opened:
    * @display: the object on which the signal is emitted
@@ -173,7 +247,7 @@ gdk_display_class_init (GdkDisplayClass *class)
                   G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GdkDisplayClass, opened),
                   NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -196,9 +270,6 @@ gdk_display_class_init (GdkDisplayClass *class)
 		  G_TYPE_NONE,
 		  1,
 		  G_TYPE_BOOLEAN);
-  g_signal_set_va_marshaller (signals[CLOSED],
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              _gdk_marshal_VOID__BOOLEANv);
 
   /**
    * GdkDisplay::seat-added:
@@ -215,7 +286,7 @@ gdk_display_class_init (GdkDisplayClass *class)
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  0, NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1, GDK_TYPE_SEAT);
 
   /**
@@ -233,7 +304,7 @@ gdk_display_class_init (GdkDisplayClass *class)
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  0, NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1, GDK_TYPE_SEAT);
 
   /**
@@ -251,7 +322,7 @@ gdk_display_class_init (GdkDisplayClass *class)
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  0, NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1, GDK_TYPE_MONITOR);
 
   /**
@@ -269,7 +340,7 @@ gdk_display_class_init (GdkDisplayClass *class)
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  0, NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1, GDK_TYPE_MONITOR);
 }
 
@@ -319,6 +390,9 @@ gdk_display_init (GdkDisplay *display)
                                                         (GDestroyNotify) g_free);
 
   display->rendering_mode = _gdk_rendering_mode;
+
+  display->composited = TRUE;
+  display->rgba = TRUE;
 }
 
 static void
@@ -502,74 +576,6 @@ gdk_display_put_event (GdkDisplay     *display,
   gdk_display_put_event_nocopy (display, gdk_event_copy (event));
 }
 
-/**
- * gdk_display_pointer_ungrab:
- * @display: a #GdkDisplay.
- * @time_: a timestap (e.g. %GDK_CURRENT_TIME).
- *
- * Release any pointer grab.
- *
- * Since: 2.2
- *
- * Deprecated: 3.0: Use gdk_device_ungrab(), together with gdk_device_grab()
- *             instead.
- */
-void
-gdk_display_pointer_ungrab (GdkDisplay *display,
-			    guint32     time_)
-{
-  GList *seats, *s;
-  GdkDevice *device;
-
-  g_return_if_fail (GDK_IS_DISPLAY (display));
-
-  seats = gdk_display_list_seats (display);
-
-  for (s = seats; s; s = s->next)
-    {
-      device = gdk_seat_get_pointer (s->data);
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-      gdk_device_ungrab (device, time_);
-      G_GNUC_END_IGNORE_DEPRECATIONS;
-    }
-
-  g_list_free (seats);
-}
-
-/**
- * gdk_display_keyboard_ungrab:
- * @display: a #GdkDisplay.
- * @time_: a timestap (e.g #GDK_CURRENT_TIME).
- *
- * Release any keyboard grab
- *
- * Since: 2.2
- *
- * Deprecated: 3.0: Use gdk_device_ungrab(), together with gdk_device_grab()
- *             instead.
- */
-void
-gdk_display_keyboard_ungrab (GdkDisplay *display,
-			     guint32     time)
-{
-  GList *seats, *s;
-  GdkDevice *device;
-
-  g_return_if_fail (GDK_IS_DISPLAY (display));
-
-  seats = gdk_display_list_seats (display);
-
-  for (s = seats; s; s = s->next)
-    {
-      device = gdk_seat_get_keyboard (s->data);
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-      gdk_device_ungrab (device, time);
-      G_GNUC_END_IGNORE_DEPRECATIONS;
-    }
-
-  g_list_free (seats);
-}
-
 /**
  * gdk_beep:
  * 
@@ -633,97 +639,6 @@ _gdk_display_enable_motion_hints (GdkDisplay *display,
     }
 }
 
-/**
- * gdk_display_get_pointer:
- * @display: a #GdkDisplay
- * @screen: (out) (allow-none) (transfer none): location to store the screen that the
- *          cursor is on, or %NULL.
- * @x: (out) (allow-none): location to store root window X coordinate of pointer, or %NULL.
- * @y: (out) (allow-none): location to store root window Y coordinate of pointer, or %NULL.
- * @mask: (out) (allow-none): location to store current modifier mask, or %NULL
- *
- * Gets the current location of the pointer and the current modifier
- * mask for a given display.
- *
- * Since: 2.2
- *
- * Deprecated: 3.0: Use gdk_device_get_position() instead.
- **/
-void
-gdk_display_get_pointer (GdkDisplay      *display,
-			 GdkScreen      **screen,
-			 gint            *x,
-			 gint            *y,
-			 GdkModifierType *mask)
-{
-  GdkScreen *default_screen;
-  GdkSeat *default_seat;
-  GdkWindow *root;
-  gdouble tmp_x, tmp_y;
-  GdkModifierType tmp_mask;
-
-  g_return_if_fail (GDK_IS_DISPLAY (display));
-
-  if (gdk_display_is_closed (display))
-    return;
-
-  default_screen = gdk_display_get_default_screen (display);
-  default_seat = gdk_display_get_default_seat (display);
-
-  /* We call _gdk_device_query_state() here manually instead of
-   * gdk_device_get_position() because we care about the modifier mask */
-
-  _gdk_device_query_state (gdk_seat_get_pointer (default_seat),
-                           gdk_screen_get_root_window (default_screen),
-                           &root, NULL,
-                           &tmp_x, &tmp_y,
-                           NULL, NULL,
-                           &tmp_mask);
-
-  if (screen)
-    *screen = gdk_window_get_screen (root);
-  if (x)
-    *x = round (tmp_x);
-  if (y)
-    *y = round (tmp_y);
-  if (mask)
-    *mask = tmp_mask;
-}
-
-/**
- * gdk_display_get_window_at_pointer:
- * @display: a #GdkDisplay
- * @win_x: (out) (allow-none): return location for x coordinate of the pointer location relative
- *    to the window origin, or %NULL
- * @win_y: (out) (allow-none): return location for y coordinate of the pointer location relative
- &    to the window origin, or %NULL
- *
- * Obtains the window underneath the mouse pointer, returning the location
- * of the pointer in that window in @win_x, @win_y for @screen. Returns %NULL
- * if the window under the mouse pointer is not known to GDK (for example, 
- * belongs to another application).
- *
- * Returns: (nullable) (transfer none): the window under the mouse
- *   pointer, or %NULL
- *
- * Since: 2.2
- *
- * Deprecated: 3.0: Use gdk_device_get_window_at_position() instead.
- **/
-GdkWindow *
-gdk_display_get_window_at_pointer (GdkDisplay *display,
-				   gint       *win_x,
-				   gint       *win_y)
-{
-  GdkDevice *pointer;
-
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-
-  pointer = gdk_seat_get_pointer (gdk_display_get_default_seat (display));
-
-  return gdk_device_get_window_at_position (pointer, win_x, win_y);
-}
-
 static void
 generate_grab_broken_event (GdkDisplay *display,
                             GdkWindow  *window,
@@ -1260,11 +1175,6 @@ _gdk_display_has_touch_grab (GdkDisplay       *display,
 {
   guint i;
 
-  g_return_val_if_fail (display, NULL);
-
-  if (!display->touch_implicit_grabs)
-    return NULL;
-
   for (i = 0; i < display->touch_implicit_grabs->len; i++)
     {
       GdkTouchGrabInfo *info;
@@ -1326,11 +1236,6 @@ _gdk_display_check_grab_ownership (GdkDisplay *display,
   GdkGrabOwnership higher_ownership, device_ownership;
   gboolean device_is_keyboard;
 
-  g_return_val_if_fail (display, TRUE);
-
-  if (!display->device_grabs)
-    return TRUE; /* No hash table, no grabs. */
-
   g_hash_table_iter_init (&iter, display->device_grabs);
   higher_ownership = device_ownership = GDK_OWNERSHIP_NONE;
   device_is_keyboard = (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD);
@@ -1423,8 +1328,7 @@ _gdk_display_pointer_info_foreach (GdkDisplay                   *display,
  * @device: device to get the grab information from
  * @grab_window: (out) (transfer none): location to store current grab window
  * @owner_events: (out): location to store boolean indicating whether
- *   the @owner_events flag to gdk_keyboard_grab() or
- *   gdk_pointer_grab() was %TRUE.
+ *   the @owner_events flag to gdk_device_grab() was %TRUE.
  *
  * Determines information about the current keyboard grab.
  * This is not public API and must not be used by applications.
@@ -1458,71 +1362,6 @@ gdk_device_grab_info (GdkDisplay  *display,
     return FALSE;
 }
 
-/**
- * gdk_device_grab_info_libgtk_only:
- * @display: the display for which to get the grab information
- * @device: device to get the grab information from
- * @grab_window: (out) (transfer none): location to store current grab window
- * @owner_events: (out): location to store boolean indicating whether
- *   the @owner_events flag to gdk_keyboard_grab() or
- *   gdk_pointer_grab() was %TRUE.
- *
- * Determines information about the current keyboard grab.
- * This is not public API and must not be used by applications.
- *
- * Returns: %TRUE if this application currently has the
- *  keyboard grabbed.
- *
- * Deprecated: 3.16: The symbol was never meant to be used outside
- *   of GTK+
- */
-gboolean
-gdk_device_grab_info_libgtk_only (GdkDisplay  *display,
-                                  GdkDevice   *device,
-                                  GdkWindow  **grab_window,
-                                  gboolean    *owner_events)
-{
-  return gdk_device_grab_info (display, device, grab_window, owner_events);
-}
-
-/**
- * gdk_display_pointer_is_grabbed:
- * @display: a #GdkDisplay
- *
- * Test if the pointer is grabbed.
- *
- * Returns: %TRUE if an active X pointer grab is in effect
- *
- * Since: 2.2
- *
- * Deprecated: 3.0: Use gdk_display_device_is_grabbed() instead.
- */
-gboolean
-gdk_display_pointer_is_grabbed (GdkDisplay *display)
-{
-  GList *seats, *s;
-  GdkDevice *device;
-
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), TRUE);
-
-  seats = gdk_display_list_seats (display);
-
-  for (s = seats; s; s = s->next)
-    {
-      device = gdk_seat_get_pointer (s->data);
-
-      if (gdk_display_device_is_grabbed (display, device))
-        {
-          g_list_free (seats);
-          return TRUE;
-        }
-    }
-
-  g_list_free (seats);
-
-  return FALSE;
-}
-
 /**
  * gdk_display_device_is_grabbed:
  * @display: a #GdkDisplay
@@ -1590,48 +1429,6 @@ gdk_display_get_name (GdkDisplay *display)
   return GDK_DISPLAY_GET_CLASS (display)->get_name (display);
 }
 
-/**
- * gdk_display_get_n_screens:
- * @display: a #GdkDisplay
- *
- * Gets the number of screen managed by the @display.
- *
- * Returns: number of screens.
- *
- * Since: 2.2
- *
- * Deprecated: 3.10: The number of screens is always 1.
- */
-gint
-gdk_display_get_n_screens (GdkDisplay *display)
-{
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), 0);
-
-  return 1;
-}
-
-/**
- * gdk_display_get_screen:
- * @display: a #GdkDisplay
- * @screen_num: the screen number
- *
- * Returns a screen object for one of the screens of the display.
- *
- * Returns: (transfer none): the #GdkScreen object
- *
- * Since: 2.2
- * Deprecated: 3.20: There is only one screen; use gdk_display_get_default_screen() to get it.
- */
-GdkScreen *
-gdk_display_get_screen (GdkDisplay *display,
-			gint        screen_num)
-{
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-  g_return_val_if_fail (screen_num == 0, NULL);
-
-  return gdk_display_get_default_screen (display);
-}
-
 /**
  * gdk_display_get_default_screen:
  * @display: a #GdkDisplay
@@ -1867,69 +1664,6 @@ gdk_display_supports_input_shapes (GdkDisplay *display)
   return GDK_DISPLAY_GET_CLASS (display)->supports_input_shapes (display);
 }
 
-/**
- * gdk_display_supports_composite:
- * @display: a #GdkDisplay
- *
- * Returns %TRUE if gdk_window_set_composited() can be used
- * to redirect drawing on the window using compositing.
- *
- * Currently this only works on X11 with XComposite and
- * XDamage extensions available.
- *
- * Returns: %TRUE if windows may be composited.
- *
- * Since: 2.12
- *
- * Deprecated: 3.16: Compositing is an outdated technology that
- *   only ever worked on X11.
- */
-gboolean
-gdk_display_supports_composite (GdkDisplay *display)
-{
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
-
-  return GDK_DISPLAY_GET_CLASS (display)->supports_composite (display);
-}
-
-/**
- * gdk_display_list_devices:
- * @display: a #GdkDisplay
- *
- * Returns the list of available input devices attached to @display.
- * The list is statically allocated and should not be freed.
- *
- * Returns: (transfer none) (element-type GdkDevice):
- *     a list of #GdkDevice
- *
- * Since: 2.2
- *
- * Deprecated: 3.0: Use gdk_device_manager_list_devices() instead.
- **/
-GList *
-gdk_display_list_devices (GdkDisplay *display)
-{
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-
-  if (!display->input_devices)
-    {
-      GdkSeat *seat;
-
-      seat = gdk_display_get_default_seat (display);
-
-      /* For backwards compatibility we only include pointing
-       * devices (the core pointer and the slaves).
-       * We store the list since this deprecated function does
-       * not transfer the list ownership.
-       */
-      display->input_devices = gdk_seat_get_slaves (seat, GDK_SEAT_CAPABILITY_ALL_POINTING);
-      display->input_devices = g_list_prepend (display->input_devices, gdk_seat_get_pointer (seat));
-      g_list_foreach (display->input_devices, (GFunc) g_object_ref, NULL);
-    }
-
-  return display->input_devices;
-}
-
 static GdkAppLaunchContext *
 gdk_display_real_get_app_launch_context (GdkDisplay *display)
 {
@@ -2085,43 +1819,6 @@ gdk_display_get_maximal_cursor_size (GdkDisplay *display,
                                                             height);
 }
 
-/**
- * gdk_display_warp_pointer:
- * @display: a #GdkDisplay
- * @screen: the screen of @display to warp the pointer to
- * @x: the x coordinate of the destination
- * @y: the y coordinate of the destination
- *
- * Warps the pointer of @display to the point @x,@y on
- * the screen @screen, unless the pointer is confined
- * to a window by a grab, in which case it will be moved
- * as far as allowed by the grab. Warping the pointer
- * creates events as if the user had moved the mouse
- * instantaneously to the destination.
- *
- * Note that the pointer should normally be under the
- * control of the user. This function was added to cover
- * some rare use cases like keyboard navigation support
- * for the color picker in the #GtkColorSelectionDialog.
- *
- * Since: 2.8
- *
- * Deprecated: 3.0: Use gdk_device_warp() instead.
- */
-void
-gdk_display_warp_pointer (GdkDisplay *display,
-                          GdkScreen  *screen,
-                          gint        x,
-                          gint        y)
-{
-  GdkDevice *pointer;
-
-  g_return_if_fail (GDK_IS_DISPLAY (display));
-
-  pointer = gdk_seat_get_pointer (gdk_display_get_default_seat (display));
-  gdk_device_warp (pointer, screen, x, y);
-}
-
 gulong
 _gdk_display_get_next_serial (GdkDisplay *display)
 {
@@ -2234,22 +1931,22 @@ _gdk_display_create_window_impl (GdkDisplay       *display,
                                  GdkWindow        *real_parent,
                                  GdkScreen        *screen,
                                  GdkEventMask      event_mask,
-                                 GdkWindowAttr    *attributes,
-                                 gint              attributes_mask)
+                                 GdkWindowAttr    *attributes)
 {
   GDK_DISPLAY_GET_CLASS (display)->create_window_impl (display,
                                                        window,
                                                        real_parent,
                                                        screen,
                                                        event_mask,
-                                                       attributes,
-                                                       attributes_mask);
+                                                       attributes);
 }
 
 GdkWindow *
 _gdk_display_create_window (GdkDisplay *display)
 {
-  return g_object_new (GDK_DISPLAY_GET_CLASS (display)->window_type, NULL);
+  return g_object_new (GDK_DISPLAY_GET_CLASS (display)->window_type,
+                       "display", display,
+                       NULL);
 }
 
 /**
@@ -2314,38 +2011,41 @@ void
 gdk_error_trap_push (void)
 {
   GdkDisplayManager *manager;
+  GdkDisplayClass *class;
   GdkGlobalErrorTrap *trap;
-  GSList *displays;
   GSList *l;
 
   manager = gdk_display_manager_get ();
-  displays = gdk_display_manager_list_displays (manager);
+  class = GDK_DISPLAY_GET_CLASS (gdk_display_manager_get_default_display (manager));
 
-  trap = g_slice_new0 (GdkGlobalErrorTrap);
-  for (l = displays; l != NULL; l = l->next)
-    {
-      GdkDisplay *display = l->data;
-      GdkDisplayClass *class = GDK_DISPLAY_GET_CLASS (display);
+  if (class->push_error_trap == NULL)
+    return;
 
-      if (class->push_error_trap != NULL)
-        {
-          class->push_error_trap (display);
-          trap->displays = g_slist_prepend (trap->displays, g_object_ref (display));
-        }
-    }
+  trap = g_slice_new (GdkGlobalErrorTrap);
+  trap->displays = gdk_display_manager_list_displays (manager);
+
+  g_slist_foreach (trap->displays, (GFunc) g_object_ref, NULL);
+  for (l = trap->displays; l != NULL; l = l->next)
+    class->push_error_trap (l->data);
 
   g_queue_push_head (&gdk_error_traps, trap);
-
-  g_slist_free (displays);
 }
 
 static gint
 gdk_error_trap_pop_internal (gboolean need_code)
 {
+  GdkDisplayManager *manager;
+  GdkDisplayClass *class;
   GdkGlobalErrorTrap *trap;
   gint result;
   GSList *l;
 
+  manager = gdk_display_manager_get ();
+  class = GDK_DISPLAY_GET_CLASS (gdk_display_manager_get_default_display (manager));
+
+  if (class->pop_error_trap == NULL)
+    return 0;
+
   trap = g_queue_pop_head (&gdk_error_traps);
 
   g_return_val_if_fail (trap != NULL, 0);
@@ -2353,9 +2053,7 @@ gdk_error_trap_pop_internal (gboolean need_code)
   result = 0;
   for (l = trap->displays; l != NULL; l = l->next)
     {
-      GdkDisplay *display = l->data;
-      GdkDisplayClass *class = GDK_DISPLAY_GET_CLASS (display);
-      gint code = class->pop_error_trap (display, !need_code);
+      gint code = class->pop_error_trap (l->data, !need_code);
 
       /* we use the error on the last display listed, why not. */
       if (code != 0)
@@ -2435,23 +2133,94 @@ gdk_display_set_rendering_mode (GdkDisplay       *display,
   display->rendering_mode = mode;
 }
 
-void
-gdk_display_set_debug_updates (GdkDisplay *display,
-                               gboolean    debug_updates)
+/**
+ * gdk_display_is_composited:
+ * @display: a #GdkDisplay
+ *
+ * Returns whether windows can reasonably be expected to have
+ * their alpha channel drawn correctly on the screen. Check
+ * gdk_display_is_rgba() for wether the display supports an
+ * alpha channel.
+ *
+ * On X11 this function returns whether a compositing manager is
+ * compositing on @display.
+ *
+ * On modern displays, this value is always %TRUE.
+ *
+ * Returns: Whether windows with RGBA visuals can reasonably be
+ * expected to have their alpha channels drawn correctly on the screen.
+ *
+ * Since: 3.90
+ **/
+gboolean
+gdk_display_is_composited (GdkDisplay *display)
 {
-  display->debug_updates = debug_updates;
-  display->debug_updates_set = TRUE;
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
+
+  return display->composited;
 }
 
-gboolean
-gdk_display_get_debug_updates (GdkDisplay *display)
+void
+gdk_display_set_composited (GdkDisplay *display,
+                            gboolean    composited)
 {
-  if (display->debug_updates_set)
-    return display->debug_updates;
-  else
-    return _gdk_debug_updates;
+  g_return_if_fail (GDK_IS_DISPLAY (display));
+
+  if (display->composited == composited)
+    return;
+
+  display->composited = composited;
+
+  g_object_notify_by_pspec (G_OBJECT (display), props[PROP_COMPOSITED]);
 }
 
+/**
+ * gdk_display_is_rgba:
+ * @display: a #GdkDisplay
+ *
+ * Returns wether windows on this @display are created with an
+ * alpha channel.
+ *
+ * Even if a %TRUE is returned, it is possible that the
+ * window’s alpha channel won’t be honored when displaying the
+ * window on the screen: in particular, for X an appropriate
+ * windowing manager and compositing manager must be running to
+ * provide appropriate display. Use gdk_display_is_composited()
+ * to check if that is the case.
+ *
+ * For setting an overall opacity for a top-level window, see
+ * gdk_window_set_opacity().
+ *
+ * On modern displays, this value is always %TRUE.
+ *
+ * Returns: %TRUE if windows are created with an alpha channel or
+ *     %FALSE if the display does not support this functionality.
+ *
+ * Since: 3.90
+ **/
+gboolean
+gdk_display_is_rgba (GdkDisplay *display)
+{
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
+
+  return display->rgba;
+}
+
+void
+gdk_display_set_rgba (GdkDisplay *display,
+                      gboolean    rgba)
+{
+  g_return_if_fail (GDK_IS_DISPLAY (display));
+
+  if (display->rgba == rgba)
+    return;
+
+  display->rgba = rgba;
+
+  g_object_notify_by_pspec (G_OBJECT (display), props[PROP_RGBA]);
+}
+
+
 void
 gdk_display_add_seat (GdkDisplay *display,
                       GdkSeat    *seat)
@@ -2547,34 +2316,6 @@ gdk_display_get_n_monitors (GdkDisplay *display)
   return GDK_DISPLAY_GET_CLASS (display)->get_n_monitors (display);
 }
 
-static GdkMonitor *
-get_fallback_monitor (GdkDisplay *display)
-{
-  static GdkMonitor *monitor = NULL;
-  GdkScreen *screen;
-
-  if (monitor == NULL)
-    {
-      g_warning ("%s does not implement the monitor vfuncs", G_OBJECT_TYPE_NAME (display));
-      monitor = gdk_monitor_new (display);
-      gdk_monitor_set_manufacturer (monitor, "fallback");
-      gdk_monitor_set_position (monitor, 0, 0);
-      gdk_monitor_set_scale_factor (monitor, 1);
-    }
-
-  screen = gdk_display_get_default_screen (display);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  gdk_monitor_set_size (monitor,
-                        gdk_screen_get_width (screen),
-                        gdk_screen_get_height (screen));
-  gdk_monitor_set_physical_size (monitor,
-                                 gdk_screen_get_width_mm (screen),
-                                 gdk_screen_get_height_mm (screen));
-G_GNUC_END_IGNORE_DEPRECATIONS
-
-  return monitor;
-}
-
 /**
  * gdk_display_get_monitor:
  * @display: a #GdkDisplay
@@ -2592,9 +2333,6 @@ gdk_display_get_monitor (GdkDisplay *display,
 {
   g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
 
-  if (GDK_DISPLAY_GET_CLASS (display)->get_monitor == NULL)
-    return get_fallback_monitor (display);
-
   return GDK_DISPLAY_GET_CLASS (display)->get_monitor (display, monitor_num);
 }
 
@@ -2609,8 +2347,13 @@ gdk_display_get_monitor (GdkDisplay *display,
  * manager to place the windows, specialized desktop applications
  * such as panels should place themselves on the primary monitor.
  *
- * Returns: (nullable) (transfer none): the primary monitor, or %NULL if no primary
- *     monitor is configured by the user
+ * If no monitor is the designated primary monitor, any monitor
+ * (usually the first) may be returned. To make sure there is a dedicated
+ * primary monitor, use gdk_monitor_is_primary() on the returned monitor.
+ *
+ * Returns: (transfer none): the primary monitor, or any monitor if no
+ *     primary monitor is configured by the user
+ *
  * Since: 3.22
  */
 GdkMonitor *
@@ -2618,10 +2361,7 @@ gdk_display_get_primary_monitor (GdkDisplay *display)
 {
   g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
 
-  if (GDK_DISPLAY_GET_CLASS (display)->get_primary_monitor)
-    return GDK_DISPLAY_GET_CLASS (display)->get_primary_monitor (display);
-
-  return NULL;
+  return GDK_DISPLAY_GET_CLASS (display)->get_primary_monitor (display);
 }
 
 /**
