@@ -24,6 +24,10 @@
 #include "gdkmonitor.h"
 #include "gdkinternals.h"
 
+#ifdef GDK_RENDERING_VULKAN
+#include <vulkan/vulkan.h>
+#endif
+
 G_BEGIN_DECLS
 
 #define GDK_DISPLAY_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GDK_TYPE_DISPLAY, GdkDisplayClass))
@@ -120,9 +124,16 @@ struct _GdkDisplay
   guint double_click_time;  /* Maximum time between clicks in msecs */
   guint double_click_distance;   /* Maximum distance between clicks in pixels */
 
-  guint has_gl_extension_texture_non_power_of_two : 1;
-  guint has_gl_extension_texture_rectangle : 1;
+#ifdef GDK_RENDERING_VULKAN
+  VkInstance vk_instance;
+  VkDebugReportCallbackEXT vk_debug_callback;
+  VkPhysicalDevice vk_physical_device;
+  VkDevice vk_device;
+  VkQueue vk_queue;
+  uint32_t vk_queue_family_index;
 
+  guint vulkan_refcount;
+#endif /* GDK_RENDERING_VULKAN */
   guint rgba : 1;
   guint composited : 1;
 
@@ -136,6 +147,8 @@ struct _GdkDisplayClass
   GObjectClass parent_class;
 
   GType window_type;          /* type for native windows for this display, set in class_init */
+  GType vk_context_type;      /* type for GdkVulkanContext, must be set if vk_extension_name != NULL */
+  const char *vk_extension_name; /* Name of required windowing vulkan extension or %NULL (default) if Vulkan isn't supported */
 
   const gchar *              (*get_name)           (GdkDisplay *display);
   GdkScreen *                (*get_default_screen) (GdkDisplay *display);
@@ -178,9 +191,6 @@ struct _GdkDisplayClass
 
   GdkAppLaunchContext *      (*get_app_launch_context) (GdkDisplay *display);
 
-  void                       (*before_process_all_updates) (GdkDisplay *display);
-  void                       (*after_process_all_updates)  (GdkDisplay *display);
-
   gulong                     (*get_next_serial) (GdkDisplay *display);
 
   void                       (*notify_startup_complete) (GdkDisplay  *display,
