@@ -65,14 +65,14 @@ GDK_AVAILABLE_IN_ALL GType gdk_setting_action_get_type (void) G_GNUC_CONST;
 GDK_AVAILABLE_IN_ALL GType gdk_owner_change_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_OWNER_CHANGE (gdk_owner_change_get_type ())
 
-/* enumerations from "gdkframeclock.h" */
-GDK_AVAILABLE_IN_ALL GType gdk_frame_clock_phase_get_type (void) G_GNUC_CONST;
-#define GDK_TYPE_FRAME_CLOCK_PHASE (gdk_frame_clock_phase_get_type ())
-
 /* enumerations from "gdkmonitor.h" */
 GDK_AVAILABLE_IN_ALL GType gdk_subpixel_layout_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_SUBPIXEL_LAYOUT (gdk_subpixel_layout_get_type ())
 
+/* enumerations from "gdkframeclock.h" */
+GDK_AVAILABLE_IN_ALL GType gdk_frame_clock_phase_get_type (void) G_GNUC_CONST;
+#define GDK_TYPE_FRAME_CLOCK_PHASE (gdk_frame_clock_phase_get_type ())
+
 /* enumerations from "gdkproperty.h" */
 GDK_AVAILABLE_IN_ALL GType gdk_prop_mode_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_PROP_MODE (gdk_prop_mode_get_type ())
@@ -112,8 +112,6 @@ GDK_AVAILABLE_IN_ALL GType gdk_visual_type_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_VISUAL_TYPE (gdk_visual_type_get_type ())
 
 /* enumerations from "gdkwindow.h" */
-GDK_AVAILABLE_IN_ALL GType gdk_window_window_class_get_type (void) G_GNUC_CONST;
-#define GDK_TYPE_WINDOW_WINDOW_CLASS (gdk_window_window_class_get_type ())
 GDK_AVAILABLE_IN_ALL GType gdk_window_type_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_WINDOW_TYPE (gdk_window_type_get_type ())
 GDK_AVAILABLE_IN_ALL GType gdk_window_hints_get_type (void) G_GNUC_CONST;
