@@ -663,6 +663,7 @@ gdk_event_copy (const GdkEvent *event)
       new_private->source_device = private->source_device ? g_object_ref (private->source_device) : NULL;
       new_private->seat = private->seat;
       new_private->tool = private->tool;
+      g_set_object (&new_private->user_data, private->user_data);
     }
 
   switch (event->any.type)
@@ -920,6 +921,14 @@ gdk_event_get_time (const GdkEvent *event)
       case GDK_DROP_START:
       case GDK_DROP_FINISHED:
 	return event->dnd.time;
+      case GDK_PAD_BUTTON_PRESS:
+      case GDK_PAD_BUTTON_RELEASE:
+        return event->pad_button.time;
+      case GDK_PAD_RING:
+      case GDK_PAD_STRIP:
+        return event->pad_axis.time;
+      case GDK_PAD_GROUP_MODE:
+        return event->pad_group_mode.time;
       case GDK_CLIENT_EVENT:
       case GDK_VISIBILITY_NOTIFY:
       case GDK_CONFIGURE:
@@ -1023,6 +1032,11 @@ gdk_event_get_state (const GdkEvent        *event,
       case GDK_SETTING:
       case GDK_OWNER_CHANGE:
       case GDK_GRAB_BROKEN:
+      case GDK_PAD_BUTTON_PRESS:
+      case GDK_PAD_BUTTON_RELEASE:
+      case GDK_PAD_RING:
+      case GDK_PAD_STRIP:
+      case GDK_PAD_GROUP_MODE:
       case GDK_EVENT_LAST:
         /* no state field */
         break;
@@ -1185,6 +1199,59 @@ gdk_event_get_root_coords (const GdkEvent *event,
   return fetched;
 }
 
+void
+gdk_event_set_coords (GdkEvent *event,
+                      gdouble   x,
+                      gdouble   y)
+{
+  g_return_if_fail (event != NULL);
+
+  switch (event->type)
+    {
+    case GDK_CONFIGURE:
+      event->configure.x = x;
+      event->configure.y = y;
+      break;
+    case GDK_ENTER_NOTIFY:
+    case GDK_LEAVE_NOTIFY:
+      event->crossing.x = x;
+      event->crossing.y = y;
+      break;
+    case GDK_SCROLL:
+      event->scroll.x = x;
+      event->scroll.y = y;
+      break;
+    case GDK_BUTTON_PRESS:
+    case GDK_2BUTTON_PRESS:
+    case GDK_3BUTTON_PRESS:
+    case GDK_BUTTON_RELEASE:
+      event->button.x = x;
+      event->button.y = y;
+      break;
+    case GDK_TOUCH_BEGIN:
+    case GDK_TOUCH_UPDATE:
+    case GDK_TOUCH_END:
+    case GDK_TOUCH_CANCEL:
+      event->touch.x = x;
+      event->touch.y = y;
+      break;
+    case GDK_MOTION_NOTIFY:
+      event->motion.x = x;
+      event->motion.y = y;
+      break;
+    case GDK_TOUCHPAD_SWIPE:
+      event->touchpad_swipe.x = x;
+      event->touchpad_swipe.y = y;
+      break;
+    case GDK_TOUCHPAD_PINCH:
+      event->touchpad_pinch.x = x;
+      event->touchpad_pinch.y = y;
+      break;
+    default:
+      break;
+    }
+}
+
 /**
  * gdk_event_get_button:
  * @event: a #GdkEvent
@@ -1213,6 +1280,10 @@ gdk_event_get_button (const GdkEvent *event,
     case GDK_BUTTON_RELEASE:
       number = event->button.button;
       break;
+    case GDK_PAD_BUTTON_PRESS:
+    case GDK_PAD_BUTTON_RELEASE:
+      number = event->pad_button.button;
+      break;
     default:
       fetched = FALSE;
       break;
@@ -2507,3 +2578,28 @@ gdk_event_get_scancode (GdkEvent *event)
   private = (GdkEventPrivate *) event;
   return private->key_scancode;
 }
+
+void
+gdk_event_set_user_data (GdkEvent *event,
+                         GObject  *user_data)
+{
+  GdkEventPrivate *private;
+
+  if (!gdk_event_is_allocated (event))
+    return;
+
+  private = (GdkEventPrivate *) event;
+  g_set_object (&private->user_data, user_data);
+}
+
+GObject *
+gdk_event_get_user_data (const GdkEvent *event)
+{
+  GdkEventPrivate *private;
+
+  if (!gdk_event_is_allocated (event))
+    return NULL;
+
+  private = (GdkEventPrivate *) event;
+  return private->user_data;
+}
