@@ -1,7 +1,4 @@
-#include "gdkmarshalers.h"
-
-#include	<glib-object.h>
-
+#include <glib-object.h>
 
 #ifdef G_ENABLE_DEBUG
 #define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
@@ -49,12 +46,101 @@
 #define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
 #endif /* !G_ENABLE_DEBUG */
 
-
 /* VOID:OBJECT (./gdkmarshalers.list:1) */
+/* Prototype for -Wmissing-prototypes */
+extern
+void _gdk_marshal_VOID__OBJECT (GClosure     *closure,
+                                GValue       *return_value,
+                                guint         n_param_values,
+                                const GValue *param_values,
+                                gpointer      invocation_hint,
+                                gpointer      marshal_data);
+void
+_gdk_marshal_VOID__OBJECT (GClosure     *closure,
+                           GValue       *return_value G_GNUC_UNUSED,
+                           guint         n_param_values,
+                           const GValue *param_values,
+                           gpointer      invocation_hint G_GNUC_UNUSED,
+                           gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__OBJECT) (gpointer data1,
+                                             gpointer arg1,
+                                             gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__OBJECT callback;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_object (param_values + 1),
+            data2);
+}
 
 /* VOID:BOOLEAN (./gdkmarshalers.list:2) */
+/* Prototype for -Wmissing-prototypes */
+extern
+void _gdk_marshal_VOID__BOOLEAN (GClosure     *closure,
+                                 GValue       *return_value,
+                                 guint         n_param_values,
+                                 const GValue *param_values,
+                                 gpointer      invocation_hint,
+                                 gpointer      marshal_data);
+void
+_gdk_marshal_VOID__BOOLEAN (GClosure     *closure,
+                            GValue       *return_value G_GNUC_UNUSED,
+                            guint         n_param_values,
+                            const GValue *param_values,
+                            gpointer      invocation_hint G_GNUC_UNUSED,
+                            gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__BOOLEAN) (gpointer data1,
+                                              gboolean arg1,
+                                              gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
+  gpointer data1, data2;
+  GMarshalFunc_VOID__BOOLEAN callback;
+
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_boolean (param_values + 1),
+            data2);
+}
 
 /* VOID:POINTER,POINTER,POINTER (./gdkmarshalers.list:3) */
+/* Prototype for -Wmissing-prototypes */
+extern
+void _gdk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
+                                                 GValue       *return_value,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint,
+                                                 gpointer      marshal_data);
 void
 _gdk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                             GValue       *return_value G_GNUC_UNUSED,
@@ -63,14 +149,14 @@ _gdk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                             gpointer      invocation_hint G_GNUC_UNUSED,
                                             gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_POINTER_POINTER) (gpointer     data1,
-                                                              gpointer     arg_1,
-                                                              gpointer     arg_2,
-                                                              gpointer     arg_3,
-                                                              gpointer     data2);
-  GMarshalFunc_VOID__POINTER_POINTER_POINTER callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__POINTER_POINTER_POINTER) (gpointer data1,
+                                                              gpointer arg1,
+                                                              gpointer arg2,
+                                                              gpointer arg3,
+                                                              gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__POINTER_POINTER_POINTER callback;
 
   g_return_if_fail (n_param_values == 4);
 
@@ -94,19 +180,27 @@ _gdk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
 }
 
 /* OBJECT:VOID (./gdkmarshalers.list:4) */
+/* Prototype for -Wmissing-prototypes */
+extern
+void _gdk_marshal_OBJECT__VOID (GClosure     *closure,
+                                GValue       *return_value,
+                                guint         n_param_values,
+                                const GValue *param_values,
+                                gpointer      invocation_hint,
+                                gpointer      marshal_data);
 void
 _gdk_marshal_OBJECT__VOID (GClosure     *closure,
-                           GValue       *return_value G_GNUC_UNUSED,
+                           GValue       *return_value,
                            guint         n_param_values,
                            const GValue *param_values,
                            gpointer      invocation_hint G_GNUC_UNUSED,
                            gpointer      marshal_data)
 {
-  typedef GObject* (*GMarshalFunc_OBJECT__VOID) (gpointer     data1,
-                                                 gpointer     data2);
-  GMarshalFunc_OBJECT__VOID callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef GObject* (*GMarshalFunc_OBJECT__VOID) (gpointer data1,
+                                                 gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_OBJECT__VOID callback;
   GObject* v_return;
 
   g_return_if_fail (return_value != NULL);
@@ -131,21 +225,29 @@ _gdk_marshal_OBJECT__VOID (GClosure     *closure,
 }
 
 /* OBJECT:DOUBLE,DOUBLE (./gdkmarshalers.list:5) */
+/* Prototype for -Wmissing-prototypes */
+extern
+void _gdk_marshal_OBJECT__DOUBLE_DOUBLE (GClosure     *closure,
+                                         GValue       *return_value,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint,
+                                         gpointer      marshal_data);
 void
 _gdk_marshal_OBJECT__DOUBLE_DOUBLE (GClosure     *closure,
-                                    GValue       *return_value G_GNUC_UNUSED,
+                                    GValue       *return_value,
                                     guint         n_param_values,
                                     const GValue *param_values,
                                     gpointer      invocation_hint G_GNUC_UNUSED,
                                     gpointer      marshal_data)
 {
-  typedef GObject* (*GMarshalFunc_OBJECT__DOUBLE_DOUBLE) (gpointer     data1,
-                                                          gdouble      arg_1,
-                                                          gdouble      arg_2,
-                                                          gpointer     data2);
-  GMarshalFunc_OBJECT__DOUBLE_DOUBLE callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef GObject* (*GMarshalFunc_OBJECT__DOUBLE_DOUBLE) (gpointer data1,
+                                                          gdouble arg1,
+                                                          gdouble arg2,
+                                                          gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_OBJECT__DOUBLE_DOUBLE callback;
   GObject* v_return;
 
   g_return_if_fail (return_value != NULL);
@@ -172,21 +274,29 @@ _gdk_marshal_OBJECT__DOUBLE_DOUBLE (GClosure     *closure,
 }
 
 /* BOXED:INT,INT (./gdkmarshalers.list:6) */
+/* Prototype for -Wmissing-prototypes */
+extern
+void _gdk_marshal_BOXED__INT_INT (GClosure     *closure,
+                                  GValue       *return_value,
+                                  guint         n_param_values,
+                                  const GValue *param_values,
+                                  gpointer      invocation_hint,
+                                  gpointer      marshal_data);
 void
 _gdk_marshal_BOXED__INT_INT (GClosure     *closure,
-                             GValue       *return_value G_GNUC_UNUSED,
+                             GValue       *return_value,
                              guint         n_param_values,
                              const GValue *param_values,
                              gpointer      invocation_hint G_GNUC_UNUSED,
                              gpointer      marshal_data)
 {
-  typedef gpointer (*GMarshalFunc_BOXED__INT_INT) (gpointer     data1,
-                                                   gint         arg_1,
-                                                   gint         arg_2,
-                                                   gpointer     data2);
-  GMarshalFunc_BOXED__INT_INT callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef gpointer (*GMarshalFunc_BOXED__INT_INT) (gpointer data1,
+                                                   gint arg1,
+                                                   gint arg2,
+                                                   gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_BOXED__INT_INT callback;
   gpointer v_return;
 
   g_return_if_fail (return_value != NULL);
@@ -213,6 +323,14 @@ _gdk_marshal_BOXED__INT_INT (GClosure     *closure,
 }
 
 /* VOID:DOUBLE,DOUBLE,POINTER,POINTER (./gdkmarshalers.list:7) */
+/* Prototype for -Wmissing-prototypes */
+extern
+void _gdk_marshal_VOID__DOUBLE_DOUBLE_POINTER_POINTER (GClosure     *closure,
+                                                       GValue       *return_value,
+                                                       guint         n_param_values,
+                                                       const GValue *param_values,
+                                                       gpointer      invocation_hint,
+                                                       gpointer      marshal_data);
 void
 _gdk_marshal_VOID__DOUBLE_DOUBLE_POINTER_POINTER (GClosure     *closure,
                                                   GValue       *return_value G_GNUC_UNUSED,
@@ -221,15 +339,15 @@ _gdk_marshal_VOID__DOUBLE_DOUBLE_POINTER_POINTER (GClosure     *closure,
                                                   gpointer      invocation_hint G_GNUC_UNUSED,
                                                   gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__DOUBLE_DOUBLE_POINTER_POINTER) (gpointer     data1,
-                                                                    gdouble      arg_1,
-                                                                    gdouble      arg_2,
-                                                                    gpointer     arg_3,
-                                                                    gpointer     arg_4,
-                                                                    gpointer     data2);
-  GMarshalFunc_VOID__DOUBLE_DOUBLE_POINTER_POINTER callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__DOUBLE_DOUBLE_POINTER_POINTER) (gpointer data1,
+                                                                    gdouble arg1,
+                                                                    gdouble arg2,
+                                                                    gpointer arg3,
+                                                                    gpointer arg4,
+                                                                    gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__DOUBLE_DOUBLE_POINTER_POINTER callback;
 
   g_return_if_fail (n_param_values == 5);
 
@@ -254,6 +372,14 @@ _gdk_marshal_VOID__DOUBLE_DOUBLE_POINTER_POINTER (GClosure     *closure,
 }
 
 /* VOID:POINTER,POINTER,BOOLEAN,BOOLEAN (./gdkmarshalers.list:8) */
+/* Prototype for -Wmissing-prototypes */
+extern
+void _gdk_marshal_VOID__POINTER_POINTER_BOOLEAN_BOOLEAN (GClosure     *closure,
+                                                         GValue       *return_value,
+                                                         guint         n_param_values,
+                                                         const GValue *param_values,
+                                                         gpointer      invocation_hint,
+                                                         gpointer      marshal_data);
 void
 _gdk_marshal_VOID__POINTER_POINTER_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                     GValue       *return_value G_GNUC_UNUSED,
@@ -262,15 +388,15 @@ _gdk_marshal_VOID__POINTER_POINTER_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                     gpointer      invocation_hint G_GNUC_UNUSED,
                                                     gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_POINTER_BOOLEAN_BOOLEAN) (gpointer     data1,
-                                                                      gpointer     arg_1,
-                                                                      gpointer     arg_2,
-                                                                      gboolean     arg_3,
-                                                                      gboolean     arg_4,
-                                                                      gpointer     data2);
-  GMarshalFunc_VOID__POINTER_POINTER_BOOLEAN_BOOLEAN callback;
-  GCClosure *cc = (GCClosure*) closure;
+  typedef void (*GMarshalFunc_VOID__POINTER_POINTER_BOOLEAN_BOOLEAN) (gpointer data1,
+                                                                      gpointer arg1,
+                                                                      gpointer arg2,
+                                                                      gboolean arg3,
+                                                                      gboolean arg4,
+                                                                      gpointer data2);
+  GCClosure *cc = (GCClosure *) closure;
   gpointer data1, data2;
+  GMarshalFunc_VOID__POINTER_POINTER_BOOLEAN_BOOLEAN callback;
 
   g_return_if_fail (n_param_values == 5);
 
