@@ -26,16 +26,10 @@
 #include "gdkquartzcursor.h"
 #include "gdkprivate-quartz.h"
 #include "gdkquartzdevice-core.h"
-#include "gdkinternal-quartz.h"
-#include "gdkquartz-cocoa-access.h"
 
 struct _GdkQuartzDeviceCore
 {
   GdkDevice parent_instance;
-
-  gboolean active;
-  NSUInteger device_id;
-  unsigned long long unique_id;
 };
 
 struct _GdkQuartzDeviceCoreClass
@@ -134,14 +128,14 @@ gdk_quartz_device_core_get_state (GdkDevice       *device,
                                   gdouble         *axes,
                                   GdkModifierType *mask)
 {
-  gint x_int, y_int;
+  gdouble x_pos, y_pos;
 
-  gdk_window_get_device_position (window, device, &x_int, &y_int, mask);
+  gdk_window_get_device_position_double (window, device, &x_pos, &y_pos, mask);
 
   if (axes)
     {
-      axes[0] = x_int;
-      axes[1] = y_int;
+      axes[0] = x_pos;
+      axes[1] = y_pos;
     }
 }
 
@@ -165,7 +159,7 @@ translate_coords_to_child_coords (GdkWindow *parent,
       *x -= tmp_x;
       *y -= tmp_y;
 
-      current = gdk_window_get_effective_parent (current);
+      current = gdk_window_get_parent (current);
     }
 }
 
@@ -215,7 +209,7 @@ gdk_quartz_device_core_query_state_helper (GdkWindow       *window,
       return NULL;
     }
 
-  toplevel = gdk_window_get_effective_toplevel (window);
+  toplevel = gdk_window_get_toplevel (window);
 
   if (mask)
     *mask = _gdk_quartz_events_get_current_keyboard_modifiers () |
@@ -229,9 +223,11 @@ gdk_quartz_device_core_query_state_helper (GdkWindow       *window,
     }
   else
     {
+      GdkWindowImplQuartz *impl;
       NSWindow *nswindow;
 
-      nswindow = gdk_quartz_window_get_nswindow (window);
+      impl = GDK_WINDOW_IMPL_QUARTZ (toplevel->impl);
+      nswindow = impl->toplevel;
 
       point = [nswindow mouseLocationOutsideOfEventStream];
 
@@ -367,36 +363,3 @@ gdk_quartz_device_core_select_window_events (GdkDevice    *device,
 {
   /* The mask is set in the common code. */
 }
-
-void
-_gdk_quartz_device_core_set_active (GdkDevice  *device,
-                                    gboolean    active,
-                                    NSUInteger  device_id)
-{
-  GdkQuartzDeviceCore *self = GDK_QUARTZ_DEVICE_CORE (device);
-
-  self->active = active;
-  self->device_id = device_id;
-}
-
-gboolean
-_gdk_quartz_device_core_is_active (GdkDevice  *device,
-                                   NSUInteger  device_id)
-{
-  GdkQuartzDeviceCore *self = GDK_QUARTZ_DEVICE_CORE (device);
-
-  return (self->active && self->device_id == device_id);
-}
-
-void
-_gdk_quartz_device_core_set_unique (GdkDevice          *device,
-                                    unsigned long long  unique_id)
-{
-  GDK_QUARTZ_DEVICE_CORE (device)->unique_id = unique_id;
-}
-
-unsigned long long
-_gdk_quartz_device_core_get_unique (GdkDevice *device)
-{
-  return GDK_QUARTZ_DEVICE_CORE (device)->unique_id;
-}
