@@ -29,16 +29,9 @@
 #include <wayland-egl.h>
 #include <gdk/wayland/tablet-unstable-v2-client-protocol.h>
 #include <gdk/wayland/gtk-shell-client-protocol.h>
-#include <gdk/wayland/xdg-shell-client-protocol.h>
 #include <gdk/wayland/xdg-shell-unstable-v6-client-protocol.h>
 #include <gdk/wayland/xdg-foreign-unstable-v1-client-protocol.h>
 #include <gdk/wayland/keyboard-shortcuts-inhibit-unstable-v1-client-protocol.h>
-#include <gdk/wayland/server-decoration-client-protocol.h>
-#include <gdk/wayland/xdg-output-unstable-v1-client-protocol.h>
-#include <gdk/wayland/primary-selection-unstable-v1-client-protocol.h>
-#ifdef HAVE_XDG_ACTIVATION
-#include <gdk/wayland/xdg-activation-v1-client-protocol.h>
-#endif
 
 #include <glib.h>
 #include <gdk/gdkkeys.h>
@@ -52,19 +45,13 @@
 
 G_BEGIN_DECLS
 
-#define GDK_WAYLAND_MAX_THEME_SCALE 4
+#define GDK_WAYLAND_MAX_THEME_SCALE 2
 #define GDK_WAYLAND_THEME_SCALES_COUNT GDK_WAYLAND_MAX_THEME_SCALE
 
 #define GDK_ZWP_POINTER_GESTURES_V1_VERSION 1
 
 typedef struct _GdkWaylandSelection GdkWaylandSelection;
 
-typedef enum _GdkWaylandShellVariant
-{
-  GDK_WAYLAND_SHELL_VARIANT_XDG_SHELL,
-  GDK_WAYLAND_SHELL_VARIANT_ZXDG_SHELL_V6,
-} GdkWaylandShellVariant;
-
 struct _GdkWaylandDisplay
 {
   GdkDisplay parent_instance;
@@ -76,35 +63,22 @@ struct _GdkWaylandDisplay
   /* Most recent serial */
   guint32 serial;
 
-  uint32_t xdg_wm_base_id;
-  int xdg_wm_base_version;
-  uint32_t zxdg_shell_v6_id;
-  GdkWaylandShellVariant shell_variant;
-
   /* Wayland fields below */
   struct wl_display *wl_display;
   struct wl_registry *wl_registry;
   struct wl_compositor *compositor;
   struct wl_shm *shm;
-  struct xdg_wm_base *xdg_wm_base;
-  struct zxdg_shell_v6 *zxdg_shell_v6;
+  struct zxdg_shell_v6 *xdg_shell;
   struct gtk_shell1 *gtk_shell;
   struct wl_input_device *input_device;
   struct wl_data_device_manager *data_device_manager;
   struct wl_subcompositor *subcompositor;
   struct zwp_pointer_gestures_v1 *pointer_gestures;
-  struct gtk_primary_selection_device_manager *gtk_primary_selection_manager;
-  struct zwp_primary_selection_device_manager_v1 *zwp_primary_selection_manager_v1;
+  struct gtk_primary_selection_device_manager *primary_selection_manager;
   struct zwp_tablet_manager_v2 *tablet_manager;
   struct zxdg_exporter_v1 *xdg_exporter;
   struct zxdg_importer_v1 *xdg_importer;
   struct zwp_keyboard_shortcuts_inhibit_manager_v1 *keyboard_shortcuts_inhibit;
-  struct org_kde_kwin_server_decoration_manager *server_decoration_manager;
-  struct zxdg_output_manager_v1 *xdg_output_manager;
-  uint32_t xdg_output_version;
-#ifdef HAVE_XDG_ACTIVATION
-  struct xdg_activation_v1 *xdg_activation;
-#endif
 
   GList *async_roundtrips;
 
@@ -131,12 +105,6 @@ struct _GdkWaylandDisplay
   int seat_version;
   int data_device_manager_version;
   int gtk_shell_version;
-  int xdg_output_manager_version;
-#ifdef HAVE_XDG_ACTIVATION
-  int xdg_activation_version;
-#endif
-
-  uint32_t server_decoration_mode;
 
   struct xkb_context *xkb_context;
 
@@ -156,7 +124,6 @@ struct _GdkWaylandDisplay
   guint have_egl_buffer_age : 1;
   guint have_egl_swap_buffers_with_damage : 1;
   guint have_egl_surfaceless_context : 1;
-  EGLint egl_min_swap_interval;
 };
 
 struct _GdkWaylandDisplayClass
