@@ -498,21 +498,15 @@ _gdk_wayland_window_register_dnd (GdkWindow *window)
 }
 
 static GdkWindow *
-create_dnd_window (GdkScreen *screen)
+create_dnd_window (GdkDisplay *display)
 {
-  GdkWindowAttr attrs;
-  guint mask;
+  GdkWindow *window;
 
-  attrs.x = attrs.y = 0;
-  attrs.width = attrs.height = 100;
-  attrs.wclass = GDK_INPUT_OUTPUT;
-  attrs.window_type = GDK_WINDOW_TEMP;
-  attrs.type_hint = GDK_WINDOW_TYPE_HINT_DND;
-  attrs.visual = gdk_screen_get_system_visual (screen);
+  window = gdk_window_new_popup (display, 0, &(GdkRectangle) { 0, 0, 100, 100 });
 
-  mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_TYPE_HINT;
-
-  return gdk_window_new (gdk_screen_get_root_window (screen), &attrs, mask);
+  gdk_window_set_type_hint (window, GDK_WINDOW_TYPE_HINT_DND);
+  
+  return window;
 }
 
 GdkDragContext *
@@ -535,7 +529,7 @@ _gdk_wayland_window_drag_begin (GdkWindow *window,
 
   gdk_drag_context_set_device (context, device);
 
-  context_wayland->dnd_window = create_dnd_window (gdk_window_get_screen (window));
+  context_wayland->dnd_window = create_dnd_window (gdk_window_get_display (window));
   context_wayland->dnd_surface = gdk_wayland_window_get_wl_surface (context_wayland->dnd_window);
   context_wayland->data_source =
     gdk_wayland_selection_get_data_source (window,
@@ -549,15 +543,6 @@ _gdk_wayland_window_drag_begin (GdkWindow *window,
       g_free (mimetype);
     }
 
-  /* If there's no targets this is local DnD, ensure we create a target for it */
-  if (!context->targets)
-    {
-      gchar *local_dnd_mime;
-      local_dnd_mime = g_strdup_printf ("application/gtk+-local-dnd-%x", getpid());
-      wl_data_source_offer (context_wayland->data_source, local_dnd_mime);
-      g_free (local_dnd_mime);
-    }
-
   return context;
 }
 
