@@ -44,7 +44,6 @@ struct _GdkWin32GLContext
   /* other items */
   guint is_attached : 1;
   guint do_frame_sync : 1;
-  guint do_blit_swap : 1;
 };
 
 struct _GdkWin32GLContextClass
@@ -58,23 +57,10 @@ _gdk_win32_window_create_gl_context (GdkWindow *window,
                                      GdkGLContext *share,
                                      GError **error);
 
-void
-_gdk_win32_window_invalidate_for_new_frame (GdkWindow *window,
-                                            cairo_region_t *update_area);
-
-void
-_gdk_win32_gl_context_end_frame (GdkGLContext *context,
-                                 cairo_region_t *painted,
-                                 cairo_region_t *damage);
-
 gboolean
 _gdk_win32_display_make_gl_context_current (GdkDisplay *display,
                                             GdkGLContext *context);
 
-gboolean
-_gdk_win32_gl_context_realize (GdkGLContext *context,
-                               GError **error);
-
 G_END_DECLS
 
 #endif /* __GDK_WIN32_GL_CONTEXT__ */
