@@ -17,6 +17,8 @@ GDK_AVAILABLE_IN_ALL GType gsk_blend_mode_get_type (void) G_GNUC_CONST;
 #define GSK_TYPE_BLEND_MODE (gsk_blend_mode_get_type ())
 GDK_AVAILABLE_IN_ALL GType gsk_corner_get_type (void) G_GNUC_CONST;
 #define GSK_TYPE_CORNER (gsk_corner_get_type ())
+GDK_AVAILABLE_IN_ALL GType gsk_serialization_error_get_type (void) G_GNUC_CONST;
+#define GSK_TYPE_SERIALIZATION_ERROR (gsk_serialization_error_get_type ())
 G_END_DECLS
 
 #endif /* __GSK_ENUM_TYPES_H__ */
