@@ -22,12 +22,8 @@
 #define SHADER_VERSION_GL3              150
 
 typedef struct {
-  int render_target_id;
-  int vao_id;
-  int buffer_id;
-  int texture_id;
-  int program_id;
-
+  int id;
+  /* Common locations (gl_common)*/
   int mvp_location;
   int source_location;
   int mask_location;
@@ -35,9 +31,33 @@ typedef struct {
   int position_location;
   int alpha_location;
   int blendMode_location;
-} RenderData;
+
+  /* Shader-specific locations */
+  union {
+    struct {
+      int color_location;
+    };
+  };
+} Program;
 
 typedef struct {
+  int render_target_id;
+  int vao_id;
+  int buffer_id;
+  int texture_id;
+  int program_id;
+
+  Program *program;
+} RenderData;
+
+enum {
+  MODE_COLOR = 1,
+  MODE_TEXTURE,
+  N_MODES
+};
+
+typedef struct {
+  int mode;
   /* Back pointer to the node, only meant for comparison */
   GskRenderNode *node;
 
@@ -51,6 +71,15 @@ typedef struct {
   float opacity;
   float z;
 
+  union {
+    struct {
+      GdkRGBA color;
+    } color_data;
+    struct {
+      int a,b;
+    } texture_data;
+  };
+
   const char *name;
 
   GskBlendMode blend_mode;
@@ -61,6 +90,8 @@ typedef struct {
   GArray *children;
 } RenderItem;
 
+
+
 enum {
   MVP,
   SOURCE,
@@ -93,6 +124,8 @@ typedef enum {
   RENDER_SCISSOR
 } RenderMode;
 
+#define NUM_PROGRAMS 3
+
 struct _GskGLRenderer
 {
   GskRenderer parent_instance;
@@ -112,8 +145,16 @@ struct _GskGLRenderer
   GskGLProfiler *gl_profiler;
   GskShaderBuilder *shader_builder;
 
-  int blend_program_id;
-  int blit_program_id;
+  union {
+    struct {
+      Program blend_program;
+      Program blit_program;
+      Program color_program;
+    };
+    struct {
+      Program programs[NUM_PROGRAMS];
+    };
+  };
 
   GArray *render_items;
 
@@ -140,6 +181,7 @@ gsk_gl_renderer_dispose (GObject *gobject)
   GskGLRenderer *self = GSK_GL_RENDERER (gobject);
 
   g_clear_object (&self->gl_context);
+  g_clear_pointer (&self->render_items, g_array_unref);
 
   G_OBJECT_CLASS (gsk_gl_renderer_parent_class)->dispose (gobject);
 }
@@ -191,6 +233,27 @@ gsk_gl_renderer_destroy_buffers (GskGLRenderer *self)
   self->has_buffers = FALSE;
 }
 
+static void
+init_common_locations (GskGLRenderer *self,
+                       Program       *prog)
+{
+  prog->source_location =
+    gsk_shader_builder_get_uniform_location (self->shader_builder, prog->id, self->uniforms[SOURCE]);
+  prog->mask_location =
+    gsk_shader_builder_get_uniform_location (self->shader_builder, prog->id, self->uniforms[MASK]);
+  prog->mvp_location =
+    gsk_shader_builder_get_uniform_location (self->shader_builder, prog->id, self->uniforms[MVP]);
+  prog->alpha_location =
+    gsk_shader_builder_get_uniform_location (self->shader_builder, prog->id, self->uniforms[ALPHA]);
+  prog->blendMode_location =
+    gsk_shader_builder_get_uniform_location (self->shader_builder, prog->id, self->uniforms[BLEND_MODE]);
+
+  prog->position_location =
+    gsk_shader_builder_get_attribute_location (self->shader_builder, prog->id, self->attributes[POSITION]);
+  prog->uv_location =
+    gsk_shader_builder_get_attribute_location (self->shader_builder, prog->id, self->attributes[UV]);
+}
+
 static gboolean
 gsk_gl_renderer_create_programs (GskGLRenderer  *self,
                                  GError        **error)
@@ -245,8 +308,12 @@ gsk_gl_renderer_create_programs (GskGLRenderer  *self,
   if (GSK_RENDER_MODE_CHECK (SHADERS))
     gsk_shader_builder_add_define (builder, "GSK_DEBUG", "1");
 #endif
+  /* Keep a pointer to query for the uniform and attribute locations
+   * when rendering the scene
+   */
+  self->shader_builder = builder;
 
-  self->blend_program_id =
+  self->blend_program.id = 
     gsk_shader_builder_create_program (builder, "blend.vs.glsl", "blend.fs.glsl", &shader_error);
   if (shader_error != NULL)
     {
@@ -256,8 +323,9 @@ gsk_gl_renderer_create_programs (GskGLRenderer  *self,
       g_object_unref (builder);
       goto out;
     }
+  init_common_locations (self, &self->blend_program);
 
-  self->blit_program_id =
+  self->blit_program.id =
     gsk_shader_builder_create_program (builder, "blit.vs.glsl", "blit.fs.glsl", &shader_error);
   if (shader_error != NULL)
     {
@@ -267,11 +335,24 @@ gsk_gl_renderer_create_programs (GskGLRenderer  *self,
       g_object_unref (builder);
       goto out;
     }
+  init_common_locations (self, &self->blit_program);
 
-  /* Keep a pointer to query for the uniform and attribute locations
-   * when rendering the scene
-   */
-  self->shader_builder = builder;
+  self->color_program.id =
+    gsk_shader_builder_create_program (builder, "color.vs.glsl", "color.fs.glsl", &shader_error);
+  if (shader_error != NULL)
+    {
+      g_propagate_prefixed_error (error,
+                                  shader_error,
+                                  "Unable to create 'color' program: ");
+      g_object_unref (builder);
+      goto out;
+    }
+  init_common_locations (self, &self->color_program);
+  self->color_program.color_location = gsk_shader_builder_get_uniform_location (self->shader_builder,
+                                                                                self->color_program.id,
+                                                                                g_quark_from_string("uColor"));
+  self->color_program.color_location = glGetUniformLocation(self->color_program.id, "uColor");
+  g_assert(self->color_program.color_location >= 0);
 
   res = TRUE;
 
@@ -443,25 +524,43 @@ render_item (GskGLRenderer *self,
 
   gsk_gl_driver_bind_vao (self->gl_driver, item->render_data.vao_id);
 
-  glUseProgram (item->render_data.program_id);
+  glUseProgram (item->render_data.program->id);
 
-  if (item->render_data.texture_id != 0)
+  switch(item->mode)
     {
-      /* Use texture unit 0 for the source */
-      glUniform1i (item->render_data.source_location, 0);
-      gsk_gl_driver_bind_source_texture (self->gl_driver, item->render_data.texture_id);
-
-      if (item->parent_data != NULL)
+      case MODE_COLOR:
         {
-          glUniform1i (item->render_data.blendMode_location, item->blend_mode);
+          glUniform4f (item->render_data.program->color_location,
+                       item->color_data.color.red,
+                       item->color_data.color.green,
+                       item->color_data.color.blue,
+                       item->color_data.color.alpha);
+        }
+      break;
 
-          /* Use texture unit 1 for the mask */
-          if (item->parent_data->texture_id != 0)
+      case MODE_TEXTURE:
+        {
+          g_assert(item->render_data.texture_id != 0);
+          /* Use texture unit 0 for the source */
+          glUniform1i (item->render_data.program->source_location, 0);
+          gsk_gl_driver_bind_source_texture (self->gl_driver, item->render_data.texture_id);
+
+          if (item->parent_data != NULL)
             {
-              glUniform1i (item->render_data.mask_location, 1);
-              gsk_gl_driver_bind_mask_texture (self->gl_driver, item->parent_data->texture_id);
+              glUniform1i (item->render_data.program->blendMode_location, item->blend_mode);
+
+              /* Use texture unit 1 for the mask */
+              if (item->parent_data->texture_id != 0)
+                {
+                  glUniform1i (item->render_data.program->mask_location, 1);
+                  gsk_gl_driver_bind_mask_texture (self->gl_driver, item->parent_data->texture_id);
+                }
             }
         }
+      break;
+
+      default:
+        g_assert_not_reached ();
     }
 
   /* Pass the opacity component */
@@ -470,12 +569,12 @@ render_item (GskGLRenderer *self,
   else
     opacity = item->opacity;
 
-  glUniform1f (item->render_data.alpha_location, opacity);
+  glUniform1f (item->render_data.program->alpha_location, opacity);
 
   /* Pass the mvp to the vertex shader */
   GSK_NOTE (TRANSFORMS, graphene_matrix_print (&item->mvp));
   graphene_matrix_to_float (&item->mvp, mvp);
-  glUniformMatrix4fv (item->render_data.mvp_location, 1, GL_FALSE, mvp);
+  glUniformMatrix4fv (item->render_data.program->mvp_location, 1, GL_FALSE, mvp);
 
   /* Draw the quad */
   GSK_NOTE2 (OPENGL, TRANSFORMS,
@@ -517,21 +616,21 @@ render_item (GskGLRenderer *self,
       gsk_gl_driver_bind_vao (self->gl_driver, item->render_data.vao_id);
 
       /* Since we're rendering the target texture, we only need the blit program */
-      glUseProgram (self->blit_program_id);
+      glUseProgram (self->blit_program.id);
 
       /* Use texture unit 0 for the render target */
-      glUniform1i (item->render_data.source_location, 0);
+      glUniform1i (item->render_data.program->source_location, 0);
       gsk_gl_driver_bind_source_texture (self->gl_driver, item->render_data.render_target_id);
 
       /* Pass the opacity component; if we got here, we know that the original render
        * target is neither fully opaque nor at full opacity
        */
-      glUniform1f (item->render_data.alpha_location, item->opacity);
+      glUniform1f (item->render_data.program->alpha_location, item->opacity);
 
       /* Pass the mvp to the vertex shader */
       GSK_NOTE (TRANSFORMS, graphene_matrix_print (&item->mvp));
       graphene_matrix_to_float (&item->mvp, mvp);
-      glUniformMatrix4fv (item->render_data.mvp_location, 1, GL_FALSE, mvp);
+      glUniformMatrix4fv (item->render_data.program->mvp_location, 1, GL_FALSE, mvp);
 
       /* Draw the quad */
       GSK_NOTE2 (OPENGL, TRANSFORMS,
@@ -666,28 +765,15 @@ gsk_gl_renderer_add_render_item (GskGLRenderer           *self,
 
   /* Select the program to use */
   if (parent != NULL)
-    program_id = self->blend_program_id;
+    {
+      item.render_data.program = &self->blend_program;
+      program_id = self->blend_program.id;
+    }
   else
-    program_id = self->blit_program_id;
-
-  item.render_data.program_id = program_id;
-
-  /* Retrieve all the uniforms and attributes */
-  item.render_data.source_location =
-    gsk_shader_builder_get_uniform_location (self->shader_builder, program_id, self->uniforms[SOURCE]);
-  item.render_data.mask_location =
-    gsk_shader_builder_get_uniform_location (self->shader_builder, program_id, self->uniforms[MASK]);
-  item.render_data.mvp_location =
-    gsk_shader_builder_get_uniform_location (self->shader_builder, program_id, self->uniforms[MVP]);
-  item.render_data.alpha_location =
-    gsk_shader_builder_get_uniform_location (self->shader_builder, program_id, self->uniforms[ALPHA]);
-  item.render_data.blendMode_location =
-    gsk_shader_builder_get_uniform_location (self->shader_builder, program_id, self->uniforms[BLEND_MODE]);
-
-  item.render_data.position_location =
-    gsk_shader_builder_get_attribute_location (self->shader_builder, program_id, self->attributes[POSITION]);
-  item.render_data.uv_location =
-    gsk_shader_builder_get_attribute_location (self->shader_builder, program_id, self->attributes[UV]);
+    {
+      item.render_data.program = &self->blit_program;
+      program_id = self->blit_program.id;
+    }
 
   if (render_node_needs_render_target (node))
     {
@@ -717,6 +803,7 @@ gsk_gl_renderer_add_render_item (GskGLRenderer           *self,
                                                                              texture,
                                                                              gl_min_filter,
                                                                              gl_mag_filter);
+        item.mode = MODE_TEXTURE;
       }
       break;
 
@@ -740,14 +827,71 @@ gsk_gl_renderer_add_render_item (GskGLRenderer           *self,
                                                  surface,
                                                  gl_min_filter,
                                                  gl_mag_filter);
+        item.mode = MODE_TEXTURE;
       }
       break;
 
+    case GSK_COLOR_NODE:
+      {
+        const GdkRGBA *c = gsk_color_node_peek_color(node);
+        program_id = self->color_program.id;
+        item.render_data.program = &self->color_program;
+        item.mode = MODE_COLOR;
+        item.color_data.color= *c;
+      }
+      break;
+
+    case GSK_COLOR_MATRIX_NODE:
+      {
+        GskRenderNode *child = gsk_color_matrix_node_get_child (node);
+
+        gsk_gl_renderer_add_render_item (self, projection, modelview, render_items, child, ritem);
+      }
+      return;
+
+    case GSK_SHADOW_NODE:
+      {
+        GskRenderNode *child = gsk_shadow_node_get_child (node);
+
+        gsk_gl_renderer_add_render_item (self, projection, modelview, render_items, child, ritem);
+      }
+      return;
+
+    case GSK_REPEAT_NODE:
+      {
+        GskRenderNode *child = gsk_repeat_node_get_child (node);
+
+        gsk_gl_renderer_add_render_item (self, projection, modelview, render_items, child, ritem);
+      }
+      return;
+
+    case GSK_BLEND_NODE:
+      {
+        GskRenderNode *child = gsk_blend_node_get_bottom_child (node);
+
+        gsk_gl_renderer_add_render_item (self, projection, modelview, render_items, child, ritem);
+
+        child = gsk_blend_node_get_top_child (node);
+        gsk_gl_renderer_add_render_item (self, projection, modelview, render_items, child, ritem);
+      }
+      return;
+
+    case GSK_CROSS_FADE_NODE:
+      {
+        GskRenderNode *child = gsk_cross_fade_node_get_start_child (node);
+
+        gsk_gl_renderer_add_render_item (self, projection, modelview, render_items, child, ritem);
+
+        child = gsk_cross_fade_node_get_end_child (node);
+        gsk_gl_renderer_add_render_item (self, projection, modelview, render_items, child, ritem);
+      }
+      return;
+
     case GSK_CONTAINER_NODE:
       {
-        guint i;
+        guint i, p;
 
-        for (i = 0; i < gsk_container_node_get_n_children (node); i++)
+        for (i = 0, p = gsk_container_node_get_n_children (node); i < p; i++)
           {
             GskRenderNode *child = gsk_container_node_get_child (node, i);
             gsk_gl_renderer_add_render_item (self, projection, modelview, render_items, child, ritem);
@@ -779,13 +923,14 @@ gsk_gl_renderer_add_render_item (GskGLRenderer           *self,
         cairo_t *cr;
 
         surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                                              ceil (node->bounds.size.width),
-                                              ceil (node->bounds.size.height));
+                                              item.size.width,
+                                              item.size.height);
+        cairo_surface_set_device_scale (surface, scale_factor, scale_factor);
         cr = cairo_create (surface);
         cairo_translate (cr, -node->bounds.origin.x, -node->bounds.origin.y);
 
         gsk_render_node_draw (node, cr);
-        
+
         cairo_destroy (cr);
 
         /* Upload the Cairo surface to a GL texture */
@@ -799,10 +944,13 @@ gsk_gl_renderer_add_render_item (GskGLRenderer           *self,
                                                  GL_NEAREST, GL_NEAREST);
 
         cairo_surface_destroy (surface);
+        item.mode = MODE_TEXTURE;
       }
       break;
     }
 
+  item.render_data.program_id = program_id;
+
   /* Create the vertex buffers holding the geometry of the quad */
   {
     GskQuadVertex vertex_data[N_VERTICES] = {
@@ -817,8 +965,8 @@ gsk_gl_renderer_add_render_item (GskGLRenderer           *self,
 
     item.render_data.vao_id =
       gsk_gl_driver_create_vao_for_quad (self->gl_driver,
-                                         item.render_data.position_location,
-                                         item.render_data.uv_location,
+                                         item.render_data.program->position_location,
+                                         item.render_data.program->uv_location,
                                          N_VERTICES,
                                          vertex_data);
   }
@@ -850,8 +998,6 @@ gsk_gl_renderer_validate_tree (GskGLRenderer           *self,
 
   gdk_gl_context_make_current (self->gl_context);
 
-  self->render_items = g_array_new (FALSE, FALSE, sizeof (RenderItem));
-
   gsk_gl_driver_begin_frame (self->gl_driver);
 
   GSK_NOTE (OPENGL, g_print ("RenderNode -> RenderItem\n"));
@@ -875,7 +1021,7 @@ gsk_gl_renderer_clear_tree (GskGLRenderer *self)
 
   gdk_gl_context_make_current (self->gl_context);
 
-  g_clear_pointer (&self->render_items, g_array_unref);
+  g_array_remove_range (self->render_items, 0, self->render_items->len);
 
   removed_textures = gsk_gl_driver_collect_textures (self->gl_driver);
   removed_vaos = gsk_gl_driver_collect_vaos (self->gl_driver);
@@ -907,8 +1053,13 @@ gsk_gl_renderer_setup_render_mode (GskGLRenderer *self)
         GdkDrawingContext *context = gsk_renderer_get_drawing_context (GSK_RENDERER (self));
         GdkWindow *window = gsk_renderer_get_window (GSK_RENDERER (self));
         GdkRectangle extents;
+        int scale_factor = gsk_renderer_get_scale_factor (GSK_RENDERER (self));
+
         cairo_region_get_extents (gdk_drawing_context_get_clip (context), &extents);
-        glScissor (extents.x, gdk_window_get_height (window) - extents.height - extents.y, extents.width, extents.height);
+
+        glScissor (extents.x * scale_factor,
+                   (gdk_window_get_height (window) - extents.height - extents.y) * scale_factor,
+                   extents.width * scale_factor, extents.height * scale_factor);
         glEnable (GL_SCISSOR_TEST);
         break;
       }
@@ -1087,6 +1238,8 @@ gsk_gl_renderer_init (GskGLRenderer *self)
 
   graphene_matrix_init_identity (&self->mvp);
 
+  self->render_items = g_array_new (FALSE, FALSE, sizeof (RenderItem));
+
 #ifdef G_ENABLE_DEBUG
   {
     GskProfiler *profiler = gsk_renderer_get_profiler (GSK_RENDERER (self));
