@@ -166,13 +166,16 @@ GskRenderNode *         gsk_blend_node_new                      (GskRenderNode
                                                                  GskRenderNode            *top,
                                                                  GskBlendMode              blend_mode);
 
+GDK_AVAILABLE_IN_3_92
+GskBlendMode            gsk_blend_node_get_blend_mode           (GskRenderNode            *node);
+
 GDK_AVAILABLE_IN_3_90
 GskRenderNode *         gsk_cross_fade_node_new                 (GskRenderNode            *start,
                                                                  GskRenderNode            *end,
                                                                  double                    progress);
 
 GDK_AVAILABLE_IN_3_90
-void                    gsk_render_node_set_scaling_filter      (GskRenderNode *node,
+void                    gsk_render_node_set_scaling_filters     (GskRenderNode *node,
                                                                  GskScalingFilter min_filter,
                                                                  GskScalingFilter mag_filter);
 
@@ -183,7 +186,7 @@ GDK_AVAILABLE_IN_3_90
 const char *            gsk_render_node_get_name                (GskRenderNode *node);
 GDK_AVAILABLE_IN_3_90
 void                    gsk_render_node_get_bounds              (GskRenderNode   *node,
-                                                                 graphene_rect_t *frame);
+                                                                 graphene_rect_t *bounds);
 
 GDK_AVAILABLE_IN_3_90
 void                    gsk_render_node_draw                    (GskRenderNode *node,
