@@ -25,7 +25,6 @@
 #include "gtkentryprivate.h"
 #include "gtkcomboboxaccessible.h"
 #include "gtkstylecontextprivate.h"
-#include "gtkwidgetprivate.h"
 
 #define GTK_TYPE_ENTRY_ICON_ACCESSIBLE      (gtk_entry_icon_accessible_get_type ())
 #define GTK_ENTRY_ICON_ACCESSIBLE(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_ENTRY_ICON_ACCESSIBLE, GtkEntryIconAccessible))
@@ -54,13 +53,9 @@ struct _GtkEntryIconAccessibleClass
   AtkObjectClass parent_class;
 };
 
-static void atk_action_interface_init (AtkActionIface *iface);
-
 static void icon_atk_action_interface_init (AtkActionIface *iface);
 static void icon_atk_component_interface_init (AtkComponentIface *iface);
 
-GType gtk_entry_icon_accessible_get_type (void);
-
 G_DEFINE_TYPE_WITH_CODE (GtkEntryIconAccessible, gtk_entry_icon_accessible, ATK_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (ATK_TYPE_ACTION, icon_atk_action_interface_init)
                          G_IMPLEMENT_INTERFACE (ATK_TYPE_COMPONENT, icon_atk_component_interface_init))
@@ -937,8 +932,7 @@ gtk_entry_accessible_get_default_attributes (AtkText *text)
   attributes = _gtk_pango_get_default_attributes (attributes,
                                                   gtk_entry_get_layout (GTK_ENTRY (widget)));
   attributes = _gtk_style_context_get_attributes (attributes,
-                                                  gtk_widget_get_style_context (widget),
-                                                  gtk_widget_get_state_flags (widget));
+                                                  gtk_widget_get_style_context (widget));
 
   return attributes;
 }
@@ -959,7 +953,6 @@ gtk_entry_accessible_get_character_extents (AtkText      *text,
   gint index, x_layout, y_layout;
   GdkWindow *window;
   gint x_window, y_window;
-  GtkAllocation allocation;
 
   widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
@@ -975,13 +968,11 @@ gtk_entry_accessible_get_character_extents (AtkText      *text,
   pango_layout_index_to_pos (gtk_entry_get_layout (entry), index, &char_rect);
   pango_extents_to_pixels (&char_rect, NULL);
 
-  _gtk_widget_get_allocation (widget, &allocation);
-
   window = gtk_widget_get_window (widget);
   gdk_window_get_origin (window, &x_window, &y_window);
 
-  *x = x_window + allocation.x + x_layout + char_rect.x;
-  *y = y_window + allocation.y + y_layout + char_rect.y;
+  *x = x_window + x_layout + char_rect.x;
+  *y = y_window + y_layout + char_rect.y;
   *width = char_rect.width;
   *height = char_rect.height;
 
@@ -1433,7 +1424,7 @@ delete_text_cb (GtkEditable *editable,
   g_signal_emit_by_name (accessible,
                          "text-changed::delete",
                          start,
-                         end - start);
+                         end);
 }
 
 static gboolean
