@@ -28,7 +28,8 @@ struct _GtkSpinButtonAccessiblePrivate
 
 static void atk_value_interface_init (AtkValueIface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (GtkSpinButtonAccessible, gtk_spin_button_accessible, GTK_TYPE_ENTRY_ACCESSIBLE,
+G_DEFINE_TYPE_WITH_CODE (GtkSpinButtonAccessible, gtk_spin_button_accessible, GTK_TYPE_WIDGET_ACCESSIBLE,
+                         G_ADD_PRIVATE (GtkSpinButtonAccessible)
                          G_IMPLEMENT_INTERFACE (ATK_TYPE_VALUE, atk_value_interface_init))
 
 static void
@@ -111,17 +112,12 @@ gtk_spin_button_accessible_class_init (GtkSpinButtonAccessibleClass *klass)
   accessible_class->widget_unset = gtk_spin_button_accessible_widget_unset;
 
   widget_class->notify_gtk = gtk_spin_button_accessible_notify_gtk;
-
-  g_type_class_add_private (klass, sizeof (GtkSpinButtonAccessiblePrivate));
 }
 
 static void
 gtk_spin_button_accessible_init (GtkSpinButtonAccessible *button)
 {
-  button->priv = G_TYPE_INSTANCE_GET_PRIVATE (button,
-                                              GTK_TYPE_SPIN_BUTTON_ACCESSIBLE,
-                                              GtkSpinButtonAccessiblePrivate);
-
+  button->priv = gtk_spin_button_accessible_get_instance_private (button);
 }
 
 static void
@@ -209,6 +205,68 @@ gtk_spin_button_accessible_set_current_value (AtkValue     *obj,
   return TRUE;
 }
 
+static void
+gtk_spin_button_accessible_get_value_and_text (AtkValue  *obj,
+                                         gdouble   *value,
+                                         gchar    **text)
+{
+  GtkWidget *widget;
+  GtkAdjustment *adjustment;
+
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+  adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (widget));
+  if (adjustment == NULL)
+    return;
+
+  *value = gtk_adjustment_get_value (adjustment);
+  *text = NULL;
+}
+
+static AtkRange *
+gtk_spin_button_accessible_get_range (AtkValue *obj)
+{
+  GtkWidget *widget;
+  GtkAdjustment *adjustment;
+
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+  adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (widget));
+  if (adjustment == NULL)
+    return NULL;
+
+  return atk_range_new (gtk_adjustment_get_lower (adjustment),
+                        gtk_adjustment_get_upper (adjustment),
+                        NULL);
+}
+
+static void
+gtk_spin_button_accessible_set_value (AtkValue      *obj,
+                                const gdouble  value)
+{
+  GtkWidget *widget;
+  GtkAdjustment *adjustment;
+
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+  adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (widget));
+  if (adjustment == NULL)
+    return;
+
+  gtk_adjustment_set_value (adjustment, value);
+}
+
+static gdouble
+gtk_spin_button_accessible_get_increment (AtkValue *obj)
+{
+  GtkWidget *widget;
+  GtkAdjustment *adjustment;
+
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+  adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (widget));
+  if (adjustment == NULL)
+    return 0;
+
+  return gtk_adjustment_get_minimum_increment (adjustment);
+}
+
 static void
 atk_value_interface_init (AtkValueIface *iface)
 {
@@ -217,4 +275,9 @@ atk_value_interface_init (AtkValueIface *iface)
   iface->get_minimum_value = gtk_spin_button_accessible_get_minimum_value;
   iface->get_minimum_increment = gtk_spin_button_accessible_get_minimum_increment;
   iface->set_current_value = gtk_spin_button_accessible_set_current_value;
+
+  iface->get_value_and_text = gtk_spin_button_accessible_get_value_and_text;
+  iface->get_range = gtk_spin_button_accessible_get_range;
+  iface->set_value = gtk_spin_button_accessible_set_value;
+  iface->get_increment = gtk_spin_button_accessible_get_increment;
 }
