@@ -30,6 +30,7 @@
 #include <gtk/gtk.h>
 #include "gtktextviewaccessibleprivate.h"
 #include "gtktextbufferprivate.h"
+#include "gtktextviewprivate.h"
 #include "gtk/gtkwidgetprivate.h"
 
 struct _GtkTextViewAccessiblePrivate
@@ -1124,15 +1125,15 @@ gtk_text_view_accessible_get_default_attributes (AtkText *text)
                                        text_attrs->appearance.underline);
 
   value = g_strdup_printf ("%u,%u,%u",
-                           text_attrs->appearance.bg_color.red,
-                           text_attrs->appearance.bg_color.green,
-                           text_attrs->appearance.bg_color.blue);
+                           (guint)(text_attrs->appearance.bg_rgba->red * 65535),
+                           (guint)(text_attrs->appearance.bg_rgba->green * 65535),
+                           (guint)(text_attrs->appearance.bg_rgba->blue * 65535));
   attributes = add_text_attribute (attributes, ATK_TEXT_ATTR_BG_COLOR, value);
 
   value = g_strdup_printf ("%u,%u,%u",
-                           text_attrs->appearance.fg_color.red,
-                           text_attrs->appearance.fg_color.green,
-                           text_attrs->appearance.fg_color.blue);
+                           (guint)(text_attrs->appearance.fg_rgba->red),
+                           (guint)(text_attrs->appearance.fg_rgba->green),
+                           (guint)(text_attrs->appearance.fg_rgba->blue));
   attributes = add_text_attribute (attributes, ATK_TEXT_ATTR_FG_COLOR, value);
 
   value = g_strdup_printf ("%g", text_attrs->font_scale);
@@ -1341,7 +1342,7 @@ gtk_text_view_accessible_set_run_attributes (AtkEditableText *text,
   GtkTextIter start;
   GtkTextIter end;
   gint j;
-  GdkColor *color;
+  GdkRGBA *color;
   gchar** RGB_vals;
   GSList *l;
 
@@ -1450,21 +1451,21 @@ gtk_text_view_accessible_set_run_attributes (AtkEditableText *text,
       else if (!strcmp (name, atk_text_attribute_get_name (ATK_TEXT_ATTR_BG_COLOR)))
         {
           RGB_vals = g_strsplit (value, ",", 3);
-          color = g_malloc (sizeof (GdkColor));
-          color->red = atoi (RGB_vals[0]);
-          color->green = atoi (RGB_vals[1]);
-          color->blue = atoi (RGB_vals[2]);
-          g_object_set (G_OBJECT (tag), "background-gdk", color, NULL);
+          color = g_malloc (sizeof (GdkRGBA));
+          color->red = atoi (RGB_vals[0]) / 65535.0;
+          color->green = atoi (RGB_vals[1]) / 65535.0;
+          color->blue = atoi (RGB_vals[2]) / 65535.0;
+          g_object_set (G_OBJECT (tag), "background-rgba", color, NULL);
         }
  
       else if (!strcmp (name, atk_text_attribute_get_name (ATK_TEXT_ATTR_FG_COLOR)))
         {
           RGB_vals = g_strsplit (value, ",", 3);
-          color = g_malloc (sizeof (GdkColor));
-          color->red = atoi (RGB_vals[0]);
-          color->green = atoi (RGB_vals[1]);
-          color->blue = atoi (RGB_vals[2]);
-          g_object_set (G_OBJECT (tag), "foreground-gdk", color, NULL);
+          color = g_malloc (sizeof (GdkRGBA));
+          color->red = atoi (RGB_vals[0]) / 65535.0;
+          color->green = atoi (RGB_vals[1]) / 65535.0;
+          color->blue = atoi (RGB_vals[2]) / 65535.0;
+          g_object_set (G_OBJECT (tag), "foreground-rgba", color, NULL);
         }
 
       else if (!strcmp (name, atk_text_attribute_get_name (ATK_TEXT_ATTR_STRETCH)))
