@@ -281,18 +281,12 @@ gtk_widget_accessible_ref_relation_set (AtkObject *obj)
               GtkWidget *temp_widget;
 
               temp_widget = gtk_widget_get_parent (widget);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-              if (GTK_IS_ALIGNMENT (temp_widget))
+              if (GTK_IS_BOX (temp_widget))
                 {
-                  temp_widget = gtk_widget_get_parent (temp_widget);
-                  if (GTK_IS_BOX (temp_widget))
-                    {
-                      label = find_label (temp_widget);
-                      if (!label)
-                        label = find_label (gtk_widget_get_parent (temp_widget));
-                    }
+                  label = find_label (temp_widget);
+                  if (!label)
+                    label = find_label (gtk_widget_get_parent (temp_widget));
                 }
-G_GNUC_END_IGNORE_DEPRECATIONS
             }
           else if (GTK_IS_COMBO_BOX (widget))
             /*
@@ -483,9 +477,6 @@ gtk_widget_accessible_notify_gtk (GObject    *obj,
     {
       gtk_widget_accessible_update_tooltip (GTK_WIDGET_ACCESSIBLE (atk_obj),
                                             widget);
-
-      if (atk_obj->description == NULL)
-        g_object_notify (G_OBJECT (atk_obj), "accessible-description");
       return;
     }
   else if (g_strcmp0 (pspec->name, "visible") == 0)
@@ -642,12 +633,11 @@ gtk_widget_accessible_grab_focus (AtkComponent *component)
   if (gtk_widget_is_toplevel (toplevel))
     {
 #ifdef GDK_WINDOWING_X11
-      if (GDK_IS_X11_DISPLAY (gtk_widget_get_display (toplevel)))
-        gtk_window_present_with_time (GTK_WINDOW (toplevel),
-                                      gdk_x11_get_server_time (gtk_widget_get_window (widget)));
-      else
+      gtk_window_present_with_time (GTK_WINDOW (toplevel),
+      gdk_x11_get_server_time (gtk_widget_get_window (widget)));
+#else
+      gtk_window_present (GTK_WINDOW (toplevel));
 #endif
-        gtk_window_present (GTK_WINDOW (toplevel));
     }
   return TRUE;
 }
