@@ -90,25 +90,16 @@ gtk_window_accessible_initialize (AtkObject *obj,
                                   gpointer   data)
 {
   GtkWidget *widget = GTK_WIDGET (data);
-  const gchar *name;
 
   ATK_OBJECT_CLASS (gtk_window_accessible_parent_class)->initialize (obj, data);
 
   g_signal_connect (data, "window-state-event", G_CALLBACK (window_state_event_cb), NULL);
   _gtk_widget_accessible_set_layer (GTK_WIDGET_ACCESSIBLE (obj), ATK_LAYER_WINDOW);
 
-  name = gtk_widget_get_name (widget);
-
-  if (!g_strcmp0 (name, "gtk-tooltip"))
-    obj->role = ATK_ROLE_TOOL_TIP;
-  else if (gtk_window_get_window_type (GTK_WINDOW (widget)) == GTK_WINDOW_POPUP)
+  if (gtk_window_get_window_type (GTK_WINDOW (widget)) == GTK_WINDOW_POPUP)
     obj->role = ATK_ROLE_WINDOW;
   else
     obj->role = ATK_ROLE_FRAME;
-
-  /* Notify that tooltip is showing */
-  if (obj->role == ATK_ROLE_TOOL_TIP && gtk_widget_get_mapped (widget))
-    atk_object_notify_state_change (obj, ATK_STATE_SHOWING, 1);
 }
 
 static GtkWidget *
@@ -260,7 +251,7 @@ gtk_window_accessible_ref_state_set (AtkObject *accessible)
 
   window = GTK_WINDOW (widget);
 
-  if (gtk_window_has_toplevel_focus (window) && gtk_window_is_active (window))
+  if (gtk_window_is_active (window))
     atk_state_set_add_state (state_set, ATK_STATE_ACTIVE);
 
   gdk_window = gtk_widget_get_window (widget);
@@ -324,6 +315,40 @@ gtk_window_accessible_ref_child (AtkObject *object,
   return g_object_ref (gtk_widget_get_accessible (ref_child));
 }
 
+static AtkAttributeSet *
+gtk_widget_accessible_get_attributes (AtkObject *obj)
+{
+  GtkWidget *window;
+  GdkWindowTypeHint hint;
+  AtkAttributeSet *attributes;
+  AtkAttribute *attr;
+  GEnumClass *class;
+  GEnumValue *value;
+
+  attributes = ATK_OBJECT_CLASS (gtk_window_accessible_parent_class)->get_attributes (obj);
+
+  attr = g_new (AtkAttribute, 1);
+  attr->name = g_strdup ("window-type");
+
+  window = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+  hint = gtk_window_get_type_hint (GTK_WINDOW (window));
+
+  class = g_type_class_ref (GDK_TYPE_WINDOW_TYPE_HINT);
+  for (value = class->values; value->value_name; value++)
+    {
+      if (hint == value->value)
+        {
+          attr->value = g_strdup (value->value_nick);
+          break;
+        }
+    }
+  g_type_class_unref (class);
+
+  attributes = g_slist_append (attributes, attr);
+
+  return attributes;
+}
+
 static void
 gtk_window_accessible_class_init (GtkWindowAccessibleClass *klass)
 {
@@ -340,6 +365,7 @@ gtk_window_accessible_class_init (GtkWindowAccessibleClass *klass)
   class->focus_event = gtk_window_accessible_focus_event;
   class->get_n_children = gtk_window_accessible_get_n_children;
   class->ref_child = gtk_window_accessible_ref_child;
+  class->get_attributes = gtk_widget_accessible_get_attributes;
 }
 
 static void
