@@ -69,7 +69,7 @@ gtk_actionable_default_init (GtkActionableInterface *iface)
 {
   g_object_interface_install_property (iface,
     g_param_spec_string ("action-name", P_("Action name"),
-                         P_("The name of the associated action, like 'app.quit'"),
+                         P_("The name of the associated action, like “app.quit”"),
                          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_interface_install_property (iface,
@@ -102,7 +102,7 @@ gtk_actionable_get_action_name (GtkActionable *actionable)
 /**
  * gtk_actionable_set_action_name:
  * @actionable: a #GtkActionable widget
- * @action_name: an action name, or %NULL
+ * @action_name: (nullable): an action name, or %NULL
  *
  * Specifies the name of the action with which this widget should be
  * associated.  If @action_name is %NULL then the widget will be
