@@ -83,10 +83,6 @@ GtkAdjustment*   gtk_adjustment_new             (gdouble          value,
                                                  gdouble          page_increment,
                                                  gdouble          page_size);
 
-GDK_AVAILABLE_IN_ALL
-void       gtk_adjustment_changed               (GtkAdjustment   *adjustment);
-GDK_AVAILABLE_IN_ALL
-void       gtk_adjustment_value_changed         (GtkAdjustment   *adjustment);
 GDK_AVAILABLE_IN_ALL
 void       gtk_adjustment_clamp_page            (GtkAdjustment   *adjustment,
                                                  gdouble          lower,
