@@ -101,17 +101,6 @@ GDK_AVAILABLE_IN_3_4
 void             gtk_application_set_menubar   (GtkApplication    *application,
                                                 GMenuModel        *menubar);
 
-GDK_DEPRECATED_IN_3_14_FOR(gtk_application_set_accels_for_action)
-void             gtk_application_add_accelerator    (GtkApplication  *application,
-                                                     const gchar     *accelerator,
-                                                     const gchar     *action_name,
-                                                     GVariant        *parameter);
-
-GDK_DEPRECATED_IN_3_14_FOR(gtk_application_set_accels_for_action)
-void             gtk_application_remove_accelerator (GtkApplication *application,
-                                                     const gchar    *action_name,
-                                                     GVariant       *parameter);
-
 typedef enum
 {
   GTK_APPLICATION_INHIBIT_LOGOUT  = (1 << 0),
@@ -145,6 +134,10 @@ gchar **         gtk_application_list_action_descriptions        (GtkApplication
 GDK_AVAILABLE_IN_3_12
 gchar **         gtk_application_get_accels_for_action           (GtkApplication       *application,
                                                                   const gchar          *detailed_action_name);
+GDK_AVAILABLE_IN_3_14
+gchar **         gtk_application_get_actions_for_accel           (GtkApplication       *application,
+                                                                  const gchar          *accel);
+
 
 GDK_AVAILABLE_IN_3_12
 void             gtk_application_set_accels_for_action           (GtkApplication       *application,
@@ -158,6 +151,8 @@ GDK_AVAILABLE_IN_3_14
 GMenu *          gtk_application_get_menu_by_id                  (GtkApplication       *application,
                                                                   const gchar          *id);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GtkApplication, g_object_unref)
+
 G_END_DECLS
 
 #endif /* __GTK_APPLICATION_H__ */
