@@ -60,14 +60,12 @@
 #include "gtksizerequest.h"
 #include "gtkwidgetprivate.h"
 #include "gtkcontainerprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkintl.h"
 
 
 struct _GtkButtonBoxPrivate
 {
   GtkButtonBoxStyle layout_style;
-  GtkCssGadget *gadget;
 };
 
 enum {
@@ -92,17 +90,17 @@ static void gtk_button_box_get_property       (GObject           *object,
                                                guint              prop_id,
                                                GValue            *value,
                                                GParamSpec        *pspec);
-static void gtk_button_box_snapshot           (GtkWidget         *widget,
-                                               GtkSnapshot       *snapshot);
-static void gtk_button_box_measure_           (GtkWidget         *widget,
-                                               GtkOrientation     orientation,
-                                               int                for_size,
-                                               int               *minimum,
-                                               int               *natural,
-                                               int               *minimum_baseline,
-                                               int               *natural_baseline);
-static void gtk_button_box_size_allocate      (GtkWidget         *widget,
-                                               GtkAllocation     *allocation);
+static void gtk_button_box_measure           (GtkWidget         *widget,
+                                              GtkOrientation     orientation,
+                                              int                for_size,
+                                              int               *minimum,
+                                              int               *natural,
+                                              int               *minimum_baseline,
+                                              int               *natural_baseline);
+static void gtk_button_box_size_allocate      (GtkWidget           *widget,
+                                               const GtkAllocation *allocation,
+                                               int                  baseline,
+                                               GtkAllocation       *out_clip);
 static void gtk_button_box_remove             (GtkContainer      *container,
                                                GtkWidget         *widget);
 static void gtk_button_box_set_child_property (GtkContainer      *container,
@@ -116,27 +114,6 @@ static void gtk_button_box_get_child_property (GtkContainer      *container,
                                                GValue            *value,
                                                GParamSpec        *pspec);
 
-static void     gtk_button_box_measure         (GtkCssGadget        *gadget,
-                                                GtkOrientation       orientation,
-                                                int                  for_size,
-                                                int                 *minimum,
-                                                int                 *natural,
-                                                int                 *minimum_baseline,
-                                                int                 *natural_baseline,
-                                                gpointer             unused);
-static void     gtk_button_box_allocate        (GtkCssGadget        *gadget,
-                                                const GtkAllocation *allocation,
-                                                int                  baseline,
-                                                GtkAllocation       *out_clip,
-                                                gpointer             unused);
-static gboolean gtk_button_box_render          (GtkCssGadget        *gadget,
-                                                GtkSnapshot         *snapshot,
-                                                int                  x,
-                                                int                  y,
-                                                int                  width,
-                                                int                  height,
-                                                gpointer             data);
-
 #define DEFAULT_LAYOUT_STYLE GTK_BUTTONBOX_EDGE
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkButtonBox, gtk_button_box, GTK_TYPE_BOX)
@@ -148,16 +125,6 @@ gtk_button_box_add (GtkContainer *container,
   gtk_box_pack_start (GTK_BOX (container), widget);
 }
 
-static void
-gtk_button_box_finalize (GObject *object)
-{
-  GtkButtonBox *button_box = GTK_BUTTON_BOX (object);
-
-  g_clear_object (&button_box->priv->gadget);
-
-  G_OBJECT_CLASS (gtk_button_box_parent_class)->finalize (object);
-}
-
 static void
 gtk_button_box_class_init (GtkButtonBoxClass *class)
 {
@@ -171,11 +138,9 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
 
   gobject_class->set_property = gtk_button_box_set_property;
   gobject_class->get_property = gtk_button_box_get_property;
-  gobject_class->finalize = gtk_button_box_finalize;
 
-  widget_class->measure = gtk_button_box_measure_;
+  widget_class->measure = gtk_button_box_measure;
   widget_class->size_allocate = gtk_button_box_size_allocate;
-  widget_class->snapshot= gtk_button_box_snapshot;
 
   container_class->remove = gtk_button_box_remove;
   container_class->add = gtk_button_box_add;
@@ -210,47 +175,6 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
   gtk_widget_class_set_css_name (widget_class, "buttonbox");
 }
 
-static void
-gtk_button_box_snapshot_forall (GtkWidget *child,
-                                gpointer   snapshot)
-{
-  gtk_widget_snapshot_child (gtk_widget_get_parent (child),
-                             child,
-                             snapshot);
-}
-
-static gboolean
-gtk_button_box_render (GtkCssGadget *gadget,
-                       GtkSnapshot  *snapshot,
-                       int           x,
-                       int           y,
-                       int           width,
-                       int           height,
-                       gpointer      unused)
-{
-  gtk_container_forall (GTK_CONTAINER (gtk_css_gadget_get_owner (gadget)),
-                        gtk_button_box_snapshot_forall,
-                        snapshot);
-
-  return FALSE;
-}
-
-
-static void
-gtk_button_box_snapshot (GtkWidget   *widget,
-                         GtkSnapshot *snapshot)
-{
-  GtkButtonBoxPrivate *priv = GTK_BUTTON_BOX (widget)->priv;
-  GtkCssGadget *gadget;
-
-  if (priv->layout_style == GTK_BUTTONBOX_EXPAND)
-    gadget = gtk_box_get_gadget (GTK_BOX (widget));
-  else
-    gadget = priv->gadget;
-
-  gtk_css_gadget_snapshot (gadget, snapshot);
-}
-
 static void
 gtk_button_box_init (GtkButtonBox *button_box)
 {
@@ -258,14 +182,6 @@ gtk_button_box_init (GtkButtonBox *button_box)
   button_box->priv->layout_style = DEFAULT_LAYOUT_STYLE;
 
   gtk_box_set_spacing (GTK_BOX (button_box), 0);
-
-  button_box->priv->gadget = gtk_css_custom_gadget_new_for_node (gtk_widget_get_css_node (GTK_WIDGET (button_box)),
-                                                         GTK_WIDGET (button_box),
-                                                         gtk_button_box_measure,
-                                                         gtk_button_box_allocate,
-                                                         gtk_button_box_render,
-                                                         NULL,
-                                                         NULL);
 }
 
 static void
@@ -783,41 +699,7 @@ gtk_button_box_size_request (GtkWidget      *widget,
 }
 
 static void
-gtk_button_box_measure (GtkCssGadget   *gadget,
-                        GtkOrientation  orientation,
-                        int             for_size,
-                        int            *minimum,
-                        int            *natural,
-                        int            *minimum_baseline,
-                        int            *natural_baseline,
-                        gpointer        unused)
-{
-  GtkWidget *widget;
-  GtkRequisition requisition;
-  int baseline;
-  int *pb;
-
-  if (minimum_baseline || natural_baseline)
-    pb = &baseline;
-  else
-    pb = NULL;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-  gtk_button_box_size_request (widget, &requisition, pb);
-
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    *minimum = *natural = requisition.width;
-  else
-    *minimum = *natural = requisition.height;
-
-  if (minimum_baseline)
-    *minimum_baseline = baseline;
-  if (natural_baseline)
-    *natural_baseline = baseline;
-}
-
-static void
-gtk_button_box_measure_ (GtkWidget      *widget,
+gtk_button_box_measure (GtkWidget      *widget,
                         GtkOrientation  orientation,
                         int             for_size,
                         int            *minimum,
@@ -826,53 +708,49 @@ gtk_button_box_measure_ (GtkWidget      *widget,
                         int            *natural_baseline)
 {
   GtkButtonBoxPrivate *priv = gtk_button_box_get_instance_private (GTK_BUTTON_BOX (widget));
-  GtkCssGadget *gadget;
 
   if (priv->layout_style == GTK_BUTTONBOX_EXPAND)
-    gadget = gtk_box_get_gadget (GTK_BOX (widget));
+    {
+      GTK_WIDGET_CLASS (gtk_button_box_parent_class)->measure (widget, orientation, for_size,
+                                                               minimum, natural,
+                                                               minimum_baseline, natural_baseline);
+    }
   else
-    gadget = priv->gadget;
+    {
+      GtkRequisition requisition;
+      int baseline;
+      int *pb;
 
-  gtk_css_gadget_get_preferred_size (gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
+      if (minimum_baseline || natural_baseline)
+        pb = &baseline;
+      else
+        pb = NULL;
+
+      gtk_button_box_size_request (widget, &requisition, pb);
+
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
+        *minimum = *natural = requisition.width;
+      else
+        *minimum = *natural = requisition.height;
+
+      if (orientation == GTK_ORIENTATION_VERTICAL)
+        {
+          if (minimum_baseline)
+            *minimum_baseline = baseline;
+          if (natural_baseline)
+            *natural_baseline = baseline;
+        }
+    }
 }
 
 static void
-gtk_button_box_size_allocate (GtkWidget     *widget,
-                              GtkAllocation *allocation)
+gtk_button_box_size_allocate (GtkWidget           *widget,
+                              const GtkAllocation *allocation,
+                              int                  baseline,
+                              GtkAllocation       *out_clip)
 {
-  GtkButtonBoxPrivate *priv = GTK_BUTTON_BOX (widget)->priv;
-  GtkCssGadget *gadget;
-  GdkRectangle clip;
-
-  if (priv->layout_style == GTK_BUTTONBOX_EXPAND)
-    gadget = gtk_box_get_gadget (GTK_BOX (widget));
-  else
-    gadget = priv->gadget;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_button_box_allocate (GtkCssGadget        *gadget,
-                         const GtkAllocation *allocation,
-                         int                  baseline,
-                         GtkAllocation       *out_clip,
-                         gpointer             unused)
-{
-  GtkWidget *widget;
-  GtkButtonBoxPrivate *priv;
-  GtkButtonBox *bbox;
+  GtkButtonBox *bbox = GTK_BUTTON_BOX (widget);
+  GtkButtonBoxPrivate *priv = bbox->priv;
   GList *children, *list;
   GtkAllocation child_allocation;
   gint nvis_children;
@@ -897,10 +775,16 @@ gtk_button_box_allocate (GtkCssGadget        *gadget,
   gint baseline_height;
   gint child_baseline;
   gint i;
+  GdkRectangle child_clip;
 
-  widget = gtk_css_gadget_get_owner (gadget);
-  bbox = GTK_BUTTON_BOX (widget);
-  priv = bbox->priv;
+  if (priv->layout_style == GTK_BUTTONBOX_EXPAND)
+    {
+      GTK_WIDGET_CLASS (gtk_button_box_parent_class)->size_allocate (widget,
+                                                                     allocation,
+                                                                     baseline,
+                                                                     out_clip);
+      return;
+    }
 
   orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (widget));
   spacing = gtk_box_get_spacing (GTK_BOX (widget));
@@ -1150,7 +1034,8 @@ gtk_button_box_allocate (GtkCssGadget        *gadget,
                 }
             }
 
-          gtk_widget_size_allocate_with_baseline (child, &child_allocation, child_baseline);
+          gtk_widget_size_allocate (child, &child_allocation, child_baseline, &child_clip);
+          gdk_rectangle_union (out_clip, &child_clip, out_clip);
           i++;
         }
     }
@@ -1159,8 +1044,6 @@ gtk_button_box_allocate (GtkCssGadget        *gadget,
   g_free (widths);
   g_free (heights);
   g_free (baselines);
-
-  gtk_container_get_children_clip (GTK_CONTAINER (widget), out_clip);
 }
 
 /**
