@@ -37,8 +37,6 @@
 
 #include "config.h"
 #include "gtkbin.h"
-#include "gtksizerequest.h"
-#include "gtkintl.h"
 
 
 struct _GtkBinPrivate
@@ -62,11 +60,22 @@ static void               gtk_bin_measure                         (GtkWidget
                                                                    int            *natural,
                                                                    int            *minimum_baseline,
                                                                    int            *natural_baseline);
-static void               gtk_bin_size_allocate                   (GtkWidget           *widget,
-                                                                   GtkAllocation       *allocation);
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GtkBin, gtk_bin, GTK_TYPE_CONTAINER)
 
+static void
+gtk_bin_size_allocate (GtkWidget           *widget,
+                       const GtkAllocation *allocation,
+                       int                  baseline,
+                       GtkAllocation       *out_clip)
+{
+  GtkBin *bin = GTK_BIN (widget);
+  GtkBinPrivate *priv = gtk_bin_get_instance_private (bin);
+
+  if (priv->child && gtk_widget_get_visible (priv->child))
+    gtk_widget_size_allocate (priv->child, allocation, baseline, out_clip);
+}
+
 static void
 gtk_bin_class_init (GtkBinClass *class)
 {
@@ -85,16 +94,13 @@ gtk_bin_class_init (GtkBinClass *class)
 static void
 gtk_bin_init (GtkBin *bin)
 {
-  bin->priv = gtk_bin_get_instance_private (bin);
-
   gtk_widget_set_has_window (GTK_WIDGET (bin), FALSE);
 }
 
-
 static GType
 gtk_bin_child_type (GtkContainer *container)
 {
-  GtkBinPrivate *priv = GTK_BIN (container)->priv;
+  GtkBinPrivate *priv = gtk_bin_get_instance_private (GTK_BIN (container));
 
   if (!priv->child)
     return GTK_TYPE_WIDGET;
@@ -107,7 +113,7 @@ gtk_bin_add (GtkContainer *container,
 	     GtkWidget    *child)
 {
   GtkBin *bin = GTK_BIN (container);
-  GtkBinPrivate *priv = bin->priv;
+  GtkBinPrivate *priv = gtk_bin_get_instance_private (bin);
 
   if (priv->child != NULL)
     {
@@ -130,7 +136,7 @@ gtk_bin_remove (GtkContainer *container,
 		GtkWidget    *child)
 {
   GtkBin *bin = GTK_BIN (container);
-  GtkBinPrivate *priv = bin->priv;
+  GtkBinPrivate *priv = gtk_bin_get_instance_private (bin);
   gboolean widget_was_visible;
 
   g_return_if_fail (priv->child == child);
@@ -153,7 +159,7 @@ gtk_bin_forall (GtkContainer *container,
 		gpointer      callback_data)
 {
   GtkBin *bin = GTK_BIN (container);
-  GtkBinPrivate *priv = bin->priv;
+  GtkBinPrivate *priv = gtk_bin_get_instance_private (bin);
 
   if (priv->child)
     (* callback) (priv->child, callback_data);
@@ -185,21 +191,6 @@ gtk_bin_measure (GtkWidget *widget,
     }
 }
 
-static void
-gtk_bin_size_allocate (GtkWidget     *widget,
-                       GtkAllocation *allocation)
-{
-  GtkBin *bin = GTK_BIN (widget);
-  GtkBinPrivate *priv = bin->priv;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  if (priv->child && gtk_widget_get_visible (priv->child))
-    {
-      gtk_widget_size_allocate (priv->child, allocation);
-    }
-}
-
 /**
  * gtk_bin_get_child:
  * @bin: a #GtkBin
@@ -208,19 +199,24 @@ gtk_bin_size_allocate (GtkWidget     *widget,
  * no child widget. The returned widget does not have a reference
  * added, so you do not need to unref it.
  *
- * Returns: (transfer none): pointer to child of the #GtkBin
+ * Returns: (transfer none) (nullable): the child of @bin, or %NULL if it does
+ * not have a child.
  **/
 GtkWidget*
 gtk_bin_get_child (GtkBin *bin)
 {
+  GtkBinPrivate *priv = gtk_bin_get_instance_private (bin);
+
   g_return_val_if_fail (GTK_IS_BIN (bin), NULL);
 
-  return bin->priv->child;
+  return priv->child;
 }
 
 void
 _gtk_bin_set_child (GtkBin    *bin,
                     GtkWidget *widget)
 {
-  bin->priv->child = widget;
+  GtkBinPrivate *priv = gtk_bin_get_instance_private (bin);
+
+  priv->child = widget;
 }
