@@ -79,7 +79,6 @@
 #include "gtkbox.h"
 #include "gtkboxprivate.h"
 #include "gtkcontainerprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkcssnodeprivate.h"
 #include "gtkcsspositionvalueprivate.h"
 #include "gtkintl.h"
@@ -117,7 +116,6 @@ typedef struct _GtkBoxChild        GtkBoxChild;
 struct _GtkBoxPrivate
 {
   GList          *children;
-  GtkCssGadget   *gadget;
 
   GtkOrientation  orientation;
   gint16          spacing;
@@ -144,7 +142,9 @@ struct _GtkBoxChild
 };
 
 static void gtk_box_size_allocate         (GtkWidget              *widget,
-                                           GtkAllocation          *allocation);
+                                           const GtkAllocation    *allocation,
+                                           int                     baseline,
+                                           GtkAllocation          *out_clip);
 
 static void gtk_box_direction_changed  (GtkWidget        *widget,
                                         GtkTextDirection  previous_direction);
@@ -185,24 +185,11 @@ static void gtk_box_measure (GtkWidget      *widget,
                              int            *natural,
                              int            *minimum_baseline,
                              int            *natural_baseline);
-static void gtk_box_snapshot (GtkWidget   *widget,
-                              GtkSnapshot *snapshot);
 
 G_DEFINE_TYPE_WITH_CODE (GtkBox, gtk_box, GTK_TYPE_CONTAINER,
                          G_ADD_PRIVATE (GtkBox)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL))
 
-static void
-gtk_box_dispose (GObject *object)
-{
-  GtkBox *box = GTK_BOX (object);
-  GtkBoxPrivate *priv = box->priv;
-
-  g_clear_object (&priv->gadget);
-
-  G_OBJECT_CLASS (gtk_box_parent_class)->dispose (object);
-}
-
 static void
 gtk_box_class_init (GtkBoxClass *class)
 {
@@ -212,9 +199,7 @@ gtk_box_class_init (GtkBoxClass *class)
 
   object_class->set_property = gtk_box_set_property;
   object_class->get_property = gtk_box_get_property;
-  object_class->dispose = gtk_box_dispose;
 
-  widget_class->snapshot = gtk_box_snapshot;
   widget_class->size_allocate = gtk_box_size_allocate;
   widget_class->measure = gtk_box_measure;
   widget_class->direction_changed = gtk_box_direction_changed;
@@ -343,48 +328,27 @@ gtk_box_get_property (GObject    *object,
     }
 }
 
-static gboolean
-gtk_box_snapshot_contents (GtkCssGadget *gadget,
-                           GtkSnapshot  *snapshot,
-                           int           x,
-                           int           y,
-                           int           width,
-                           int           height,
-                           gpointer      data)
-{
-  GTK_WIDGET_CLASS (gtk_box_parent_class)->snapshot (gtk_css_gadget_get_owner (gadget), snapshot);
-
-  return FALSE;
-}
-
-static void
-gtk_box_snapshot (GtkWidget   *widget,
-                  GtkSnapshot *snapshot)
-{
-  gtk_css_gadget_snapshot (GTK_BOX (widget)->priv->gadget, snapshot);
-}
-
 static void
 count_expand_children (GtkBox *box,
                        gint *visible_children,
                        gint *expand_children)
 {
   GtkBoxPrivate  *private = box->priv;
-  GList       *children;
-  GtkBoxChild *child;
+  GtkWidget *child;
 
   *visible_children = *expand_children = 0;
 
-  for (children = private->children; children; children = children->next)
+  for (child = _gtk_widget_get_first_child (GTK_WIDGET (box));
+       child != NULL;
+       child = _gtk_widget_get_next_sibling (child))
     {
-      child = children->data;
+      if (_gtk_widget_get_visible (child))
+        {
+          *visible_children += 1;
 
-      if (_gtk_widget_get_visible (child->widget))
-	{
-	  *visible_children += 1;
-          if (gtk_widget_compute_expand (child->widget, private->orientation))
-	    *expand_children += 1;
-	}
+          if (gtk_widget_compute_expand (child, private->orientation))
+            *expand_children += 1;
+        }
     }
 }
 
@@ -407,6 +371,7 @@ get_spacing (GtkBox *box)
 static void
 gtk_box_size_allocate_no_center (GtkWidget           *widget,
                                  const GtkAllocation *allocation,
+                                 int                  baseline,
                                  GdkRectangle        *out_clip)
 {
   GtkBox *box = GTK_BOX (widget);
@@ -423,7 +388,6 @@ gtk_box_size_allocate_no_center (GtkWidget           *widget,
   gint minimum_above, natural_above;
   gint minimum_below, natural_below;
   gboolean have_baseline;
-  gint baseline;
 
   GtkPackType packing;
 
@@ -471,24 +435,6 @@ gtk_box_size_allocate_no_center (GtkWidget           *widget,
                           &sizes[i].minimum_size, &sizes[i].natural_size,
                           NULL, NULL);
 
-      /* Assert the api is working properly */
-      if (sizes[i].minimum_size < 0)
-	g_error ("GtkBox child %s minimum %s: %d < 0 for %s %d",
-		 gtk_widget_get_name (GTK_WIDGET (child->widget)),
-		 (private->orientation == GTK_ORIENTATION_HORIZONTAL) ? "width" : "height",
-		 sizes[i].minimum_size,
-		 (private->orientation == GTK_ORIENTATION_HORIZONTAL) ? "height" : "width",
-		 (private->orientation == GTK_ORIENTATION_HORIZONTAL) ? allocation->height : allocation->width);
-
-      if (sizes[i].natural_size < sizes[i].minimum_size)
-	g_error ("GtkBox child %s natural %s: %d < minimum %d for %s %d",
-		 gtk_widget_get_name (GTK_WIDGET (child->widget)),
-		 (private->orientation == GTK_ORIENTATION_HORIZONTAL) ? "width" : "height",
-		 sizes[i].natural_size,
-		 sizes[i].minimum_size,
-		 (private->orientation == GTK_ORIENTATION_HORIZONTAL) ? "height" : "width",
-		 (private->orientation == GTK_ORIENTATION_HORIZONTAL) ? allocation->height : allocation->width);
-
       children_minimum_size += sizes[i].minimum_size;
 
       sizes[i].data = child;
@@ -578,7 +524,7 @@ gtk_box_size_allocate_no_center (GtkWidget           *widget,
 	  sizes[i].natural_size = child_size;
 
 	  if (private->orientation == GTK_ORIENTATION_HORIZONTAL &&
-	      gtk_widget_get_valign (child->widget) == GTK_ALIGN_BASELINE)
+              gtk_widget_get_valign (child->widget) == GTK_ALIGN_BASELINE)
 	    {
 	      int child_allocation_width;
 	      int child_minimum_height, child_natural_height;
@@ -606,7 +552,11 @@ gtk_box_size_allocate_no_center (GtkWidget           *widget,
 	}
     }
 
-  baseline = gtk_widget_get_allocated_baseline (widget);
+  if (private->orientation == GTK_ORIENTATION_VERTICAL)
+    baseline = -1;
+
+  /* we only calculate our own baseline if we don't get one passed from the parent
+   * and any of the child widgets explicitly request one */
   if (baseline == -1 && have_baseline)
     {
       gint height = MAX (1, allocation->height);
@@ -708,8 +658,7 @@ gtk_box_size_allocate_no_center (GtkWidget           *widget,
 		  child_allocation.y -= child_size;
 		}
 	    }
-	  gtk_widget_size_allocate_with_baseline (child->widget, &child_allocation, baseline);
-          gtk_widget_get_clip (child->widget, &clip);
+	  gtk_widget_size_allocate (child->widget, &child_allocation, baseline, &clip);
           gdk_rectangle_union (&clip, out_clip, out_clip);
 
 	  i++;
@@ -718,33 +667,12 @@ gtk_box_size_allocate_no_center (GtkWidget           *widget,
 }
 
 static void
-gtk_box_allocate_contents (GtkCssGadget        *gadget,
-                           const GtkAllocation *allocation,
-                           int                  baseline,
-                           GtkAllocation       *out_clip,
-                           gpointer             unused)
+gtk_box_size_allocate (GtkWidget           *widget,
+                       const GtkAllocation *allocation,
+                       int                  baseline,
+                       GtkAllocation       *out_clip)
 {
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-
-  *out_clip = *allocation;
-  gtk_box_size_allocate_no_center (widget, allocation, out_clip);
-}
-
-static void
-gtk_box_size_allocate (GtkWidget     *widget,
-                       GtkAllocation *allocation)
-{
-  GtkBoxPrivate *priv = GTK_BOX (widget)->priv;
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
+  gtk_box_size_allocate_no_center (widget, allocation, baseline, out_clip);
 }
 
 static GType
@@ -1125,22 +1053,6 @@ gtk_box_get_size (GtkWidget      *widget,
     *natural_baseline = nat_baseline;
 }
 
-static void
-gtk_box_measure (GtkWidget      *widget,
-                 GtkOrientation  orientation,
-                 int             for_size,
-                 int            *minimum,
-                 int            *natural,
-                 int            *minimum_baseline,
-                 int            *natural_baseline)
-{
-  gtk_css_gadget_get_preferred_size (GTK_BOX (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
-}
-
 static void
 gtk_box_compute_size_for_opposing_orientation (GtkBox *box,
 					       gint    avail_size,
@@ -1190,20 +1102,6 @@ gtk_box_compute_size_for_opposing_orientation (GtkBox *box,
                               &sizes[i].minimum_size, &sizes[i].natural_size,
                               NULL, NULL);
 
-	  /* Assert the api is working properly */
-	  if (sizes[i].minimum_size < 0)
-	    g_error ("GtkBox child %s minimum %s: %d < 0",
-		     gtk_widget_get_name (GTK_WIDGET (child->widget)),
-		     (private->orientation == GTK_ORIENTATION_HORIZONTAL) ? "width" : "height",
-		     sizes[i].minimum_size);
-
-	  if (sizes[i].natural_size < sizes[i].minimum_size)
-	    g_error ("GtkBox child %s natural %s: %d < minimum %d",
-		     gtk_widget_get_name (GTK_WIDGET (child->widget)),
-		     (private->orientation == GTK_ORIENTATION_HORIZONTAL) ? "width" : "height",
-		     sizes[i].natural_size,
-		     sizes[i].minimum_size);
-
           children_minimum_size += sizes[i].minimum_size;
 
 	  sizes[i].data = child;
@@ -1414,17 +1312,16 @@ gtk_box_compute_size_for_orientation (GtkBox *box,
     *natural_size = required_natural;
 }
 
+
 static void
-gtk_box_get_content_size (GtkCssGadget   *gadget,
-                          GtkOrientation  orientation,
-                          gint            for_size,
-                          gint           *minimum,
-                          gint           *natural,
-                          gint           *minimum_baseline,
-                          gint           *natural_baseline,
-                          gpointer        unused)
+gtk_box_measure (GtkWidget      *widget,
+                 GtkOrientation  orientation,
+                 int             for_size,
+                 int            *minimum,
+                 int            *natural,
+                 int            *minimum_baseline,
+                 int            *natural_baseline)
 {
-  GtkWidget     *widget  = gtk_css_gadget_get_owner (gadget);
   GtkBox        *box     = GTK_BOX (widget);
   GtkBoxPrivate *private = box->priv;
 
@@ -1433,15 +1330,15 @@ gtk_box_get_content_size (GtkCssGadget   *gadget,
   else
     {
       if (private->orientation != orientation)
-	gtk_box_compute_size_for_opposing_orientation (box, for_size, minimum, natural, minimum_baseline, natural_baseline);
+        gtk_box_compute_size_for_opposing_orientation (box, for_size, minimum, natural, minimum_baseline, natural_baseline);
       else
-	{
-	  if (minimum_baseline)
-	    *minimum_baseline = -1;
-	  if (natural_baseline)
-	    *natural_baseline = -1;
-	  gtk_box_compute_size_for_orientation (box, for_size, minimum, natural);
-	}
+        {
+          if (minimum_baseline)
+            *minimum_baseline = -1;
+          if (natural_baseline)
+            *natural_baseline = -1;
+          gtk_box_compute_size_for_orientation (box, for_size, minimum, natural);
+        }
     }
 }
 
@@ -1463,23 +1360,9 @@ gtk_box_init (GtkBox *box)
   private->spacing = 0;
   private->baseline_pos = GTK_BASELINE_POSITION_CENTER;
 
-  private->gadget = gtk_css_custom_gadget_new_for_node (gtk_widget_get_css_node (GTK_WIDGET (box)),
-                                                        GTK_WIDGET (box),
-                                                        gtk_box_get_content_size,
-                                                        gtk_box_allocate_contents,
-                                                        gtk_box_snapshot_contents,
-                                                        NULL,
-                                                        NULL);
-
   _gtk_orientable_set_style_classes (GTK_ORIENTABLE (box));
 }
 
-GtkCssGadget *
-gtk_box_get_gadget (GtkBox *box)
-{
-  return box->priv->gadget;
-}
-
 /**
  * gtk_box_new:
  * @orientation: the box’s orientation.
