@@ -20,16 +20,11 @@
 #define __GTK_BOX_PRIVATE_H__
 
 #include "gtkbox.h"
-#include "gtkcssgadgetprivate.h"
 
 G_BEGIN_DECLS
 
-
 GList      *_gtk_box_get_children       (GtkBox         *box);
 
-GtkCssGadget *gtk_box_get_gadget (GtkBox *box);
-
-
 G_END_DECLS
 
 #endif /* __GTK_BOX_PRIVATE_H__ */
