@@ -47,7 +47,7 @@
  *
  * Button-like widgets like #GtkToggleButton, #GtkMenuButton, #GtkVolumeButton,
  * #GtkLockButton, #GtkColorButton, #GtkFontButton or #GtkFileChooserButton use
- * style classes such as .toggle, .popup, .scale, .lock, .color, .font, .file
+ * style classes such as .toggle, .popup, .scale, .lock, .color, .file
  * to differentiate themselves from a plain GtkButton.
  */
 
@@ -70,7 +70,6 @@
 #include "a11y/gtkbuttonaccessible.h"
 #include "gtkapplicationprivate.h"
 #include "gtkactionhelper.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkcontainerprivate.h"
 
 /* Time out before giving up on getting a key release when animating
@@ -120,9 +119,6 @@ static void gtk_button_get_property   (GObject            *object,
 static void gtk_button_screen_changed (GtkWidget          *widget,
 				       GdkScreen          *previous_screen);
 static void gtk_button_unrealize (GtkWidget * widget);
-static void gtk_button_size_allocate (GtkWidget * widget,
-				      GtkAllocation * allocation);
-static void gtk_button_snapshot (GtkWidget * widget, GtkSnapshot *snapshot);
 static gint gtk_button_grab_broken (GtkWidget * widget,
 				    GdkEventGrabBroken * event);
 static gint gtk_button_key_release (GtkWidget * widget, GdkEventKey * event);
@@ -152,27 +148,7 @@ static void gtk_button_measure_ (GtkWidget      *widget,
                                  int            *natural,
                                  int            *minimum_baseline,
                                  int            *natural_baseline);
-static void     gtk_button_measure  (GtkCssGadget        *gadget,
-                                     GtkOrientation       orientation,
-                                     int                  for_size,
-                                     int                 *minimum_size,
-                                     int                 *natural_size,
-                                     int                 *minimum_baseline,
-                                     int                 *natural_baseline,
-                                     gpointer             data);
-static void     gtk_button_allocate (GtkCssGadget        *gadget,
-                                     const GtkAllocation *allocation,
-                                     int                  baseline,
-                                     GtkAllocation       *out_clip,
-                                     gpointer             data);
 static void gtk_button_set_child_type (GtkButton *button, guint child_type);
-static gboolean gtk_button_render   (GtkCssGadget        *gadget,
-                                     GtkSnapshot         *snapshot,
-                                     int                  x,
-                                     int                  y,
-                                     int                  width,
-                                     int                  height,
-                                     gpointer             data);
 
 static GParamSpec *props[LAST_PROP] = { NULL, };
 static guint button_signals[LAST_SIGNAL] = { 0 };
@@ -204,6 +180,13 @@ gtk_button_remove (GtkContainer *container, GtkWidget *child)
   GTK_CONTAINER_CLASS (gtk_button_parent_class)->remove (container, child);
 }
 
+static void
+gtk_button_unmap (GtkWidget *widget)
+{
+  GTK_BUTTON (widget)->priv->in_button = FALSE;
+  GTK_WIDGET_CLASS (gtk_button_parent_class)->unmap (widget);
+}
+
 static void
 gtk_button_class_init (GtkButtonClass *klass)
 {
@@ -223,14 +206,13 @@ gtk_button_class_init (GtkButtonClass *klass)
   widget_class->measure = gtk_button_measure_;
   widget_class->screen_changed = gtk_button_screen_changed;
   widget_class->unrealize = gtk_button_unrealize;
-  widget_class->size_allocate = gtk_button_size_allocate;
-  widget_class->snapshot = gtk_button_snapshot;
   widget_class->grab_broken_event = gtk_button_grab_broken;
   widget_class->key_release_event = gtk_button_key_release;
   widget_class->enter_notify_event = gtk_button_enter_notify;
   widget_class->leave_notify_event = gtk_button_leave_notify;
   widget_class->state_flags_changed = gtk_button_state_flags_changed;
   widget_class->grab_notify = gtk_button_grab_notify;
+  widget_class->unmap = gtk_button_unmap;
 
   container_class->add    = gtk_button_add;
   container_class->remove = gtk_button_remove;
@@ -353,12 +335,11 @@ touch_release_in_button (GtkButton *button)
     }
 
   gdk_event_get_coords (event, &x, &y);
-  gtk_widget_get_allocation (GTK_WIDGET (button), &allocation);
+  gtk_widget_get_own_allocation (GTK_WIDGET (button), &allocation);
 
   gdk_event_free (event);
 
-  if (x >= 0 && x <= allocation.width &&
-      y >= 0 && y <= allocation.height)
+  if (gdk_rectangle_contains_point (&allocation, x, y))
     return TRUE;
 
   return FALSE;
@@ -402,10 +383,10 @@ multipress_gesture_update_cb (GtkGesture       *gesture,
   if (sequence != gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture)))
     return;
 
-  gtk_widget_get_allocation (GTK_WIDGET (button), &allocation);
+  gtk_widget_get_own_allocation (GTK_WIDGET (button), &allocation);
   gtk_gesture_get_point (gesture, sequence, &x, &y);
 
-  in_button = (x >= 0 && y >= 0 && x < allocation.width && y < allocation.height);
+  in_button = gdk_rectangle_contains_point (&allocation, x, y);
 
   if (priv->in_button != in_button)
     {
@@ -465,14 +446,6 @@ gtk_button_init (GtkButton *button)
   g_signal_connect (priv->gesture, "cancel", G_CALLBACK (multipress_gesture_cancel_cb), button);
   gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (priv->gesture), GTK_PHASE_CAPTURE);
 
-  priv->gadget = gtk_css_custom_gadget_new_for_node (gtk_widget_get_css_node (GTK_WIDGET (button)),
-                                                     GTK_WIDGET (button),
-                                                     gtk_button_measure,
-                                                     gtk_button_allocate,
-                                                     gtk_button_render,
-                                                     NULL,
-                                                     NULL);
-
 }
 
 static void
@@ -482,7 +455,6 @@ gtk_button_finalize (GObject *object)
   GtkButtonPrivate *priv = button->priv;
 
   g_clear_object (&priv->gesture);
-  g_clear_object (&priv->gadget);
 
   G_OBJECT_CLASS (gtk_button_parent_class)->finalize (object);
 }
@@ -777,73 +749,6 @@ gtk_button_unrealize (GtkWidget *widget)
   GTK_WIDGET_CLASS (gtk_button_parent_class)->unrealize (widget);
 }
 
-static void
-gtk_button_size_allocate (GtkWidget     *widget,
-			  GtkAllocation *allocation)
-{
-  GtkButton *button = GTK_BUTTON (widget);
-  GtkButtonPrivate *priv = button->priv;
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-  gtk_css_gadget_allocate (priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_button_allocate (GtkCssGadget        *gadget,
-                     const GtkAllocation *allocation,
-                     int                  baseline,
-                     GtkAllocation       *out_clip,
-                     gpointer             unused)
-{
-  GtkWidget *widget;
-  GtkWidget *child;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-
-  *out_clip = *allocation;
-
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child && gtk_widget_get_visible (child))
-    {
-      GtkAllocation clip;
-
-      gtk_widget_size_allocate_with_baseline (child, (GtkAllocation *)allocation, baseline);
-      gtk_widget_get_clip (child, &clip);
-      gdk_rectangle_union (&clip, out_clip, out_clip);
-    }
-}
-
-static void
-gtk_button_snapshot (GtkWidget   *widget,
-                     GtkSnapshot *snapshot)
-{
-  gtk_css_gadget_snapshot (GTK_BUTTON (widget)->priv->gadget, snapshot);
-}
-
-static gboolean
-gtk_button_render (GtkCssGadget *gadget,
-                   GtkSnapshot  *snapshot,
-                   int           x,
-                   int           y,
-                   int           width,
-                   int           height,
-                   gpointer      data)
-{
-  GtkWidget *widget;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-
-  GTK_WIDGET_CLASS (gtk_button_parent_class)->snapshot (widget, snapshot);
-
-  return gtk_widget_has_visible_focus (widget);
-}
-
 static void
 gtk_button_do_release (GtkButton *button,
                        gboolean   emit_clicked)
@@ -993,42 +898,6 @@ gtk_button_finish_activate (GtkButton *button,
     gtk_button_clicked (button);
 }
 
-
-static void
-gtk_button_measure (GtkCssGadget   *gadget,
-		    GtkOrientation  orientation,
-                    int             for_size,
-		    int            *minimum,
-		    int            *natural,
-		    int            *minimum_baseline,
-		    int            *natural_baseline,
-                    gpointer        data)
-{
-  GtkWidget *widget;
-  GtkWidget *child;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-  child = gtk_bin_get_child (GTK_BIN (widget));
-
-  if (child && gtk_widget_get_visible (child))
-    {
-       gtk_widget_measure (child,
-                           orientation,
-                           for_size,
-                           minimum, natural,
-                           minimum_baseline, natural_baseline);
-    }
-  else
-    {
-      *minimum = 0;
-      *natural = 0;
-      if (minimum_baseline)
-        *minimum_baseline = 0;
-      if (natural_baseline)
-        *natural_baseline = 0;
-    }
-}
-
 static void
 gtk_button_measure_ (GtkWidget      *widget,
                      GtkOrientation  orientation,
@@ -1038,11 +907,16 @@ gtk_button_measure_ (GtkWidget      *widget,
                      int            *minimum_baseline,
                      int            *natural_baseline)
 {
-  gtk_css_gadget_get_preferred_size (GTK_BUTTON (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
+  GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
+
+  if (child && gtk_widget_get_visible (child))
+    {
+       gtk_widget_measure (child,
+                           orientation,
+                           for_size,
+                           minimum, natural,
+                           minimum_baseline, natural_baseline);
+    }
 }
 
 /**
@@ -1184,11 +1058,7 @@ gtk_button_update_state (GtkButton *button)
   else
     depressed = priv->in_button && priv->button_down;
 
-  new_state = gtk_widget_get_state_flags (GTK_WIDGET (button)) &
-    ~(GTK_STATE_FLAG_PRELIGHT | GTK_STATE_FLAG_ACTIVE);
-
-  if (priv->in_button)
-    new_state |= GTK_STATE_FLAG_PRELIGHT;
+  new_state = gtk_widget_get_state_flags (GTK_WIDGET (button)) & ~(GTK_STATE_FLAG_ACTIVE);
 
   if (depressed)
     new_state |= GTK_STATE_FLAG_ACTIVE;
