@@ -58,11 +58,7 @@ struct _GtkButton
 /**
  * GtkButtonClass:
  * @parent_class: The parent class.
- * @pressed: Signal emitted when the button is pressed. Deprecated: 2.8.
- * @released: Signal emitted when the button is released. Deprecated: 2.8.
  * @clicked: Signal emitted when the button has been activated (pressed and released).
- * @enter: Signal emitted when the pointer enters the button. Deprecated: 2.8.
- * @leave: Signal emitted when the pointer leaves the button. Deprecated: 2.8.
  * @activate: Signal that causes the button to animate press then
  *    release. Applications should never connect to this signal, but use
  *    the @clicked signal.
@@ -73,11 +69,7 @@ struct _GtkButtonClass
 
   /*< public >*/
 
-  void (* pressed)  (GtkButton *button);
-  void (* released) (GtkButton *button);
   void (* clicked)  (GtkButton *button);
-  void (* enter)    (GtkButton *button);
-  void (* leave)    (GtkButton *button);
   void (* activate) (GtkButton *button);
 
   /*< private >*/
@@ -99,24 +91,14 @@ GtkWidget*     gtk_button_new_with_label    (const gchar    *label);
 GDK_AVAILABLE_IN_3_10
 GtkWidget*     gtk_button_new_from_icon_name (const gchar    *icon_name,
 					      GtkIconSize     size);
-GDK_DEPRECATED_IN_3_10_FOR(gtk_button_new_with_label)
-GtkWidget*     gtk_button_new_from_stock    (const gchar    *stock_id);
 GDK_AVAILABLE_IN_ALL
 GtkWidget*     gtk_button_new_with_mnemonic (const gchar    *label);
 GDK_AVAILABLE_IN_ALL
 void           gtk_button_clicked           (GtkButton      *button);
-GDK_DEPRECATED
-void           gtk_button_pressed           (GtkButton      *button);
-GDK_DEPRECATED
-void           gtk_button_released          (GtkButton      *button);
-GDK_DEPRECATED
-void           gtk_button_enter             (GtkButton      *button);
-GDK_DEPRECATED
-void           gtk_button_leave             (GtkButton      *button);
 
 GDK_AVAILABLE_IN_ALL
 void                  gtk_button_set_relief         (GtkButton      *button,
-						     GtkReliefStyle  newstyle);
+						     GtkReliefStyle  relief);
 GDK_AVAILABLE_IN_ALL
 GtkReliefStyle        gtk_button_get_relief         (GtkButton      *button);
 GDK_AVAILABLE_IN_ALL
@@ -129,42 +111,12 @@ void                  gtk_button_set_use_underline  (GtkButton      *button,
 						     gboolean        use_underline);
 GDK_AVAILABLE_IN_ALL
 gboolean              gtk_button_get_use_underline  (GtkButton      *button);
-GDK_DEPRECATED_IN_3_10
-void                  gtk_button_set_use_stock      (GtkButton      *button,
-						     gboolean        use_stock);
-GDK_DEPRECATED_IN_3_10
-gboolean              gtk_button_get_use_stock      (GtkButton      *button);
-GDK_AVAILABLE_IN_ALL
-void                  gtk_button_set_focus_on_click (GtkButton      *button,
-						     gboolean        focus_on_click);
-GDK_AVAILABLE_IN_ALL
-gboolean              gtk_button_get_focus_on_click (GtkButton      *button);
-GDK_DEPRECATED_IN_3_14
-void                  gtk_button_set_alignment      (GtkButton      *button,
-						     gfloat          xalign,
-						     gfloat          yalign);
-GDK_DEPRECATED_IN_3_14
-void                  gtk_button_get_alignment      (GtkButton      *button,
-						     gfloat         *xalign,
-						     gfloat         *yalign);
-GDK_AVAILABLE_IN_ALL
-void                  gtk_button_set_image          (GtkButton      *button,
-					             GtkWidget      *image);
-GDK_AVAILABLE_IN_ALL
-GtkWidget*            gtk_button_get_image          (GtkButton      *button);
-GDK_AVAILABLE_IN_ALL
-void                  gtk_button_set_image_position (GtkButton      *button,
-						     GtkPositionType position);
-GDK_AVAILABLE_IN_ALL
-GtkPositionType       gtk_button_get_image_position (GtkButton      *button);
-GDK_AVAILABLE_IN_3_6
-void                  gtk_button_set_always_show_image (GtkButton   *button,
-                                                        gboolean     always_show);
-GDK_AVAILABLE_IN_3_6
-gboolean              gtk_button_get_always_show_image (GtkButton   *button);
 
-GDK_AVAILABLE_IN_ALL
-GdkWindow*            gtk_button_get_event_window   (GtkButton      *button);
+GDK_AVAILABLE_IN_3_90
+void                  gtk_button_set_icon_name      (GtkButton      *button,
+                                                     const char     *icon_name);
+GDK_AVAILABLE_IN_3_90
+const char *          gtk_button_get_icon_name      (GtkButton      *button);
 
 
 G_END_DECLS
