@@ -21,7 +21,6 @@
 
 #include "gtkactionhelper.h"
 #include "gtkgesturesingle.h"
-#include "gtkcssgadgetprivate.h"
 
 G_BEGIN_DECLS
 
@@ -29,7 +28,6 @@ G_BEGIN_DECLS
 struct _GtkButtonPrivate
 {
   GtkActionHelper       *action_helper;
-  GtkCssGadget          *gadget;
 
   GdkDevice             *grab_keyboard;
 
@@ -38,7 +36,6 @@ struct _GtkButtonPrivate
   guint                  activate_timeout;
 
   guint          button_down           : 1;
-  guint          constructed           : 1;
   guint          in_button             : 1;
   guint          use_underline         : 1;
   guint          child_type            : 2;
