@@ -36,19 +36,20 @@
  *
  * # GtkCenterBox as GtkBuildable
  *
- * The GtkCenterBox implementation of the GtkBuildable interface supports
+ * The GtkCenterBox implementation of the #GtkBuildable interface supports
  * placing children in the 3 positions by specifying “start”, “center” or
  * “end” as the “type” attribute of a <child> element.
  *
  * # CSS nodes
  *
- * GtkCenterBox uses a single CSS node with name box.
+ * GtkCenterBox uses a single CSS node with the name “box”,
  *
  * In horizontal orientation, the nodes of the children are always arranged
  * from left to right. So :first-child will always select the leftmost child,
  * regardless of text direction.
  *
- * In vertical orientation, the are arranged from top to bottom.
+ * In vertical orientation, the nodes of the children are arranged from top to
+ * bottom.
  */
 
 #include "config.h"
@@ -379,12 +380,13 @@ gtk_center_box_measure (GtkWidget      *widget,
 }
 
 static void
-gtk_center_box_size_allocate (GtkWidget     *widget,
-                              GtkAllocation *allocation)
+gtk_center_box_size_allocate (GtkWidget           *widget,
+                              const GtkAllocation *allocation,
+                              int                  baseline,
+                              GtkAllocation       *out_clip)
 {
   GtkCenterBox *self = GTK_CENTER_BOX (widget);
   GtkAllocation child_allocation;
-  GtkAllocation clip = *allocation;
   GtkAllocation child_clip;
   GtkWidget *child[3];
   int child_size[3];
@@ -392,16 +394,12 @@ gtk_center_box_size_allocate (GtkWidget     *widget,
   GtkRequestedSize sizes[3];
   int size;
   int for_size;
-  int baseline;
   int i;
 
-  GTK_WIDGET_CLASS (gtk_center_box_parent_class)->size_allocate (widget, allocation);
-
   if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       size = allocation->width;
       for_size = allocation->height;
-      baseline = gtk_widget_get_allocated_baseline (widget);
     }
   else
     {
@@ -448,7 +446,7 @@ gtk_center_box_size_allocate (GtkWidget     *widget,
 
       for (i = 0; i < 3; i++)
         {
-          if (gtk_widget_get_valign (child[i]) == GTK_ALIGN_BASELINE)
+          if (child[i] && gtk_widget_get_valign (child[i]) == GTK_ALIGN_BASELINE)
             {
               int child_min_height, child_nat_height;
               int child_min_baseline, child_nat_baseline;
@@ -528,12 +526,9 @@ gtk_center_box_size_allocate (GtkWidget     *widget,
           child_allocation.height = child_size[i];
         }
 
-      gtk_widget_size_allocate_with_baseline (child[i], &child_allocation, allocation->y + baseline);
-      gtk_widget_get_clip (child[i], &child_clip);
-      gdk_rectangle_union (&clip, &clip, &child_clip);
+      gtk_widget_size_allocate (child[i], &child_allocation, allocation->y + baseline, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
     }
-
-  gtk_widget_set_clip (widget, &clip);
 }
 
 static void
@@ -727,9 +722,9 @@ gtk_center_box_new (void)
 /**
  * gtk_center_box_set_start_widget:
  * @self: a #GtkCenterBox
- * @child: the child
+ * @child: (nullable): the new start widget, or %NULL
  *
- * Sets the start widget.
+ * Sets the start widget. To remove the existing start widget, pass %NULL.
  *
  * Since: 3.92
  */
@@ -750,15 +745,15 @@ gtk_center_box_set_start_widget (GtkCenterBox *self,
 /**
  * gtk_center_box_set_center_widget:
  * @self: a #GtkCenterBox
- * @child: the child
+ * @child: (nullable): the new center widget, or %NULL
  *
- * Sets the center widget.
+ * Sets the center widget. To remove the existing center widget, pas %NULL.
  *
  * Since: 3.92
  */
 void
 gtk_center_box_set_center_widget (GtkCenterBox *self,
-                                 GtkWidget     *child)
+                                  GtkWidget    *child)
 {
   if (self->center_widget)
     gtk_widget_unparent (self->center_widget);
@@ -773,9 +768,9 @@ gtk_center_box_set_center_widget (GtkCenterBox *self,
 /**
  * gtk_center_box_set_end_widget:
  * @self: a #GtkCenterBox
- * @child: the child
+ * @child: (nullable): the new end widget, or %NULL
  *
- * Sets the end widget.
+ * Sets the end widget. To remove the existing end widget, pass %NULL.
  *
  * Since: 3.92
  */
@@ -797,9 +792,9 @@ gtk_center_box_set_end_widget (GtkCenterBox *self,
  * gtk_center_box_get_start_widget:
  * @self: a #GtkCenterBox
  *
- * Gets the start widget.
+ * Gets the start widget, or %NULL if there is none.
  *
- * Returns: the start widget.
+ * Returns: (transfer none) (nullable): the start widget.
  *
  * Since: 3.92
  */
@@ -813,9 +808,9 @@ gtk_center_box_get_start_widget (GtkCenterBox *self)
  * gtk_center_box_get_center_widget:
  * @self: a #GtkCenterBox
  *
- * Gets the center widget.
+ * Gets the center widget, or %NULL if there is none.
  *
- * Returns: the center widget.
+ * Returns: (transfer none) (nullable): the center widget.
  *
  * Since: 3.92
  */
@@ -829,9 +824,9 @@ gtk_center_box_get_center_widget (GtkCenterBox *self)
  * gtk_center_box_get_end_widget:
  * @self: a #GtkCenterBox
  *
- * Gets the end widget.
+ * Gets the end widget, or %NULL if there is none.
  *
- * Returns: the end widget.
+ * Returns: (transfer none) (nullable): the end widget.
  *
  * Since: 3.92
  */
