@@ -23,7 +23,6 @@
  */
 
 #include "config.h"
-#include "gtkbuiltiniconprivate.h"
 #include "gtkcheckmenuitemprivate.h"
 #include "gtkmenuitemprivate.h"
 #include "gtkaccellabel.h"
@@ -34,6 +33,7 @@
 #include "gtkcssnodeprivate.h"
 #include "gtkcssstylepropertyprivate.h"
 #include "gtkwidgetprivate.h"
+#include "gtkiconprivate.h"
 
 /**
  * SECTION:gtkcheckmenuitem
@@ -60,12 +60,9 @@
  * with name check, which gets the .left or .right style class.
  */
 
-
-#define INDICATOR_SIZE 16
-
 struct _GtkCheckMenuItemPrivate
 {
-  GtkCssGadget *indicator_gadget;
+  GtkWidget *indicator_widget;
 
   guint active             : 1;
   guint draw_as_radio      : 1;
@@ -84,13 +81,9 @@ enum {
   PROP_DRAW_AS_RADIO
 };
 
-static void gtk_check_menu_item_snapshot             (GtkWidget             *widget,
-                                                      GtkSnapshot           *snapshot);
 static void gtk_check_menu_item_activate             (GtkMenuItem           *menu_item);
 static void gtk_check_menu_item_toggle_size_request  (GtkMenuItem           *menu_item,
                                                       gint                  *requisition);
-static void gtk_real_check_menu_item_snapshot_indicator (GtkCheckMenuItem   *check_menu_item,
-                                                      GtkSnapshot           *snapshot);
 static void gtk_check_menu_item_set_property         (GObject               *object,
                                                       guint                  prop_id,
                                                       const GValue          *value,
@@ -111,53 +104,47 @@ G_DEFINE_TYPE_WITH_CODE (GtkCheckMenuItem, gtk_check_menu_item, GTK_TYPE_MENU_IT
                          G_ADD_PRIVATE (GtkCheckMenuItem))
 
 static void
-gtk_check_menu_item_size_allocate (GtkWidget     *widget,
-                                   GtkAllocation *allocation)
+gtk_check_menu_item_size_allocate (GtkWidget           *widget,
+                                   const GtkAllocation *allocation,
+                                   int                  baseline,
+                                   GtkAllocation       *out_clip)
 {
-  GtkAllocation clip, widget_clip;
-  GtkAllocation content_alloc, indicator_alloc;
-  GtkCssGadget *menu_item_gadget;
+  GtkAllocation child_clip;
+  GtkAllocation indicator_alloc;
   GtkCheckMenuItem *check_menu_item = GTK_CHECK_MENU_ITEM (widget);
   GtkCheckMenuItemPrivate *priv = check_menu_item->priv;
-  gint content_baseline, toggle_size;
+  gint toggle_size;
 
-  GTK_WIDGET_CLASS (gtk_check_menu_item_parent_class)->size_allocate
-    (widget, allocation);
+  GTK_WIDGET_CLASS (gtk_check_menu_item_parent_class)->size_allocate (widget,
+                                                                      allocation,
+                                                                      baseline,
+                                                                      out_clip);
 
-  menu_item_gadget = _gtk_menu_item_get_gadget (GTK_MENU_ITEM (widget));
-  gtk_css_gadget_get_content_allocation (menu_item_gadget,
-                                         &content_alloc, &content_baseline);
-
-  gtk_css_gadget_get_preferred_size (priv->indicator_gadget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     -1,
-                                     &indicator_alloc.width, NULL,
-                                     NULL, NULL);
-  gtk_css_gadget_get_preferred_size (priv->indicator_gadget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     -1,
-                                     &indicator_alloc.height, NULL,
-                                     NULL, NULL);
+  gtk_widget_measure (priv->indicator_widget,
+                      GTK_ORIENTATION_HORIZONTAL,
+                      -1,
+                      &indicator_alloc.width, NULL,
+                      NULL, NULL);
+  gtk_widget_measure (priv->indicator_widget,
+                      GTK_ORIENTATION_VERTICAL,
+                      -1,
+                      &indicator_alloc.height, NULL,
+                      NULL, NULL);
   toggle_size = GTK_MENU_ITEM (check_menu_item)->priv->toggle_size;
 
   if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
-    indicator_alloc.x = content_alloc.x +
-      (toggle_size - indicator_alloc.width) / 2;
+    indicator_alloc.x = (toggle_size - indicator_alloc.width) / 2;
   else
-    indicator_alloc.x = content_alloc.x + content_alloc.width - toggle_size +
+    indicator_alloc.x = allocation->width - toggle_size +
       (toggle_size - indicator_alloc.width) / 2;
 
-  indicator_alloc.y = content_alloc.y +
-    (content_alloc.height - indicator_alloc.height) / 2;
+  indicator_alloc.y = (allocation->height - indicator_alloc.height) / 2;
 
-  gtk_css_gadget_allocate (check_menu_item->priv->indicator_gadget,
-                           &indicator_alloc,
-                           content_baseline,
-                           &clip);
-
-  gtk_widget_get_clip (widget, &widget_clip);
-  gdk_rectangle_union (&widget_clip, &clip, &widget_clip);
-  gtk_widget_set_clip (widget, &widget_clip);
+  gtk_widget_size_allocate (priv->indicator_widget,
+                            &indicator_alloc,
+                            baseline,
+                            &child_clip);
+  gdk_rectangle_union (out_clip, &child_clip, out_clip);
 }
 
 static void
@@ -165,7 +152,7 @@ gtk_check_menu_item_finalize (GObject *object)
 {
   GtkCheckMenuItemPrivate *priv = GTK_CHECK_MENU_ITEM (object)->priv;
 
-  g_clear_object (&priv->indicator_gadget);
+  gtk_widget_unparent (priv->indicator_widget);
 
   G_OBJECT_CLASS (gtk_check_menu_item_parent_class)->finalize (object);
 }
@@ -213,14 +200,11 @@ gtk_check_menu_item_class_init (GtkCheckMenuItemClass *klass)
                                                          FALSE,
                                                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
-  widget_class->snapshot = gtk_check_menu_item_snapshot;
-
   menu_item_class->activate = gtk_check_menu_item_activate;
   menu_item_class->hide_on_activate = FALSE;
   menu_item_class->toggle_size_request = gtk_check_menu_item_toggle_size_request;
   
   klass->toggled = NULL;
-  klass->snapshot_indicator = gtk_real_check_menu_item_snapshot_indicator;
 
   /**
    * GtkCheckMenuItem::toggled:
@@ -343,11 +327,11 @@ gtk_check_menu_item_toggle_size_request (GtkMenuItem *menu_item,
   g_return_if_fail (GTK_IS_CHECK_MENU_ITEM (menu_item));
 
   check_menu_item = GTK_CHECK_MENU_ITEM (menu_item);
-  gtk_css_gadget_get_preferred_size (check_menu_item->priv->indicator_gadget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     -1,
-                                     requisition, NULL,
-                                     NULL, NULL);
+  gtk_widget_measure (check_menu_item->priv->indicator_widget,
+                      GTK_ORIENTATION_HORIZONTAL,
+                      -1,
+                      requisition, NULL,
+                      NULL, NULL);
 }
 
 /**
@@ -369,13 +353,14 @@ update_node_state (GtkCheckMenuItem *check_menu_item)
   GtkStateFlags state;
 
   state = gtk_widget_get_state_flags (GTK_WIDGET (check_menu_item));
+  state &= ~(GTK_STATE_FLAG_CHECKED | GTK_STATE_FLAG_INCONSISTENT);
 
   if (priv->inconsistent)
     state |= GTK_STATE_FLAG_INCONSISTENT;
   if (priv->active)
     state |= GTK_STATE_FLAG_CHECKED;
 
-  gtk_css_gadget_set_state (priv->indicator_gadget, state);
+  gtk_widget_set_state_flags (priv->indicator_widget, state, TRUE);
 }
 
 /**
@@ -455,7 +440,7 @@ gtk_check_menu_item_set_draw_as_radio (GtkCheckMenuItem *check_menu_item,
   if (draw_as_radio != priv->draw_as_radio)
     {
       priv->draw_as_radio = draw_as_radio;
-      indicator_node = gtk_css_gadget_get_node (priv->indicator_gadget);
+      indicator_node = gtk_widget_get_css_node (priv->indicator_widget);
       if (draw_as_radio)
         gtk_css_node_set_name (indicator_node, I_("radio"));
       else
@@ -493,27 +478,11 @@ gtk_check_menu_item_init (GtkCheckMenuItem *check_menu_item)
   priv = check_menu_item->priv = gtk_check_menu_item_get_instance_private (check_menu_item);
   priv->active = FALSE;
 
-  priv->indicator_gadget =
-    gtk_builtin_icon_new ("check",
-                          GTK_WIDGET (check_menu_item),
-                          _gtk_menu_item_get_gadget (GTK_MENU_ITEM (check_menu_item)),
-                          NULL);
+  priv->indicator_widget = gtk_icon_new ("check");
+  gtk_widget_set_parent (priv->indicator_widget, GTK_WIDGET (check_menu_item));
   update_node_state (check_menu_item);
 }
 
-static void
-gtk_check_menu_item_snapshot (GtkWidget   *widget,
-                              GtkSnapshot *snapshot)
-{
-  GtkCheckMenuItem *check_menu_item = GTK_CHECK_MENU_ITEM (widget);
-
-  if (GTK_WIDGET_CLASS (gtk_check_menu_item_parent_class)->snapshot)
-    GTK_WIDGET_CLASS (gtk_check_menu_item_parent_class)->snapshot (widget, snapshot);
-
-  if (GTK_CHECK_MENU_ITEM_GET_CLASS (check_menu_item)->snapshot_indicator)
-    GTK_CHECK_MENU_ITEM_GET_CLASS (check_menu_item)->snapshot_indicator (check_menu_item, snapshot);
-}
-
 static void
 gtk_check_menu_item_activate (GtkMenuItem *menu_item)
 {
@@ -549,42 +518,37 @@ static void
 gtk_check_menu_item_direction_changed (GtkWidget        *widget,
                                        GtkTextDirection  previous_dir)
 {
-  GtkCheckMenuItem *check_menu_item = GTK_CHECK_MENU_ITEM (widget);
-  GtkCheckMenuItemPrivate *priv = check_menu_item->priv;
-  GtkCssNode *indicator_node, *widget_node, *node;
+  GtkCheckMenuItemPrivate *priv = GTK_CHECK_MENU_ITEM (widget)->priv;
+  GtkStyleContext *context;
+  GtkWidget *child;
 
-  indicator_node = gtk_css_gadget_get_node (priv->indicator_gadget);
-  widget_node = gtk_widget_get_css_node (widget);
+
+  context = gtk_widget_get_style_context (priv->indicator_widget);
 
   if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
     {
-      gtk_css_node_remove_class (indicator_node, g_quark_from_static_string (GTK_STYLE_CLASS_LEFT));
-      gtk_css_node_add_class (indicator_node, g_quark_from_static_string (GTK_STYLE_CLASS_RIGHT));
+      gtk_style_context_add_class (context, GTK_STYLE_CLASS_LEFT);
+      gtk_style_context_remove_class (context, GTK_STYLE_CLASS_RIGHT);
 
-      node = gtk_css_node_get_last_child (widget_node);
-      if (node != indicator_node)
-        gtk_css_node_insert_after (widget_node, indicator_node, node);
+      child = gtk_widget_get_last_child (widget);
+
+      if (child != priv->indicator_widget)
+        gtk_widget_insert_before (widget, priv->indicator_widget, NULL);
     }
   else
     {
-      gtk_css_node_add_class (indicator_node, g_quark_from_static_string (GTK_STYLE_CLASS_LEFT));
-      gtk_css_node_remove_class (indicator_node, g_quark_from_static_string (GTK_STYLE_CLASS_RIGHT));
+      gtk_style_context_add_class (context, GTK_STYLE_CLASS_RIGHT);
+      gtk_style_context_remove_class (context, GTK_STYLE_CLASS_LEFT);
 
-      node = gtk_css_node_get_first_child (widget_node);
-      if (node != indicator_node)
-        gtk_css_node_insert_before (widget_node, indicator_node, node);
+      child = gtk_widget_get_first_child (widget);
+
+      if (child != priv->indicator_widget)
+        gtk_widget_insert_after (widget, priv->indicator_widget, NULL);
     }
 
   GTK_WIDGET_CLASS (gtk_check_menu_item_parent_class)->direction_changed (widget, previous_dir);
 }
 
-static void
-gtk_real_check_menu_item_snapshot_indicator (GtkCheckMenuItem *check_menu_item,
-                                             GtkSnapshot      *snapshot)
-{
-  gtk_css_gadget_snapshot (check_menu_item->priv->indicator_gadget, snapshot);
-}
-
 static void
 gtk_check_menu_item_get_property (GObject     *object,
                                   guint        prop_id,
@@ -657,8 +621,8 @@ _gtk_check_menu_item_set_active (GtkCheckMenuItem *check_menu_item,
   update_node_state (check_menu_item);
 }
 
-GtkCssGadget *
-_gtk_check_menu_item_get_indicator_gadget (GtkCheckMenuItem *check_menu_item)
+GtkWidget *
+_gtk_check_menu_item_get_indicator_widget (GtkCheckMenuItem *check_menu_item)
 {
-  return check_menu_item->priv->indicator_gadget;
+  return check_menu_item->priv->indicator_widget;
 }
