@@ -268,6 +268,15 @@ gtk_clipboard_get (GdkAtom selection)
   return gtk_clipboard_get_for_display (gdk_display_get_default (), selection);
 }
 
+GtkClipboard *
+gtk_clipboard_get_default (GdkDisplay *display)
+{
+  g_return_val_if_fail (display != NULL, NULL);
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
+
+  return gtk_clipboard_get_for_display (display, GDK_SELECTION_CLIPBOARD);
+}
+
 static void
 clipboard_owner_destroyed (gpointer data)
 {
@@ -503,8 +512,11 @@ void
 gtk_clipboard_clear (GtkClipboard *clipboard)
 {
   clipboard_unset (clipboard);
-
-  [clipboard->pasteboard declareTypes:nil owner:nil];
+#ifdef AVAILABLE_MAC_OS_X_VERSION_10_6_AND_LATER
+    [clipboard->pasteboard clearContents];
+#else
+    [clipboard->pasteboard declareTypes:nil owner:nil];
+#endif
 }
 
 static void
@@ -753,7 +765,6 @@ GdkPixbuf *
 gtk_clipboard_wait_for_image (GtkClipboard *clipboard)
 {
   GdkAtom target = gdk_atom_intern_static_string("image/tiff");
-  int i;
   GtkSelectionData *data;
 
   data = gtk_clipboard_wait_for_contents (clipboard, target);
@@ -1107,3 +1118,11 @@ _gtk_clipboard_store_all (void)
       list = list->next;
     }
 }
+
+GdkAtom
+gtk_clipboard_get_selection (GtkClipboard *clipboard)
+{
+  g_return_val_if_fail (GTK_IS_CLIPBOARD (clipboard), GDK_NONE);
+
+  return clipboard->selection;
+}
