@@ -189,6 +189,9 @@ GtkClipboard *gtk_clipboard_get_for_display (GdkDisplay   *display,
 GDK_AVAILABLE_IN_ALL
 GtkClipboard *gtk_clipboard_get             (GdkAtom       selection);
 
+GDK_AVAILABLE_IN_3_16
+GtkClipboard *gtk_clipboard_get_default     (GdkDisplay    *display);
+
 GDK_AVAILABLE_IN_ALL
 GdkDisplay   *gtk_clipboard_get_display     (GtkClipboard *clipboard);
 
@@ -287,10 +290,8 @@ void gtk_clipboard_set_can_store (GtkClipboard         *clipboard,
 GDK_AVAILABLE_IN_ALL
 void gtk_clipboard_store         (GtkClipboard   *clipboard);
 
-/* private */
-void     _gtk_clipboard_handle_event    (GdkEventOwnerChange *event);
-
-void     _gtk_clipboard_store_all       (void);
+GDK_AVAILABLE_IN_3_22
+GdkAtom gtk_clipboard_get_selection (GtkClipboard *clipboard);
 
 G_END_DECLS
 
