@@ -179,13 +179,13 @@ create_surface (GtkColorPlane *plane)
 }
 
 static void
-plane_size_allocate (GtkWidget     *widget,
-                     GtkAllocation *allocation)
+plane_size_allocate (GtkWidget           *widget,
+                     const GtkAllocation *allocation,
+                     int                   baseline,
+                     GtkAllocation        *out_clip)
 {
   GtkColorPlane *plane = GTK_COLOR_PLANE (widget);
 
-  GTK_WIDGET_CLASS (gtk_color_plane_parent_class)->size_allocate (widget, allocation);
-
   create_surface (plane);
 }
 
