@@ -28,9 +28,7 @@
 #include "gtkmenushell.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
-#include "gtkiconhelperprivate.h"
 #include "gtkcssnodeprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkroundedboxprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtkstylecontextprivate.h"
@@ -60,8 +58,7 @@ struct _GtkColorSwatchPrivate
 
   GtkGesture *long_press_gesture;
   GtkGesture *multipress_gesture;
-  GtkCssGadget *gadget;
-  GtkCssGadget *overlay_gadget;
+  GtkWidget *overlay_widget;
 
   GtkWidget *popover;
 };
@@ -86,29 +83,14 @@ static guint signals[LAST_SIGNAL];
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkColorSwatch, gtk_color_swatch, GTK_TYPE_WIDGET)
 
+#define INTENSITY(r, g, b) ((r) * 0.30 + (g) * 0.59 + (b) * 0.11)
 static void
 swatch_snapshot (GtkWidget   *widget,
                  GtkSnapshot *snapshot)
 {
-  gtk_css_gadget_snapshot (GTK_COLOR_SWATCH (widget)->priv->gadget, snapshot);
-}
-
-#define INTENSITY(r, g, b) ((r) * 0.30 + (g) * 0.59 + (b) * 0.11)
-
-static gboolean
-gtk_color_swatch_render (GtkCssGadget *gadget,
-                         GtkSnapshot  *snapshot,
-                         int           x,
-                         int           y,
-                         int           width,
-                         int           height,
-                         gpointer      data)
-{
-  GtkWidget *widget;
   GtkColorSwatch *swatch;
   GtkStyleContext *context;
 
-  widget = gtk_css_gadget_get_owner (gadget);
   swatch = GTK_COLOR_SWATCH (widget);
   context = gtk_widget_get_style_context (widget);
 
@@ -116,21 +98,16 @@ gtk_color_swatch_render (GtkCssGadget *gadget,
     {
       cairo_pattern_t *pattern;
       cairo_matrix_t matrix;
-      GtkAllocation allocation, border_allocation;
+      GtkAllocation border_allocation;
       GskRoundedRect content_box;
 
-      gtk_widget_get_allocation (widget, &allocation);
-      gtk_css_gadget_get_border_allocation (gadget, &border_allocation, NULL);
-
-      border_allocation.x -= allocation.x;
-      border_allocation.y -= allocation.y;
+      gtk_widget_get_border_allocation (widget, &border_allocation);
 
       gtk_rounded_boxes_init_for_style (NULL,
                                         NULL,
                                         &content_box,
                                         gtk_style_context_lookup_style (context),
-                                        border_allocation.x,
-                                        border_allocation.y,
+                                        0, 0,
                                         border_allocation.width,
                                         border_allocation.height);
       gtk_snapshot_push_rounded_clip (snapshot,
@@ -177,11 +154,10 @@ gtk_color_swatch_render (GtkCssGadget *gadget,
       gtk_snapshot_pop (snapshot);
     }
 
-  gtk_css_gadget_snapshot (swatch->priv->overlay_gadget, snapshot);
-
-  return gtk_widget_has_visible_focus (widget);
+  gtk_widget_snapshot_child (widget, swatch->priv->overlay_widget, snapshot);
 }
 
+
 static void
 drag_set_color_icon (GdkDragContext *context,
                      const GdkRGBA  *color)
@@ -272,39 +248,6 @@ swatch_drag_data_received (GtkWidget        *widget,
   gtk_color_swatch_set_rgba (GTK_COLOR_SWATCH (widget), &color);
 }
 
-static void
-gtk_color_swatch_measure (GtkCssGadget   *gadget,
-                          GtkOrientation  orientation,
-                          int             for_size,
-                          int            *minimum,
-                          int            *natural,
-                          int            *minimum_baseline,
-                          int            *natural_baseline,
-                          gpointer        unused)
-{
-  GtkWidget *widget;
-  GtkColorSwatch *swatch;
-  gint w, h, min;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-  swatch = GTK_COLOR_SWATCH (widget);
-
-  gtk_css_gadget_get_preferred_size (swatch->priv->overlay_gadget,
-                                     orientation,
-                                     -1,
-                                     minimum, natural,
-                                     NULL, NULL);
-
-  gtk_widget_get_size_request (widget, &w, &h);
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    min = w < 0 ? 48 : w;
-  else
-    min = h < 0 ? 32 : h;
-
-  *minimum = MAX (*minimum, min);
-  *natural = MAX (*natural, min);
-}
-
 static gboolean
 swatch_key_press (GtkWidget   *widget,
                   GdkEventKey *event)
@@ -437,30 +380,17 @@ tap_action (GtkGestureMultiPress *gesture,
 }
 
 static void
-swatch_size_allocate (GtkWidget     *widget,
-                      GtkAllocation *allocation)
+swatch_size_allocate (GtkWidget           *widget,
+                      const GtkAllocation *allocation,
+                      int                  baseline,
+                      GtkAllocation       *out_clip)
 {
   GtkColorSwatch *swatch = GTK_COLOR_SWATCH (widget);
-  GtkAllocation clip, clip2;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (swatch->priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-  gtk_css_gadget_allocate (swatch->priv->overlay_gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip2);
-
-  gdk_rectangle_union (&clip, &clip2, &clip);
-
-  gtk_widget_set_clip (widget, &clip);
+  gtk_widget_size_allocate (swatch->priv->overlay_widget, allocation, -1, out_clip);
 }
 
 static void
-gtk_color_swatch_measure_ (GtkWidget *widget,
+gtk_color_swatch_measure (GtkWidget *widget,
                           GtkOrientation  orientation,
                           int             for_size,
                           int            *minimum,
@@ -468,11 +398,25 @@ gtk_color_swatch_measure_ (GtkWidget *widget,
                           int            *minimum_baseline,
                           int            *natural_baseline)
 {
-  gtk_css_gadget_get_preferred_size (GTK_COLOR_SWATCH (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
+  GtkColorSwatch *swatch;
+  gint w, h, min;
+
+  swatch = GTK_COLOR_SWATCH (widget);
+
+  gtk_widget_measure (swatch->priv->overlay_widget,
+                      orientation,
+                      -1,
+                      minimum, natural,
+                      NULL, NULL);
+
+  gtk_widget_get_size_request (widget, &w, &h);
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    min = w < 0 ? 48 : w;
+  else
+    min = h < 0 ? 32 : h;
+
+  *minimum = MAX (*minimum, min);
+  *natural = MAX (*natural, min);
 }
 
 
@@ -487,14 +431,14 @@ swatch_popup_menu (GtkWidget *widget)
 static void
 update_icon (GtkColorSwatch *swatch)
 {
-  GtkIconHelper *icon_helper = GTK_ICON_HELPER (swatch->priv->overlay_gadget);
+  GtkImage *image = GTK_IMAGE (swatch->priv->overlay_widget);
 
   if (swatch->priv->icon)
-    _gtk_icon_helper_set_icon_name (icon_helper, swatch->priv->icon, GTK_ICON_SIZE_BUTTON);
+    gtk_image_set_from_icon_name (image, swatch->priv->icon, GTK_ICON_SIZE_BUTTON);
   else if (gtk_widget_get_state_flags (GTK_WIDGET (swatch)) & GTK_STATE_FLAG_SELECTED)
-    _gtk_icon_helper_set_icon_name (icon_helper, "object-select-symbolic", GTK_ICON_SIZE_BUTTON);
+    gtk_image_set_from_icon_name (image, "object-select-symbolic", GTK_ICON_SIZE_BUTTON);
   else
-    _gtk_icon_helper_clear (icon_helper);
+    gtk_image_clear (image);
 }
 
 static void
@@ -503,9 +447,6 @@ swatch_state_flags_changed (GtkWidget     *widget,
 {
   GtkColorSwatch *swatch = GTK_COLOR_SWATCH (widget);
 
-  gtk_css_gadget_set_state (swatch->priv->gadget, gtk_widget_get_state_flags (widget));
-  gtk_css_gadget_set_state (swatch->priv->overlay_gadget, gtk_widget_get_state_flags (widget));
-
   update_icon (swatch);
 
   GTK_WIDGET_CLASS (gtk_color_swatch_parent_class)->state_flags_changed (widget, previous_state);
@@ -571,8 +512,7 @@ swatch_finalize (GObject *object)
   GtkColorSwatch *swatch = GTK_COLOR_SWATCH (object);
 
   g_free (swatch->priv->icon);
-  g_clear_object (&swatch->priv->gadget);
-  g_clear_object (&swatch->priv->overlay_gadget);
+  gtk_widget_unparent (swatch->priv->overlay_widget);
 
   G_OBJECT_CLASS (gtk_color_swatch_parent_class)->finalize (object);
 }
@@ -605,7 +545,7 @@ gtk_color_swatch_class_init (GtkColorSwatchClass *class)
   object_class->finalize = swatch_finalize;
   object_class->dispose = swatch_dispose;
 
-  widget_class->measure = gtk_color_swatch_measure_;
+  widget_class->measure = gtk_color_swatch_measure;
   widget_class->snapshot = swatch_snapshot;
   widget_class->drag_begin = swatch_drag_begin;
   widget_class->drag_data_get = swatch_drag_data_get;
@@ -648,8 +588,6 @@ gtk_color_swatch_class_init (GtkColorSwatchClass *class)
 static void
 gtk_color_swatch_init (GtkColorSwatch *swatch)
 {
-  GtkCssNode *widget_node;
-
   swatch->priv = gtk_color_swatch_get_instance_private (swatch);
   swatch->priv->use_alpha = TRUE;
   swatch->priv->selectable = TRUE;
@@ -669,20 +607,12 @@ gtk_color_swatch_init (GtkColorSwatch *swatch)
   g_signal_connect (swatch->priv->multipress_gesture, "pressed",
                     G_CALLBACK (tap_action), swatch);
 
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (swatch));
-  swatch->priv->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                             GTK_WIDGET (swatch),
-                                                             gtk_color_swatch_measure,
-                                                             NULL,
-                                                             gtk_color_swatch_render,
-                                                             NULL,
-                                                             NULL);
-  gtk_css_gadget_add_class (swatch->priv->gadget, "activatable");
-
-  swatch->priv->overlay_gadget = gtk_icon_helper_new_named ("overlay", GTK_WIDGET (swatch));
-  _gtk_icon_helper_set_force_scale_pixbuf (GTK_ICON_HELPER (swatch->priv->overlay_gadget), TRUE);
-  gtk_css_node_set_parent (gtk_css_gadget_get_node (swatch->priv->overlay_gadget), widget_node);
+  gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (swatch)), "activatable");
 
+  swatch->priv->overlay_widget = g_object_new (GTK_TYPE_IMAGE,
+                                               "css-name", "overlay",
+                                               NULL);
+  gtk_widget_set_parent (swatch->priv->overlay_widget, GTK_WIDGET (swatch));
 }
 
 /* Public API {{{1 */
