@@ -56,7 +56,7 @@ gtk_css_image_fallback_get_aspect_ratio (GtkCssImage *image)
   GtkCssImageFallback *fallback = GTK_CSS_IMAGE_FALLBACK (image);
 
   if (fallback->used < 0)
-    return 1;
+    return 0;
 
   return _gtk_css_image_get_aspect_ratio (fallback->images[fallback->used]);
 }
@@ -79,7 +79,7 @@ gtk_css_image_fallback_snapshot (GtkCssImage *image,
       else
         color = &red;
 
-      gtk_snapshot_append_color_node (snapshot, color, &GRAPHENE_RECT_INIT (0, 0, width, height), "image() Fallback Color");
+      gtk_snapshot_append_color (snapshot, color, &GRAPHENE_RECT_INIT (0, 0, width, height), "image() Fallback Color");
     }
   else
     gtk_css_image_snapshot (fallback->images[fallback->used], snapshot, width, height);
