@@ -25,6 +25,7 @@
 
 #include "gtk/gtkcssparserprivate.h"
 #include "gtk/gtkcsstypesprivate.h"
+#include "gtk/gtksnapshotprivate.h"
 
 G_BEGIN_DECLS
 
@@ -59,8 +60,7 @@ struct _GtkCssImageClass
                                                     guint                       property_id,
                                                     GtkStyleProviderPrivate    *provider,
                                                     GtkCssStyle                *style,
-                                                    GtkCssStyle                *parent_style,
-                                                    GtkCssDependencies         *dependencies);
+                                                    GtkCssStyle                *parent_style);
   /* compare two images for equality */
   gboolean     (* equal)                           (GtkCssImage                *image1,
                                                     GtkCssImage                *image2);
@@ -75,6 +75,10 @@ struct _GtkCssImageClass
                                                     cairo_t                    *cr,
                                                     double                      width,
                                                     double                      height);
+  void         (* snapshot)                        (GtkCssImage                *image,
+                                                    GtkSnapshot                *snapshot,
+                                                    double                      width,
+                                                    double                      height);
   /* parse CSS, return TRUE on success */
   gboolean     (* parse)                           (GtkCssImage                *image,
                                                     GtkCssParser               *parser);
@@ -96,8 +100,7 @@ GtkCssImage *  _gtk_css_image_compute              (GtkCssImage                *
                                                     guint                       property_id,
                                                     GtkStyleProviderPrivate    *provider,
                                                     GtkCssStyle                *style,
-                                                    GtkCssStyle                *parent_style,
-                                                    GtkCssDependencies         *dependencies);
+                                                    GtkCssStyle                *parent_style);
 gboolean       _gtk_css_image_equal                (GtkCssImage                *image1,
                                                     GtkCssImage                *image2);
 GtkCssImage *  _gtk_css_image_transition           (GtkCssImage                *start,
@@ -109,6 +112,10 @@ void           _gtk_css_image_draw                 (GtkCssImage                *
                                                     cairo_t                    *cr,
                                                     double                      width,
                                                     double                      height);
+void           gtk_css_image_snapshot              (GtkCssImage                *image,
+                                                    GtkSnapshot                *snapshot,
+                                                    double                      width,
+                                                    double                      height);
 void           _gtk_css_image_print                (GtkCssImage                *image,
                                                     GString                    *string);
 
