@@ -44,7 +44,6 @@
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
-#include "gtkutilsprivate.h"
 #include "gtkversion.h"
 
 /**
@@ -62,11 +61,11 @@
  * gtk_style_context_add_provider_for_screen().
 
  * In addition, certain files will be read when GTK+ is initialized. First, the
- * file `$XDG_CONFIG_HOME/gtk-3.0/gtk.css` is loaded if it exists. Then, GTK+
+ * file `$XDG_CONFIG_HOME/gtk-4.0/gtk.css` is loaded if it exists. Then, GTK+
  * loads the first existing file among
- * `XDG_DATA_HOME/themes/THEME/gtk-VERSION/gtk.css`,
- * `$HOME/.themes/THEME/gtk-VERSION/gtk.css`,
- * `$XDG_DATA_DIRS/themes/THEME/gtk-VERSION/gtk.css` and
+ * `XDG_DATA_HOME/themes/theme-name/gtk-VERSION/gtk.css`,
+ * `$HOME/.themes/theme-name/gtk-VERSION/gtk.css`,
+ * `$XDG_DATA_DIRS/themes/theme-name/gtk-VERSION/gtk.css` and
  * `DATADIR/share/themes/THEME/gtk-VERSION/gtk.css`, where `THEME` is the name of
  * the current theme (see the #GtkSettings:gtk-theme-name setting), `DATADIR`
  * is the prefix configured when GTK+ was compiled (unless overridden by the
@@ -82,7 +81,6 @@
 typedef struct GtkCssRuleset GtkCssRuleset;
 typedef struct _GtkCssScanner GtkCssScanner;
 typedef struct _PropertyValue PropertyValue;
-typedef struct _WidgetPropertyValue WidgetPropertyValue;
 typedef enum ParserScope ParserScope;
 typedef enum ParserSymbol ParserSymbol;
 
@@ -92,24 +90,14 @@ struct _PropertyValue {
   GtkCssSection       *section;
 };
 
-struct _WidgetPropertyValue {
-  WidgetPropertyValue *next;
-  char *name;
-  char *value;
-
-  GtkCssSection *section;
-};
-
 struct GtkCssRuleset
 {
   GtkCssSelector *selector;
   GtkCssSelectorTree *selector_match;
-  WidgetPropertyValue *widget_style;
   PropertyValue *styles;
   GtkBitmask *set_styles;
   guint n_styles;
   guint owns_styles : 1;
-  guint owns_widget_style : 1;
 };
 
 struct _GtkCssScanner
@@ -146,17 +134,15 @@ static guint css_provider_signals[LAST_SIGNAL] = { 0 };
 static void gtk_css_provider_finalize (GObject *object);
 static void gtk_css_style_provider_iface_init (GtkStyleProviderIface *iface);
 static void gtk_css_style_provider_private_iface_init (GtkStyleProviderPrivateInterface *iface);
-static void widget_property_value_list_free (WidgetPropertyValue *head);
 static void gtk_css_style_provider_emit_error (GtkStyleProviderPrivate *provider,
                                                GtkCssSection           *section,
                                                const GError            *error);
 
-static gboolean
+static void
 gtk_css_provider_load_internal (GtkCssProvider *css_provider,
                                 GtkCssScanner  *scanner,
                                 GFile          *file,
-                                const char     *data,
-                                GError        **error);
+                                const char     *data);
 
 GQuark
 gtk_css_provider_error_quark (void)
@@ -257,8 +243,6 @@ gtk_css_ruleset_init_copy (GtkCssRuleset       *new,
   /* First copy takes over ownership */
   if (ruleset->owns_styles)
     ruleset->owns_styles = FALSE;
-  if (ruleset->owns_widget_style)
-    ruleset->owns_widget_style = FALSE;
   if (new->set_styles)
     new->set_styles = _gtk_bitmask_copy (new->set_styles);
 }
@@ -281,82 +265,12 @@ gtk_css_ruleset_clear (GtkCssRuleset *ruleset)
     }
   if (ruleset->set_styles)
     _gtk_bitmask_free (ruleset->set_styles);
-  if (ruleset->owns_widget_style)
-    widget_property_value_list_free (ruleset->widget_style);
   if (ruleset->selector)
     _gtk_css_selector_free (ruleset->selector);
 
   memset (ruleset, 0, sizeof (GtkCssRuleset));
 }
 
-static WidgetPropertyValue *
-widget_property_value_new (char *name, GtkCssSection *section)
-{
-  WidgetPropertyValue *value;
-
-  value = g_slice_new0 (WidgetPropertyValue);
-
-  value->name = name;
-  if (gtk_keep_css_sections)
-    value->section = gtk_css_section_ref (section);
-
-  return value;
-}
-
-static void
-widget_property_value_free (WidgetPropertyValue *value)
-{
-  g_free (value->value);
-  g_free (value->name);
-  if (value->section)
-    gtk_css_section_unref (value->section);
-
-  g_slice_free (WidgetPropertyValue, value);
-}
-
-static void
-widget_property_value_list_free (WidgetPropertyValue *head)
-{
-  WidgetPropertyValue *l, *next;
-  for (l = head; l != NULL; l = next)
-    {
-      next = l->next;
-      widget_property_value_free (l);
-    }
-}
-
-static WidgetPropertyValue *
-widget_property_value_list_remove_name (WidgetPropertyValue *head, const char *name)
-{
-  WidgetPropertyValue *l, **last;
-
-  last = &head;
-
-  for (l = head; l != NULL; l = l->next)
-    {
-      if (strcmp (l->name, name) == 0)
-	{
-	  *last = l->next;
-	  widget_property_value_free (l);
-	  break;
-	}
-
-      last = &l->next;
-    }
-
-  return head;
-}
-
-static void
-gtk_css_ruleset_add_style (GtkCssRuleset *ruleset,
-                           char          *name,
-                           WidgetPropertyValue *value)
-{
-  value->next = widget_property_value_list_remove_name (ruleset->widget_style, name);
-  ruleset->widget_style = value;
-  ruleset->owns_widget_style = TRUE;
-}
-
 static void
 gtk_css_ruleset_add (GtkCssRuleset       *ruleset,
                      GtkCssStyleProperty *property,
@@ -624,92 +538,9 @@ verify_tree_get_change_results (GtkCssProvider *provider,
 }
 
 
-static gboolean
-gtk_css_provider_get_style_property (GtkStyleProvider *provider,
-                                     GtkWidgetPath    *path,
-                                     GtkStateFlags     state,
-                                     GParamSpec       *pspec,
-                                     GValue           *value)
-{
-  GtkCssProvider *css_provider = GTK_CSS_PROVIDER (provider);
-  GtkCssProviderPrivate *priv = css_provider->priv;
-  WidgetPropertyValue *val;
-  GPtrArray *tree_rules;
-  GtkCssMatcher matcher;
-  gboolean found = FALSE;
-  gchar *prop_name;
-  gint i;
-
-  if (state == gtk_widget_path_iter_get_state (path, -1))
-    {
-      gtk_widget_path_ref (path);
-    }
-  else
-    {
-      path = gtk_widget_path_copy (path);
-      gtk_widget_path_iter_set_state (path, -1, state);
-    }
-
-  if (!_gtk_css_matcher_init (&matcher, path, NULL))
-    {
-      gtk_widget_path_unref (path);
-      return FALSE;
-    }
-
-  tree_rules = _gtk_css_selector_tree_match_all (priv->tree, &matcher);
-  if (tree_rules)
-    {
-      verify_tree_match_results (css_provider, &matcher, tree_rules);
-
-      prop_name = g_strdup_printf ("-%s-%s",
-                                   g_type_name (pspec->owner_type),
-                                   pspec->name);
-
-      for (i = tree_rules->len - 1; i >= 0; i--)
-        {
-          GtkCssRuleset *ruleset = tree_rules->pdata[i];
-
-          if (ruleset->widget_style == NULL)
-            continue;
-
-          for (val = ruleset->widget_style; val != NULL; val = val->next)
-            {
-              if (strcmp (val->name, prop_name) == 0)
-                {
-                  GtkCssScanner *scanner;
-
-	          scanner = gtk_css_scanner_new (css_provider,
-                                                 NULL,
-                                                 val->section,
-                                                 val->section != NULL ? gtk_css_section_get_file (val->section) : NULL,
-                                                 val->value);
-                  if (!val->section)
-                    gtk_css_scanner_push_section (scanner, GTK_CSS_SECTION_VALUE);
-                  found = _gtk_css_style_funcs_parse_value (value, scanner->parser);
-                  if (!val->section)
-                    gtk_css_scanner_pop_section (scanner, GTK_CSS_SECTION_VALUE);
-                  gtk_css_scanner_destroy (scanner);
-	          break;
-                }
-            }
-
-          if (found)
-            break;
-        }
-
-      g_free (prop_name);
-      g_ptr_array_free (tree_rules, TRUE);
-    }
-
-  gtk_widget_path_unref (path);
-
-  return found;
-}
-
 static void
 gtk_css_style_provider_iface_init (GtkStyleProviderIface *iface)
 {
-  iface->get_style_property = gtk_css_provider_get_style_property;
 }
 
 static GtkCssValue *
@@ -918,12 +749,6 @@ css_provider_commit (GtkCssProvider *css_provider,
 
   priv = css_provider->priv;
 
-  if (ruleset->styles == NULL && ruleset->widget_style == NULL)
-    {
-      g_slist_free_full (selectors, (GDestroyNotify) _gtk_css_selector_free);
-      return;
-    }
-
   for (l = selectors; l; l = l->next)
     {
       GtkCssRuleset new;
@@ -968,37 +793,6 @@ gtk_css_provider_reset (GtkCssProvider *css_provider)
 
 }
 
-static void
-gtk_css_provider_propagate_error (GtkCssProvider  *provider,
-                                  GtkCssSection   *section,
-                                  const GError    *error,
-                                  GError         **propagate_to)
-{
-
-  char *s;
-
-  /* don't fail for deprecations */
-  if (g_error_matches (error, GTK_CSS_PROVIDER_ERROR, GTK_CSS_PROVIDER_ERROR_DEPRECATED))
-    {
-      s = _gtk_css_section_to_string (section);
-      g_warning ("Theme parsing error: %s: %s", s, error->message);
-      g_free (s);
-      return;
-    }
-
-  /* we already set an error. And we'd like to keep the first one */
-  if (*propagate_to)
-    return;
-
-  *propagate_to = g_error_copy (error);
-  if (section)
-    {
-      s = _gtk_css_section_to_string (section);
-      g_prefix_error (propagate_to, "%s", s);
-      g_free (s);
-    }
-}
-
 static gboolean
 parse_import (GtkCssScanner *scanner)
 {
@@ -1053,7 +847,6 @@ parse_import (GtkCssScanner *scanner)
       gtk_css_provider_load_internal (scanner->provider,
                                       scanner,
                                       file,
-                                      NULL,
                                       NULL);
     }
 
@@ -1345,67 +1138,6 @@ parse_selector_list (GtkCssScanner *scanner)
   return selectors;
 }
 
-static gboolean
-name_is_style_property (const char *name)
-{
-  if (name[0] != '-')
-    return FALSE;
-
-  if (g_str_has_prefix (name, "-gtk-"))
-    return FALSE;
-
-  return TRUE;
-}
-
-static void
-warn_if_deprecated (GtkCssScanner *scanner,
-                    const gchar   *name)
-{
-  gchar *n = NULL;
-  gchar *p;
-  const gchar *type_name;
-  const gchar *property_name;
-  GType type;
-  GTypeClass *class = NULL;
-  GParamSpec *pspec;
-
-  n = g_strdup (name);
-
-  /* skip initial - */
-  type_name = n + 1;
-
-  p = strchr (type_name, '-');
-  if (!p)
-    goto out;
-
-  p[0] = '\0';
-  property_name = p + 1;
-
-  type = g_type_from_name (type_name);
-  if (type == G_TYPE_INVALID ||
-      !g_type_is_a (type, GTK_TYPE_WIDGET))
-    goto out;
-
-  class = g_type_class_ref (type);
-  pspec = gtk_widget_class_find_style_property (GTK_WIDGET_CLASS (class), property_name);
-  if (!pspec)
-    goto out;
-
-  if (!(pspec->flags & G_PARAM_DEPRECATED))
-    goto out;
-
-  _gtk_css_parser_error_full (scanner->parser,
-                              GTK_CSS_PROVIDER_ERROR_DEPRECATED,
-                              "The style property %s:%s is deprecated and shouldn't be "
-                              "used anymore. It will be removed in a future version",
-                              g_type_name (pspec->owner_type), pspec->name);
-
-out:
-  g_free (n);
-  if (class)
-    g_type_class_unref (class);
-}
-
 static void
 parse_declaration (GtkCssScanner *scanner,
                    GtkCssRuleset *ruleset)
@@ -1420,19 +1152,6 @@ parse_declaration (GtkCssScanner *scanner,
     goto check_for_semicolon;
 
   property = _gtk_style_property_lookup (name);
-  if (property == NULL && !name_is_style_property (name))
-    {
-      gtk_css_provider_error (scanner->provider,
-                              scanner,
-                              GTK_CSS_PROVIDER_ERROR,
-                              GTK_CSS_PROVIDER_ERROR_NAME,
-                              "'%s' is not a valid property name",
-                              name);
-      _gtk_css_parser_resync (scanner->parser, TRUE, '}');
-      g_free (name);
-      gtk_css_scanner_pop_section (scanner, GTK_CSS_SECTION_DECLARATION);
-      return;
-    }
 
   if (property != NULL && strcmp (name, property->name) != 0)
     {
@@ -1522,34 +1241,6 @@ parse_declaration (GtkCssScanner *scanner,
         }
 
 
-      gtk_css_scanner_pop_section (scanner, GTK_CSS_SECTION_VALUE);
-    }
-  else if (name_is_style_property (name))
-    {
-      char *value_str;
-
-      warn_if_deprecated (scanner, name);
-
-      gtk_css_scanner_push_section (scanner, GTK_CSS_SECTION_VALUE);
-
-      value_str = _gtk_css_parser_read_value (scanner->parser);
-      if (value_str)
-        {
-          WidgetPropertyValue *val;
-
-          val = widget_property_value_new (name, scanner->section);
-	  val->value = value_str;
-
-          gtk_css_ruleset_add_style (ruleset, name, val);
-        }
-      else
-        {
-          _gtk_css_parser_resync (scanner->parser, TRUE, '}');
-          gtk_css_scanner_pop_section (scanner, GTK_CSS_SECTION_VALUE);
-          gtk_css_scanner_pop_section (scanner, GTK_CSS_SECTION_DECLARATION);
-          return;
-        }
-
       gtk_css_scanner_pop_section (scanner, GTK_CSS_SECTION_VALUE);
     }
   else
@@ -1716,34 +1407,24 @@ gtk_css_provider_postprocess (GtkCssProvider *css_provider)
 #endif
 }
 
-static gboolean
+static void
 gtk_css_provider_load_internal (GtkCssProvider *css_provider,
                                 GtkCssScanner  *parent,
                                 GFile          *file,
-                                const char     *text,
-                                GError        **error)
+                                const char     *text)
 {
-  GBytes *free_bytes = NULL;
   GtkCssScanner *scanner;
-  gulong error_handler;
-
-  if (error)
-    error_handler = g_signal_connect (css_provider,
-                                      "parsing-error",
-                                      G_CALLBACK (gtk_css_provider_propagate_error),
-                                      error);
-  else
-    error_handler = 0; /* silence gcc */
+  char *free_data = NULL;
 
   if (text == NULL)
     {
       GError *load_error = NULL;
 
-      free_bytes = gtk_file_load_bytes (file, NULL, &load_error);
-
-      if (free_bytes != NULL)
+      if (g_file_load_contents (file, NULL,
+                                &free_data, NULL,
+                                NULL, &load_error))
         {
-          text = g_bytes_get_data (free_bytes, NULL);
+          text = free_data;
         }
       else
         {
@@ -1792,22 +1473,7 @@ gtk_css_provider_load_internal (GtkCssProvider *css_provider,
         gtk_css_provider_postprocess (css_provider);
     }
 
-  if (free_bytes)
-    g_bytes_unref (free_bytes);
-
-  if (error)
-    {
-      g_signal_handler_disconnect (css_provider, error_handler);
-
-      if (*error)
-        {
-          /* We clear all contents from the provider for backwards compat reasons */
-          gtk_css_provider_reset (css_provider);
-          return FALSE;
-        }
-    }
-
-  return TRUE;
+  g_free (free_data);
 }
 
 /**
@@ -1817,27 +1483,19 @@ gtk_css_provider_load_internal (GtkCssProvider *css_provider,
  * @length: the length of @data in bytes, or -1 for NUL terminated strings. If
  *   @length is not -1, the code will assume it is not NUL terminated and will
  *   potentially do a copy.
- * @error: (out) (allow-none): return location for a #GError, or %NULL
  *
  * Loads @data into @css_provider, and by doing so clears any previously loaded
  * information.
- *
- * Returns: %TRUE. The return value is deprecated and %FALSE will only be
- *     returned for backwards compatibility reasons if an @error is not 
- *     %NULL and a loading error occurred. To track errors while loading
- *     CSS, connect to the #GtkCssProvider::parsing-error signal.
  **/
-gboolean
+void
 gtk_css_provider_load_from_data (GtkCssProvider  *css_provider,
                                  const gchar     *data,
-                                 gssize           length,
-                                 GError         **error)
+                                 gssize           length)
 {
   char *free_data;
-  gboolean ret;
 
-  g_return_val_if_fail (GTK_IS_CSS_PROVIDER (css_provider), FALSE);
-  g_return_val_if_fail (data != NULL, FALSE);
+  g_return_if_fail (GTK_IS_CSS_PROVIDER (css_provider));
+  g_return_if_fail (data != NULL);
 
   if (length < 0)
     {
@@ -1852,80 +1510,57 @@ gtk_css_provider_load_from_data (GtkCssProvider  *css_provider,
 
   gtk_css_provider_reset (css_provider);
 
-  ret = gtk_css_provider_load_internal (css_provider, NULL, NULL, data, error);
+  gtk_css_provider_load_internal (css_provider, NULL, NULL, data);
 
   g_free (free_data);
 
   _gtk_style_provider_private_changed (GTK_STYLE_PROVIDER_PRIVATE (css_provider));
-
-  return ret;
 }
 
 /**
  * gtk_css_provider_load_from_file:
  * @css_provider: a #GtkCssProvider
  * @file: #GFile pointing to a file to load
- * @error: (out) (allow-none): return location for a #GError, or %NULL
  *
  * Loads the data contained in @file into @css_provider, making it
  * clear any previously loaded information.
- *
- * Returns: %TRUE. The return value is deprecated and %FALSE will only be
- *     returned for backwards compatibility reasons if an @error is not 
- *     %NULL and a loading error occurred. To track errors while loading
- *     CSS, connect to the #GtkCssProvider::parsing-error signal.
  **/
-gboolean
+void
 gtk_css_provider_load_from_file (GtkCssProvider  *css_provider,
-                                 GFile           *file,
-                                 GError         **error)
+                                 GFile           *file)
 {
-  gboolean success;
-
-  g_return_val_if_fail (GTK_IS_CSS_PROVIDER (css_provider), FALSE);
-  g_return_val_if_fail (G_IS_FILE (file), FALSE);
+  g_return_if_fail (GTK_IS_CSS_PROVIDER (css_provider));
+  g_return_if_fail (G_IS_FILE (file));
 
   gtk_css_provider_reset (css_provider);
 
-  success = gtk_css_provider_load_internal (css_provider, NULL, file, NULL, error);
+  gtk_css_provider_load_internal (css_provider, NULL, file, NULL);
 
   _gtk_style_provider_private_changed (GTK_STYLE_PROVIDER_PRIVATE (css_provider));
-
-  return success;
 }
 
 /**
  * gtk_css_provider_load_from_path:
  * @css_provider: a #GtkCssProvider
  * @path: the path of a filename to load, in the GLib filename encoding
- * @error: (out) (allow-none): return location for a #GError, or %NULL
  *
  * Loads the data contained in @path into @css_provider, making it clear
  * any previously loaded information.
- *
- * Returns: %TRUE. The return value is deprecated and %FALSE will only be
- *     returned for backwards compatibility reasons if an @error is not 
- *     %NULL and a loading error occurred. To track errors while loading
- *     CSS, connect to the #GtkCssProvider::parsing-error signal.
  **/
-gboolean
+void
 gtk_css_provider_load_from_path (GtkCssProvider  *css_provider,
-                                 const gchar     *path,
-                                 GError         **error)
+                                 const gchar     *path)
 {
   GFile *file;
-  gboolean result;
 
-  g_return_val_if_fail (GTK_IS_CSS_PROVIDER (css_provider), FALSE);
-  g_return_val_if_fail (path != NULL, FALSE);
+  g_return_if_fail (GTK_IS_CSS_PROVIDER (css_provider));
+  g_return_if_fail (path != NULL);
 
   file = g_file_new_for_path (path);
   
-  result = gtk_css_provider_load_from_file (css_provider, file, error);
+  gtk_css_provider_load_from_file (css_provider, file);
 
   g_object_unref (file);
-
-  return result;
 }
 
 /**
@@ -1959,7 +1594,7 @@ gtk_css_provider_load_from_resource (GtkCssProvider *css_provider,
   file = g_file_new_for_uri (uri);
   g_free (uri);
 
-  gtk_css_provider_load_from_file (css_provider, file, NULL);
+  gtk_css_provider_load_from_file (css_provider, file);
 
   g_object_unref (file);
 }
@@ -1972,8 +1607,6 @@ gtk_css_provider_load_from_resource (GtkCssProvider *css_provider,
  *
  * Returns: (transfer none): The provider used for fallback styling.
  *          This memory is owned by GTK+, and you must not free it.
- *
- * Deprecated: 3.24: Use gtk_css_provider_new() instead.
  **/
 GtkCssProvider *
 gtk_css_provider_get_default (void)
@@ -2016,14 +1649,11 @@ _gtk_css_provider_get_theme_dir (GtkCssProvider *provider)
 
 /*
  * Look for
- * $dir/$subdir/gtk-3.16/gtk-$variant.css
- * $dir/$subdir/gtk-3.14/gtk-$variant.css
+ * $dir/$subdir/gtk-4.16/gtk-$variant.css
+ * $dir/$subdir/gtk-4.14/gtk-$variant.css
  *  ...
- * $dir/$subdir/gtk-3.0/gtk-$variant.css
+ * $dir/$subdir/gtk-4.0/gtk-$variant.css
  * and return the first found file.
- * We don't check versions before 3.14,
- * since those GTK+ versions didn't have
- * the versioned loading mechanism.
  */
 static gchar *
 _gtk_css_find_theme_dir (const gchar *dir,
@@ -2052,7 +1682,7 @@ _gtk_css_find_theme_dir (const gchar *dir,
       if (i < 14)
         i = 0;
 
-      subsubdir = g_strdup_printf ("gtk-3.%d", i);
+      subsubdir = g_strdup_printf ("gtk-4.%d", i);
       path = g_build_filename (base, subsubdir, file, NULL);
       g_free (subsubdir);
 
@@ -2163,7 +1793,7 @@ _gtk_css_provider_load_named (GtkCssProvider *provider,
       if (resource != NULL)
         g_resources_register (resource);
 
-      gtk_css_provider_load_from_path (provider, path, NULL);
+      gtk_css_provider_load_from_path (provider, path);
 
       /* Only set this after load, as load_from_path will clear it */
       provider->priv->resource = resource;
@@ -2240,20 +1870,10 @@ compare_properties (gconstpointer a, gconstpointer b, gpointer style)
                  _gtk_style_property_get_name (GTK_STYLE_PROPERTY (styles[*ub].property)));
 }
 
-static int
-compare_names (gconstpointer a, gconstpointer b)
-{
-  const WidgetPropertyValue *aa = a;
-  const WidgetPropertyValue *bb = b;
-  return strcmp (aa->name, bb->name);
-}
-
 static void
 gtk_css_ruleset_print (const GtkCssRuleset *ruleset,
                        GString             *str)
 {
-  GList *values, *walk;
-  WidgetPropertyValue *widget_value;
   guint i;
 
   _gtk_css_selector_tree_match_print (ruleset->selector_match, str);
@@ -2283,29 +1903,6 @@ gtk_css_ruleset_print (const GtkCssRuleset *ruleset,
       g_free (sorted);
     }
 
-  if (ruleset->widget_style)
-    {
-      values = NULL;
-      for (widget_value = ruleset->widget_style; widget_value != NULL; widget_value = widget_value->next)
-	values = g_list_prepend (values, widget_value);
-
-      /* so the output is identical for identical selector styles */
-      values = g_list_sort (values, compare_names);
-
-      for (walk = values; walk; walk = walk->next)
-        {
-	  widget_value = walk->data;
-
-          g_string_append (str, "  ");
-          g_string_append (str, widget_value->name);
-          g_string_append (str, ": ");
-          g_string_append (str, widget_value->value);
-          g_string_append (str, ";\n");
-        }
-
-      g_list_free (values);
-    }
-
   g_string_append (str, "}\n");
 }
 
