@@ -69,8 +69,6 @@ GtkCssValue *
 gtk_css_style_get_value (GtkCssStyle *style,
                           guint        id)
 {
-  gtk_internal_return_val_if_fail (GTK_IS_CSS_STYLE (style), NULL);
-
   return GTK_CSS_STYLE_GET_CLASS (style)->get_value (style, id);
 }
 
@@ -227,7 +225,6 @@ gtk_css_style_get_pango_attributes (GtkCssStyle *style)
   const GdkRGBA *color;
   const GdkRGBA *decoration_color;
   gint letter_spacing;
-  const char *font_feature_settings;
 
   /* text-decoration */
   decoration_line = _gtk_css_text_decoration_line_value_get (gtk_css_style_get_value (style, GTK_CSS_PROPERTY_TEXT_DECORATION_LINE));
@@ -263,13 +260,6 @@ gtk_css_style_get_pango_attributes (GtkCssStyle *style)
       attrs = add_pango_attr (attrs, pango_attr_letter_spacing_new (letter_spacing * PANGO_SCALE));
     }
 
-  /* font-feature-settings */
-  font_feature_settings = _gtk_css_string_value_get (gtk_css_style_get_value (style, GTK_CSS_PROPERTY_FONT_FEATURE_SETTINGS));
-  if (font_feature_settings != NULL)
-    {
-      attrs = add_pango_attr (attrs, pango_attr_font_features_new (font_feature_settings));
-    }
-
   return attrs;
 }
 
