@@ -22,6 +22,46 @@
 #include "gtkcssnumbervalueprivate.h"
 #include "gtkstylecontextprivate.h"
 
+cairo_operator_t
+_gtk_css_blend_mode_get_operator (GskBlendMode mode)
+{
+  switch (mode)
+    {
+    case GSK_BLEND_MODE_COLOR:
+      return CAIRO_OPERATOR_HSL_COLOR;
+    case GSK_BLEND_MODE_COLOR_BURN:
+      return CAIRO_OPERATOR_COLOR_BURN;
+    case GSK_BLEND_MODE_COLOR_DODGE:
+      return CAIRO_OPERATOR_COLOR_DODGE;
+    case GSK_BLEND_MODE_DARKEN:
+      return CAIRO_OPERATOR_DARKEN;
+    case GSK_BLEND_MODE_DIFFERENCE:
+      return CAIRO_OPERATOR_DIFFERENCE;
+    case GSK_BLEND_MODE_EXCLUSION:
+      return CAIRO_OPERATOR_EXCLUSION;
+    case GSK_BLEND_MODE_HARD_LIGHT:
+      return CAIRO_OPERATOR_HARD_LIGHT;
+    case GSK_BLEND_MODE_HUE:
+      return CAIRO_OPERATOR_HSL_HUE;
+    case GSK_BLEND_MODE_LIGHTEN:
+      return CAIRO_OPERATOR_LIGHTEN;
+    case GSK_BLEND_MODE_LUMINOSITY:
+      return CAIRO_OPERATOR_HSL_LUMINOSITY;
+    case GSK_BLEND_MODE_MULTIPLY:
+      return CAIRO_OPERATOR_MULTIPLY;
+    case GSK_BLEND_MODE_OVERLAY:
+      return CAIRO_OPERATOR_OVERLAY;
+    case GSK_BLEND_MODE_SATURATION:
+      return CAIRO_OPERATOR_HSL_SATURATION;
+    case GSK_BLEND_MODE_SCREEN:
+      return CAIRO_OPERATOR_SCREEN;
+
+    case GSK_BLEND_MODE_DEFAULT:
+    default:
+      return CAIRO_OPERATOR_OVER;
+    }
+}
+
 GtkCssChange
 _gtk_css_change_for_sibling (GtkCssChange match)
 {
@@ -138,6 +178,44 @@ gtk_css_change_print (GtkCssChange  change,
     }
 }
 
+GtkCssDimension
+gtk_css_unit_get_dimension (GtkCssUnit unit)
+{
+  switch (unit)
+    {
+    case GTK_CSS_NUMBER:
+      return GTK_CSS_DIMENSION_NUMBER;
+
+    case GTK_CSS_PERCENT:
+      return GTK_CSS_DIMENSION_PERCENTAGE;
+
+    case GTK_CSS_PX:
+    case GTK_CSS_PT:
+    case GTK_CSS_EM:
+    case GTK_CSS_EX:
+    case GTK_CSS_REM:
+    case GTK_CSS_PC:
+    case GTK_CSS_IN:
+    case GTK_CSS_CM:
+    case GTK_CSS_MM:
+      return GTK_CSS_DIMENSION_LENGTH;
+
+    case GTK_CSS_RAD:
+    case GTK_CSS_DEG:
+    case GTK_CSS_GRAD:
+    case GTK_CSS_TURN:
+      return GTK_CSS_DIMENSION_ANGLE;
+
+    case GTK_CSS_S:
+    case GTK_CSS_MS:
+      return GTK_CSS_DIMENSION_TIME;
+
+    default:
+      g_assert_not_reached ();
+      return GTK_CSS_DIMENSION_PERCENTAGE;
+    }
+}
+
 char *
 gtk_css_change_to_string (GtkCssChange change)
 {
