@@ -126,7 +126,7 @@
  *  // Add the label, and show everything we’ve added
  *
  *  gtk_container_add (GTK_CONTAINER (content_area), label);
- *  gtk_widget_show_all (dialog);
+ *  gtk_widget_show (dialog);
  * }
  * ]|
  *
@@ -443,7 +443,6 @@ add_cb (GtkContainer *container,
     g_warning ("Content added to the action area of a dialog using header bars");
 
   gtk_widget_set_visible (priv->action_box, TRUE);
-  gtk_widget_set_no_show_all (priv->action_box, FALSE);
 }
 
 static void
@@ -495,7 +494,6 @@ gtk_dialog_constructed (GObject *object)
     }
 
   gtk_widget_set_visible (priv->action_box, !priv->use_header_bar);
-  gtk_widget_set_no_show_all (priv->action_box, priv->use_header_bar);
 }
 
 static void
