@@ -28,6 +28,7 @@
 #include "gtkprivate.h"
 #include "gtksnapshot.h"
 #include "gtkstylecontext.h"
+#include "gtkwidgetprivate.h"
 
 typedef struct _GtkDrawingAreaPrivate GtkDrawingAreaPrivate;
 
@@ -60,8 +61,7 @@ static GParamSpec *props[LAST_PROP] = { NULL, };
  * creating a drawing area, the application may want to connect to:
  *
  * - Mouse and button press signals to respond to input from
- *   the user. (Use gtk_widget_add_events() to enable events
- *   you wish to receive.)
+ *   the user.
  *
  * - The #GtkWidget::realize signal to take any necessary actions
  *   when the widget is instantiated on a particular display.
@@ -79,8 +79,7 @@ static GParamSpec *props[LAST_PROP] = { NULL, };
  *
  * Note that GDK automatically clears the exposed area before causing a
  * redraw, and that drawing is implicitly clipped to the exposed
- * area. If you want to have a theme-provided background, you need
- * to call gtk_render_background() in your ::draw method.
+ * area.
  *
  * ## Simple GtkDrawingArea usage
  *
@@ -95,8 +94,6 @@ static GParamSpec *props[LAST_PROP] = { NULL, };
  *
  *   context = gtk_widget_get_style_context (GTK_WIDGET (area));
  *
- *   gtk_render_background (context, cr, 0, 0, width, height);
- *
  *   cairo_arc (cr,
  *              width / 2.0, height / 2.0,
  *              MIN (width, height) / 2.0,
@@ -235,21 +232,22 @@ gtk_drawing_area_snapshot (GtkWidget   *widget,
   GtkDrawingArea *self = GTK_DRAWING_AREA (widget);
   GtkDrawingAreaPrivate *priv = gtk_drawing_area_get_instance_private (self);
   cairo_t *cr;
+  int width, height;
 
   if (!priv->draw_func)
     return;
 
+  gtk_widget_get_content_size (widget, &width, &height);
+
   cr = gtk_snapshot_append_cairo (snapshot,
                                   &GRAPHENE_RECT_INIT (
                                       0, 0,
-                                      gtk_widget_get_allocated_width (widget),
-                                      gtk_widget_get_allocated_height (widget)
+                                      width, height
                                   ),
                                   "DrawingAreaContents");
   priv->draw_func (self,
                    cr,
-                   gtk_widget_get_allocated_width (widget),
-                   gtk_widget_get_allocated_height (widget),
+                   width, height,
                    priv->draw_func_target);
   cairo_destroy (cr);
 }
