@@ -119,9 +119,6 @@
 #include "gtkdnd.h"
 #include "a11y/gtkexpanderaccessible.h"
 #include "gtkstylecontextprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
-#include "gtkboxgadgetprivate.h"
-#include "gtkbuiltiniconprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtkcontainerprivate.h"
 #include "gtkiconprivate.h"
@@ -145,12 +142,12 @@ struct _GtkExpanderPrivate
 {
   GtkWidget        *label_widget;
 
-  GtkCssGadget     *gadget;
+  GtkWidget        *box;
   GtkWidget        *title_widget;
   GtkWidget        *arrow_widget;
+  GtkWidget        *child;
 
   GtkGesture       *multipress_gesture;
-  gint              spacing;
 
   guint             expand_timer;
 
@@ -173,11 +170,10 @@ static void gtk_expander_get_property (GObject          *object,
                                        GParamSpec       *pspec);
 
 static void     gtk_expander_destroy        (GtkWidget        *widget);
-static void     gtk_expander_size_allocate  (GtkWidget        *widget,
-                                             GtkAllocation    *allocation);
-static void     gtk_expander_snapshot       (GtkWidget        *widget,
-                                             GtkSnapshot      *snapshot);
-
+static void     gtk_expander_size_allocate  (GtkWidget           *widget,
+                                             const GtkAllocation *allocation,
+                                             int                  baseline,
+                                             GtkAllocation       *out_clip);
 static gboolean gtk_expander_enter_notify   (GtkWidget        *widget,
                                              GdkEventCrossing *event);
 static gboolean gtk_expander_leave_notify   (GtkWidget        *widget,
@@ -232,19 +228,23 @@ static void     gesture_multipress_released_cb (GtkGestureMultiPress *gesture,
                                                 gdouble               y,
                                                 GtkExpander          *expander);
 
-G_DEFINE_TYPE_WITH_CODE (GtkExpander, gtk_expander, GTK_TYPE_BIN,
+G_DEFINE_TYPE_WITH_CODE (GtkExpander, gtk_expander, GTK_TYPE_CONTAINER,
                          G_ADD_PRIVATE (GtkExpander)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
                                                 gtk_expander_buildable_init))
 
 static void
-gtk_expander_finalize (GObject *obj)
+gtk_expander_forall (GtkContainer *container,
+                     GtkCallback   callback,
+                     gpointer      user_data)
 {
-  GtkExpanderPrivate *priv = gtk_expander_get_instance_private (GTK_EXPANDER (obj));
+  GtkExpanderPrivate *priv = gtk_expander_get_instance_private (GTK_EXPANDER (container));
 
-  gtk_widget_unparent (priv->title_widget);
+  if (priv->child)
+    (*callback) (priv->child, user_data);
 
-  G_OBJECT_CLASS (gtk_expander_parent_class)->finalize (obj);
+  if (priv->label_widget)
+    (*callback) (priv->label_widget, user_data);
 }
 
 static void
@@ -258,13 +258,11 @@ gtk_expander_class_init (GtkExpanderClass *klass)
   widget_class    = (GtkWidgetClass *) klass;
   container_class = (GtkContainerClass *) klass;
 
-  gobject_class->finalize     = gtk_expander_finalize;
   gobject_class->set_property = gtk_expander_set_property;
   gobject_class->get_property = gtk_expander_get_property;
 
   widget_class->destroy              = gtk_expander_destroy;
   widget_class->size_allocate        = gtk_expander_size_allocate;
-  widget_class->snapshot             = gtk_expander_snapshot;
   widget_class->enter_notify_event   = gtk_expander_enter_notify;
   widget_class->leave_notify_event   = gtk_expander_leave_notify;
   widget_class->focus                = gtk_expander_focus;
@@ -275,6 +273,7 @@ gtk_expander_class_init (GtkExpanderClass *klass)
 
   container_class->add    = gtk_expander_add;
   container_class->remove = gtk_expander_remove;
+  container_class->forall = gtk_expander_forall;
 
   klass->activate = gtk_expander_activate;
 
@@ -359,7 +358,6 @@ static void
 gtk_expander_init (GtkExpander *expander)
 {
   GtkExpanderPrivate *priv;
-  GtkCssNode *widget_node;
 
   expander->priv = priv = gtk_expander_get_instance_private (expander);
 
@@ -367,7 +365,7 @@ gtk_expander_init (GtkExpander *expander)
   gtk_widget_set_has_window (GTK_WIDGET (expander), FALSE);
 
   priv->label_widget = NULL;
-  priv->spacing = 0;
+  priv->child = NULL;
 
   priv->expanded = FALSE;
   priv->use_underline = FALSE;
@@ -377,14 +375,13 @@ gtk_expander_init (GtkExpander *expander)
   priv->expand_timer = 0;
   priv->resize_toplevel = 0;
 
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (expander));
-  priv->gadget = gtk_box_gadget_new_for_node (widget_node, GTK_WIDGET (expander));
-  gtk_box_gadget_set_orientation (GTK_BOX_GADGET (priv->gadget), GTK_ORIENTATION_VERTICAL);
+  priv->box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_widget_set_parent (priv->box, GTK_WIDGET (expander));
+
   priv->title_widget = g_object_new (GTK_TYPE_BOX,
                                      "css-name", "title",
                                      NULL);
-  gtk_widget_set_parent (priv->title_widget, GTK_WIDGET (expander));
-  gtk_box_gadget_insert_widget (GTK_BOX_GADGET (priv->gadget), -1, priv->title_widget);
+  gtk_container_add (GTK_CONTAINER (priv->box), priv->title_widget);
 
   priv->arrow_widget = gtk_icon_new ("arrow");
   gtk_style_context_add_class (gtk_widget_get_style_context (priv->arrow_widget),
@@ -515,35 +512,28 @@ gtk_expander_destroy (GtkWidget *widget)
       priv->expand_timer = 0;
     }
 
+  if (priv->box)
+    {
+      gtk_widget_unparent (priv->box);
+      priv->box = NULL;
+      priv->child = NULL;
+      priv->label_widget = NULL;
+      priv->arrow_widget = NULL;
+    }
+
   g_clear_object (&priv->multipress_gesture);
 
   GTK_WIDGET_CLASS (gtk_expander_parent_class)->destroy (widget);
-
-  g_clear_object (&priv->gadget);
 }
 
 static void
-gtk_expander_size_allocate (GtkWidget     *widget,
-                            GtkAllocation *allocation)
+gtk_expander_size_allocate (GtkWidget           *widget,
+                            const GtkAllocation *allocation,
+                            int                  baseline,
+                            GtkAllocation       *out_clip)
 {
   GtkExpanderPrivate *priv = GTK_EXPANDER (widget)->priv;
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_expander_snapshot (GtkWidget   *widget,
-                       GtkSnapshot *snapshot)
-{
-  gtk_css_gadget_snapshot (GTK_EXPANDER (widget)->priv->gadget, snapshot);
+  gtk_widget_size_allocate (priv->box, allocation, baseline, out_clip);
 }
 
 static void
@@ -554,20 +544,16 @@ gesture_multipress_pressed_cb (GtkGestureMultiPress *gesture,
                                GtkExpander          *expander)
 {
   GtkExpanderPrivate *priv = expander->priv;
-  GtkAllocation title_allocation, allocation;
+  GtkAllocation title_allocation;
 
-  gtk_widget_get_allocation (GTK_WIDGET (expander), &allocation);
-  gtk_widget_get_allocation (priv->title_widget, &title_allocation);
+  gtk_widget_get_own_allocation (priv->title_widget, &title_allocation);
+  gtk_widget_translate_coordinates (priv->title_widget, GTK_WIDGET (expander),
+                                    title_allocation.x, title_allocation.y,
+                                    &title_allocation.x, &title_allocation.y);
   /* Coordinates are in the widget coordinate system, so transform
    * the title_allocation to it.
    */
-  title_allocation.x -= allocation.x;
-  title_allocation.y -= allocation.y;
-
-  priv->pressed_in_title = (x >= title_allocation.x &&
-                            x < title_allocation.x + title_allocation.width &&
-                            y >= title_allocation.y &&
-                            y < title_allocation.y + title_allocation.height);
+  priv->pressed_in_title = gdk_rectangle_contains_point (&title_allocation, x, y);
 }
 
 static void
@@ -739,7 +725,7 @@ focus_in_site (GtkExpander      *expander,
         return FALSE;
     case FOCUS_CHILD:
       {
-        GtkWidget *child = gtk_bin_get_child (GTK_BIN (expander));
+        GtkWidget *child = expander->priv->child;
 
         if (child && gtk_widget_get_child_visible (child))
           return gtk_widget_child_focus (child, direction);
@@ -831,7 +817,7 @@ static void
 gtk_expander_resize_toplevel (GtkExpander *expander)
 {
   GtkExpanderPrivate *priv = expander->priv;
-  GtkWidget *child = gtk_bin_get_child (GTK_BIN (expander));
+  GtkWidget *child = priv->child;
 
   if (child && priv->resize_toplevel &&
       gtk_widget_get_realized (GTK_WIDGET (expander)))
@@ -906,13 +892,21 @@ gtk_expander_add (GtkContainer *container,
                   GtkWidget    *widget)
 {
   GtkExpander *expander = GTK_EXPANDER (container);
+  GtkExpanderPrivate *priv = gtk_expander_get_instance_private (expander);
 
-  GTK_CONTAINER_CLASS (gtk_expander_parent_class)->add (container, widget);
+  if (priv->expanded)
+    {
+      gtk_container_add (GTK_CONTAINER (priv->box), widget);
+    }
+  else
+    {
+      if (g_object_is_floating (widget))
+        g_object_ref_sink (widget);
 
-  gtk_widget_queue_resize (GTK_WIDGET (container));
+      g_object_ref (widget);
+    }
 
-  if (expander->priv->expanded)
-    gtk_box_gadget_insert_widget (GTK_BOX_GADGET (expander->priv->gadget), -1, widget);
+  priv->child = widget;
 }
 
 static void
@@ -920,12 +914,18 @@ gtk_expander_remove (GtkContainer *container,
                      GtkWidget    *widget)
 {
   GtkExpander *expander = GTK_EXPANDER (container);
+  GtkExpanderPrivate *priv = gtk_expander_get_instance_private (expander);
 
-  if (GTK_EXPANDER (expander)->priv->label_widget == widget)
+  if (priv->label_widget == widget)
     gtk_expander_set_label_widget (expander, NULL);
   else
     {
-      gtk_box_gadget_remove_widget (GTK_BOX_GADGET (expander->priv->gadget), widget);
+      gtk_container_remove (GTK_CONTAINER (priv->box), widget);
+      if (!priv->expanded)
+        {
+          /* We hold an extra ref */
+          g_object_unref (widget);
+        }
       GTK_CONTAINER_CLASS (gtk_expander_parent_class)->remove (container, widget);
     }
 }
@@ -945,11 +945,14 @@ gtk_expander_measure (GtkWidget      *widget,
                       int            *minimum_baseline,
                       int            *natural_baseline)
 {
-  gtk_css_gadget_get_preferred_size (GTK_EXPANDER (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
+  GtkExpander *expander = GTK_EXPANDER (widget);
+  GtkExpanderPrivate *priv = gtk_expander_get_instance_private (expander);
+
+  gtk_widget_measure (priv->box,
+                       orientation,
+                       for_size,
+                       minimum, natural,
+                       minimum_baseline, natural_baseline);
 }
 
 /**
@@ -1024,16 +1027,20 @@ gtk_expander_set_expanded (GtkExpander *expander,
 
   update_node_state (expander);
 
-  child = gtk_bin_get_child (GTK_BIN (expander));
+  child = priv->child;
 
   if (child)
     {
       if (priv->expanded)
-        gtk_box_gadget_insert_widget (GTK_BOX_GADGET (priv->gadget), 1, child);
+        {
+          gtk_container_add (GTK_CONTAINER (priv->box), child);
+          g_object_unref (priv->child);
+        }
       else
-        gtk_box_gadget_remove_widget (GTK_BOX_GADGET (priv->gadget), child);
-
-      gtk_widget_queue_resize (GTK_WIDGET (expander));
+        {
+          g_object_ref (priv->child);
+          gtk_container_remove (GTK_CONTAINER (priv->box), child);
+        }
       gtk_expander_resize_toplevel (expander);
     }
 
