@@ -381,25 +381,15 @@ gtk_file_chooser_button_measure (GtkWidget       *widget,
 }
 
 static void
-gtk_file_chooser_button_snapshot (GtkWidget   *widget,
-                                  GtkSnapshot *snapshot)
+gtk_file_chooser_button_size_allocate (GtkWidget           *widget,
+                                       const GtkAllocation *allocation,
+                                       int                  baseline,
+                                       GtkAllocation       *out_clip)
 {
   GtkFileChooserButton *button = GTK_FILE_CHOOSER_BUTTON (widget);
   GtkFileChooserButtonPrivate *priv = gtk_file_chooser_button_get_instance_private (button);
 
-  gtk_widget_snapshot_child (widget, priv->child, snapshot);
-}
-
-static void
-gtk_file_chooser_button_size_allocate (GtkWidget     *widget,
-                                       GtkAllocation *allocation)
-{
-  GtkFileChooserButton *button = GTK_FILE_CHOOSER_BUTTON (widget);
-  GtkFileChooserButtonPrivate *priv = gtk_file_chooser_button_get_instance_private (button);
-
-  GTK_WIDGET_CLASS (gtk_file_chooser_button_parent_class)->size_allocate (widget, allocation);
-
-  gtk_widget_size_allocate (priv->child, allocation);
+  gtk_widget_size_allocate (priv->child, allocation, baseline, out_clip);
 }
 
 static void
@@ -427,7 +417,6 @@ gtk_file_chooser_button_class_init (GtkFileChooserButtonClass * class)
   widget_class->state_flags_changed = gtk_file_chooser_button_state_flags_changed;
   widget_class->measure = gtk_file_chooser_button_measure;
   widget_class->size_allocate = gtk_file_chooser_button_size_allocate;
-  widget_class->snapshot = gtk_file_chooser_button_snapshot;
 
   /**
    * GtkFileChooserButton::file-set:
