@@ -27,6 +27,7 @@
  * @Short_description: A container which allows you to position
  * widgets at fixed coordinates
  * @Title: GtkFixed
+ * @See_also: #GtkLayout
  *
  * The #GtkFixed widget is a container which can place child widgets
  * at fixed positions and with fixed sizes, given in pixels. #GtkFixed
@@ -43,18 +44,19 @@
  * - Fonts other than the one you used to write the app will of course
  *   change the size of widgets containing text; keep in mind that
  *   users may use a larger font because of difficulty reading the
- *   default, or they may be using Windows or the framebuffer port of
- *   GTK+, where different fonts are available.
+ *   default, or they may be using a different OS that provides different fonts.
  *
  * - Translation of text into other languages changes its size. Also,
  *   display of non-English text will use a different font in many
  *   cases.
  *
- * In addition, the fixed widget can’t properly be mirrored in
- * right-to-left languages such as Hebrew and Arabic. i.e. normally
- * GTK+ will flip the interface to put labels to the right of the
- * thing they label, but it can’t do that with #GtkFixed. So your
- * application will not be usable in right-to-left languages.
+ * In addition, #GtkFixed does not pay attention to text direction and thus may
+ * produce unwanted results if your app is run under right-to-left languages
+ * such as Hebrew or Arabic. That is: normally GTK+ will order containers
+ * appropriately for the text direction, e.g. to put labels to the right of the
+ * thing they label when using an RTL language, but it can’t do that with
+ * #GtkFixed. So if you need to reorder widgets depending on the text direction,
+ * you would need to manually detect it and adjust child positions accordingly.
  *
  * Finally, fixed positioning makes it kind of annoying to add/remove
  * GUI elements, since you have to reposition all the other
@@ -64,6 +66,9 @@
  * If you know none of these things are an issue for your application,
  * and prefer the simplicity of #GtkFixed, by all means use the
  * widget. But you should be aware of the tradeoffs.
+ *
+ * See also #GtkLayout, which shares the ability to perform fixed positioning
+ * of child widgets and additionally adds custom drawing and scrollability.
  */
 
 #include "config.h"
@@ -87,7 +92,6 @@ enum {
   CHILD_PROP_Y
 };
 
-static void gtk_fixed_realize       (GtkWidget        *widget);
 static void gtk_fixed_measure (GtkWidget      *widget,
                                GtkOrientation  orientation,
                                int             for_size,
@@ -97,8 +101,10 @@ static void gtk_fixed_measure (GtkWidget      *widget,
                                int            *natural_baseline);
 
 
-static void gtk_fixed_size_allocate (GtkWidget        *widget,
-                                     GtkAllocation    *allocation);
+static void gtk_fixed_size_allocate (GtkWidget           *widget,
+                                     const GtkAllocation *allocation,
+                                     int                  baseline,
+                                     GtkAllocation       *out_clip);
 static void gtk_fixed_snapshot      (GtkWidget        *widget,
                                      GtkSnapshot      *snapshot);
 static void gtk_fixed_add           (GtkContainer     *container,
@@ -132,7 +138,6 @@ gtk_fixed_class_init (GtkFixedClass *class)
   widget_class = (GtkWidgetClass*) class;
   container_class = (GtkContainerClass*) class;
 
-  widget_class->realize = gtk_fixed_realize;
   widget_class->measure = gtk_fixed_measure;
   widget_class->size_allocate = gtk_fixed_size_allocate;
   widget_class->snapshot = gtk_fixed_snapshot;
@@ -172,9 +177,9 @@ gtk_fixed_init (GtkFixed *fixed)
 {
   fixed->priv = gtk_fixed_get_instance_private (fixed);
 
-  gtk_widget_set_has_window (GTK_WIDGET (fixed), FALSE);
-
   fixed->priv->children = NULL;
+
+  gtk_widget_set_has_window (GTK_WIDGET (fixed), FALSE);
 }
 
 /**
@@ -346,28 +351,6 @@ gtk_fixed_get_child_property (GtkContainer *container,
     }
 }
 
-static void
-gtk_fixed_realize (GtkWidget *widget)
-{
-  GtkAllocation allocation;
-  GdkWindow *window;
-
-  if (!gtk_widget_get_has_window (widget))
-    GTK_WIDGET_CLASS (gtk_fixed_parent_class)->realize (widget);
-  else
-    {
-      gtk_widget_set_realized (widget, TRUE);
-
-      gtk_widget_get_allocation (widget, &allocation);
-
-      window = gdk_window_new_child (gtk_widget_get_parent_window (widget),
-                                     GDK_ALL_EVENTS_MASK,
-                                     &allocation);
-      gtk_widget_set_window (widget, window);
-      gtk_widget_register_window (widget, window);
-    }
-}
-
 static void
 gtk_fixed_measure (GtkWidget      *widget,
                    GtkOrientation  orientation,
@@ -409,8 +392,10 @@ gtk_fixed_measure (GtkWidget      *widget,
 }
 
 static void
-gtk_fixed_size_allocate (GtkWidget     *widget,
-                         GtkAllocation *allocation)
+gtk_fixed_size_allocate (GtkWidget           *widget,
+                         const GtkAllocation *allocation,
+                         int                  baseline,
+                         GtkAllocation       *out_clip)
 {
   GtkFixed *fixed = GTK_FIXED (widget);
   GtkFixedPrivate *priv = fixed->priv;
@@ -419,18 +404,6 @@ gtk_fixed_size_allocate (GtkWidget     *widget,
   GtkRequisition child_requisition;
   GList *children;
 
-  gtk_widget_set_allocation (widget, allocation);
-
-  if (gtk_widget_get_has_window (widget))
-    {
-      if (gtk_widget_get_realized (widget))
-        gdk_window_move_resize (gtk_widget_get_window (widget),
-                                allocation->x,
-                                allocation->y,
-                                allocation->width,
-                                allocation->height);
-    }
-
   for (children = priv->children; children; children = children->next)
     {
       child = children->data;
@@ -442,15 +415,9 @@ gtk_fixed_size_allocate (GtkWidget     *widget,
       child_allocation.x = child->x;
       child_allocation.y = child->y;
 
-      if (!gtk_widget_get_has_window (widget))
-        {
-          child_allocation.x += allocation->x;
-          child_allocation.y += allocation->y;
-        }
-
       child_allocation.width = child_requisition.width;
       child_allocation.height = child_requisition.height;
-      gtk_widget_size_allocate (child->widget, &child_allocation);
+      gtk_widget_size_allocate (child->widget, &child_allocation, -1, out_clip);
     }
 }
 
