@@ -58,7 +58,7 @@
  *
  * # CSS nodes
  *
- * GtkFontButton has a single CSS node with name button and style class .font.
+ * GtkFontButton has a single CSS node with name fontbutton.
  */
 
 
@@ -463,25 +463,15 @@ gtk_font_button_measure (GtkWidget       *widget,
 }
 
 static void
-gtk_font_button_snapshot (GtkWidget   *widget,
-                          GtkSnapshot *snapshot)
+gtk_font_button_size_allocate (GtkWidget           *widget,
+                               const GtkAllocation *allocation,
+                               int                  baseline,
+                               GtkAllocation       *out_clip)
 {
   GtkFontButton *button = GTK_FONT_BUTTON (widget);
   GtkFontButtonPrivate *priv = gtk_font_button_get_instance_private (button);
 
-  gtk_widget_snapshot_child (widget, priv->button, snapshot);
-}
-
-static void
-gtk_font_button_size_allocate (GtkWidget     *widget,
-                               GtkAllocation *allocation)
-{
-  GtkFontButton *button = GTK_FONT_BUTTON (widget);
-  GtkFontButtonPrivate *priv = gtk_font_button_get_instance_private (button);
-
-  GTK_WIDGET_CLASS (gtk_font_button_parent_class)->size_allocate (widget, allocation);
-
-  gtk_widget_size_allocate (priv->button, allocation);
+  gtk_widget_size_allocate (priv->button, allocation, baseline, out_clip);
 }
 
 static void
@@ -499,7 +489,6 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
 
   widget_class->measure = gtk_font_button_measure;
   widget_class->size_allocate = gtk_font_button_size_allocate;
-  widget_class->snapshot = gtk_font_button_snapshot;
 
   klass->font_set = NULL;
 
@@ -622,7 +611,7 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
                                                 g_cclosure_marshal_VOID__VOID,
                                                 G_TYPE_NONE, 0);
 
-  gtk_widget_class_set_css_name (widget_class, "button");
+  gtk_widget_class_set_css_name (widget_class, "fontbutton");
 }
 
 static void
