@@ -355,7 +355,7 @@ output_cb (GtkSpinButton *spin,
   adjustment = gtk_spin_button_get_adjustment (spin);
   value = gtk_adjustment_get_value (adjustment);
   text = g_strdup_printf ("%2.4g", value);
-  gtk_entry_set_text (GTK_ENTRY (spin), text);
+  gtk_spin_button_set_text (spin, text);
   g_free (text);
 
   return TRUE;
@@ -593,15 +593,17 @@ gtk_font_chooser_widget_snapshot (GtkWidget   *widget,
 }
 
 static void
-gtk_font_chooser_widget_size_allocate (GtkWidget     *widget,
-                                       GtkAllocation *allocation)
+gtk_font_chooser_widget_size_allocate (GtkWidget           *widget,
+                                       const GtkAllocation *allocation,
+                                       int                  baseline,
+                                       GtkAllocation       *out_clip)
 {
   GtkFontChooserWidget *self = GTK_FONT_CHOOSER_WIDGET (widget);
   GtkFontChooserWidgetPrivate *priv = gtk_font_chooser_widget_get_instance_private (self);
 
-  GTK_WIDGET_CLASS (gtk_font_chooser_widget_parent_class)->size_allocate (widget, allocation);
+  GTK_WIDGET_CLASS (gtk_font_chooser_widget_parent_class)->size_allocate (widget, allocation, -1, out_clip);
 
-  gtk_widget_size_allocate (priv->grid, allocation);
+  gtk_widget_size_allocate (priv->grid, allocation, -1, out_clip);
 }
 
 static void
