@@ -27,7 +27,6 @@
 #include "gtksizerequest.h"
 #include "gtkwidgetprivate.h"
 #include "gtkcontainerprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkcsspositionvalueprivate.h"
 #include "gtkstylecontextprivate.h"
 #include "gtkprivate.h"
@@ -110,8 +109,6 @@ struct _GtkGridPrivate
   GList *children;
   GList *row_properties;
 
-  GtkCssGadget *gadget;
-
   GtkOrientation orientation;
   gint baseline_row;
 
@@ -399,8 +396,6 @@ gtk_grid_finalize (GObject *object)
 
   g_list_free_full (priv->row_properties, (GDestroyNotify)gtk_grid_row_properties_free);
 
-  g_clear_object (&priv->gadget);
-
   G_OBJECT_CLASS (gtk_grid_parent_class)->finalize (object);
 }
 
@@ -1500,7 +1495,7 @@ gtk_grid_get_size_for_size (GtkGrid        *grid,
 }
 
 static void
-gtk_grid_measure_ (GtkWidget     *widget,
+gtk_grid_measure (GtkWidget     *widget,
                   GtkOrientation  orientation,
                   int             for_size,
                   int            *minimum,
@@ -1508,24 +1503,6 @@ gtk_grid_measure_ (GtkWidget     *widget,
                   int            *minimum_baseline,
                   int            *natural_baseline)
 {
-  gtk_css_gadget_get_preferred_size (GTK_GRID (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
-}
-
-static void
-gtk_grid_measure (GtkCssGadget   *gadget,
-                  GtkOrientation  orientation,
-                  int             for_size,
-                  int            *minimum,
-                  int            *natural,
-                  int            *minimum_baseline,
-                  int            *natural_baseline,
-                  gpointer        data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
   GtkGrid *grid = GTK_GRID (widget);
 
   if ((orientation == GTK_ORIENTATION_HORIZONTAL &&
@@ -1598,38 +1575,19 @@ gtk_grid_request_allocate_children (GtkGridRequest      *request,
         child_allocation.x = allocation->x + allocation->width
                              - (child_allocation.x - allocation->x) - child_allocation.width;
 
-      gtk_widget_size_allocate_with_baseline (child->widget, &child_allocation, baseline);
-      gtk_widget_get_clip (child->widget, &child_clip);
-      gdk_rectangle_union (&child_clip, out_clip, out_clip);
+      gtk_widget_size_allocate (child->widget, &child_allocation, baseline, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
     }
 }
 
 #define GET_SIZE(allocation, orientation) (orientation == GTK_ORIENTATION_HORIZONTAL ? allocation->width : allocation->height)
 
 static void
-gtk_grid_size_allocate (GtkWidget     *widget,
-                        GtkAllocation *allocation)
+gtk_grid_size_allocate (GtkWidget          *widget,
+                        const GtkAllocation *allocation,
+                        int                  baseline,
+                        GtkAllocation       *out_clip)
 {
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (GTK_GRID (widget)->priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_grid_allocate (GtkCssGadget        *gadget,
-                   const GtkAllocation *allocation,
-                   int                  baseline,
-                   GtkAllocation       *out_clip,
-                   gpointer             data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
   GtkGrid *grid = GTK_GRID (widget);
   GtkGridPrivate *priv = grid->priv;
   GtkGridRequest request;
@@ -1663,31 +1621,9 @@ gtk_grid_allocate (GtkCssGadget        *gadget,
   gtk_grid_request_position (&request, 0);
   gtk_grid_request_position (&request, 1);
 
-  *out_clip = *allocation;
   gtk_grid_request_allocate_children (&request, allocation, out_clip);
 }
 
-static gboolean
-gtk_grid_render (GtkCssGadget *gadget,
-                 GtkSnapshot  *snapshot,
-                 int           x,
-                 int           y,
-                 int           width,
-                 int           height,
-                 gpointer      data)
-{
-  GTK_WIDGET_CLASS (gtk_grid_parent_class)->snapshot (gtk_css_gadget_get_owner (gadget), snapshot);
-
-  return FALSE;
-}
-
-static void
-gtk_grid_snapshot (GtkWidget   *widget,
-                   GtkSnapshot *snapshot)
-{
-  gtk_css_gadget_snapshot (GTK_GRID (widget)->priv->gadget, snapshot);
-}
-
 static void
 gtk_grid_class_init (GtkGridClass *class)
 {
@@ -1700,8 +1636,7 @@ gtk_grid_class_init (GtkGridClass *class)
   object_class->finalize = gtk_grid_finalize;
 
   widget_class->size_allocate = gtk_grid_size_allocate;
-  widget_class->measure = gtk_grid_measure_;
-  widget_class->snapshot = gtk_grid_snapshot;
+  widget_class->measure = gtk_grid_measure;
 
   container_class->add = gtk_grid_add;
   container_class->remove = gtk_grid_remove;
@@ -1804,15 +1739,6 @@ gtk_grid_init (GtkGrid *grid)
   priv->linedata[0].homogeneous = FALSE;
   priv->linedata[1].homogeneous = FALSE;
 
-  priv->gadget = gtk_css_custom_gadget_new_for_node (gtk_widget_get_css_node (GTK_WIDGET (grid)),
-                                                     GTK_WIDGET (grid),
-                                                     gtk_grid_measure,
-                                                     gtk_grid_allocate,
-                                                     gtk_grid_render,
-                                                     NULL,
-                                                     NULL);
-
-
   _gtk_orientable_set_style_classes (GTK_ORIENTABLE (grid));
 }
 
