@@ -26,7 +26,6 @@
 #include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
 #include "gtkcssnodeprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkwindowprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtkcontainerprivate.h"
@@ -83,7 +82,6 @@ struct _GtkHeaderBarPrivate
 
   GtkWidget *titlebar_icon;
 
-  GtkCssGadget *gadget;
   guint shows_app_menu : 1;
 };
 
@@ -354,6 +352,7 @@ _gtk_header_bar_update_window_buttons (GtkHeaderBar *bar)
                   gtk_style_context_add_class (gtk_widget_get_style_context (button), "titlebutton");
                   gtk_style_context_add_class (gtk_widget_get_style_context (button), "icon");
                   gtk_widget_set_size_request (button, 20, 20);
+
                   if (!_gtk_header_bar_update_window_icon (bar, window))
                     {
                       gtk_widget_destroy (button);
@@ -374,12 +373,16 @@ _gtk_header_bar_update_window_buttons (GtkHeaderBar *bar)
                   image = gtk_image_new ();
                   gtk_container_add (GTK_CONTAINER (button), image);
                   gtk_widget_set_can_focus (button, FALSE);
+
                   accessible = gtk_widget_get_accessible (button);
                   if (GTK_IS_ACCESSIBLE (accessible))
                     atk_object_set_name (accessible, _("Application menu"));
+
                   priv->titlebar_icon = image;
                   if (!_gtk_header_bar_update_window_icon (bar, window))
-                    gtk_image_set_from_icon_name (GTK_IMAGE (priv->titlebar_icon), "process-stop-symbolic", GTK_ICON_SIZE_MENU);
+                    gtk_image_set_from_icon_name (GTK_IMAGE (priv->titlebar_icon),
+                                                  "application-x-executable-symbolic", GTK_ICON_SIZE_MENU);
+
                   priv->shows_app_menu = TRUE;
                 }
               else if (strcmp (t[j], "minimize") == 0 &&
@@ -395,6 +398,7 @@ _gtk_header_bar_update_window_buttons (GtkHeaderBar *bar)
                   gtk_widget_set_can_focus (button, FALSE);
                   g_signal_connect_swapped (button, "clicked",
                                             G_CALLBACK (gtk_window_iconify), window);
+
                   accessible = gtk_widget_get_accessible (button);
                   if (GTK_IS_ACCESSIBLE (accessible))
                     atk_object_set_name (accessible, _("Minimize"));
@@ -417,6 +421,7 @@ _gtk_header_bar_update_window_buttons (GtkHeaderBar *bar)
                   gtk_widget_set_can_focus (button, FALSE);
                   g_signal_connect_swapped (button, "clicked",
                                             G_CALLBACK (_gtk_window_toggle_maximized), window);
+
                   accessible = gtk_widget_get_accessible (button);
                   if (GTK_IS_ACCESSIBLE (accessible))
                     atk_object_set_name (accessible, maximized ? _("Restore") : _("Maximize"));
@@ -434,6 +439,7 @@ _gtk_header_bar_update_window_buttons (GtkHeaderBar *bar)
                   gtk_widget_set_can_focus (button, FALSE);
                   g_signal_connect_swapped (button, "clicked",
                                             G_CALLBACK (gtk_window_close), window);
+
                   accessible = gtk_widget_get_accessible (button);
                   if (GTK_IS_ACCESSIBLE (accessible))
                     atk_object_set_name (accessible, _("Close"));
@@ -703,7 +709,7 @@ gtk_header_bar_compute_size_for_orientation (GtkWidget *widget,
 
       if (gtk_widget_get_visible (child->widget))
         {
-          gtk_widget_measure (widget, GTK_ORIENTATION_HORIZONTAL, avail_size,
+          gtk_widget_measure (child->widget, GTK_ORIENTATION_HORIZONTAL, avail_size,
                               &child_size, &child_natural, NULL, NULL);
 
           required_size += child_size;
@@ -878,17 +884,14 @@ gtk_header_bar_compute_size_for_opposing_orientation (GtkWidget *widget,
 }
 
 static void
-gtk_header_bar_get_content_size (GtkCssGadget   *gadget,
-                                 GtkOrientation  orientation,
-                                 gint            for_size,
-                                 gint           *minimum,
-                                 gint           *natural,
-                                 gint           *minimum_baseline,
-                                 gint           *natural_baseline,
-                                 gpointer        unused)
+gtk_header_bar_measure (GtkWidget      *widget,
+                        GtkOrientation  orientation,
+                        int             for_size,
+                        int            *minimum,
+                        int            *natural,
+                        int            *minimum_baseline,
+                        int            *natural_baseline)
 {
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-
   if (for_size < 0)
     gtk_header_bar_get_size (widget, orientation, minimum, natural);
   else if (orientation == GTK_ORIENTATION_HORIZONTAL)
@@ -898,48 +901,14 @@ gtk_header_bar_get_content_size (GtkCssGadget   *gadget,
 }
 
 static void
-gtk_header_bar_measure (GtkWidget      *widget,
-                        GtkOrientation  orientation,
-                        int             for_size,
-                        int            *minimum,
-                        int            *natural,
-                        int            *minimum_baseline,
-                        int            *natural_baseline)
+gtk_header_bar_size_allocate (GtkWidget           *widget,
+                              const GtkAllocation *allocation,
+                              int                  baseline,
+                              GtkAllocation       *out_clip)
 {
   GtkHeaderBarPrivate *priv = gtk_header_bar_get_instance_private (GTK_HEADER_BAR (widget));
-
-  gtk_css_gadget_get_preferred_size (priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
-}
-
-static void
-gtk_header_bar_size_allocate (GtkWidget     *widget,
-                              GtkAllocation *allocation)
-{
-  GtkHeaderBarPrivate *priv = gtk_header_bar_get_instance_private (GTK_HEADER_BAR (widget));
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (priv->gadget, allocation, gtk_widget_get_allocated_baseline (widget), &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_header_bar_allocate_contents (GtkCssGadget        *gadget,
-                                  const GtkAllocation *allocation,
-                                  int                  baseline,
-                                  GtkAllocation       *out_clip,
-                                  gpointer             unused)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
   GtkWidget *title_widget;
   GtkHeaderBar *bar = GTK_HEADER_BAR (widget);
-  GtkHeaderBarPrivate *priv = gtk_header_bar_get_instance_private (bar);
   GtkRequestedSize *sizes;
   gint width, height;
   gint nvis_children;
@@ -959,7 +928,7 @@ gtk_header_bar_allocate_contents (GtkCssGadget        *gadget,
   gint x;
   gint child_size;
   GtkTextDirection direction;
-  GtkAllocation child_clip;
+  GtkAllocation child_clip = {0, };
 
   direction = gtk_widget_get_direction (widget);
   nvis_children = count_visible_children (bar);
@@ -1136,9 +1105,8 @@ gtk_header_bar_allocate_contents (GtkCssGadget        *gadget,
           if (direction == GTK_TEXT_DIR_RTL)
             child_allocation.x = allocation->x + allocation->width - (child_allocation.x - allocation->x) - child_allocation.width;
 
-          gtk_widget_size_allocate (child->widget, &child_allocation);
-          gtk_widget_get_clip (child->widget, &child_clip);
-          gdk_rectangle_union (&child_clip, out_clip, out_clip);
+          gtk_widget_size_allocate (child->widget, &child_allocation, baseline, &child_clip);
+          gdk_rectangle_union (out_clip, &child_clip, out_clip);
 
         next:
           i++;
@@ -1175,9 +1143,8 @@ gtk_header_bar_allocate_contents (GtkCssGadget        *gadget,
 
   if (title_widget != NULL)
     {
-      gtk_widget_size_allocate (title_widget, &child_allocation);
-      gtk_widget_get_clip (title_widget, &child_clip);
-      gdk_rectangle_union (&child_clip, out_clip, out_clip);
+      gtk_widget_size_allocate (title_widget, &child_allocation, baseline, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
     }
 
   child_allocation.y = allocation->y;
@@ -1191,9 +1158,8 @@ gtk_header_bar_allocate_contents (GtkCssGadget        *gadget,
       else
         child_allocation.x = allocation->x + allocation->width - start_width + priv->spacing;
       child_allocation.width = start_width - priv->spacing;
-      gtk_widget_size_allocate (priv->titlebar_start_box, &child_allocation);
-      gtk_widget_get_clip (priv->titlebar_start_box, &child_clip);
-      gdk_rectangle_union (&child_clip, out_clip, out_clip);
+      gtk_widget_size_allocate (priv->titlebar_start_box, &child_allocation, baseline, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
     }
 
   if (priv->titlebar_end_box)
@@ -1204,9 +1170,8 @@ gtk_header_bar_allocate_contents (GtkCssGadget        *gadget,
       else
         child_allocation.x = allocation->x + allocation->width - end_width + priv->spacing;
       child_allocation.width = end_width - priv->spacing;
-      gtk_widget_size_allocate (priv->titlebar_end_box, &child_allocation);
-      gtk_widget_get_clip (priv->titlebar_end_box, &child_clip);
-      gdk_rectangle_union (&child_clip, out_clip, out_clip);
+      gtk_widget_size_allocate (priv->titlebar_end_box, &child_allocation, baseline, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
     }
 }
 
@@ -1467,8 +1432,6 @@ gtk_header_bar_finalize (GObject *object)
   g_free (priv->subtitle);
   g_free (priv->decoration_layout);
 
-  g_clear_object (&priv->gadget);
-
   G_OBJECT_CLASS (gtk_header_bar_parent_class)->finalize (object);
 }
 
@@ -1805,31 +1768,6 @@ gtk_header_bar_set_child_property (GtkContainer *container,
     }
 }
 
-static void
-gtk_header_bar_snapshot (GtkWidget   *widget,
-                         GtkSnapshot *snapshot)
-{
-  GtkHeaderBarPrivate *priv = gtk_header_bar_get_instance_private (GTK_HEADER_BAR (widget));
-
-  gtk_css_gadget_snapshot (priv->gadget, snapshot);
-}
-
-static gboolean
-gtk_header_bar_render_contents (GtkCssGadget *gadget,
-                                GtkSnapshot  *snapshot,
-                                int           x,
-                                int           y,
-                                int           width,
-                                int           height,
-                                gpointer      unused)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-
-  GTK_WIDGET_CLASS (gtk_header_bar_parent_class)->snapshot (widget, snapshot);
-
-  return FALSE;
-}
-
 static void
 gtk_header_bar_realize (GtkWidget *widget)
 {
@@ -1913,7 +1851,6 @@ gtk_header_bar_class_init (GtkHeaderBarClass *class)
   widget_class->destroy = gtk_header_bar_destroy;
   widget_class->size_allocate = gtk_header_bar_size_allocate;
   widget_class->measure = gtk_header_bar_measure;
-  widget_class->snapshot = gtk_header_bar_snapshot;
   widget_class->realize = gtk_header_bar_realize;
   widget_class->unrealize = gtk_header_bar_unrealize;
   widget_class->hierarchy_changed = gtk_header_bar_hierarchy_changed;
@@ -2045,7 +1982,6 @@ static void
 gtk_header_bar_init (GtkHeaderBar *bar)
 {
   GtkHeaderBarPrivate *priv;
-  GtkCssNode *widget_node;
 
   priv = gtk_header_bar_get_instance_private (bar);
 
@@ -2064,16 +2000,6 @@ gtk_header_bar_init (GtkHeaderBar *bar)
 
   init_sizing_box (bar);
   construct_label_box (bar);
-
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (bar));
-  priv->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                     GTK_WIDGET (bar),
-                                                     gtk_header_bar_get_content_size,
-                                                     gtk_header_bar_allocate_contents,
-                                                     gtk_header_bar_render_contents,
-                                                     NULL,
-                                                     NULL);
-
 }
 
 static void
