@@ -23,7 +23,7 @@
 #include "gtk/gtkimage.h"
 #include "gtk/gtktypes.h"
 
-#include "gtkcssgadgetprivate.h"
+#include "gtkcssnodeprivate.h"
 #include "gtkimagedefinitionprivate.h"
 
 G_BEGIN_DECLS
@@ -56,22 +56,20 @@ typedef struct _GtkIconHelperPrivate GtkIconHelperPrivate;
 
 struct _GtkIconHelper
 {
-  GtkCssGadget parent;
+  GObject parent_instance;
 
   GtkIconHelperPrivate *priv;
 };
 
 struct _GtkIconHelperClass
 {
-  GtkCssGadgetClass parent_class;
+  GObjectClass parent_class;
 };
 
 GType gtk_icon_helper_get_type (void) G_GNUC_CONST;
 
 GtkIconHelper *gtk_icon_helper_new (GtkCssNode *node,
                                     GtkWidget  *owner);
-GtkCssGadget *gtk_icon_helper_new_named (const char *name,
-                                          GtkWidget  *owner);
 
 void _gtk_icon_helper_clear (GtkIconHelper *self);
 
