@@ -40,7 +40,6 @@
 #include "gtkcssshadowsvalueprivate.h"
 #include "gtkstylecontextprivate.h"
 #include "gtkwidgetprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
 
 #include "a11y/gtkimageaccessible.h"
 
@@ -69,49 +68,6 @@
  * The image file may contain an animation, if so the #GtkImage will
  * display an animation (#GdkPixbufAnimation) instead of a static image.
  *
- * #GtkImage is a “no window” widget (has no #GdkWindow of its own),
- * so by default does not receive events. If you want to receive events
- * on the image, such as button clicks, place the image inside a
- * #GtkEventBox, then connect to the event signals on the event box.
- *
- * ## Handling button press events on a #GtkImage.
- *
- * |[<!-- language="C" -->
- *   static gboolean
- *   button_press_callback (GtkWidget      *event_box,
- *                          GdkEventButton *event,
- *                          gpointer        data)
- *   {
- *     g_print ("Event box clicked at coordinates %f,%f\n",
- *              event->x, event->y);
- *
- *     // Returning TRUE means we handled the event, so the signal
- *     // emission should be stopped (don’t call any further callbacks
- *     // that may be connected). Return FALSE to continue invoking callbacks.
- *     return TRUE;
- *   }
- *
- *   static GtkWidget*
- *   create_image (void)
- *   {
- *     GtkWidget *image;
- *     GtkWidget *event_box;
- *
- *     image = gtk_image_new_from_file ("myfile.png");
- *
- *     event_box = gtk_event_box_new ();
- *
- *     gtk_container_add (GTK_CONTAINER (event_box), image);
- *
- *     g_signal_connect (G_OBJECT (event_box),
- *                       "button_press_event",
- *                       G_CALLBACK (button_press_callback),
- *                       image);
- *
- *     return image;
- *   }
- * ]|
- *
  * Sometimes an application will want to avoid depending on external data
  * files, such as image files. GTK+ comes with a program to avoid this,
  * called “gdk-pixbuf-csource”. This library
@@ -132,8 +88,6 @@ struct _GtkImagePrivate
   GdkPixbufAnimationIter *animation_iter;
   gint animation_timeout;
 
-  GtkCssGadget *gadget;
-
   float baseline_align;
 
   gchar                *filename;       /* Only used with GTK_IMAGE_ANIMATION, GTK_IMAGE_PIXBUF */
@@ -144,8 +98,10 @@ struct _GtkImagePrivate
 #define DEFAULT_ICON_SIZE GTK_ICON_SIZE_BUTTON
 static void gtk_image_snapshot             (GtkWidget    *widget,
                                             GtkSnapshot  *snapshot);
-static void gtk_image_size_allocate        (GtkWidget    *widget,
-                                            GtkAllocation*allocation);
+static void gtk_image_size_allocate        (GtkWidget           *widget,
+                                            const GtkAllocation *allocation,
+                                            int                  baseline,
+                                            GtkAllocation       *out_clip);
 static void gtk_image_unmap                (GtkWidget    *widget);
 static void gtk_image_unrealize            (GtkWidget    *widget);
 static void gtk_image_measure (GtkWidget      *widget,
@@ -155,25 +111,9 @@ static void gtk_image_measure (GtkWidget      *widget,
                                int           *natural,
                                int           *minimum_baseline,
                                int           *natural_baseline);
-static void gtk_image_get_content_size (GtkCssGadget   *gadget,
-                                        GtkOrientation  orientation,
-                                        gint            for_size,
-                                        gint           *minimum,
-                                        gint           *natural,
-                                        gint           *minimum_baseline,
-                                        gint           *natural_baseline,
-                                        gpointer        unused);
-static gboolean gtk_image_render_contents (GtkCssGadget *gadget,
-                                           GtkSnapshot  *snapshot,
-                                           int           x,
-                                           int           y,
-                                           int           width,
-                                           int           height,
-                                           gpointer      data);
 
 static void gtk_image_style_updated        (GtkWidget    *widget);
 static void gtk_image_finalize             (GObject      *object);
-static void gtk_image_reset                (GtkImage     *image);
 
 static void gtk_image_set_property         (GObject      *object,
                                             guint         prop_id,
@@ -368,14 +308,6 @@ gtk_image_init (GtkImage *image)
 
   priv->icon_helper = gtk_icon_helper_new (widget_node, GTK_WIDGET (image));
   _gtk_icon_helper_set_icon_size (priv->icon_helper, DEFAULT_ICON_SIZE);
-
-  priv->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                     GTK_WIDGET (image),
-                                                     gtk_image_get_content_size,
-                                                     NULL,
-                                                     gtk_image_render_contents,
-                                                     NULL, NULL);
-
 }
 
 static void
@@ -384,7 +316,6 @@ gtk_image_finalize (GObject *object)
   GtkImage *image = GTK_IMAGE (object);
 
   g_clear_object (&image->priv->icon_helper);
-  g_clear_object (&image->priv->gadget);
 
   g_free (image->priv->filename);
   g_free (image->priv->resource_path);
@@ -1299,28 +1230,17 @@ gtk_image_reset_anim_iter (GtkImage *image)
 }
 
 static void
-gtk_image_size_allocate (GtkWidget     *widget,
-                         GtkAllocation *allocation)
+gtk_image_size_allocate (GtkWidget           *widget,
+                         const GtkAllocation *allocation,
+                         int                  baseline,
+                         GtkAllocation       *out_clip)
 {
-  GtkAllocation clip;
-  GdkRectangle extents;
-
-  gtk_widget_set_allocation (widget, allocation);
-  gtk_css_gadget_allocate (GTK_IMAGE (widget)->priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
   _gtk_style_context_get_icon_extents (gtk_widget_get_style_context (widget),
-                                       &extents,
+                                       out_clip,
                                        allocation->x,
                                        allocation->y,
                                        allocation->width,
                                        allocation->height);
-
-  gdk_rectangle_union (&clip, &extents, &clip);
-
-  gtk_widget_set_clip (widget, &clip);
 }
 
 static void
@@ -1418,66 +1338,18 @@ gtk_image_get_baseline_align (GtkImage *image)
   return image->priv->baseline_align;
 }
 
-static void
-gtk_image_get_content_size (GtkCssGadget   *gadget,
-                            GtkOrientation  orientation,
-                            gint            for_size,
-                            gint           *minimum,
-                            gint           *natural,
-                            gint           *minimum_baseline,
-                            gint           *natural_baseline,
-                            gpointer        unused)
-{
-  GtkWidget *widget;
-  gint width, height;
-  float baseline_align;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-
-  _gtk_icon_helper_get_size (GTK_IMAGE (widget)->priv->icon_helper,
-                             &width, &height);
-
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    {
-      *minimum = *natural = width;
-    }
-  else
-    {
-      baseline_align = gtk_image_get_baseline_align (GTK_IMAGE (widget));
-      *minimum = *natural = height;
-      if (minimum_baseline)
-        *minimum_baseline = height * baseline_align;
-      if (natural_baseline)
-        *natural_baseline = height * baseline_align;
-    }
-
-}
-
 static void
 gtk_image_snapshot (GtkWidget   *widget,
                     GtkSnapshot *snapshot)
 {
-  gtk_css_gadget_snapshot (GTK_IMAGE (widget)->priv->gadget,
-                           snapshot);
-}
-
-static gboolean
-gtk_image_render_contents (GtkCssGadget *gadget,
-                           GtkSnapshot  *snapshot,
-                           int           x,
-                           int           y,
-                           int           width,
-                           int           height,
-                           gpointer      data)
-{
-  GtkWidget *widget;
-  GtkImage *image;
-  GtkImagePrivate *priv;
+  GtkImage *image = GTK_IMAGE (widget);
+  GtkImagePrivate *priv = gtk_image_get_instance_private (image);
+  int x, y, width, height;
   gint w, h, baseline;
 
-  widget = gtk_css_gadget_get_owner (gadget);
-  image = GTK_IMAGE (widget);
-  priv = image->priv;
+  gtk_widget_get_content_size (widget, &width, &height);
+  x = 0;
+  y = 0;
 
   _gtk_icon_helper_get_size (priv->icon_helper, &w, &h);
 
@@ -1496,7 +1368,7 @@ gtk_image_render_contents (GtkCssGadget *gadget,
       GdkPixbuf *pixbuf = get_animation_frame (image);
 
       gtk_snapshot_render_icon (snapshot, context, pixbuf, x, y);
-      
+
       g_object_unref (pixbuf);
     }
   else
@@ -1505,8 +1377,6 @@ gtk_image_render_contents (GtkCssGadget *gadget,
       gtk_icon_helper_snapshot (priv->icon_helper, snapshot);
       gtk_snapshot_offset (snapshot, -x, -y);
     }
-
-  return FALSE;
 }
 
 static void
@@ -1560,8 +1430,22 @@ gtk_image_set_from_definition (GtkImage           *image,
   g_object_thaw_notify (G_OBJECT (image));
 }
 
-static void
-gtk_image_reset (GtkImage *image)
+GtkImageDefinition *
+gtk_image_get_definition (GtkImage *image)
+{
+  return gtk_icon_helper_get_definition (image->priv->icon_helper);
+}
+
+/**
+ * gtk_image_clear:
+ * @image: a #GtkImage
+ *
+ * Resets the image to be empty.
+ *
+ * Since: 2.8
+ */
+void
+gtk_image_clear (GtkImage *image)
 {
   GtkImagePrivate *priv = image->priv;
   GtkImageType storage_type;
@@ -1597,20 +1481,6 @@ gtk_image_reset (GtkImage *image)
   g_object_thaw_notify (G_OBJECT (image));
 }
 
-/**
- * gtk_image_clear:
- * @image: a #GtkImage
- *
- * Resets the image to be empty.
- *
- * Since: 2.8
- */
-void
-gtk_image_clear (GtkImage *image)
-{
-  gtk_image_reset (image);
-}
-
 static void
 gtk_image_measure (GtkWidget      *widget,
                    GtkOrientation  orientation,
@@ -1620,11 +1490,25 @@ gtk_image_measure (GtkWidget      *widget,
                    int           *minimum_baseline,
                    int           *natural_baseline)
 {
-  gtk_css_gadget_get_preferred_size (GTK_IMAGE (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
+  gint width, height;
+  float baseline_align;
+
+  _gtk_icon_helper_get_size (GTK_IMAGE (widget)->priv->icon_helper,
+                             &width, &height);
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    {
+      *minimum = *natural = width;
+    }
+  else
+    {
+      baseline_align = gtk_image_get_baseline_align (GTK_IMAGE (widget));
+      *minimum = *natural = height;
+      if (minimum_baseline)
+        *minimum_baseline = height * baseline_align;
+      if (natural_baseline)
+        *natural_baseline = height * baseline_align;
+    }
 }
 
 static void
