@@ -19,12 +19,9 @@
 
 #include "gtkimagedefinitionprivate.h"
 
-#include "deprecated/gtkiconfactory.h"
-
 typedef struct _GtkImageDefinitionEmpty GtkImageDefinitionEmpty;
 typedef struct _GtkImageDefinitionPixbuf GtkImageDefinitionPixbuf;
 typedef struct _GtkImageDefinitionStock GtkImageDefinitionStock;
-typedef struct _GtkImageDefinitionIconSet GtkImageDefinitionIconSet;
 typedef struct _GtkImageDefinitionAnimation GtkImageDefinitionAnimation;
 typedef struct _GtkImageDefinitionIconName GtkImageDefinitionIconName;
 typedef struct _GtkImageDefinitionGIcon GtkImageDefinitionGIcon;
@@ -50,13 +47,6 @@ struct _GtkImageDefinitionStock {
   char *id;
 };
 
-struct _GtkImageDefinitionIconSet {
-  GtkImageType type;
-  gint ref_count;
-
-  GtkIconSet *icon_set;
-};
-
 struct _GtkImageDefinitionAnimation {
   GtkImageType type;
   gint ref_count;
@@ -92,7 +82,6 @@ union _GtkImageDefinition
   GtkImageDefinitionEmpty empty;
   GtkImageDefinitionPixbuf pixbuf;
   GtkImageDefinitionStock stock;
-  GtkImageDefinitionIconSet icon_set;
   GtkImageDefinitionAnimation animation;
   GtkImageDefinitionIconName icon_name;
   GtkImageDefinitionGIcon gicon;
@@ -114,21 +103,18 @@ gtk_image_definition_alloc (GtkImageType type)
     sizeof (GtkImageDefinitionEmpty),
     sizeof (GtkImageDefinitionPixbuf),
     sizeof (GtkImageDefinitionStock),
-    sizeof (GtkImageDefinitionIconSet),
     sizeof (GtkImageDefinitionAnimation),
     sizeof (GtkImageDefinitionIconName),
     sizeof (GtkImageDefinitionGIcon),
     sizeof (GtkImageDefinitionSurface)
   };
   GtkImageDefinition *def;
-  GtkImageDefinitionEmpty *empty_def;
 
   g_assert (type < G_N_ELEMENTS (sizes));
 
   def = g_malloc0 (sizes[type]);
-  empty_def = (GtkImageDefinitionEmpty *) def;
-  empty_def->type = type;
-  empty_def->ref_count = 1;
+  def->type = type;
+  def->empty.ref_count = 1;
 
   return def;
 }
@@ -138,49 +124,13 @@ gtk_image_definition_new_pixbuf (GdkPixbuf *pixbuf,
                                  int        scale)
 {
   GtkImageDefinition *def;
-  GtkImageDefinitionPixbuf *pixbuf_def;
 
   if (pixbuf == NULL || scale <= 0)
     return NULL;
 
   def = gtk_image_definition_alloc (GTK_IMAGE_PIXBUF);
-  pixbuf_def = (GtkImageDefinitionPixbuf *) def;
-  pixbuf_def->pixbuf = g_object_ref (pixbuf);
-  pixbuf_def->scale = scale;
-
-  return def;
-}
-
-GtkImageDefinition *
-gtk_image_definition_new_stock (const char *stock_id)
-{
-  GtkImageDefinition *def;
-  GtkImageDefinitionStock *stock_def;
-
-  if (stock_id == NULL || stock_id[0] == '\0')
-    return NULL;
-
-  def = gtk_image_definition_alloc (GTK_IMAGE_STOCK);
-  stock_def = (GtkImageDefinitionStock *) def;
-  stock_def->id = g_strdup (stock_id);
-
-  return def;
-}
-
-GtkImageDefinition *
-gtk_image_definition_new_icon_set (GtkIconSet *icon_set)
-{
-  GtkImageDefinition *def;
-  GtkImageDefinitionIconSet *icon_set_def;
-
-  if (icon_set == NULL)
-    return NULL;
-
-  def = gtk_image_definition_alloc (GTK_IMAGE_ICON_SET);
-  icon_set_def = (GtkImageDefinitionIconSet *) def;
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-  icon_set_def->icon_set = gtk_icon_set_ref (icon_set);
-G_GNUC_END_IGNORE_DEPRECATIONS;
+  def->pixbuf.pixbuf = g_object_ref (pixbuf);
+  def->pixbuf.scale = scale;
 
   return def;
 }
@@ -190,15 +140,13 @@ gtk_image_definition_new_animation (GdkPixbufAnimation *animation,
                                     int                 scale)
 {
   GtkImageDefinition *def;
-  GtkImageDefinitionAnimation *animation_def;
 
   if (animation == NULL || scale <= 0)
     return NULL;
 
   def = gtk_image_definition_alloc (GTK_IMAGE_ANIMATION);
-  animation_def = (GtkImageDefinitionAnimation *) def;
-  animation_def->animation = g_object_ref (animation);
-  animation_def->scale = scale;
+  def->animation.animation = g_object_ref (animation);
+  def->animation.scale = scale;
 
   return def;
 }
@@ -207,14 +155,12 @@ GtkImageDefinition *
 gtk_image_definition_new_icon_name (const char *icon_name)
 {
   GtkImageDefinition *def;
-  GtkImageDefinitionIconName *icon_name_def;
 
   if (icon_name == NULL || icon_name[0] == '\0')
     return NULL;
 
   def = gtk_image_definition_alloc (GTK_IMAGE_ICON_NAME);
-  icon_name_def = (GtkImageDefinitionIconName *) def;
-  icon_name_def->icon_name = g_strdup (icon_name);
+  def->icon_name.icon_name = g_strdup (icon_name);
 
   return def;
 }
@@ -223,14 +169,12 @@ GtkImageDefinition *
 gtk_image_definition_new_gicon (GIcon *gicon)
 {
   GtkImageDefinition *def;
-  GtkImageDefinitionGIcon *gicon_def;
 
   if (gicon == NULL)
     return NULL;
 
   def = gtk_image_definition_alloc (GTK_IMAGE_GICON);
-  gicon_def = (GtkImageDefinitionGIcon *) def;
-  gicon_def->gicon = g_object_ref (gicon);
+  def->gicon.gicon = g_object_ref (gicon);
 
   return def;
 }
@@ -239,14 +183,12 @@ GtkImageDefinition *
 gtk_image_definition_new_surface (cairo_surface_t *surface)
 {
   GtkImageDefinition *def;
-  GtkImageDefinitionSurface *surface_def;
 
   if (surface == NULL)
     return NULL;
 
   def = gtk_image_definition_alloc (GTK_IMAGE_SURFACE);
-  surface_def = (GtkImageDefinitionSurface *) def;
-  surface_def->surface = cairo_surface_reference (surface);
+  def->surface.surface = cairo_surface_reference (surface);
 
   return def;
 }
@@ -254,10 +196,7 @@ gtk_image_definition_new_surface (cairo_surface_t *surface)
 GtkImageDefinition *
 gtk_image_definition_ref (GtkImageDefinition *def)
 {
-  GtkImageDefinitionEmpty *empty_def;
-
-  empty_def = (GtkImageDefinitionEmpty *) def;
-  empty_def->ref_count++;
+  def->empty.ref_count++;
 
   return def;
 }
@@ -265,19 +204,9 @@ gtk_image_definition_ref (GtkImageDefinition *def)
 void
 gtk_image_definition_unref (GtkImageDefinition *def)
 {
-  GtkImageDefinitionEmpty *empty_def;
-  GtkImageDefinitionPixbuf *pixbuf_def;
-  GtkImageDefinitionAnimation *animation_def;
-  GtkImageDefinitionSurface *surface_def;
-  GtkImageDefinitionStock *stock_def;
-  GtkImageDefinitionIconSet *icon_set_def;
-  GtkImageDefinitionIconName *icon_name_def;
-  GtkImageDefinitionGIcon *gicon_def;
+  def->empty.ref_count--;
 
-  empty_def = (GtkImageDefinitionEmpty *) def;
-  empty_def->ref_count--;
-
-  if (empty_def->ref_count > 0)
+  if (def->empty.ref_count > 0)
     return;
 
   switch (def->type)
@@ -287,34 +216,19 @@ gtk_image_definition_unref (GtkImageDefinition *def)
       g_assert_not_reached ();
       break;
     case GTK_IMAGE_PIXBUF:
-      pixbuf_def = (GtkImageDefinitionPixbuf *) def;
-      g_object_unref (pixbuf_def->pixbuf);
+      g_object_unref (def->pixbuf.pixbuf);
       break;
     case GTK_IMAGE_ANIMATION:
-      animation_def = (GtkImageDefinitionAnimation *) def;
-      g_object_unref (animation_def->animation);
+      g_object_unref (def->animation.animation);
       break;
     case GTK_IMAGE_SURFACE:
-      surface_def = (GtkImageDefinitionSurface *) def;
-      cairo_surface_destroy (surface_def->surface);
-      break;
-    case GTK_IMAGE_STOCK:
-      stock_def = (GtkImageDefinitionStock *) def;
-      g_free (stock_def->id);
-      break;
-    case GTK_IMAGE_ICON_SET:
-      icon_set_def = (GtkImageDefinitionIconSet *) def;
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-      gtk_icon_set_unref (icon_set_def->icon_set);
-G_GNUC_END_IGNORE_DEPRECATIONS;
+      cairo_surface_destroy (def->surface.surface);
       break;
     case GTK_IMAGE_ICON_NAME:
-      icon_name_def = (GtkImageDefinitionIconName *) def;
-      g_free (icon_name_def->icon_name);
+      g_free (def->icon_name.icon_name);
       break;
     case GTK_IMAGE_GICON:
-      gicon_def = (GtkImageDefinitionGIcon *) def;
-      g_object_unref (gicon_def->gicon);
+      g_object_unref (def->gicon.gicon);
       break;
     }
 
@@ -336,8 +250,6 @@ gtk_image_definition_get_scale (const GtkImageDefinition *def)
       g_assert_not_reached ();
     case GTK_IMAGE_EMPTY:
     case GTK_IMAGE_SURFACE:
-    case GTK_IMAGE_STOCK:
-    case GTK_IMAGE_ICON_SET:
     case GTK_IMAGE_ICON_NAME:
     case GTK_IMAGE_GICON:
       return 1;
@@ -357,24 +269,6 @@ gtk_image_definition_get_pixbuf (const GtkImageDefinition *def)
   return def->pixbuf.pixbuf;
 }
 
-const gchar *
-gtk_image_definition_get_stock (const GtkImageDefinition *def)
-{
-  if (def->type != GTK_IMAGE_STOCK)
-    return NULL;
-
-  return def->stock.id;
-}
-
-GtkIconSet *
-gtk_image_definition_get_icon_set (const GtkImageDefinition *def)
-{
-  if (def->type != GTK_IMAGE_ICON_SET)
-    return NULL;
-
-  return def->icon_set.icon_set;
-}
-
 GdkPixbufAnimation *
 gtk_image_definition_get_animation (const GtkImageDefinition *def)
 {
