@@ -163,11 +163,6 @@ enum
 static GParamSpec *props[LAST_PROP] = { NULL, };
 static guint signals[LAST_SIGNAL];
 
-#define ACTION_AREA_DEFAULT_BORDER 5
-#define ACTION_AREA_DEFAULT_SPACING 6
-#define CONTENT_AREA_DEFAULT_BORDER 8
-#define CONTENT_AREA_DEFAULT_SPACING 16
-
 static void     gtk_info_bar_set_property (GObject        *object,
                                            guint           prop_id,
                                            const GValue   *value,
@@ -523,8 +518,7 @@ gtk_info_bar_add_action_widget (GtkInfoBar *info_bar,
   else
     g_warning ("Only 'activatable' widgets can be packed into the action area of a GtkInfoBar");
 
-  gtk_box_pack_end (GTK_BOX (info_bar->priv->action_area),
-                    child, FALSE, FALSE);
+  gtk_box_pack_end (GTK_BOX (info_bar->priv->action_area), child);
   if (response_id == GTK_RESPONSE_HELP)
     gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (info_bar->priv->action_area),
                                         child, TRUE);
