@@ -44,19 +44,16 @@
  * @Short_description: Infinite scrollable area containing child widgets
  *   and/or custom drawing
  * @Title: GtkLayout
- * @See_also: #GtkDrawingArea, #GtkScrolledWindow
+ * @See_also: #GtkDrawingArea, #GtkFixed
  *
- * #GtkLayout is similar to #GtkDrawingArea in that it’s a “blank slate”
- * and doesn’t do anything but paint a blank background by default. It's
- * different in that it supports scrolling natively (you can add it to a
- * #GtkScrolledWindow), and it can contain child widgets, since it’s a
- * #GtkContainer. However if you’re just going to draw, a #GtkDrawingArea
- * is a better choice since it has lower overhead.
+ * #GtkLayout is similar to #GtkDrawingArea in that it’s a “blank slate” and
+ * doesn’t do anything except paint a blank background by default. It’s
+ * different in that it supports scrolling natively due to implementing
+ * #GtkScrollable, and can contain child widgets since it’s a #GtkContainer.
  *
- * When handling expose events on a #GtkLayout, you must draw to
- * GTK_LAYOUT (layout)->bin_window, rather than to
- * GTK_WIDGET (layout)->window, as you would for a drawing
- * area.
+ * If you just want to draw, a #GtkDrawingArea is a better choice since it has
+ * lower overhead. If you just need to position child widgets at specific
+ * points, then #GtkFixed provides that functionality on its own.
  */
 
 
@@ -79,7 +76,6 @@ struct _GtkLayoutPrivate
   /* Properties */
 
   GdkVisibilityState visibility;
-  GdkWindow *bin_window;
 
   GList *children;
 
@@ -120,8 +116,6 @@ static void gtk_layout_set_property       (GObject        *object,
                                            const GValue   *value,
                                            GParamSpec     *pspec);
 static void gtk_layout_finalize           (GObject        *object);
-static void gtk_layout_realize            (GtkWidget      *widget);
-static void gtk_layout_unrealize          (GtkWidget      *widget);
 static void gtk_layout_map                (GtkWidget      *widget);
 static void gtk_layout_measure (GtkWidget *widget,
                                 GtkOrientation  orientation,
@@ -130,8 +124,10 @@ static void gtk_layout_measure (GtkWidget *widget,
                                 int            *natural,
                                 int            *minimum_baseline,
                                 int            *natural_baseline);
-static void gtk_layout_size_allocate      (GtkWidget      *widget,
-                                           GtkAllocation  *allocation);
+static void gtk_layout_size_allocate      (GtkWidget          *widget,
+                                           const GtkAllocation *allocation,
+                                           int                 baseline,
+                                           GtkAllocation      *out_clip);
 static void gtk_layout_add                (GtkContainer   *container,
 					   GtkWidget      *widget);
 static void gtk_layout_remove             (GtkContainer   *container,
@@ -189,24 +185,6 @@ gtk_layout_new (GtkAdjustment *hadjustment,
   return GTK_WIDGET (layout);
 }
 
-/**
- * gtk_layout_get_bin_window:
- * @layout: a #GtkLayout
- *
- * Retrieve the bin window of the layout used for drawing operations.
- *
- * Returns: (transfer none): a #GdkWindow
- *
- * Since: 2.14
- **/
-GdkWindow*
-gtk_layout_get_bin_window (GtkLayout *layout)
-{
-  g_return_val_if_fail (GTK_IS_LAYOUT (layout), NULL);
-
-  return layout->priv->bin_window;
-}
-
 static void
 gtk_layout_set_hadjustment_values (GtkLayout *layout)
 {
@@ -389,9 +367,6 @@ gtk_layout_put (GtkLayout     *layout,
 
   priv->children = g_list_append (priv->children, child);
 
-  if (gtk_widget_get_realized (GTK_WIDGET (layout)))
-    gtk_widget_set_parent_window (child->widget, priv->bin_window);
-
   gtk_widget_set_parent (child_widget, GTK_WIDGET (layout));
 }
 
@@ -468,12 +443,10 @@ gtk_layout_set_size (GtkLayout     *layout,
 		     guint          height)
 {
   GtkLayoutPrivate *priv;
-  GtkWidget *widget;
 
   g_return_if_fail (GTK_IS_LAYOUT (layout));
 
   priv = layout->priv;
-  widget = GTK_WIDGET (layout);
 
   g_object_freeze_notify (G_OBJECT (layout));
   if (width != priv->width)
@@ -488,16 +461,6 @@ gtk_layout_set_size (GtkLayout     *layout,
      }
   g_object_thaw_notify (G_OBJECT (layout));
 
-  if (gtk_widget_get_realized (widget))
-    {
-      GtkAllocation allocation;
-
-      gtk_widget_get_allocation (widget, &allocation);
-      width = MAX (width, allocation.width);
-      height = MAX (height, allocation.height);
-      gdk_window_resize (priv->bin_window, width, height);
-    }
-
   gtk_layout_set_hadjustment_values (layout);
   gtk_layout_set_vadjustment_values (layout);
 }
@@ -595,8 +558,6 @@ gtk_layout_class_init (GtkLayoutClass *class)
 						     G_MAXINT,
 						     100,
 						     GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
-  widget_class->realize = gtk_layout_realize;
-  widget_class->unrealize = gtk_layout_unrealize;
   widget_class->map = gtk_layout_map;
   widget_class->measure = gtk_layout_measure;
   widget_class->size_allocate = gtk_layout_size_allocate;
@@ -756,8 +717,6 @@ gtk_layout_init (GtkLayout *layout)
   priv->hadjustment = NULL;
   priv->vadjustment = NULL;
 
-  priv->bin_window = NULL;
-
   priv->scroll_x = 0;
   priv->scroll_y = 0;
   priv->visibility = GDK_VISIBILITY_PARTIAL;
@@ -765,39 +724,6 @@ gtk_layout_init (GtkLayout *layout)
   priv->freeze_count = 0;
 }
 
-/* Widget methods
- */
-static void 
-gtk_layout_realize (GtkWidget *widget)
-{
-  GtkLayout *layout = GTK_LAYOUT (widget);
-  GtkLayoutPrivate *priv = layout->priv;
-  GtkAllocation allocation;
-  GList *tmp_list;
-
-  GTK_WIDGET_CLASS (gtk_layout_parent_class)->realize (widget);
-
-  gtk_widget_get_allocation (widget, &allocation);
-
-  priv->bin_window = gdk_window_new_child (gtk_widget_get_window (widget),
-                                           GDK_ALL_EVENTS_MASK,
-                                           &(GdkRectangle) {
-                                             allocation.x - gtk_adjustment_get_value (priv->hadjustment),
-                                             allocation.y - gtk_adjustment_get_value (priv->vadjustment),
-                                             MAX (priv->width, allocation.width),
-                                             MAX (priv->height, allocation.height)});
-  gtk_widget_register_window (widget, priv->bin_window);
-
-  tmp_list = priv->children;
-  while (tmp_list)
-    {
-      GtkLayoutChild *child = tmp_list->data;
-      tmp_list = tmp_list->next;
-
-      gtk_widget_set_parent_window (child->widget, priv->bin_window);
-    }
-}
-
 static void
 gtk_layout_map (GtkWidget *widget)
 {
@@ -819,21 +745,6 @@ gtk_layout_map (GtkWidget *widget)
 	    gtk_widget_map (child->widget);
 	}
     }
-
-  gdk_window_show (priv->bin_window);
-}
-
-static void 
-gtk_layout_unrealize (GtkWidget *widget)
-{
-  GtkLayout *layout = GTK_LAYOUT (widget);
-  GtkLayoutPrivate *priv = layout->priv;
-
-  gtk_widget_unregister_window (widget, priv->bin_window);
-  gdk_window_destroy (priv->bin_window);
-  priv->bin_window = NULL;
-
-  GTK_WIDGET_CLASS (gtk_layout_parent_class)->unrealize (widget);
 }
 
 static void
@@ -850,15 +761,15 @@ gtk_layout_measure (GtkWidget *widget,
 
 
 static void
-gtk_layout_size_allocate (GtkWidget     *widget,
-			  GtkAllocation *allocation)
+gtk_layout_size_allocate (GtkWidget           *widget,
+                          const GtkAllocation *allocation,
+                          int                  baseline,
+                          GtkAllocation       *out_clip)
 {
   GtkLayout *layout = GTK_LAYOUT (widget);
   GtkLayoutPrivate *priv = layout->priv;
   GList *tmp_list;
 
-  gtk_widget_set_allocation (widget, allocation);
-
   tmp_list = priv->children;
 
   while (tmp_list)
@@ -869,19 +780,8 @@ gtk_layout_size_allocate (GtkWidget     *widget,
       gtk_layout_allocate_child (layout, child);
     }
 
-  if (gtk_widget_get_realized (widget))
-    {
-      gdk_window_move_resize (priv->bin_window,
-                              allocation->x - gtk_adjustment_get_value (priv->hadjustment),
-                              allocation->y - gtk_adjustment_get_value (priv->vadjustment),
-                              MAX (priv->width, allocation->width),
-                              MAX (priv->height, allocation->height));
-    }
-
   gtk_layout_set_hadjustment_values (layout);
   gtk_layout_set_vadjustment_values (layout);
-
-  gtk_widget_set_clip (widget, allocation);
 }
 
 /* Container methods
@@ -950,6 +850,7 @@ gtk_layout_allocate_child (GtkLayout      *layout,
 {
   GtkAllocation allocation;
   GtkRequisition requisition;
+  GtkAllocation child_clip;
 
   allocation.x = child->x;
   allocation.y = child->y;
@@ -957,8 +858,8 @@ gtk_layout_allocate_child (GtkLayout      *layout,
   gtk_widget_get_preferred_size (child->widget, &requisition, NULL);
   allocation.width = requisition.width;
   allocation.height = requisition.height;
-  
-  gtk_widget_size_allocate (child->widget, &allocation);
+
+  gtk_widget_size_allocate (child->widget, &allocation, -1, &child_clip);
 }
 
 /* Callbacks */
@@ -972,13 +873,5 @@ gtk_layout_adjustment_changed (GtkAdjustment *adjustment,
   if (priv->freeze_count)
     return;
 
-  if (gtk_widget_get_realized (GTK_WIDGET (layout)))
-    {
-      GtkAllocation allocation;
-
-      gtk_widget_get_allocation (GTK_WIDGET (layout), &allocation);
-      gdk_window_move (priv->bin_window,
-		       allocation.x - gtk_adjustment_get_value (priv->hadjustment),
-		       allocation.y - gtk_adjustment_get_value (priv->vadjustment));
-    }
+  gtk_widget_queue_allocate (GTK_WIDGET (layout));
 }
