@@ -430,24 +430,29 @@ gtk_level_bar_allocate_trough_continuous (GtkLevelBar *self,
   GtkAllocation block_area, clip;
   gdouble fill_percentage;
   gboolean inverted;
+  int block_min;
 
   inverted = gtk_level_bar_get_real_inverted (self);
 
   /* allocate the empty (unfilled) part */
   block_area = *allocation;
-  gtk_widget_size_allocate_with_baseline (self->priv->block_widget[inverted ? 0 : 1],
-                                          &block_area,
-                                          baseline);
-  gtk_widget_get_clip (self->priv->block_widget[inverted ? 0 : 1], out_clip);
+  gtk_widget_size_allocate (self->priv->block_widget[inverted ? 0 : 1],
+                            &block_area,
+                            baseline,
+                            out_clip);
 
   /* now allocate the filled part */
   block_area = *allocation;
   fill_percentage = (self->priv->cur_value - self->priv->min_value) /
     (self->priv->max_value - self->priv->min_value);
 
+  gtk_widget_measure (self->priv->block_widget[inverted ? 1 : 0], self->priv->orientation, -1,
+                      &block_min, NULL, NULL, NULL);
+
   if (self->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       block_area.width = (gint) floor (block_area.width * fill_percentage);
+      block_area.width = MAX (block_area.width, block_min);
 
       if (inverted)
         block_area.x += allocation->width - block_area.width;
@@ -455,15 +460,16 @@ gtk_level_bar_allocate_trough_continuous (GtkLevelBar *self,
   else
     {
       block_area.height = (gint) floor (block_area.height * fill_percentage);
+      block_area.height = MAX (block_area.height, block_min);
 
       if (inverted)
         block_area.y += allocation->height - block_area.height;
     }
 
-  gtk_widget_size_allocate_with_baseline (self->priv->block_widget[inverted ? 1 : 0],
-                                          &block_area,
-                                          baseline);
-  gtk_widget_get_clip (self->priv->block_widget[inverted ? 1 : 0], &clip);
+  gtk_widget_size_allocate (self->priv->block_widget[inverted ? 1 : 0],
+                            &block_area,
+                            baseline,
+                            &clip);
   gdk_rectangle_intersect (out_clip, &clip, out_clip);
 }
 
@@ -501,10 +507,10 @@ gtk_level_bar_allocate_trough_discrete (GtkLevelBar *self,
 
   for (i = 0; i < num_blocks; i++)
     {
-      gtk_widget_size_allocate_with_baseline (self->priv->block_widget[i],
-                                              &block_area,
-                                              baseline);
-      gtk_widget_get_clip (self->priv->block_widget[i], &clip);
+      gtk_widget_size_allocate (self->priv->block_widget[i],
+                                &block_area,
+                                baseline,
+                                &clip);
       gdk_rectangle_intersect (out_clip, &clip, out_clip);
 
       if (self->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
@@ -530,18 +536,14 @@ gtk_level_bar_allocate_trough (GtkGizmo            *gizmo,
 }
 
 static void
-gtk_level_bar_size_allocate (GtkWidget     *widget,
-                             GtkAllocation *allocation)
+gtk_level_bar_size_allocate (GtkWidget           *widget,
+                             const GtkAllocation *allocation,
+                             int                  baseline,
+                             GtkAllocation       *out_clip)
 {
   GtkLevelBarPrivate *priv = gtk_level_bar_get_instance_private (GTK_LEVEL_BAR (widget));
-  GtkAllocation clip;
 
-  GTK_WIDGET_CLASS (gtk_level_bar_parent_class)->size_allocate (widget, allocation);
-
-  gtk_widget_size_allocate (priv->trough_widget, allocation);
-  gtk_widget_get_clip (priv->trough_widget, &clip);
-
-  gtk_widget_set_clip (widget, &clip);
+  gtk_widget_size_allocate (priv->trough_widget, allocation, baseline, out_clip);
 }
 
 static void
