@@ -1471,6 +1471,7 @@ handle_pointing_event (GdkEvent *event)
 {
   GtkWidget *target = NULL, *old_target = NULL, *widget;
   GtkWindow *toplevel;
+  GtkWidget *toplevel_widget;
   GdkEventSequence *sequence;
   GdkDevice *device;
   gdouble x, y;
@@ -1480,10 +1481,12 @@ handle_pointing_event (GdkEvent *event)
   if (!device || !gdk_event_get_coords (event, &x, &y))
     return widget;
 
-  toplevel = GTK_WINDOW (gtk_widget_get_toplevel (widget));
-  if (!GTK_IS_WINDOW (toplevel))
+  toplevel_widget = gtk_widget_get_toplevel (widget);
+  if (!GTK_IS_WINDOW (toplevel_widget))
     return widget;
 
+  toplevel = GTK_WINDOW (toplevel_widget);
+
   sequence = gdk_event_get_event_sequence (event);
 
   switch (event->type)
@@ -2417,6 +2420,7 @@ gtk_get_event_target (const GdkEvent *event)
 /**
  * gtk_get_event_target_with_type:
  * @event: a #GdkEvent
+ * @type: the type to look for
  *
  * If @event is %NULL or the event was not associated with any widget,
  * returns %NULL, otherwise returns first widget found from the event
