@@ -27,9 +27,7 @@
 #include <string.h>
 
 #include "gtkaccellabel.h"
-#include "gtkbuiltiniconprivate.h"
 #include "gtkcontainerprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtkmenuprivate.h"
@@ -47,6 +45,7 @@
 #include "a11y/gtkmenuitemaccessible.h"
 #include "gtkstylecontextprivate.h"
 #include "gtkcssstylepropertyprivate.h"
+#include "gtkiconprivate.h"
 
 #define MENU_POPUP_DELAY     225
 
@@ -246,49 +245,17 @@ gtk_menu_item_actionable_interface_init (GtkActionableInterface *iface)
   iface->get_action_target_value = gtk_menu_item_get_action_target_value;
 }
 
-static gboolean
-gtk_menu_item_render (GtkCssGadget *gadget,
-                      GtkSnapshot  *snapshot,
-                      int           x,
-                      int           y,
-                      int           width,
-                      int           height,
-                      gpointer      data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkMenuItem *menu_item = GTK_MENU_ITEM (widget);
-  GtkMenuItemPrivate *priv = menu_item->priv;
-  GtkWidget *parent;
-
-  parent = gtk_widget_get_parent (widget);
-
-  if (priv->submenu && !GTK_IS_MENU_BAR (parent))
-    gtk_css_gadget_snapshot (priv->arrow_gadget, snapshot);
-
-  GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->snapshot (widget, snapshot);
-
-  return FALSE;
-}
-
 static void
-gtk_menu_item_snapshot (GtkWidget   *widget,
-                        GtkSnapshot *snapshot)
+gtk_menu_item_size_allocate (GtkWidget           *widget,
+                             const GtkAllocation *allocation,
+                             int                  baseline,
+                             GtkAllocation       *out_clip)
 {
-  gtk_css_gadget_snapshot (GTK_MENU_ITEM (widget)->priv->gadget, snapshot);
-}
-
-static void
-gtk_menu_item_allocate (GtkCssGadget        *gadget,
-                        const GtkAllocation *allocation,
-                        int                  baseline,
-                        GtkAllocation       *out_clip,
-                        gpointer             data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
   GtkMenuItem *menu_item = GTK_MENU_ITEM (widget);
   GtkMenuItemPrivate *priv = menu_item->priv;
   GtkAllocation child_allocation;
   GtkAllocation arrow_clip = { 0 };
+  GtkAllocation child_clip = *allocation;
   GtkTextDirection direction;
   GtkPackDirection child_pack_dir;
   GtkWidget *child;
@@ -332,16 +299,16 @@ gtk_menu_item_allocate (GtkCssGadget        *gadget,
 	{
           GtkAllocation arrow_alloc;
 
-          gtk_css_gadget_get_preferred_size (priv->arrow_gadget,
-                                             GTK_ORIENTATION_HORIZONTAL,
-                                             -1,
-                                             &arrow_alloc.width, NULL,
-                                             NULL, NULL);
-          gtk_css_gadget_get_preferred_size (priv->arrow_gadget,
-                                             GTK_ORIENTATION_VERTICAL,
-                                             -1,
-                                             &arrow_alloc.height, NULL,
-                                             NULL, NULL);
+          gtk_widget_measure (priv->arrow_widget,
+                              GTK_ORIENTATION_HORIZONTAL,
+                              -1,
+                              &arrow_alloc.width, NULL,
+                              NULL, NULL);
+          gtk_widget_measure (priv->arrow_widget,
+                              GTK_ORIENTATION_VERTICAL,
+                              -1,
+                              &arrow_alloc.height, NULL,
+                              NULL, NULL);
 
           if (direction == GTK_TEXT_DIR_LTR)
             {
@@ -358,42 +325,20 @@ gtk_menu_item_allocate (GtkCssGadget        *gadget,
           arrow_alloc.y = child_allocation.y +
             (child_allocation.height - arrow_alloc.height) / 2;
 
-          gtk_css_gadget_allocate (priv->arrow_gadget,
-                                   &arrow_alloc,
-                                   baseline,
-                                   &arrow_clip);
-	}
+          gtk_widget_size_allocate(priv->arrow_widget, &arrow_alloc, baseline, &arrow_clip);
+          gdk_rectangle_union (out_clip, &arrow_clip, out_clip);
+        }
 
       child_allocation.width = MAX (1, child_allocation.width);
 
-      gtk_widget_size_allocate (child, &child_allocation);
-
-      gtk_container_get_children_clip (GTK_CONTAINER (widget), out_clip);
-      gdk_rectangle_union (out_clip, &arrow_clip, out_clip);
+      gtk_widget_size_allocate (child, &child_allocation, baseline, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
     }
 
   if (priv->submenu)
     gtk_menu_reposition (GTK_MENU (priv->submenu));
 }
 
-static void
-gtk_menu_item_size_allocate (GtkWidget     *widget,
-                             GtkAllocation *allocation)
-{
-  GtkMenuItem *menu_item = GTK_MENU_ITEM (widget);
-  GtkMenuItemPrivate *priv = menu_item->priv;
-  GtkAllocation clip;
-  
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
 static void
 gtk_menu_item_accel_width_foreach (GtkWidget *widget,
                                    gpointer   data)
@@ -441,11 +386,11 @@ gtk_menu_item_real_get_width (GtkWidget *widget,
 	{
           gint arrow_size;
 
-          gtk_css_gadget_get_preferred_size (priv->arrow_gadget,
-                                             GTK_ORIENTATION_HORIZONTAL,
-                                             -1,
-                                             &arrow_size, NULL,
-                                             NULL, NULL);
+          gtk_widget_measure (priv->arrow_widget,
+                              GTK_ORIENTATION_HORIZONTAL,
+                              -1,
+                              &arrow_size, NULL,
+                              NULL, NULL);
 
           min_width += arrow_size;
           nat_width = min_width;
@@ -493,11 +438,11 @@ gtk_menu_item_real_get_height (GtkWidget *widget,
       gint arrow_size = 0;
 
       if ((priv->submenu && !GTK_IS_MENU_BAR (parent)) || priv->reserve_indicator)
-        gtk_css_gadget_get_preferred_size (priv->arrow_gadget,
-                                           GTK_ORIENTATION_VERTICAL,
-                                           -1,
-                                           &arrow_size, NULL,
-                                           NULL, NULL);
+        gtk_widget_measure (priv->arrow_widget,
+                            GTK_ORIENTATION_VERTICAL,
+                            -1,
+                            &arrow_size, NULL,
+                            NULL, NULL);
 
       if (for_size != -1)
         {
@@ -533,37 +478,18 @@ gtk_menu_item_real_get_height (GtkWidget *widget,
 }
 
 static void
-gtk_menu_item_measure (GtkCssGadget   *gadget,
+gtk_menu_item_measure (GtkWidget      *widget,
                        GtkOrientation  orientation,
-                       int             size,
+                       int             for_size,
                        int            *minimum,
                        int            *natural,
                        int            *minimum_baseline,
-                       int            *natural_baseline,
-                       gpointer        data)
+                       int            *natural_baseline)
 {
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     gtk_menu_item_real_get_width (widget, minimum, natural);
   else
-    gtk_menu_item_real_get_height (widget, size, minimum, natural);
-}
-
-static void
-gtk_menu_item_measure_ (GtkWidget      *widget,
-                        GtkOrientation  orientation,
-                        int             for_size,
-                        int            *minimum,
-                        int            *natural,
-                        int            *minimum_baseline,
-                        int            *natural_baseline)
-{
-  gtk_css_gadget_get_preferred_size (GTK_MENU_ITEM (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
+    gtk_menu_item_real_get_height (widget, for_size, minimum, natural);
 }
 
 
@@ -581,13 +507,12 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
 
   widget_class->destroy = gtk_menu_item_destroy;
   widget_class->size_allocate = gtk_menu_item_size_allocate;
-  widget_class->snapshot = gtk_menu_item_snapshot;
   widget_class->enter_notify_event = gtk_menu_item_enter;
   widget_class->leave_notify_event = gtk_menu_item_leave;
   widget_class->mnemonic_activate = gtk_menu_item_mnemonic_activate;
   widget_class->parent_set = gtk_menu_item_parent_set;
   widget_class->can_activate_accel = gtk_menu_item_can_activate_accel;
-  widget_class->measure = gtk_menu_item_measure_;
+  widget_class->measure = gtk_menu_item_measure;
   widget_class->direction_changed = gtk_menu_item_direction_changed;
 
   container_class->forall = gtk_menu_item_forall;
@@ -747,7 +672,6 @@ static void
 gtk_menu_item_init (GtkMenuItem *menu_item)
 {
   GtkMenuItemPrivate *priv;
-  GtkCssNode *widget_node;
 
   priv = gtk_menu_item_get_instance_private (menu_item);
   menu_item->priv = priv;
@@ -763,20 +687,6 @@ gtk_menu_item_init (GtkMenuItem *menu_item)
     priv->submenu_direction = GTK_DIRECTION_RIGHT;
   priv->submenu_placement = GTK_TOP_BOTTOM;
   priv->timer = 0;
-
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (menu_item));
-  priv->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                     GTK_WIDGET (menu_item),
-                                                     gtk_menu_item_measure,
-                                                     gtk_menu_item_allocate,
-                                                     gtk_menu_item_render,
-                                                     NULL, NULL);
-}
-
-GtkCssGadget *
-_gtk_menu_item_get_gadget (GtkMenuItem *menu_item)
-{
-  return menu_item->priv->gadget;
 }
 
 /**
@@ -838,8 +748,11 @@ gtk_menu_item_dispose (GObject *object)
 
   g_clear_object (&priv->action_helper);
 
-  g_clear_object (&priv->arrow_gadget);
-  g_clear_object (&priv->gadget);
+  if (priv->arrow_widget)
+    {
+      gtk_widget_unparent (priv->arrow_widget);
+      priv->arrow_widget = NULL;
+    }
 
   G_OBJECT_CLASS (gtk_menu_item_parent_class)->dispose (object);
 }
@@ -935,7 +848,11 @@ gtk_menu_item_detacher (GtkWidget *widget,
   g_return_if_fail (priv->submenu == (GtkWidget*) menu);
 
   priv->submenu = NULL;
-  g_clear_object (&priv->arrow_gadget);
+  if (priv->arrow_widget)
+    {
+      gtk_widget_unparent (priv->arrow_widget);
+      priv->arrow_widget = NULL;
+    }
 }
 
 static void
@@ -1000,63 +917,61 @@ gtk_menu_item_buildable_custom_finished (GtkBuildable *buildable,
 }
 
 static void
-update_node_classes (GtkMenuItem *menu_item)
+update_arrow_classes (GtkMenuItem *menu_item)
 {
   GtkMenuItemPrivate *priv = menu_item->priv;
-  GtkCssNode *arrow_node, *widget_node, *node;
+  GtkStyleContext *context;
+  GtkWidget *child;
 
-  if (!priv->arrow_gadget)
+  if (!priv->arrow_widget)
     return;
 
-  arrow_node = gtk_css_gadget_get_node (priv->arrow_gadget);
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (menu_item));
-
-  gtk_css_node_set_state (arrow_node, gtk_css_node_get_state (widget_node));
+  context = gtk_widget_get_style_context (priv->arrow_widget);
 
   if (gtk_widget_get_direction (GTK_WIDGET (menu_item)) == GTK_TEXT_DIR_RTL)
     {
-      gtk_css_node_add_class (arrow_node, g_quark_from_static_string (GTK_STYLE_CLASS_LEFT));
-      gtk_css_node_remove_class (arrow_node, g_quark_from_static_string (GTK_STYLE_CLASS_RIGHT));
+      gtk_style_context_add_class (context, GTK_STYLE_CLASS_LEFT);
+      gtk_style_context_remove_class (context, GTK_STYLE_CLASS_RIGHT);
 
-      node = gtk_css_node_get_first_child (widget_node);
-      if (node != arrow_node)
-        gtk_css_node_insert_before (widget_node, arrow_node, node);
+      child = gtk_widget_get_first_child (GTK_WIDGET (menu_item));
+
+      if (child != priv->arrow_widget)
+        gtk_widget_insert_after (GTK_WIDGET (menu_item), priv->arrow_widget, NULL);
     }
   else
     {
-      gtk_css_node_remove_class (arrow_node, g_quark_from_static_string (GTK_STYLE_CLASS_LEFT));
-      gtk_css_node_add_class (arrow_node, g_quark_from_static_string (GTK_STYLE_CLASS_RIGHT));
+      gtk_style_context_add_class (context, GTK_STYLE_CLASS_RIGHT);
+      gtk_style_context_remove_class (context, GTK_STYLE_CLASS_LEFT);
 
-      node = gtk_css_node_get_last_child (widget_node);
-      if (node != arrow_node)
-        gtk_css_node_insert_after (widget_node, arrow_node, node);
+      child = gtk_widget_get_last_child (GTK_WIDGET (menu_item));
+
+      if (child != priv->arrow_widget)
+        gtk_widget_insert_before (GTK_WIDGET (menu_item), priv->arrow_widget, NULL);
     }
 }
 
 static void
-update_arrow_gadget (GtkMenuItem *menu_item)
+update_arrow_widget (GtkMenuItem *menu_item)
 {
   GtkMenuItemPrivate *priv = menu_item->priv;
   GtkWidget *widget = GTK_WIDGET (menu_item);
-  gboolean should_have_gadget;
+  gboolean should_have_arrow;
 
-  should_have_gadget = priv->reserve_indicator ||
+  should_have_arrow = priv->reserve_indicator ||
     (priv->submenu && !GTK_IS_MENU_BAR (gtk_widget_get_parent (widget)));
 
-  if (should_have_gadget)
+  if (should_have_arrow)
     {
-      if (!priv->arrow_gadget)
+      if (!priv->arrow_widget)
         {
-          priv->arrow_gadget = gtk_builtin_icon_new ("arrow",
-                                                     widget,
-                                                     priv->gadget,
-                                                     NULL);
-          update_node_classes (menu_item);
+          priv->arrow_widget = gtk_icon_new ("arrow");
+          gtk_widget_set_parent (priv->arrow_widget, widget);
         }
     }
-  else
+  else if (priv->arrow_widget)
     {
-      g_clear_object (&priv->arrow_gadget);
+      gtk_widget_unparent (priv->arrow_widget);
+      priv->arrow_widget = NULL;
     }
 }
 
@@ -1097,7 +1012,7 @@ gtk_menu_item_set_submenu (GtkMenuItem *menu_item,
                                      gtk_menu_item_detacher);
         }
 
-      update_arrow_gadget (menu_item);
+      update_arrow_widget (menu_item);
 
       if (gtk_widget_get_parent (widget))
         gtk_widget_queue_resize (widget);
@@ -1374,9 +1289,10 @@ gtk_real_menu_item_set_label (GtkMenuItem *menu_item,
   gtk_menu_item_ensure_label (menu_item);
 
   child = gtk_bin_get_child (GTK_BIN (menu_item));
-  if (GTK_IS_LABEL (child))
+  if (GTK_IS_LABEL (child) ||
+      GTK_IS_ACCEL_LABEL (child))
     {
-      gtk_label_set_label (GTK_LABEL (child), label ? label : "");
+      g_object_set (child, "label", label ? label : "", NULL);
 
       g_object_notify_by_pspec (G_OBJECT (menu_item), menu_item_props[PROP_LABEL]);
     }
@@ -1390,8 +1306,15 @@ gtk_real_menu_item_get_label (GtkMenuItem *menu_item)
   gtk_menu_item_ensure_label (menu_item);
 
   child = gtk_bin_get_child (GTK_BIN (menu_item));
-  if (GTK_IS_LABEL (child))
-    return gtk_label_get_label (GTK_LABEL (child));
+  if (GTK_IS_LABEL (child) ||
+      GTK_IS_ACCEL_LABEL (child))
+    {
+      const char *label;
+
+      g_object_get (child, "label", &label, NULL);
+
+      return label;
+    }
 
   return NULL;
 }
@@ -1719,7 +1642,7 @@ gtk_menu_item_parent_set (GtkWidget *widget,
                                        menu->priv->accel_group,
                                        TRUE);
 
-  update_arrow_gadget (menu_item);
+  update_arrow_widget (menu_item);
 
   if (GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->parent_set)
     GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->parent_set (widget, previous_parent);
@@ -1731,7 +1654,7 @@ gtk_menu_item_direction_changed (GtkWidget        *widget,
 {
   GtkMenuItem *menu_item = GTK_MENU_ITEM (widget);
 
-  update_node_classes (menu_item);
+  update_arrow_classes (menu_item);
 
   GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->direction_changed (widget, previous_dir);
 }
@@ -1900,7 +1823,7 @@ gtk_menu_item_ensure_label (GtkMenuItem *menu_item)
 
   if (!gtk_bin_get_child (GTK_BIN (menu_item)))
     {
-      accel_label = g_object_new (GTK_TYPE_ACCEL_LABEL, "xalign", 0.0, NULL);
+      accel_label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
       gtk_widget_set_halign (accel_label, GTK_ALIGN_FILL);
       gtk_widget_set_valign (accel_label, GTK_ALIGN_CENTER);
 
@@ -1968,10 +1891,10 @@ gtk_menu_item_set_use_underline (GtkMenuItem *menu_item,
   gtk_menu_item_ensure_label (menu_item);
 
   child = gtk_bin_get_child (GTK_BIN (menu_item));
-  if (GTK_IS_LABEL (child) &&
-      gtk_label_get_use_underline (GTK_LABEL (child)) != setting)
+  if (GTK_IS_ACCEL_LABEL (child) &&
+      gtk_accel_label_get_use_underline (GTK_ACCEL_LABEL (child)) != setting)
     {
-      gtk_label_set_use_underline (GTK_LABEL (child), setting);
+      gtk_accel_label_set_use_underline (GTK_ACCEL_LABEL (child), setting);
       g_object_notify_by_pspec (G_OBJECT (menu_item), menu_item_props[PROP_USE_UNDERLINE]);
     }
 }
@@ -2000,6 +1923,9 @@ gtk_menu_item_get_use_underline (GtkMenuItem *menu_item)
   child = gtk_bin_get_child (GTK_BIN (menu_item));
   if (GTK_IS_LABEL (child))
     return gtk_label_get_use_underline (GTK_LABEL (child));
+  else if (GTK_IS_ACCEL_LABEL (child))
+    return gtk_accel_label_get_use_underline (GTK_ACCEL_LABEL (child));
+
 
   return FALSE;
 }
@@ -2031,7 +1957,7 @@ gtk_menu_item_set_reserve_indicator (GtkMenuItem *menu_item,
   if (priv->reserve_indicator != reserve)
     {
       priv->reserve_indicator = reserve;
-      update_arrow_gadget (menu_item);
+      update_arrow_widget (menu_item);
       gtk_widget_queue_resize (GTK_WIDGET (menu_item));
     }
 }
