@@ -548,7 +548,8 @@ gtk_menu_shell_real_insert (GtkMenuShell *menu_shell,
 
   priv->children = g_list_insert (priv->children, child, position);
 
-  gtk_widget_set_parent (child, GTK_WIDGET (menu_shell));
+  if (gtk_widget_get_parent (child) == NULL)
+    gtk_widget_set_parent (child, GTK_WIDGET (menu_shell));
 }
 
 /**
@@ -1812,8 +1813,6 @@ gtk_menu_shell_tracker_insert_func (GtkMenuTrackerItem *item,
       label = gtk_menu_tracker_item_get_label (item);
       if (label)
         gtk_menu_item_set_label (GTK_MENU_ITEM (widget), label);
-
-      gtk_widget_show (widget);
     }
   else if (gtk_menu_tracker_item_get_has_link (item, G_MENU_LINK_SUBMENU))
     {
@@ -1823,6 +1822,7 @@ gtk_menu_shell_tracker_insert_func (GtkMenuTrackerItem *item,
       g_object_bind_property (item, "label", widget, "text", G_BINDING_SYNC_CREATE);
 
       submenu = GTK_MENU_SHELL (gtk_menu_new ());
+      gtk_widget_hide (GTK_WIDGET (submenu));
 
       /* We recurse directly here: we could use an idle instead to
        * prevent arbitrary recursion depth.  We could also do it
@@ -1851,8 +1851,6 @@ gtk_menu_shell_tracker_insert_func (GtkMenuTrackerItem *item,
           g_signal_connect (submenu, "hide", G_CALLBACK (gtk_menu_shell_submenu_hidden), item);
           g_signal_connect (submenu, "selection-done", G_CALLBACK (gtk_menu_shell_submenu_selection_done), item);
         }
-
-      gtk_widget_show (widget);
     }
   else
     {
