@@ -35,11 +35,10 @@
 #include "gtkpopover.h"
 #include "gtkintl.h"
 #include "gtkcssnodeprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
-#include "gtkbuiltiniconprivate.h"
 #include "gtkcsstypesprivate.h"
 #include "gtkstylecontextprivate.h"
 #include "gtkcontainerprivate.h"
+#include "gtkiconprivate.h"
 
 /**
  * SECTION:gtkmodelbutton
@@ -154,8 +153,7 @@ struct _GtkModelButton
   GtkWidget *box;
   GtkWidget *image;
   GtkWidget *label;
-  GtkCssGadget *gadget;
-  GtkCssGadget *indicator_gadget;
+  GtkWidget *indicator_widget;
   gboolean active;
   gboolean centered;
   gboolean inverted;
@@ -203,8 +201,6 @@ gtk_model_button_update_state (GtkModelButton *button)
   state = gtk_widget_get_state_flags (GTK_WIDGET (button));
   indicator_state = state;
 
-  gtk_css_gadget_set_state (button->gadget, state);
-
   if (button->role == GTK_BUTTON_ROLE_CHECK)
     {
       if (button->active && !button->menu_name)
@@ -238,41 +234,41 @@ gtk_model_button_update_state (GtkModelButton *button)
         image_type = GTK_CSS_IMAGE_BUILTIN_ARROW_RIGHT;
     }
 
-  gtk_builtin_icon_set_image (GTK_BUILTIN_ICON (button->indicator_gadget), image_type);
+  gtk_icon_set_image (GTK_ICON (button->indicator_widget), image_type);
 
   if (button->iconic)
-    gtk_css_gadget_set_state (button->gadget, indicator_state);
+    gtk_widget_set_state_flags (GTK_WIDGET (button), indicator_state, TRUE);
   else
-    gtk_css_gadget_set_state (button->gadget, state);
+    gtk_widget_set_state_flags (GTK_WIDGET (button), state, TRUE);
 
-  gtk_css_gadget_set_state (button->indicator_gadget, indicator_state);
+  gtk_widget_set_state_flags (button->indicator_widget, indicator_state, TRUE);
 }
 
 static void
 update_node_ordering (GtkModelButton *button)
 {
-  GtkCssNode *widget_node, *indicator_node, *node;
+  GtkStyleContext *indicator_context;
+  GtkWidget *child;
 
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (button));
-  indicator_node = gtk_css_gadget_get_node (button->indicator_gadget);
+  indicator_context = gtk_widget_get_style_context (button->indicator_widget);
 
   if (indicator_is_left (GTK_WIDGET (button)))
     {
-      gtk_css_gadget_add_class (button->indicator_gadget, GTK_STYLE_CLASS_LEFT);
-      gtk_css_gadget_remove_class (button->indicator_gadget, GTK_STYLE_CLASS_RIGHT);
+      gtk_style_context_add_class (indicator_context, GTK_STYLE_CLASS_LEFT);
+      gtk_style_context_remove_class (indicator_context, GTK_STYLE_CLASS_RIGHT);
 
-      node = gtk_css_node_get_first_child (widget_node);
-      if (node != indicator_node)
-        gtk_css_node_insert_before (widget_node, indicator_node, node);
+      child = gtk_widget_get_first_child (GTK_WIDGET (button));
+      if (child != button->indicator_widget)
+        gtk_widget_insert_before (button->indicator_widget, GTK_WIDGET (button), child);
     }
   else
     {
-      gtk_css_gadget_remove_class (button->indicator_gadget, GTK_STYLE_CLASS_LEFT);
-      gtk_css_gadget_add_class (button->indicator_gadget, GTK_STYLE_CLASS_RIGHT);
+      gtk_style_context_add_class (indicator_context, GTK_STYLE_CLASS_RIGHT);
+      gtk_style_context_remove_class (indicator_context, GTK_STYLE_CLASS_LEFT);
 
-      node = gtk_css_node_get_last_child (widget_node);
-      if (node != indicator_node)
-        gtk_css_node_insert_after (widget_node, indicator_node, node);
+      child = gtk_widget_get_first_child (GTK_WIDGET (button));
+      if (child != button->indicator_widget)
+        gtk_widget_insert_after (button->indicator_widget, GTK_WIDGET (button), child);
     }
 }
 
@@ -304,7 +300,6 @@ update_node_name (GtkModelButton *button)
   AtkRole a11y_role;
   const gchar *indicator_name;
   gboolean indicator_visible;
-  GtkCssNode *indicator_node;
 
   accessible = gtk_widget_get_accessible (GTK_WIDGET (button));
   switch (button->role)
@@ -313,25 +308,25 @@ update_node_name (GtkModelButton *button)
       a11y_role = ATK_ROLE_PUSH_BUTTON;
       if (button->menu_name)
         {
-          indicator_name = I_("arrow");
+          indicator_name = "arrow";
           indicator_visible = TRUE;
         }
       else
         {
-          indicator_name = I_("check");
+          indicator_name = "check";
           indicator_visible = FALSE;
         }
       break;
 
     case GTK_BUTTON_ROLE_CHECK:
       a11y_role = ATK_ROLE_CHECK_BOX;
-      indicator_name = I_("check");
+      indicator_name = "check";
       indicator_visible = TRUE;
       break;
 
     case GTK_BUTTON_ROLE_RADIO:
       a11y_role = ATK_ROLE_RADIO_BUTTON;
-      indicator_name = I_("radio");
+      indicator_name = "radio";
       indicator_visible = TRUE;
       break;
 
@@ -344,9 +339,8 @@ update_node_name (GtkModelButton *button)
 
   atk_object_set_role (accessible, a11y_role);
 
-  indicator_node = gtk_css_gadget_get_node (button->indicator_gadget);
-  gtk_css_node_set_name (indicator_node, indicator_name);
-  gtk_css_node_set_visible (indicator_node, indicator_visible);
+  gtk_icon_set_css_name (GTK_ICON (button->indicator_widget), indicator_name);
+  gtk_widget_set_visible (button->indicator_widget, indicator_visible);
 }
 
 static void
@@ -455,7 +449,7 @@ gtk_model_button_set_iconic (GtkModelButton *button,
                              gboolean        iconic)
 {
   GtkCssNode *widget_node;
-  GtkCssNode *indicator_node;
+  GtkStyleContext *context;
 
   if (button->iconic == iconic)
     return;
@@ -463,24 +457,24 @@ gtk_model_button_set_iconic (GtkModelButton *button,
   button->iconic = iconic;
 
   widget_node = gtk_widget_get_css_node (GTK_WIDGET (button));
-  indicator_node = gtk_css_gadget_get_node (button->indicator_gadget);
+  context = gtk_widget_get_style_context (GTK_WIDGET (button));
   if (iconic)
     {
       gtk_css_node_set_name (widget_node, I_("button"));
-      gtk_css_gadget_add_class (button->gadget, "model");
-      gtk_css_gadget_add_class (button->gadget, "image-button");
+      gtk_style_context_add_class (context, "model");
+      gtk_style_context_add_class (context, "image-button");
       gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NORMAL);
-      gtk_css_node_set_visible (indicator_node, FALSE);
+      gtk_widget_set_visible (button->indicator_widget, FALSE);
     }
   else
     {
       gtk_css_node_set_name (widget_node, I_("modelbutton"));
-      gtk_css_gadget_remove_class (button->gadget, "model");
-      gtk_css_gadget_remove_class (button->gadget, "image-button");
+      gtk_style_context_remove_class (context, "model");
+      gtk_style_context_remove_class (context, "image-button");
       gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-      gtk_css_node_set_visible (indicator_node,
-                                button->role != GTK_BUTTON_ROLE_NORMAL ||
-                                button->menu_name == NULL);
+      gtk_widget_set_visible (button->indicator_widget,
+                              button->role != GTK_BUTTON_ROLE_NORMAL ||
+                              button->menu_name == NULL);
     }
 
   update_visibility (button);
@@ -632,288 +626,235 @@ needs_indicator (GtkModelButton *button)
 }
 
 static void
-gtk_model_button_measure_ (GtkWidget      *widget,
-                           GtkOrientation  orientation,
-                           int             for_size,
-                           int            *minimum,
-                           int            *natural,
-                           int            *minimum_baseline,
-                           int            *natural_baseline)
-{
-  GtkCssGadget *gadget;
-
-  if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
-  else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
-
-  gtk_css_gadget_get_preferred_size (gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
-}
-
-static void
-gtk_model_button_measure (GtkCssGadget   *gadget,
+gtk_model_button_measure (GtkWidget      *widget,
                           GtkOrientation  orientation,
                           int             for_size,
                           int            *minimum,
                           int            *natural,
                           int            *minimum_baseline,
-                          int            *natural_baseline,
-                          gpointer        data)
+                          int            *natural_baseline)
 {
-  GtkWidget *widget;
-  GtkModelButton *button;
-  GtkWidget *child;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-  button = GTK_MODEL_BUTTON (widget);
-  child = gtk_bin_get_child (GTK_BIN (widget));
-
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+  if (GTK_MODEL_BUTTON (widget)->iconic)
     {
-      gint check_min, check_nat;
-
-      gtk_css_gadget_get_preferred_size (button->indicator_gadget,
-                                         GTK_ORIENTATION_HORIZONTAL,
-                                         -1,
-                                         &check_min, &check_nat,
-                                         NULL, NULL);
-
-      if (child && gtk_widget_get_visible (child))
-        {
-          gtk_widget_measure (child,
-                              orientation,
-                              for_size,
-                              minimum, natural,
-                              minimum_baseline, natural_baseline);
-        }
-      else
-        {
-          *minimum = 0;
-          *natural = 0;
-        }
-
-      if (button->centered)
-        {
-          *minimum += 2 * check_min;
-          *natural += 2 * check_nat;
-        }
-      else if (needs_indicator (button))
-        {
-          *minimum += check_min;
-          *natural += check_nat;
-        }
+      GTK_WIDGET_CLASS (gtk_model_button_parent_class)->measure (widget, orientation, for_size,
+                                                                 minimum, natural,
+                                                                 minimum_baseline, natural_baseline);
     }
   else
     {
-      gint check_min, check_nat;
+      GtkModelButton *button;
+      GtkWidget *child;
 
-      gtk_css_gadget_get_preferred_size (button->indicator_gadget,
-                                         GTK_ORIENTATION_VERTICAL,
-                                         -1,
-                                         &check_min, &check_nat,
-                                         NULL, NULL);
+      button = GTK_MODEL_BUTTON (widget);
+      child = gtk_bin_get_child (GTK_BIN (widget));
 
-      if (child && gtk_widget_get_visible (child))
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
         {
-          gint child_min, child_nat;
-          gint child_min_baseline = -1, child_nat_baseline = -1;
+          gint check_min, check_nat;
 
-          if (for_size > -1)
-            {
-              if (button->centered)
-                for_size -= 2 * check_nat;
-              else if (needs_indicator (button))
-                for_size -= check_nat;
-            }
+          gtk_widget_measure (button->indicator_widget,
+                              GTK_ORIENTATION_HORIZONTAL,
+                              -1,
+                              &check_min, &check_nat,
+                               NULL, NULL);
 
-          gtk_widget_measure (child, GTK_ORIENTATION_VERTICAL,
-                              for_size,
-                              &child_min, &child_nat,
-                              &child_min_baseline, &child_nat_baseline);
-
-          if (button->centered)
+          if (child && gtk_widget_get_visible (child))
             {
-              *minimum = MAX (2 * check_min, child_min);
-              *natural = MAX (2 * check_nat, child_nat);
-            }
-          else if (needs_indicator (button))
-            {
-              *minimum = MAX (check_min, child_min);
-              *natural = MAX (check_nat, child_nat);
-            }
-          else
-            {
-              *minimum = child_min;
-              *natural = child_nat;
-            }
-
-          if (minimum_baseline && child_min_baseline >= 0)
-            *minimum_baseline = child_min_baseline + (*minimum - child_min) / 2;
-          if (natural_baseline && child_nat_baseline >= 0)
-            *natural_baseline = child_nat_baseline + (*natural - child_nat) / 2;
-        }
-      else
-        {
-          if (button->centered)
-            {
-              *minimum = 2 * check_min;
-              *natural = 2 * check_nat;
-            }
-          else if (needs_indicator (button))
-            {
-              *minimum = check_min;
-              *natural = check_nat;
+              gtk_widget_measure (child,
+                                  orientation,
+                                  for_size,
+                                  minimum, natural,
+                                  minimum_baseline, natural_baseline);
             }
           else
             {
               *minimum = 0;
               *natural = 0;
             }
+
+          if (button->centered)
+            {
+              *minimum += 2 * check_min;
+              *natural += 2 * check_nat;
+            }
+          else if (needs_indicator (button))
+            {
+              *minimum += check_min;
+              *natural += check_nat;
+            }
         }
-    }
-}
-
-static void
-gtk_model_button_size_allocate (GtkWidget     *widget,
-                                GtkAllocation *allocation)
-{
-  GtkCssGadget *gadget;
-  GdkRectangle clip;
-
-  if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
-  else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
-
-  gtk_widget_set_allocation (widget, allocation);
-  gtk_css_gadget_allocate (gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_model_button_allocate (GtkCssGadget        *gadget,
-                           const GtkAllocation *allocation,
-                           int                  baseline,
-                           GtkAllocation       *out_clip,
-                           gpointer             unused)
-{
-  GtkWidget *widget;
-  GtkModelButton *button;
-  GtkAllocation child_allocation;
-  GtkWidget *child;
-  gint check_min_width, check_nat_width;
-  gint check_min_height, check_nat_height;
-  GdkRectangle check_clip;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-  button = GTK_MODEL_BUTTON (widget);
-  child = gtk_bin_get_child (GTK_BIN (widget));
-
-
-
-  gtk_css_gadget_get_preferred_size (button->indicator_gadget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     -1,
-                                     &check_min_width, &check_nat_width,
-                                     NULL, NULL);
-  gtk_css_gadget_get_preferred_size (button->indicator_gadget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     -1,
-                                     &check_min_height, &check_nat_height,
-                                     NULL, NULL);
-
-  if (indicator_is_left (widget))
-    child_allocation.x = allocation->x;
-  else
-    child_allocation.x = allocation->x + allocation->width - check_nat_width;
-  child_allocation.y = allocation->y + (allocation->height - check_nat_height) / 2;
-  child_allocation.width = check_nat_width;
-  child_allocation.height = check_nat_height;
-
-  gtk_css_gadget_allocate (button->indicator_gadget,
-                           &child_allocation,
-                           baseline,
-                           &check_clip);
-
-  if (child && gtk_widget_get_visible (child))
-    {
-      GtkBorder border = { 0, };
-
-      if (button->centered)
+      else
         {
-          border.left = check_nat_width;
-          border.right = check_nat_width;
-        }
-      else if (needs_indicator (button))
-        {
-          if (indicator_is_left (widget))
-            border.left += check_nat_width;
+          gint check_min, check_nat;
+
+          gtk_widget_measure (button->indicator_widget,
+                              GTK_ORIENTATION_VERTICAL,
+                              -1,
+                              &check_min, &check_nat,
+                              NULL, NULL);
+
+          if (child && gtk_widget_get_visible (child))
+            {
+              gint child_min, child_nat;
+              gint child_min_baseline = -1, child_nat_baseline = -1;
+
+              if (for_size > -1)
+                {
+                  if (button->centered)
+                    for_size -= 2 * check_nat;
+                  else if (needs_indicator (button))
+                    for_size -= check_nat;
+                }
+
+              gtk_widget_measure (child, GTK_ORIENTATION_VERTICAL,
+                                  for_size,
+                                  &child_min, &child_nat,
+                                  &child_min_baseline, &child_nat_baseline);
+
+              if (button->centered)
+                {
+                  *minimum = MAX (2 * check_min, child_min);
+                  *natural = MAX (2 * check_nat, child_nat);
+                }
+              else if (needs_indicator (button))
+                {
+                  *minimum = MAX (check_min, child_min);
+                  *natural = MAX (check_nat, child_nat);
+                }
+              else
+                {
+                  *minimum = child_min;
+                  *natural = child_nat;
+                }
+
+              if (minimum_baseline && child_min_baseline >= 0)
+                *minimum_baseline = child_min_baseline + (*minimum - child_min) / 2;
+              if (natural_baseline && child_nat_baseline >= 0)
+                *natural_baseline = child_nat_baseline + (*natural - child_nat) / 2;
+            }
           else
-            border.right += check_nat_width;
+            {
+              if (button->centered)
+                {
+                  *minimum = 2 * check_min;
+                  *natural = 2 * check_nat;
+                }
+              else if (needs_indicator (button))
+                {
+                  *minimum = check_min;
+                  *natural = check_nat;
+                }
+              else
+                {
+                  *minimum = 0;
+                  *natural = 0;
+                }
+            }
         }
-
-      child_allocation.x = allocation->x + border.left;
-      child_allocation.y = allocation->y + border.top;
-      child_allocation.width = allocation->width - border.left - border.right;
-      child_allocation.height = allocation->height - border.top - border.bottom;
-
-      baseline = gtk_widget_get_allocated_baseline (widget);
-      if (baseline != -1)
-        baseline -= border.top;
-
-      gtk_widget_size_allocate_with_baseline (child, &child_allocation, baseline);
     }
+}
 
-  gtk_container_get_children_clip (GTK_CONTAINER (widget), out_clip);
-  gdk_rectangle_union (out_clip, &check_clip, out_clip);
+static void
+gtk_model_button_size_allocate (GtkWidget           *widget,
+                                const GtkAllocation *allocation,
+                                int                  baseline,
+                                GtkAllocation       *out_clip)
+{
+  if (GTK_MODEL_BUTTON (widget)->iconic)
+    {
+      GTK_WIDGET_CLASS (gtk_model_button_parent_class)->size_allocate (widget,
+                                                                       allocation,
+                                                                       baseline,
+                                                                       out_clip);
+    }
+  else
+    {
+      GtkAllocation child_clip = *allocation;
+      GtkModelButton *button;
+      GtkAllocation child_allocation;
+      GtkWidget *child;
+      gint check_min_width, check_nat_width;
+      gint check_min_height, check_nat_height;
+
+      button = GTK_MODEL_BUTTON (widget);
+      child = gtk_bin_get_child (GTK_BIN (widget));
+
+
+
+      gtk_widget_measure (button->indicator_widget,
+                          GTK_ORIENTATION_HORIZONTAL,
+                          -1,
+                          &check_min_width, &check_nat_width,
+                          NULL, NULL);
+      gtk_widget_measure (button->indicator_widget,
+                          GTK_ORIENTATION_VERTICAL,
+                          -1,
+                          &check_min_height, &check_nat_height,
+                          NULL, NULL);
+
+      if (indicator_is_left (widget))
+        child_allocation.x = allocation->x;
+      else
+        child_allocation.x = allocation->x + allocation->width - check_nat_width;
+      child_allocation.y = allocation->y + (allocation->height - check_nat_height) / 2;
+      child_allocation.width = check_nat_width;
+      child_allocation.height = check_nat_height;
+
+      gtk_widget_size_allocate (button->indicator_widget, &child_allocation, baseline, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
+
+      if (child && gtk_widget_get_visible (child))
+        {
+          GtkBorder border = { 0, };
+
+          if (button->centered)
+            {
+              border.left = check_nat_width;
+              border.right = check_nat_width;
+            }
+          else if (needs_indicator (button))
+            {
+              if (indicator_is_left (widget))
+                border.left += check_nat_width;
+              else
+                border.right += check_nat_width;
+            }
+
+          child_allocation.x = allocation->x + border.left;
+          child_allocation.y = allocation->y + border.top;
+          child_allocation.width = allocation->width - border.left - border.right;
+          child_allocation.height = allocation->height - border.top - border.bottom;
+
+          if (baseline != -1)
+            baseline -= border.top;
+
+          gtk_widget_size_allocate (child, &child_allocation, baseline, &child_clip);
+          gdk_rectangle_union (out_clip, &child_clip, out_clip);
+        }
+    }
 }
 
 static void
 gtk_model_button_snapshot (GtkWidget   *widget,
                            GtkSnapshot *snapshot)
 {
-  GtkCssGadget *gadget;
-
   if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
+    {
+      GTK_WIDGET_CLASS (gtk_model_button_parent_class)->snapshot (widget, snapshot);
+    }
   else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
+    {
+      GtkWidget *child;
+      GtkModelButton *button = GTK_MODEL_BUTTON (widget);
 
-  gtk_css_gadget_snapshot (gadget, snapshot);
-}
+      if (gtk_widget_get_visible (button->indicator_widget))
+        gtk_widget_snapshot_child (widget, button->indicator_widget, snapshot);
 
-static gboolean
-gtk_model_button_render (GtkCssGadget *gadget,
-                         GtkSnapshot  *snapshot,
-                         int           x,
-                         int           y,
-                         int           width,
-                         int           height,
-                         gpointer      data)
-{
-  GtkWidget *widget;
-  GtkModelButton *button;
-  GtkWidget *child;
+      child = gtk_bin_get_child (GTK_BIN (widget));
+      if (child)
+        gtk_widget_snapshot_child (widget, child, snapshot);
+    }
 
-  widget = gtk_css_gadget_get_owner (gadget);
-  button = GTK_MODEL_BUTTON (widget);
-
-  if (gtk_css_node_get_visible (gtk_css_gadget_get_node (button->indicator_gadget)))
-    gtk_css_gadget_snapshot (button->indicator_gadget, snapshot);
-
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child)
-    gtk_widget_snapshot_child (widget, child, snapshot);
-
-  return gtk_widget_has_visible_focus (widget);
 }
 
 static void
@@ -954,8 +895,7 @@ gtk_model_button_finalize (GObject *object)
 {
   GtkModelButton *button = GTK_MODEL_BUTTON (object);
 
-  g_clear_object (&button->indicator_gadget);
-  g_clear_object (&button->gadget);
+  gtk_widget_unparent (button->indicator_widget);
 
   G_OBJECT_CLASS (gtk_model_button_parent_class)->finalize (object);
 }
@@ -971,7 +911,7 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
   object_class->get_property = gtk_model_button_get_property;
   object_class->set_property = gtk_model_button_set_property;
 
-  widget_class->measure = gtk_model_button_measure_;
+  widget_class->measure = gtk_model_button_measure;
   widget_class->size_allocate = gtk_model_button_size_allocate;
   widget_class->snapshot = gtk_model_button_snapshot;
   widget_class->destroy = gtk_model_button_destroy;
@@ -1111,8 +1051,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
 static void
 gtk_model_button_init (GtkModelButton *button)
 {
-  GtkCssNode *widget_node;
-
   button->role = GTK_BUTTON_ROLE_NORMAL;
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   button->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
@@ -1125,21 +1063,10 @@ gtk_model_button_init (GtkModelButton *button)
   gtk_container_add (GTK_CONTAINER (button->box), button->label);
   gtk_container_add (GTK_CONTAINER (button), button->box);
 
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (button));
-  button->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                       GTK_WIDGET (button),
-                                                       gtk_model_button_measure,
-                                                       gtk_model_button_allocate,
-                                                       gtk_model_button_render,
-                                                       NULL,
-                                                       NULL);
-  button->indicator_gadget = gtk_builtin_icon_new ("check",
-                                                   GTK_WIDGET (button),
-                                                   button->gadget,
-                                                   NULL);
-  gtk_builtin_icon_set_default_size (GTK_BUILTIN_ICON (button->indicator_gadget), 16);
+  button->indicator_widget = gtk_icon_new ("check");
+  gtk_widget_set_parent (button->indicator_widget, GTK_WIDGET (button));
+  gtk_widget_hide (button->indicator_widget);
   update_node_ordering (button);
-  gtk_css_node_set_visible (gtk_css_gadget_get_node (button->indicator_gadget), FALSE);
 }
 
 /**
