@@ -67,17 +67,6 @@ gtk_model_menu_item_activate (GtkMenuItem *item)
   /* block the automatic toggle behaviour -- just do nothing */
 }
 
-static void
-gtk_model_menu_item_snapshot_indicator (GtkCheckMenuItem *check_item,
-                                        GtkSnapshot      *snapshot)
-{
-  GtkModelMenuItem *item = GTK_MODEL_MENU_ITEM (check_item);
-
-  if (item->has_indicator)
-    GTK_CHECK_MENU_ITEM_CLASS (gtk_model_menu_item_parent_class)
-      ->snapshot_indicator (check_item, snapshot);
-}
-
 static void
 gtk_model_menu_item_set_has_indicator (GtkModelMenuItem *item,
                                        gboolean          has_indicator)
@@ -87,7 +76,8 @@ gtk_model_menu_item_set_has_indicator (GtkModelMenuItem *item,
 
   item->has_indicator = has_indicator;
 
-  gtk_widget_queue_resize (GTK_WIDGET (item));
+  gtk_widget_set_visible (_gtk_check_menu_item_get_indicator_widget (GTK_CHECK_MENU_ITEM (item)),
+                          item->has_indicator);
 }
 
 static void
@@ -150,7 +140,7 @@ gtk_model_menu_item_set_icon (GtkModelMenuItem *item,
     {
       gtk_menu_item_get_label (GTK_MENU_ITEM (item));
       child = gtk_bin_get_child (GTK_BIN (item));
-      g_assert (GTK_IS_LABEL (child));
+      g_assert (GTK_IS_ACCEL_LABEL (child));
     }
 
   /* If it is a box, make sure there are no images inside of it already.
@@ -244,7 +234,7 @@ gtk_model_menu_item_set_text (GtkModelMenuItem *item,
     {
       gtk_menu_item_get_label (GTK_MENU_ITEM (item));
       child = gtk_bin_get_child (GTK_BIN (item));
-      g_assert (GTK_IS_LABEL (child));
+      g_assert (GTK_IS_ACCEL_LABEL (child));
     }
 
   if (GTK_IS_LABEL (child))
@@ -252,6 +242,11 @@ gtk_model_menu_item_set_text (GtkModelMenuItem *item,
       gtk_label_set_text_with_mnemonic (GTK_LABEL (child), text);
       return;
     }
+  else if (GTK_IS_ACCEL_LABEL (child))
+    {
+      gtk_accel_label_set_label (GTK_ACCEL_LABEL (child), text);
+      return;
+    }
 
   if (!GTK_IS_CONTAINER (child))
     return;
@@ -278,6 +273,8 @@ gtk_model_menu_item_get_text (GtkModelMenuItem *item)
 
   if (GTK_IS_LABEL (child))
     return gtk_label_get_text (GTK_LABEL (child));
+  else if (GTK_IS_ACCEL_LABEL (child))
+    return gtk_accel_label_get_label (GTK_ACCEL_LABEL (child));
 
   if (GTK_IS_CONTAINER (child))
     {
@@ -463,17 +460,16 @@ gtk_model_menu_item_set_property (GObject      *object,
 static void
 gtk_model_menu_item_init (GtkModelMenuItem *item)
 {
+  item->has_indicator = FALSE;
+  gtk_widget_hide (_gtk_check_menu_item_get_indicator_widget (GTK_CHECK_MENU_ITEM (item)));
 }
 
 static void
 gtk_model_menu_item_class_init (GtkModelMenuItemClass *class)
 {
-  GtkCheckMenuItemClass *check_class = GTK_CHECK_MENU_ITEM_CLASS (class);
   GtkMenuItemClass *item_class = GTK_MENU_ITEM_CLASS (class);
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
-  check_class->snapshot_indicator = gtk_model_menu_item_snapshot_indicator;
-
   item_class->toggle_size_request = gtk_model_menu_item_toggle_size_request;
   item_class->activate = gtk_model_menu_item_activate;
 
