@@ -33,9 +33,7 @@
  * @Title: GtkOrientable
  *
  * The #GtkOrientable interface is implemented by all widgets that can be
- * oriented horizontally or vertically. Historically, such widgets have been
- * realized as subclasses of a common base class (e.g #GtkBox/#GtkHBox/#GtkVBox
- * or #GtkScale/#GtkHScale/#GtkVScale). #GtkOrientable is more flexible in that
+ * oriented horizontally or vertically. #GtkOrientable is more flexible in that
  * it allows the orientation to be changed at runtime, allowing the widgets
  * to “flip”.
  *
@@ -62,7 +60,7 @@ gtk_orientable_default_init (GtkOrientableInterface *iface)
                                                           P_("The orientation of the orientable"),
                                                           GTK_TYPE_ORIENTATION,
                                                           GTK_ORIENTATION_HORIZONTAL,
-                                                          GTK_PARAM_READWRITE));
+                                                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 }
 
 /**
@@ -94,7 +92,7 @@ gtk_orientable_set_orientation (GtkOrientable  *orientable,
  *
  * Retrieves the orientation of the @orientable.
  *
- * Return value: the orientation of the @orientable.
+ * Returns: the orientation of the @orientable.
  *
  * Since: 2.16
  **/
