@@ -93,17 +93,12 @@ gtk_overlay_compute_child_allocation (GtkOverlay      *overlay,
                                       GtkOverlayChild *child,
                                       GtkAllocation   *widget_allocation)
 {
-  GtkAllocation allocation, overlay_allocation;
+  GtkAllocation allocation;
   gboolean result;
 
   g_signal_emit (overlay, signals[GET_CHILD_POSITION],
                  0, child->widget, &allocation, &result);
 
-  gtk_widget_get_allocation (GTK_WIDGET (overlay), &overlay_allocation);
-
-  allocation.x += overlay_allocation.x;
-  allocation.y += overlay_allocation.y;
-
   widget_allocation->x = allocation.x;
   widget_allocation->y = allocation.y;
   widget_allocation->width = allocation.width;
@@ -238,6 +233,7 @@ gtk_overlay_child_allocate (GtkOverlay      *overlay,
                             GtkOverlayChild *child)
 {
   GtkAllocation child_allocation;
+  GtkAllocation child_clip;
 
   if (!gtk_widget_get_visible (child->widget))
     return;
@@ -245,23 +241,25 @@ gtk_overlay_child_allocate (GtkOverlay      *overlay,
   gtk_overlay_compute_child_allocation (overlay, child, &child_allocation);
 
   gtk_overlay_child_update_style_classes (overlay, child->widget, &child_allocation);
-  gtk_widget_size_allocate (child->widget, &child_allocation);
+  gtk_widget_size_allocate (child->widget, &child_allocation, -1, &child_clip);
 }
 
 static void
-gtk_overlay_size_allocate (GtkWidget     *widget,
-                           GtkAllocation *allocation)
+gtk_overlay_size_allocate (GtkWidget           *widget,
+                           const GtkAllocation *allocation,
+                           int                  baseline,
+                           GtkAllocation       *out_clip)
 {
   GtkOverlay *overlay = GTK_OVERLAY (widget);
   GtkOverlayPrivate *priv = overlay->priv;
   GSList *children;
   GtkWidget *main_widget;
 
-  GTK_WIDGET_CLASS (gtk_overlay_parent_class)->size_allocate (widget, allocation);
+  GTK_WIDGET_CLASS (gtk_overlay_parent_class)->size_allocate (widget, allocation, baseline, out_clip);
 
   main_widget = gtk_bin_get_child (GTK_BIN (overlay));
   if (main_widget && gtk_widget_get_visible (main_widget))
-    gtk_widget_size_allocate (main_widget, allocation);
+    gtk_widget_size_allocate (main_widget, allocation, -1, out_clip);
 
   for (children = priv->children; children; children = children->next)
     gtk_overlay_child_allocate (overlay, children->data);
@@ -332,6 +330,19 @@ gtk_overlay_get_child_position (GtkOverlay    *overlay,
   return TRUE;
 }
 
+static void
+gtk_overlay_add (GtkContainer *container,
+                 GtkWidget    *widget)
+{
+
+  /* We only get into this path if we have no child
+   * (since GtkOverlay is a GtkBin) and the only child
+   * we can add through gtk_container_add is the main child,
+   * which we want to keep the lowest in the hierarchy. */
+  gtk_widget_insert_after (widget, GTK_WIDGET (container), NULL);
+  _gtk_bin_set_child (GTK_BIN (container), widget);
+}
+
 static void
 gtk_overlay_remove (GtkContainer *container,
                     GtkWidget    *widget)
@@ -593,6 +604,7 @@ gtk_overlay_class_init (GtkOverlayClass *klass)
 
   widget_class->size_allocate = gtk_overlay_size_allocate;
 
+  container_class->add = gtk_overlay_add;
   container_class->remove = gtk_overlay_remove;
   container_class->forall = gtk_overlay_forall;
   container_class->set_child_property = gtk_overlay_set_child_property;
