@@ -38,11 +38,11 @@
 #include "gtkwidgetprivate.h"
 #include "gtkcontainerprivate.h"
 #include "a11y/gtkpanedaccessible.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkcssnodeprivate.h"
 #include "gtkstylecontextprivate.h"
 #include "gtkcssstylepropertyprivate.h"
 #include "gtkrendericonprivate.h"
+#include "gtkgizmoprivate.h"
 
 #include <math.h>
 
@@ -117,6 +117,8 @@
  * ]|
  */
 
+#define HANDLE_EXTRA_SIZE 6
+
 enum {
   CHILD1,
   CHILD2
@@ -133,10 +135,8 @@ struct _GtkPanedPrivate
   GtkOrientation  orientation;
 
   GdkRectangle   handle_pos;
-  GdkWindow     *handle;
 
-  GtkCssGadget  *gadget;
-  GtkCssGadget  *handle_gadget;
+  GtkWidget     *handle_widget;
 
   GtkGesture    *pan_gesture;  /* Used for touch */
   GtkGesture    *drag_gesture; /* Used for mice */
@@ -148,7 +148,6 @@ struct _GtkPanedPrivate
   gint          min_position;
   gint          original_position;
 
-  guint         handle_prelit : 1;
   guint         in_recursion  : 1;
   guint         child1_resize : 1;
   guint         child1_shrink : 1;
@@ -203,29 +202,22 @@ static void     gtk_paned_get_child_property    (GtkContainer     *container,
                                                  GValue           *value,
                                                  GParamSpec       *pspec);
 static void     gtk_paned_finalize              (GObject          *object);
-static void     gtk_paned_measure_ (GtkWidget *widget,
+static void     gtk_paned_measure (GtkWidget *widget,
                                    GtkOrientation  orientation,
                                    int             for_size,
                                    int            *minimum,
                                    int            *natural,
                                    int            *minimum_baseline,
                                    int            *natural_baseline);
-static void     gtk_paned_size_allocate         (GtkWidget        *widget,
-                                                 GtkAllocation    *allocation);
-static void     gtk_paned_realize               (GtkWidget        *widget);
+static void     gtk_paned_size_allocate         (GtkWidget           *widget,
+                                                 const GtkAllocation *allocation,
+                                                 int                  baseline,
+                                                 GtkAllocation       *out_clip);
 static void     gtk_paned_unrealize             (GtkWidget        *widget);
-static void     gtk_paned_map                   (GtkWidget        *widget);
-static void     gtk_paned_unmap                 (GtkWidget        *widget);
-static void     gtk_paned_state_flags_changed   (GtkWidget        *widget,
-                                                 GtkStateFlags     previous_state);
 static void     gtk_paned_direction_changed     (GtkWidget        *widget,
                                                  GtkTextDirection  previous_direction);
 static void     gtk_paned_snapshot              (GtkWidget        *widget,
 						 GtkSnapshot      *snapshot);
-static gboolean gtk_paned_enter                 (GtkWidget        *widget,
-						 GdkEventCrossing *event);
-static gboolean gtk_paned_leave                 (GtkWidget        *widget,
-						 GdkEventCrossing *event);
 static gboolean gtk_paned_focus                 (GtkWidget        *widget,
 						 GtkDirectionType  direction);
 static void     gtk_paned_add                   (GtkContainer     *container,
@@ -294,6 +286,57 @@ add_move_binding (GtkBindingSet   *binding_set,
 				GTK_TYPE_SCROLL_TYPE, scroll);
 }
 
+static void
+get_handle_area (GtkPaned     *paned,
+                 GdkRectangle *area)
+{
+  GtkPanedPrivate *priv = gtk_paned_get_instance_private (paned);
+  int extra = 0;
+
+  gtk_widget_get_outer_allocation (priv->handle_widget, area);
+
+  if (!gtk_paned_get_wide_handle (paned))
+    extra = HANDLE_EXTRA_SIZE;
+
+  area->x -= extra;
+  area->y -= extra;
+  area->width += extra * 2;
+  area->height += extra * 2;
+}
+
+static gboolean
+gtk_paned_motion_notify (GtkWidget      *widget,
+                         GdkEventMotion *event)
+{
+  GtkPaned *paned = GTK_PANED (widget);
+  GtkPanedPrivate *priv = gtk_paned_get_instance_private (paned);
+  GdkRectangle handle_area;
+  GdkDisplay *display;
+
+  get_handle_area (paned, &handle_area);
+  display = gtk_widget_get_display (widget);
+
+  if (gdk_rectangle_contains_point (&handle_area, event->x, event->y) ||
+      priv->panning)
+    {
+      GdkCursor *cursor;
+
+      if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+        cursor = gdk_cursor_new_from_name (display, "col-resize");
+      else
+        cursor = gdk_cursor_new_from_name (display, "row-resize");
+
+      gtk_widget_set_cursor (widget, cursor);
+      g_object_unref (cursor);
+    }
+  else
+    {
+      gtk_widget_set_cursor (widget, NULL);
+    }
+
+  return GTK_WIDGET_CLASS (gtk_paned_parent_class)->motion_notify_event (widget, event);
+}
+
 static void
 gtk_paned_class_init (GtkPanedClass *class)
 {
@@ -312,17 +355,12 @@ gtk_paned_class_init (GtkPanedClass *class)
   object_class->get_property = gtk_paned_get_property;
   object_class->finalize = gtk_paned_finalize;
 
-  widget_class->measure = gtk_paned_measure_;
+  widget_class->measure = gtk_paned_measure;
   widget_class->size_allocate = gtk_paned_size_allocate;
-  widget_class->realize = gtk_paned_realize;
   widget_class->unrealize = gtk_paned_unrealize;
-  widget_class->map = gtk_paned_map;
-  widget_class->unmap = gtk_paned_unmap;
   widget_class->snapshot = gtk_paned_snapshot;
   widget_class->focus = gtk_paned_focus;
-  widget_class->enter_notify_event = gtk_paned_enter;
-  widget_class->leave_notify_event = gtk_paned_leave;
-  widget_class->state_flags_changed = gtk_paned_state_flags_changed;
+  widget_class->motion_notify_event = gtk_paned_motion_notify;
   widget_class->direction_changed = gtk_paned_direction_changed;
 
   container_class->add = gtk_paned_add;
@@ -681,24 +719,22 @@ initiates_touch_drag (GtkPaned *paned,
 {
   gint handle_size, handle_pos, drag_pos;
   GtkPanedPrivate *priv = paned->priv;
-  GtkAllocation allocation;
 
 #define TOUCH_EXTRA_AREA_WIDTH 50
-  gtk_css_gadget_get_content_allocation (priv->gadget, &allocation, NULL);
-  gtk_css_gadget_get_preferred_size (priv->handle_gadget,
-                                     priv->orientation,
-                                     -1,
-                                     NULL, &handle_size,
-                                     NULL, NULL);
+  gtk_widget_measure (priv->handle_widget,
+                      priv->orientation,
+                      -1,
+                      NULL, &handle_size,
+                      NULL, NULL);
 
   if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
-      handle_pos = priv->handle_pos.x - allocation.x;
+      handle_pos = priv->handle_pos.x;
       drag_pos = start_x;
     }
   else
     {
-      handle_pos = priv->handle_pos.y - allocation.y;
+      handle_pos = priv->handle_pos.y;
       drag_pos = start_y;
     }
 
@@ -719,7 +755,7 @@ gesture_drag_begin_cb (GtkGestureDrag *gesture,
 {
   GtkPanedPrivate *priv = paned->priv;
   GdkEventSequence *sequence;
-  GtkAllocation allocation;
+  GdkRectangle handle_area;
   const GdkEvent *event;
   GdkDevice *device;
   gboolean is_touch;
@@ -727,12 +763,13 @@ gesture_drag_begin_cb (GtkGestureDrag *gesture,
   sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
   event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
   device = gdk_event_get_source_device (event);
-  gtk_css_gadget_get_content_allocation (priv->gadget, &allocation, NULL);
-  paned->priv->panning = FALSE;
+  priv->panning = FALSE;
 
   is_touch = (event->type == GDK_TOUCH_BEGIN ||
               gdk_device_get_source (device) == GDK_SOURCE_TOUCHSCREEN);
 
+  get_handle_area (paned, &handle_area);
+
   if ((is_touch && GTK_GESTURE (gesture) == priv->drag_gesture) ||
       (!is_touch && GTK_GESTURE (gesture) == priv->pan_gesture))
     {
@@ -741,13 +778,13 @@ gesture_drag_begin_cb (GtkGestureDrag *gesture,
       return;
     }
 
-  if (event->any.window == priv->handle ||
+  if (gdk_rectangle_contains_point (&handle_area, (int)start_x, (int)start_y) ||
       (is_touch && initiates_touch_drag (paned, start_x, start_y)))
     {
       if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
-        priv->drag_pos = start_x - (priv->handle_pos.x - allocation.x);
+        priv->drag_pos = start_x - priv->handle_pos.x;
       else
-        priv->drag_pos = start_y - (priv->handle_pos.y - allocation.y);
+        priv->drag_pos = start_y - priv->handle_pos.y;
 
       gtk_gesture_set_state (GTK_GESTURE (gesture),
                              GTK_EVENT_SEQUENCE_CLAIMED);
@@ -780,8 +817,11 @@ gesture_drag_end_cb (GtkGestureDrag *gesture,
                      gdouble         offset_y,
                      GtkPaned       *paned)
 {
+
   if (!paned->priv->panning)
     gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_DENIED);
+
+  paned->priv->panning = FALSE;
 }
 
 static void
@@ -809,7 +849,6 @@ gtk_paned_set_property (GObject        *object,
                                              GTK_ORIENTATION_VERTICAL);
 
           /* state_flags_changed updates the cursor */
-          gtk_paned_state_flags_changed (GTK_WIDGET (paned), 0);
           gtk_widget_queue_resize (GTK_WIDGET (paned));
           g_object_notify_by_pspec (object, pspec);
         }
@@ -821,7 +860,7 @@ gtk_paned_set_property (GObject        *object,
       if (priv->position_set != g_value_get_boolean (value))
         {
           priv->position_set = g_value_get_boolean (value);
-          gtk_widget_queue_resize_no_redraw (GTK_WIDGET (paned));
+          gtk_widget_queue_resize (GTK_WIDGET (paned));
           g_object_notify_by_pspec (object, pspec);
         }
       break;
@@ -915,7 +954,7 @@ gtk_paned_set_child_property (GtkContainer    *container,
       break;
     }
   if (old_value != new_value)
-    gtk_widget_queue_resize_no_redraw (GTK_WIDGET (container));
+    gtk_widget_queue_resize (GTK_WIDGET (container));
 }
 
 static void
@@ -928,8 +967,10 @@ gtk_paned_get_child_property (GtkContainer *container,
   GtkPaned *paned = GTK_PANED (container);
   GtkPanedPrivate *priv = paned->priv;
 
-  g_assert (child == priv->child1 || child == priv->child2);
-  
+  if (child != priv->child1 &&
+      child != priv->child2)
+    return;
+
   switch (property_id)
     {
     case CHILD_PROP_RESIZE:
@@ -954,6 +995,7 @@ static void
 gtk_paned_finalize (GObject *object)
 {
   GtkPaned *paned = GTK_PANED (object);
+  GtkPanedPrivate *priv = gtk_paned_get_instance_private (paned);
   
   gtk_paned_set_saved_focus (paned, NULL);
   gtk_paned_set_first_paned (paned, NULL);
@@ -961,8 +1003,7 @@ gtk_paned_finalize (GObject *object)
   g_clear_object (&paned->priv->pan_gesture);
   g_clear_object (&paned->priv->drag_gesture);
 
-  g_clear_object (&paned->priv->handle_gadget);
-  g_clear_object (&paned->priv->gadget);
+  gtk_widget_unparent (priv->handle_widget);
 
   G_OBJECT_CLASS (gtk_paned_parent_class)->finalize (object);
 }
@@ -1061,11 +1102,11 @@ gtk_paned_get_preferred_size_for_orientation (GtkWidget      *widget,
     {
       gint handle_size;
 
-      gtk_css_gadget_get_preferred_size (priv->handle_gadget,
-                                         priv->orientation,
-                                         -1,
-                                         NULL, &handle_size,
-                                         NULL, NULL);
+      gtk_widget_measure (priv->handle_widget,
+                          priv->orientation,
+                          -1,
+                          NULL, &handle_size,
+                          NULL, NULL);
 
       *minimum += handle_size;
       *natural += handle_size;
@@ -1090,11 +1131,11 @@ gtk_paned_get_preferred_size_for_opposite_orientation (GtkWidget      *widget,
       gint child1_req, child2_req;
       gint handle_size;
 
-      gtk_css_gadget_get_preferred_size (priv->handle_gadget,
-                                         OPPOSITE_ORIENTATION (priv->orientation),
-                                         -1,
-                                         NULL, &handle_size,
-                                         NULL, NULL);
+      gtk_widget_measure (priv->handle_widget,
+                          OPPOSITE_ORIENTATION (priv->orientation),
+                          -1,
+                          NULL, &handle_size,
+                          NULL, NULL);
 
       gtk_widget_measure (priv->child1, priv->orientation, -1, &child1_req, NULL, NULL, NULL);
       gtk_widget_measure (priv->child2, priv->orientation, -1, &child2_req, NULL, NULL, NULL);
@@ -1139,27 +1180,7 @@ gtk_paned_get_preferred_size_for_opposite_orientation (GtkWidget      *widget,
 }
 
 static void
-gtk_paned_measure (GtkCssGadget   *gadget,
-                   GtkOrientation  orientation,
-                   int             size,
-                   int            *minimum,
-                   int            *natural,
-                   int            *minimum_baseline,
-                   int            *natural_baseline,
-                   gpointer        data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-
-  if (orientation == priv->orientation)
-    gtk_paned_get_preferred_size_for_orientation (widget, size, minimum, natural);
-  else
-    gtk_paned_get_preferred_size_for_opposite_orientation (widget, size, minimum, natural);
-}
-
-static void
-gtk_paned_measure_ (GtkWidget *widget,
+gtk_paned_measure (GtkWidget *widget,
                    GtkOrientation  orientation,
                    int             for_size,
                    int            *minimum,
@@ -1167,11 +1188,13 @@ gtk_paned_measure_ (GtkWidget *widget,
                    int            *minimum_baseline,
                    int            *natural_baseline)
 {
-  gtk_css_gadget_get_preferred_size (GTK_PANED (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
+  GtkPaned *paned = GTK_PANED (widget);
+  GtkPanedPrivate *priv = paned->priv;
+
+  if (orientation == priv->orientation)
+    gtk_paned_get_preferred_size_for_orientation (widget, for_size, minimum, natural);
+  else
+    gtk_paned_get_preferred_size_for_opposite_orientation (widget, for_size, minimum, natural);
 }
 
 static void
@@ -1203,49 +1226,28 @@ gtk_paned_set_child_visible (GtkPaned  *paned,
 }
 
 static void
-gtk_paned_size_allocate (GtkWidget     *widget,
-                         GtkAllocation *allocation)
+gtk_paned_size_allocate (GtkWidget           *widget,
+                         const GtkAllocation *allocation,
+                         int                  baseline,
+                         GtkAllocation       *out_clip)
 {
   GtkPaned *paned = GTK_PANED (widget);
   GtkPanedPrivate *priv = paned->priv;
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_paned_allocate (GtkCssGadget        *gadget,
-                    const GtkAllocation *allocation,
-                    int                  baseline,
-                    GtkAllocation       *out_clip,
-                    gpointer             data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-  GtkAllocation clip = { 0 };
+  GtkAllocation child_clip;
 
   if (priv->child1 && gtk_widget_get_visible (priv->child1) &&
       priv->child2 && gtk_widget_get_visible (priv->child2))
     {
       GtkAllocation child1_allocation;
       GtkAllocation child2_allocation;
-      GtkAllocation priv_child1_allocation;
       GdkRectangle old_handle_pos;
       gint handle_size;
 
-      gtk_css_gadget_get_preferred_size (priv->handle_gadget,
-                                         priv->orientation,
-                                         -1,
-                                         NULL, &handle_size,
-                                         NULL, NULL);
+      gtk_widget_measure (priv->handle_widget,
+                          priv->orientation,
+                          -1,
+                          NULL, &handle_size,
+                          NULL, NULL);
 
       old_handle_pos = priv->handle_pos;
 
@@ -1338,10 +1340,8 @@ gtk_paned_allocate (GtkCssGadget        *gadget,
             child2_allocation.height = child2_height;
         }
 
-      gtk_css_gadget_allocate (priv->handle_gadget,
-                               &priv->handle_pos,
-                               -1,
-                               &clip);
+      gtk_widget_size_allocate (priv->handle_widget, &priv->handle_pos, -1, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
 
       if (gtk_widget_get_mapped (widget) &&
           (old_handle_pos.x != priv->handle_pos.x ||
@@ -1357,46 +1357,17 @@ gtk_paned_allocate (GtkCssGadget        *gadget,
                                       priv->handle_pos.width, priv->handle_pos.height);
         }
 
-      if (gtk_widget_get_realized (widget))
-	{
-          GtkAllocation border_alloc;
 
-	  if (gtk_widget_get_mapped (widget))
-	    gdk_window_show (priv->handle);
+      gtk_widget_size_allocate (priv->child1, &child1_allocation, -1, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
 
-          gtk_css_gadget_get_border_allocation (priv->handle_gadget, &border_alloc, NULL);
-          gdk_window_move_resize (priv->handle,
-                                  border_alloc.x, border_alloc.y,
-                                  border_alloc.width, border_alloc.height);
-	}
-
-      /* Now allocate the childen, making sure, when resizing not to
-       * overlap the windows
-       */
-      gtk_widget_get_allocation (priv->child1, &priv_child1_allocation);
-      if (gtk_widget_get_mapped (widget) &&
-          ((priv->orientation == GTK_ORIENTATION_HORIZONTAL &&
-            priv_child1_allocation.width < child1_allocation.width) ||
-
-           (priv->orientation == GTK_ORIENTATION_VERTICAL &&
-            priv_child1_allocation.height < child1_allocation.height)))
-	{
-          gtk_widget_size_allocate (priv->child2, &child2_allocation);
-          gtk_widget_size_allocate (priv->child1, &child1_allocation);
-	}
-      else
-	{
-          gtk_widget_size_allocate (priv->child1, &child1_allocation);
-          gtk_widget_size_allocate (priv->child2, &child2_allocation);
-	}
+      gtk_widget_size_allocate (priv->child2, &child2_allocation, -1, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
     }
   else
     {
       GtkAllocation child_allocation;
 
-      if (gtk_widget_get_realized (widget))
-	gdk_window_hide (priv->handle);
-
       child_allocation.x = allocation->x;
       child_allocation.y = allocation->y;
       child_allocation.width = allocation->width;
@@ -1407,14 +1378,16 @@ gtk_paned_allocate (GtkCssGadget        *gadget,
           gtk_paned_set_child_visible (paned, CHILD1, TRUE);
           gtk_paned_set_child_visible (paned, CHILD2, FALSE);
 
-          gtk_widget_size_allocate (priv->child1, &child_allocation);
+          gtk_widget_size_allocate (priv->child1, &child_allocation, -1, &child_clip);
+          gdk_rectangle_union (out_clip, &child_clip, out_clip);
         }
       else if (priv->child2 && gtk_widget_get_visible (priv->child2))
         {
           gtk_paned_set_child_visible (paned, CHILD1, FALSE);
           gtk_paned_set_child_visible (paned, CHILD2, TRUE);
 
-          gtk_widget_size_allocate (priv->child2, &child_allocation);
+          gtk_widget_size_allocate (priv->child2, &child_allocation, -1, &child_clip);
+          gdk_rectangle_union (out_clip, &child_clip, out_clip);
         }
       else
         {
@@ -1422,49 +1395,13 @@ gtk_paned_allocate (GtkCssGadget        *gadget,
           gtk_paned_set_child_visible (paned, CHILD2, FALSE);
         }
     }
-
-  gtk_container_get_children_clip (GTK_CONTAINER (paned), out_clip);
-  gdk_rectangle_union (out_clip, &clip, out_clip);
 }
 
-static void
-gtk_paned_realize (GtkWidget *widget)
-{
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-
-  GTK_WIDGET_CLASS (gtk_paned_parent_class)->realize (widget);
-
-  priv->handle = gdk_window_new_input (gtk_widget_get_window (widget),
-                                       GDK_ALL_EVENTS_MASK,
-                                       &priv->handle_pos);
-
-  if (gtk_widget_is_sensitive (widget))
-    {
-      GdkCursor *cursor;
-
-      cursor = gdk_cursor_new_from_name (gtk_widget_get_display (widget),
-                                         priv->orientation == GTK_ORIENTATION_HORIZONTAL
-                                         ? "col-resize" : "row-resize");
-      gdk_window_set_cursor (priv->handle, cursor);
-      g_object_unref (cursor);
-    }
-
-  gtk_widget_register_window (widget, priv->handle);
-}
 
 static void
 gtk_paned_unrealize (GtkWidget *widget)
 {
   GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-
-  if (priv->handle)
-    {
-      gtk_widget_unregister_window (widget, priv->handle);
-      gdk_window_destroy (priv->handle);
-      priv->handle = NULL;
-    }
 
   gtk_paned_set_last_child1_focus (paned, NULL);
   gtk_paned_set_last_child2_focus (paned, NULL);
@@ -1474,71 +1411,33 @@ gtk_paned_unrealize (GtkWidget *widget)
   GTK_WIDGET_CLASS (gtk_paned_parent_class)->unrealize (widget);
 }
 
-static void
-gtk_paned_map (GtkWidget *widget)
-{
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-
-  if (priv->child1 && gtk_widget_get_visible (priv->child1) &&
-      priv->child2 && gtk_widget_get_visible (priv->child2))
-    gdk_window_show (priv->handle);
-
-  GTK_WIDGET_CLASS (gtk_paned_parent_class)->map (widget);
-}
-
-static void
-gtk_paned_unmap (GtkWidget *widget)
-{
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-
-  gdk_window_hide (priv->handle);
-
-  GTK_WIDGET_CLASS (gtk_paned_parent_class)->unmap (widget);
-}
-
 static void
 gtk_paned_snapshot (GtkWidget   *widget,
                     GtkSnapshot *snapshot)
 {
-  gtk_css_gadget_snapshot (GTK_PANED (widget)->priv->gadget, snapshot);
-}
+  GtkPanedPrivate *priv = gtk_paned_get_instance_private (GTK_PANED (widget));
+  GtkAllocation child_allocation;
+  int width, height;
 
-static gboolean
-gtk_paned_render (GtkCssGadget *gadget,
-                  GtkSnapshot  *snapshot,
-                  int           x,
-                  int           y,
-                  int           width,
-                  int           height,
-                  gpointer      data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-  GtkAllocation widget_allocation, child_allocation;
-
-  gtk_widget_get_allocation (widget, &widget_allocation);
+  gtk_widget_get_content_size (widget, &width, &height);
   gtk_snapshot_push_clip (snapshot,
                           &GRAPHENE_RECT_INIT (
                               0, 0,
-                              widget_allocation.width,
-                              widget_allocation.height
+                              width, height
                           ),
                           "GtkPaned");
 
   if (priv->child1 && gtk_widget_get_visible (priv->child1) &&
       priv->child2 && gtk_widget_get_visible (priv->child2))
-    gtk_css_gadget_snapshot (priv->handle_gadget, snapshot);
+    gtk_widget_snapshot_child (widget, priv->handle_widget, snapshot);
 
   if (priv->child1 && gtk_widget_get_visible (priv->child1))
     {
-      gtk_widget_get_allocation (priv->child1, &child_allocation);
+      gtk_widget_get_outer_allocation (priv->child1, &child_allocation);
       gtk_snapshot_push_clip (snapshot,
                               &GRAPHENE_RECT_INIT (
-                                  child_allocation.x - widget_allocation.x,
-                                  child_allocation.y - widget_allocation.y,
+                                  child_allocation.x,
+                                  child_allocation.y,
                                   child_allocation.width,
                                   child_allocation.height
                               ),
@@ -1549,11 +1448,11 @@ gtk_paned_render (GtkCssGadget *gadget,
 
   if (priv->child2 && gtk_widget_get_visible (priv->child2))
     {
-      gtk_widget_get_allocation (priv->child2, &child_allocation);
+      gtk_widget_get_outer_allocation (priv->child2, &child_allocation);
       gtk_snapshot_push_clip (snapshot,
                               &GRAPHENE_RECT_INIT (
-                                  child_allocation.x - widget_allocation.x,
-                                  child_allocation.y - widget_allocation.y,
+                                  child_allocation.x,
+                                  child_allocation.y,
                                   child_allocation.width,
                                   child_allocation.height
                               ),
@@ -1563,55 +1462,26 @@ gtk_paned_render (GtkCssGadget *gadget,
     }
 
   gtk_snapshot_pop (snapshot);
-
-  return FALSE;
 }
 
 static gboolean
-gtk_paned_render_handle (GtkCssGadget *gadget,
-                         GtkSnapshot  *snapshot,
-                         int           x,
-                         int           y,
-                         int           width,
-                         int           height,
-                         gpointer      data)
+gtk_paned_render_handle (GtkGizmo    *gizmo,
+                         GtkSnapshot *snapshot)
 {
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-  GtkStyleContext *context = gtk_widget_get_style_context (widget);
+  GtkWidget *widget = GTK_WIDGET (gizmo);
+  GtkCssStyle *style = gtk_css_node_get_style (gtk_widget_get_css_node (widget));
+  int width, height;
 
-  gtk_style_context_save_to_node (context, gtk_css_gadget_get_node (priv->handle_gadget));
-  gtk_snapshot_offset (snapshot, x, y);
+  gtk_widget_get_content_size (widget, &width, &height);
 
-  gtk_css_style_snapshot_icon (gtk_style_context_lookup_style (context),
+  gtk_css_style_snapshot_icon (style,
                                snapshot,
                                width, height,
                                GTK_CSS_IMAGE_BUILTIN_PANE_SEPARATOR);
 
-  gtk_snapshot_offset (snapshot, -x, -y);
-  gtk_style_context_restore (context);
-
   return FALSE;
 }
 
-static void
-update_node_state (GtkWidget *widget)
-{
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-  GtkStateFlags state;
-
-  state = gtk_widget_get_state_flags (widget);
-
-  if (gtk_widget_is_focus (widget))
-    state |= GTK_STATE_FLAG_SELECTED;
-  if (priv->handle_prelit)
-    state |= GTK_STATE_FLAG_PRELIGHT;
-
-  gtk_css_node_set_state (gtk_css_gadget_get_node (priv->handle_gadget), state);
-}
-
 static void
 connect_drag_gesture_signals (GtkPaned   *paned,
                               GtkGesture *gesture)
@@ -1629,16 +1499,10 @@ gtk_paned_init (GtkPaned *paned)
 {
   GtkPanedPrivate *priv;
   GtkGesture *gesture;
-  GtkCssNode *widget_node;
 
   gtk_widget_set_has_window (GTK_WIDGET (paned), FALSE);
   gtk_widget_set_can_focus (GTK_WIDGET (paned), TRUE);
 
-  /* We only need to redraw when the handle position moves, which is
-   * independent of the overall allocation of the GtkPaned
-   */
-  gtk_widget_set_redraw_on_allocate (GTK_WIDGET (paned), FALSE);
-
   paned->priv = gtk_paned_get_instance_private (paned);
   priv = paned->priv;
 
@@ -1646,7 +1510,6 @@ gtk_paned_init (GtkPaned *paned)
 
   priv->child1 = NULL;
   priv->child2 = NULL;
-  priv->handle = NULL;
 
   priv->handle_pos.width = 5;
   priv->handle_pos.height = 5;
@@ -1656,7 +1519,6 @@ gtk_paned_init (GtkPaned *paned)
   priv->last_child1_focus = NULL;
   priv->last_child2_focus = NULL;
   priv->in_recursion = FALSE;
-  priv->handle_prelit = FALSE;
   priv->original_position = -1;
   priv->max_position = G_MAXINT;
 
@@ -1676,27 +1538,16 @@ gtk_paned_init (GtkPaned *paned)
 
   /* Pointer gesture */
   gesture = gtk_gesture_drag_new (GTK_WIDGET (paned));
+  gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (gesture),
+                                              GTK_PHASE_CAPTURE);
   connect_drag_gesture_signals (paned, gesture);
   priv->drag_gesture = gesture;
 
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (paned));
-  priv->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                     GTK_WIDGET (paned),
-                                                     gtk_paned_measure,
-                                                     gtk_paned_allocate,
-                                                     gtk_paned_render,
-                                                     NULL,
-                                                     NULL);
-  priv->handle_gadget = gtk_css_custom_gadget_new ("separator",
-                                                   GTK_WIDGET (paned),
-                                                   priv->gadget,
-                                                   NULL,
-                                                   NULL,
-                                                   NULL,
-                                                   gtk_paned_render_handle,
-                                                   NULL,
-                                                   NULL);
-  update_node_state (GTK_WIDGET (paned));
+  priv->handle_widget = gtk_gizmo_new ("separator",
+                                       NULL,
+                                       NULL,
+                                       gtk_paned_render_handle);
+  gtk_widget_set_parent (priv->handle_widget, GTK_WIDGET (paned));
 }
 
 static gboolean
@@ -1723,10 +1574,8 @@ update_drag (GtkPaned *paned,
   gint pos;
   gint handle_size;
   gint size;
-  gint x, y;
 
-  gdk_window_get_position (priv->handle, &x, &y);
-  gtk_css_gadget_get_content_allocation (priv->gadget, &allocation, NULL);
+  gtk_widget_get_content_allocation (GTK_WIDGET (paned), &allocation);
   if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     pos = xpos;
   else
@@ -1736,11 +1585,11 @@ update_drag (GtkPaned *paned,
 
   if (is_rtl (paned))
     {
-      gtk_css_gadget_get_preferred_size (priv->handle_gadget,
-                                         GTK_ORIENTATION_HORIZONTAL,
-                                         -1,
-                                         NULL, &handle_size,
-                                         NULL, NULL);
+      gtk_widget_measure (priv->handle_widget,
+                          GTK_ORIENTATION_HORIZONTAL,
+                          -1,
+                          NULL, &handle_size,
+                          NULL, NULL);
 
       size = allocation.width - pos - handle_size;
     }
@@ -1755,48 +1604,6 @@ update_drag (GtkPaned *paned,
     gtk_paned_set_position (paned, size);
 }
 
-static gboolean
-gtk_paned_enter (GtkWidget        *widget,
-		 GdkEventCrossing *event)
-{
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-
-  if (!gtk_gesture_is_active (priv->pan_gesture))
-    {
-      priv->handle_prelit = TRUE;
-      update_node_state (widget);
-      gtk_widget_queue_draw_area (widget,
-				  priv->handle_pos.x,
-				  priv->handle_pos.y,
-				  priv->handle_pos.width,
-				  priv->handle_pos.height);
-    }
-  
-  return TRUE;
-}
-
-static gboolean
-gtk_paned_leave (GtkWidget        *widget,
-		 GdkEventCrossing *event)
-{
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-
-  if (!gtk_gesture_is_active (priv->pan_gesture))
-    {
-      priv->handle_prelit = FALSE;
-      update_node_state (widget);
-      gtk_widget_queue_draw_area (widget,
-				  priv->handle_pos.x,
-				  priv->handle_pos.y,
-				  priv->handle_pos.width,
-				  priv->handle_pos.height);
-    }
-
-  return TRUE;
-}
-
 static gboolean
 gtk_paned_focus (GtkWidget        *widget,
 		 GtkDirectionType  direction)
@@ -1815,34 +1622,6 @@ gtk_paned_focus (GtkWidget        *widget,
   return retval;
 }
 
-static void
-gtk_paned_state_flags_changed (GtkWidget     *widget,
-                               GtkStateFlags  previous_state)
-{
-  GtkPaned *paned = GTK_PANED (widget);
-  GtkPanedPrivate *priv = paned->priv;
-  GdkCursor *cursor;
-
-  if (gtk_widget_get_realized (widget))
-    {
-      if (gtk_widget_is_sensitive (widget))
-        cursor = gdk_cursor_new_from_name (gtk_widget_get_display (widget),
-                                           priv->orientation == GTK_ORIENTATION_HORIZONTAL
-                                           ? "col-resize" : "row-resize");
-      else
-        cursor = NULL;
-
-      gdk_window_set_cursor (priv->handle, cursor);
-
-      if (cursor)
-        g_object_unref (cursor);
-    }
-
-  update_node_state (widget);
-
-  GTK_WIDGET_CLASS (gtk_paned_parent_class)->state_flags_changed (widget, previous_state);
-}
-
 static void
 gtk_paned_direction_changed (GtkWidget        *widget,
                              GtkTextDirection  previous_direction)
@@ -1919,8 +1698,6 @@ gtk_paned_pack1 (GtkPaned  *paned,
 		 gboolean   shrink)
 {
   GtkPanedPrivate *priv;
-  GtkCssNode *widget_node;
-  GtkCssNode *child_node;
 
   g_return_if_fail (GTK_IS_PANED (paned));
   g_return_if_fail (GTK_IS_WIDGET (child));
@@ -1933,14 +1710,10 @@ gtk_paned_pack1 (GtkPaned  *paned,
       priv->child1_resize = resize;
       priv->child1_shrink = shrink;
 
-      widget_node = gtk_widget_get_css_node (GTK_WIDGET (paned));
-      child_node = gtk_widget_get_css_node (child);
       if (gtk_widget_get_direction (GTK_WIDGET (paned)) == GTK_TEXT_DIR_RTL)
-        gtk_css_node_insert_after (widget_node, child_node, gtk_css_gadget_get_node (priv->handle_gadget));
+        gtk_widget_insert_after (child, GTK_WIDGET (paned), priv->handle_widget);
       else
-        gtk_css_node_insert_before (widget_node, child_node, gtk_css_gadget_get_node (priv->handle_gadget));
-
-      gtk_widget_set_parent (child, GTK_WIDGET (paned));
+        gtk_widget_insert_before (child, GTK_WIDGET (paned), priv->handle_widget);
     }
 }
 
@@ -1960,8 +1733,6 @@ gtk_paned_pack2 (GtkPaned  *paned,
 		 gboolean   shrink)
 {
   GtkPanedPrivate *priv;
-  GtkCssNode *widget_node;
-  GtkCssNode *child_node;
 
   g_return_if_fail (GTK_IS_PANED (paned));
   g_return_if_fail (GTK_IS_WIDGET (child));
@@ -1974,14 +1745,10 @@ gtk_paned_pack2 (GtkPaned  *paned,
       priv->child2_resize = resize;
       priv->child2_shrink = shrink;
 
-      widget_node = gtk_widget_get_css_node (GTK_WIDGET (paned));
-      child_node = gtk_widget_get_css_node (child);
       if (gtk_widget_get_direction (GTK_WIDGET (paned)) == GTK_TEXT_DIR_RTL)
-        gtk_css_node_insert_before (widget_node, child_node, gtk_css_gadget_get_node (priv->handle_gadget));
+        gtk_widget_insert_before (child, GTK_WIDGET (paned), priv->handle_widget);
       else
-        gtk_css_node_insert_after (widget_node, child_node, gtk_css_gadget_get_node (priv->handle_gadget));
-
-      gtk_widget_set_parent (child, GTK_WIDGET (paned));
+        gtk_widget_insert_after (child, GTK_WIDGET (paned), priv->handle_widget);
     }
 }
 
@@ -2012,27 +1779,18 @@ gtk_paned_remove (GtkContainer *container,
 {
   GtkPaned *paned = GTK_PANED (container);
   GtkPanedPrivate *priv = paned->priv;
-  gboolean was_visible;
-
-  was_visible = gtk_widget_get_visible (widget);
 
   if (priv->child1 == widget)
     {
       gtk_widget_unparent (widget);
 
       priv->child1 = NULL;
-
-      if (was_visible && gtk_widget_get_visible (GTK_WIDGET (container)))
-	gtk_widget_queue_resize_no_redraw (GTK_WIDGET (container));
     }
   else if (priv->child2 == widget)
     {
       gtk_widget_unparent (widget);
 
       priv->child2 = NULL;
-
-      if (was_visible && gtk_widget_get_visible (GTK_WIDGET (container)))
-	gtk_widget_queue_resize_no_redraw (GTK_WIDGET (container));
     }
 }
 
@@ -2106,7 +1864,7 @@ gtk_paned_set_position (GtkPaned *paned,
       if (priv->child1_size != position)
         {
           g_object_notify (G_OBJECT (paned), "position");
-          gtk_widget_queue_resize_no_redraw (GTK_WIDGET (paned));
+          gtk_widget_queue_allocate (GTK_WIDGET (paned));
         }
 
       priv->child1_size = position;
@@ -2811,27 +2569,6 @@ gtk_paned_toggle_handle_focus (GtkPaned *paned)
   return FALSE;
 }
 
-/**
- * gtk_paned_get_handle_window:
- * @paned: a #GtkPaned
- *
- * Returns the #GdkWindow of the handle. This function is
- * useful when handling button or motion events because it
- * enables the callback to distinguish between the window
- * of the paned, a child and the handle.
- *
- * Returns: (transfer none): the paned’s handle window.
- *
- * Since: 2.20
- **/
-GdkWindow *
-gtk_paned_get_handle_window (GtkPaned *paned)
-{
-  g_return_val_if_fail (GTK_IS_PANED (paned), NULL);
-
-  return paned->priv->handle;
-}
-
 /**
  * gtk_paned_set_wide_handle:
  * @paned: a #GtkPaned
@@ -2845,6 +2582,7 @@ void
 gtk_paned_set_wide_handle (GtkPaned *paned,
                            gboolean  wide)
 {
+  GtkPanedPrivate *priv = gtk_paned_get_instance_private (paned);
   gboolean old_wide;
 
   g_return_if_fail (GTK_IS_PANED (paned));
@@ -2853,11 +2591,12 @@ gtk_paned_set_wide_handle (GtkPaned *paned,
   if (old_wide != wide)
     {
       if (wide)
-        gtk_css_gadget_add_class (paned->priv->handle_gadget, GTK_STYLE_CLASS_WIDE);
+        gtk_style_context_add_class (gtk_widget_get_style_context (priv->handle_widget),
+                                     GTK_STYLE_CLASS_WIDE);
       else
-        gtk_css_gadget_remove_class (paned->priv->handle_gadget, GTK_STYLE_CLASS_WIDE);
+        gtk_style_context_remove_class (gtk_widget_get_style_context (priv->handle_widget),
+                                        GTK_STYLE_CLASS_WIDE);
 
-      gtk_widget_queue_resize (GTK_WIDGET (paned));
       g_object_notify (G_OBJECT (paned), "wide-handle");
     }
 }
@@ -2875,10 +2614,10 @@ gtk_paned_set_wide_handle (GtkPaned *paned,
 gboolean
 gtk_paned_get_wide_handle (GtkPaned *paned)
 {
+  GtkPanedPrivate *priv = gtk_paned_get_instance_private (paned);
+
   g_return_val_if_fail (GTK_IS_PANED (paned), FALSE);
 
-  if (gtk_css_node_has_class (gtk_css_gadget_get_node (paned->priv->handle_gadget), g_quark_from_static_string (GTK_STYLE_CLASS_WIDE)))
-    return TRUE;
-  else
-    return FALSE;
+  return gtk_style_context_has_class (gtk_widget_get_style_context (priv->handle_widget),
+                                      GTK_STYLE_CLASS_WIDE);
 }
