@@ -28,11 +28,11 @@
  */
 #ifdef GTK_COMPILATION
 #include "gtkbutton.h"
-#include "gtkeventbox.h"
 #include "gtkimage.h"
 #include "gtkintl.h"
 #include "gtklabel.h"
 #include "gtkspinner.h"
+#include "gtkstack.h"
 #include "gtktypebuiltins.h"
 #else
 #include <gtk/gtk.h>
@@ -43,10 +43,10 @@ struct _GtkPlacesViewRow
   GtkListBoxRow  parent_instance;
 
   GtkLabel      *available_space_label;
+  GtkStack      *mount_stack;
   GtkSpinner    *busy_spinner;
   GtkButton     *eject_button;
   GtkImage      *eject_icon;
-  GtkEventBox   *event_box;
   GtkImage      *icon_image;
   GtkLabel      *name_label;
   GtkLabel      *path_label;
@@ -285,15 +285,15 @@ gtk_places_view_row_set_property (GObject      *object,
 
     case PROP_MOUNT:
       g_set_object (&self->mount, g_value_get_object (value));
-
-      /*
-       * When we hide the eject button, no size is allocated for it. Since
-       * we want to have alignment between rows, it needs an empty space
-       * when the eject button is not available. So, call then
-       * gtk_widget_set_child_visible(), which makes the button allocate the
-       * size but it stays hidden when needed.
-       */
-      gtk_widget_set_child_visible (GTK_WIDGET (self->eject_button), self->mount != NULL);
+      if (self->mount != NULL)
+        {
+          gtk_stack_set_visible_child (self->mount_stack, GTK_WIDGET (self->eject_button));
+          gtk_widget_set_child_visible (GTK_WIDGET (self->mount_stack), TRUE);
+        }
+      else
+        {
+          gtk_widget_set_child_visible (GTK_WIDGET (self->mount_stack), FALSE);
+        }
       measure_available_space (self);
       break;
 
@@ -376,10 +376,10 @@ gtk_places_view_row_class_init (GtkPlacesViewRowClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gtk/libgtk/ui/gtkplacesviewrow.ui");
 
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, available_space_label);
+  gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, mount_stack);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, busy_spinner);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, eject_button);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, eject_icon);
-  gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, event_box);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, icon_image);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, name_label);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesViewRow, path_label);
@@ -433,21 +433,21 @@ gtk_places_view_row_get_eject_button (GtkPlacesViewRow *row)
   return GTK_WIDGET (row->eject_button);
 }
 
-GtkWidget*
-gtk_places_view_row_get_event_box (GtkPlacesViewRow *row)
-{
-  g_return_val_if_fail (GTK_IS_PLACES_VIEW_ROW (row), NULL);
-
-  return GTK_WIDGET (row->event_box);
-}
-
 void
 gtk_places_view_row_set_busy (GtkPlacesViewRow *row,
                               gboolean          is_busy)
 {
   g_return_if_fail (GTK_IS_PLACES_VIEW_ROW (row));
 
-  gtk_widget_set_visible (GTK_WIDGET (row->busy_spinner), is_busy);
+  if (is_busy)
+    {
+      gtk_stack_set_visible_child (row->mount_stack, GTK_WIDGET (row->busy_spinner));
+      gtk_widget_set_child_visible (GTK_WIDGET (row->mount_stack), TRUE);
+    }
+  else
+    {
+      gtk_widget_set_child_visible (GTK_WIDGET (row->mount_stack), FALSE);
+    }
 }
 
 gboolean
