@@ -73,15 +73,33 @@
  *
  * # CSS nodes
  *
- * GtkPopover has a single css node called popover. It always gets the
- * .background style class and it gets the .menu style class if it is
- * menu-like (e.g. #GtkPopoverMenu or created using gtk_popover_new_from_model().
+ * |[<!-- language="plain" -->
+ * popover
+ * ├── arrow
+ * ╰── contents.background[.menu]
+ *     ╰── <child>
+ * ]|
+ *
+ * The contents child node always gets the .background style class and it
+ * gets the .menu style class if the popover is menu-like (e.g. #GtkPopoverMenu
+ * or created using gtk_popover_new_from_model().
  *
  * Particular uses of GtkPopover, such as touch selection popups
  * or magnifiers in #GtkEntry or #GtkTextView get style classes
  * like .touch-selection or .magnifier to differentiate from
  * plain popovers.
  *
+ * When styling a popover directly, the popover node should usually not have any
+ * background.
+ *
+ * Note that, in order to accomplish appropriate arrow visuals, #GtkPopover uses
+ * custom drawing for the arrow node. This makes it possible for the arrow to change
+ * its shape dynamically, but it also limits the possibilities of styling it using CSS.
+ * In particular, the arrow gets drawn over the content node's border so they look
+ * like one shape, which means that the border-width of the content node and the arrow
+ * node should be the same. The arrow also does not support any border shape other than
+ * solid, no border-radius, only one border width (border-bottom-width is used) and no box-shadow.
+ *
  * Since: 3.12
  */
 
@@ -113,6 +131,8 @@
 #include "gtkstylecontextprivate.h"
 #include "gtkprogresstrackerprivate.h"
 #include "gtksettingsprivate.h"
+#include "gtkgizmoprivate.h"
+#include "gtkcssnodeprivate.h"
 
 #include "gsk/gskroundedrectprivate.h"
 
@@ -151,6 +171,8 @@ enum {
 struct _GtkPopoverPrivate
 {
   GtkWidget *widget;
+  GtkWidget *contents_widget;
+  GtkCssNode *arrow_node;
   GtkWindow *window;
   GtkWidget *prev_focus_widget;
   GtkWidget *default_widget;
@@ -193,7 +215,6 @@ static void gtk_popover_update_relative_to (GtkPopover *popover,
                                             GtkWidget  *relative_to);
 static void gtk_popover_set_state          (GtkPopover *popover,
                                             guint       state);
-static void gtk_popover_invalidate_borders (GtkPopover *popover);
 static void gtk_popover_apply_modality     (GtkPopover *popover,
                                             gboolean    modal);
 
@@ -202,6 +223,49 @@ static void gtk_popover_set_scrollable_full (GtkPopover    *popover,
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkPopover, gtk_popover, GTK_TYPE_BIN)
 
+
+static void
+measure_contents (GtkGizmo       *gizmo,
+                  GtkOrientation  orientation,
+                  int             for_size,
+                  int            *minimum,
+                  int            *natural,
+                  int            *minimum_baseline,
+                  int            *natural_baseline)
+{
+  GtkPopover *popover = GTK_POPOVER (gtk_widget_get_parent (GTK_WIDGET (gizmo)));
+  GtkWidget *child = gtk_bin_get_child (GTK_BIN (popover));
+
+  if (child)
+    gtk_widget_measure (child, orientation, for_size,
+                        minimum, natural,
+                        minimum_baseline, natural_baseline);
+}
+
+static void
+allocate_contents (GtkGizmo            *gizmo,
+                   const GtkAllocation *allocation,
+                   int                  baseline,
+                   GtkAllocation       *out_clip)
+{
+  GtkPopover *popover = GTK_POPOVER (gtk_widget_get_parent (GTK_WIDGET (gizmo)));
+  GtkWidget *child = gtk_bin_get_child (GTK_BIN (popover));
+
+  if (child)
+    gtk_widget_size_allocate (child, allocation, -1, out_clip);
+}
+
+static void
+node_style_changed_cb (GtkCssNode        *node,
+                       GtkCssStyleChange *change,
+                       GtkWidget         *widget)
+{
+  if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_SIZE | GTK_CSS_AFFECTS_CLIP))
+    gtk_widget_queue_resize (widget);
+  else
+    gtk_widget_queue_draw (widget);
+}
+
 static void
 gtk_popover_init (GtkPopover *popover)
 {
@@ -218,7 +282,23 @@ gtk_popover_init (GtkPopover *popover)
   popover->priv->preferred_position = GTK_POS_TOP;
   popover->priv->constraint = GTK_POPOVER_CONSTRAINT_WINDOW;
 
-  context = gtk_widget_get_style_context (GTK_WIDGET (popover));
+  popover->priv->arrow_node = gtk_css_node_new ();
+  gtk_css_node_set_name (popover->priv->arrow_node, I_("arrow"));
+  gtk_css_node_set_parent (popover->priv->arrow_node, gtk_widget_get_css_node (widget));
+  gtk_css_node_set_state (popover->priv->arrow_node,
+                          gtk_css_node_get_state (gtk_widget_get_css_node (widget)));
+  g_signal_connect_object (popover->priv->arrow_node, "style-changed",
+                           G_CALLBACK (node_style_changed_cb), popover, 0);
+  g_object_unref (popover->priv->arrow_node);
+
+  popover->priv->contents_widget = gtk_gizmo_new ("contents",
+                                                  measure_contents,
+                                                  allocate_contents,
+                                                  NULL);
+
+  gtk_widget_set_parent (popover->priv->contents_widget, widget);
+
+  context = gtk_widget_get_style_context (popover->priv->contents_widget);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_BACKGROUND);
 }
 
@@ -309,7 +389,7 @@ gtk_popover_hide_internal (GtkPopover *popover)
   if (gtk_widget_get_realized (widget))
     {
       cairo_region_t *region = cairo_region_create ();
-      gdk_window_input_shape_combine_region (gtk_widget_get_parent_window (widget),
+      gdk_window_input_shape_combine_region (gtk_widget_get_window (widget),
                                              region, 0, 0);
       cairo_region_destroy (region);
     }
@@ -369,6 +449,21 @@ gtk_popover_dispose (GObject *object)
 
   g_clear_object (&priv->default_widget);
 
+  if (priv->contents_widget)
+    {
+      GtkWidget *child = gtk_bin_get_child (GTK_BIN (popover));
+
+      if (child)
+        {
+          /* Parent is content_widget! */
+          gtk_widget_unparent (child);
+          _gtk_bin_set_child (GTK_BIN (popover), NULL);
+        }
+
+      gtk_widget_unparent (priv->contents_widget);
+      priv->contents_widget = NULL;
+    }
+
   G_OBJECT_CLASS (gtk_popover_parent_class)->dispose (object);
 }
 
@@ -378,11 +473,30 @@ gtk_popover_realize (GtkWidget *widget)
   GtkAllocation allocation;
   GdkWindow *window;
 
-  gtk_widget_get_allocation (widget, &allocation);
+  gtk_widget_get_window_allocation (widget, &allocation);
+
+#ifdef GDK_WINDOWING_WAYLAND
+  if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)))
+    {
+      GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
+
+      g_assert (GTK_IS_WINDOW (toplevel));
+
+      window = gdk_wayland_window_new_subsurface (gtk_widget_get_display (toplevel),
+                                                  GDK_ALL_EVENTS_MASK,
+                                                  &allocation);
+
+      gdk_window_set_transient_for (window,
+                                    gtk_widget_get_window (toplevel));
+    }
+  else
+#endif
+    {
+      window = gdk_window_new_child (gtk_widget_get_parent_window (widget),
+                                     GDK_ALL_EVENTS_MASK,
+                                     &allocation);
+    }
 
-  window = gdk_window_new_child (gtk_widget_get_parent_window (widget),
-                                 GDK_ALL_EVENTS_MASK,
-                                 &(GdkRectangle) { 0, 0, allocation.width, allocation.height });
   gtk_widget_set_window (widget, window);
   gtk_widget_register_window (widget, window);
   gtk_widget_set_realized (widget, TRUE);
@@ -686,8 +800,7 @@ gtk_popover_get_gap_coords (GtkPopover      *popover,
                             gint            *tip_x_out,
                             gint            *tip_y_out,
                             gint            *final_x_out,
-                            gint            *final_y_out,
-                            GtkPositionType *gap_side_out)
+                            gint            *final_y_out)
 {
   GtkWidget *widget = GTK_WIDGET (popover);
   GtkPopoverPrivate *priv = popover->priv;
@@ -696,14 +809,14 @@ gtk_popover_get_gap_coords (GtkPopover      *popover,
   gint initial_x, initial_y;
   gint tip_x, tip_y;
   gint final_x, final_y;
-  GtkPositionType gap_side, pos;
-  GtkAllocation allocation;
+  GtkPositionType pos;
   gint border_radius;
   GtkStyleContext *context;
-  GtkBorder margin, border, widget_margin;
+  GtkBorder border;
+  int popover_width, popover_height;
 
   gtk_popover_get_pointing_to (popover, &rect);
-  gtk_widget_get_allocation (widget, &allocation);
+  gtk_widget_get_content_size (widget, &popover_width, &popover_height);
 
 #ifdef GDK_WINDOWING_WAYLAND
   if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)))
@@ -722,23 +835,10 @@ gtk_popover_get_gap_coords (GtkPopover      *popover,
     gtk_widget_translate_coordinates (priv->widget, widget,
                                       rect.x, rect.y, &rect.x, &rect.y);
 
-  get_margin (widget, &margin);
+  context = gtk_widget_get_style_context (priv->contents_widget);
+  gtk_style_context_get_border (context, &border);
 
-  if (_gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
-    {
-      widget_margin.left = gtk_widget_get_margin_start (widget);
-      widget_margin.right = gtk_widget_get_margin_end (widget);
-    }
-  else
-    {
-      widget_margin.left = gtk_widget_get_margin_end (widget);
-      widget_margin.right = gtk_widget_get_margin_start (widget);
-    }
-
-  widget_margin.top = gtk_widget_get_margin_top (widget);
-  widget_margin.bottom = gtk_widget_get_margin_bottom (widget);
-
-  context = gtk_widget_get_style_context (widget);
+  pos = get_effective_position (popover, priv->final_position);
 
   gtk_style_context_get_border (context, &border);
   gtk_style_context_get (context,
@@ -748,76 +848,64 @@ gtk_popover_get_gap_coords (GtkPopover      *popover,
 
   if (pos == GTK_POS_BOTTOM || pos == GTK_POS_RIGHT)
     {
-      tip = ((pos == GTK_POS_BOTTOM) ? border.top + widget_margin.top : border.left + widget_margin.left);
-      base = tip + TAIL_HEIGHT;
-      gap_side = (priv->final_position == GTK_POS_BOTTOM) ? GTK_POS_TOP : GTK_POS_LEFT;
+      tip = 0;
+      base = tip + TAIL_HEIGHT + border.top;
     }
   else if (pos == GTK_POS_TOP)
     {
-      base = allocation.height - TAIL_HEIGHT - border.bottom - widget_margin.bottom;
+      base = popover_height - border.bottom - TAIL_HEIGHT;
       tip = base + TAIL_HEIGHT;
-      gap_side = GTK_POS_BOTTOM;
     }
   else if (pos == GTK_POS_LEFT)
     {
-      base = allocation.width - TAIL_HEIGHT - border.right - widget_margin.right;
+      base = popover_width - border.right - TAIL_HEIGHT;
       tip = base + TAIL_HEIGHT;
-      gap_side = GTK_POS_RIGHT;
     }
   else
     g_assert_not_reached ();
 
   if (POS_IS_VERTICAL (pos))
     {
-      tip_pos = rect.x + (rect.width / 2) + widget_margin.left;
+      tip_pos = rect.x + (rect.width / 2);
       initial_x = CLAMP (tip_pos - TAIL_GAP_WIDTH / 2,
-                         border_radius + margin.left + TAIL_HEIGHT,
-                         allocation.width - TAIL_GAP_WIDTH - margin.right - border_radius - TAIL_HEIGHT);
+                         border_radius,
+                         popover_width - TAIL_GAP_WIDTH - border_radius);
       initial_y = base;
 
-      tip_x = CLAMP (tip_pos, 0, allocation.width);
+      tip_x = CLAMP (tip_pos, 0, popover_width);
       tip_y = tip;
 
       final_x = CLAMP (tip_pos + TAIL_GAP_WIDTH / 2,
-                       border_radius + margin.left + TAIL_GAP_WIDTH + TAIL_HEIGHT,
-                       allocation.width - margin.right - border_radius - TAIL_HEIGHT);
+                       border_radius + TAIL_GAP_WIDTH,
+                       popover_width - border_radius);
       final_y = base;
     }
   else
     {
-      tip_pos = rect.y + (rect.height / 2) + widget_margin.top;
+      tip_pos = rect.y + (rect.height / 2);
 
       initial_x = base;
       initial_y = CLAMP (tip_pos - TAIL_GAP_WIDTH / 2,
-                         border_radius + margin.top + TAIL_HEIGHT,
-                         allocation.height - TAIL_GAP_WIDTH - margin.bottom - border_radius - TAIL_HEIGHT);
+                         border_radius,
+                         popover_height - TAIL_GAP_WIDTH - border_radius);
 
       tip_x = tip;
-      tip_y = CLAMP (tip_pos, 0, allocation.height);
+      tip_y = CLAMP (tip_pos, 0, popover_height);
 
       final_x = base;
       final_y = CLAMP (tip_pos + TAIL_GAP_WIDTH / 2,
-                       border_radius + margin.top + TAIL_GAP_WIDTH + TAIL_HEIGHT,
-                       allocation.height - margin.right - border_radius - TAIL_HEIGHT);
+                       border_radius + TAIL_GAP_WIDTH,
+                       popover_height - border_radius);
     }
 
-  if (initial_x_out)
-    *initial_x_out = initial_x;
-  if (initial_y_out)
-    *initial_y_out = initial_y;
+  *initial_x_out = initial_x;
+  *initial_y_out = initial_y;
 
-  if (tip_x_out)
-    *tip_x_out = tip_x;
-  if (tip_y_out)
-    *tip_y_out = tip_y;
+  *tip_x_out = tip_x;
+  *tip_y_out = tip_y;
 
-  if (final_x_out)
-    *final_x_out = final_x;
-  if (final_y_out)
-    *final_y_out = final_y;
-
-  if (gap_side_out)
-    *gap_side_out = gap_side;
+  *final_x_out = final_x;
+  *final_y_out = final_y;
 }
 
 static void
@@ -875,16 +963,20 @@ gtk_popover_apply_tail_path (GtkPopover *popover,
   gint initial_x, initial_y;
   gint tip_x, tip_y;
   gint final_x, final_y;
+  GtkStyleContext *context;
+  GtkBorder border;
 
   if (!popover->priv->widget)
     return;
 
+  context = gtk_widget_get_style_context (popover->priv->contents_widget);
+  gtk_style_context_get_border (context, &border);
+
   cairo_set_line_width (cr, 1);
   gtk_popover_get_gap_coords (popover,
                               &initial_x, &initial_y,
                               &tip_x, &tip_y,
-                              &final_x, &final_y,
-                              NULL);
+                              &final_x, &final_y);
 
   cairo_move_to (cr, initial_x, initial_y);
   cairo_line_to (cr, tip_x, tip_y);
@@ -951,7 +1043,7 @@ gtk_popover_update_shape (GtkPopover *popover)
   gtk_widget_shape_combine_region (widget, region);
   cairo_region_destroy (region);
 
-  gdk_window_set_child_shapes (gtk_widget_get_parent_window (widget));
+  gdk_window_set_child_shapes (gtk_widget_get_window (widget));
 }
 
 static void
@@ -1085,97 +1177,63 @@ gtk_popover_update_position (GtkPopover *popover)
         gtk_popover_update_shape (popover);
 
       priv->current_position = priv->final_position;
-      gtk_popover_invalidate_borders (popover);
     }
 
   _gtk_popover_update_child_visible (popover);
 }
 
+GtkWidget *
+gtk_popover_get_contents_widget (GtkPopover *popover)
+{
+  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
+
+  return priv->contents_widget;
+}
+
 static void
 gtk_popover_snapshot (GtkWidget   *widget,
                       GtkSnapshot *snapshot)
 {
   GtkPopover *popover = GTK_POPOVER (widget);
+  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
   GtkStyleContext *context;
-  GtkAllocation allocation, clip;
-  GtkWidget *child;
   GtkBorder border;
-  GdkRGBA border_color;
-  int rect_x, rect_y, rect_w, rect_h;
-  gint initial_x, initial_y, final_x, final_y;
-  gint gap_start, gap_end;
-  GtkPositionType gap_side;
   graphene_rect_t bounds;
   cairo_t *cr;
+  int width, height;
 
-  context = gtk_widget_get_style_context (widget);
+  /* Draw the child first so we can draw the arrow (partially) over it */
+  gtk_widget_snapshot_child (widget, priv->contents_widget, snapshot);
 
-  gtk_widget_get_allocation (widget, &allocation);
-  gtk_widget_get_clip (widget, &clip);
-
-  gtk_style_context_get_border (context, &border);
-  gtk_popover_get_rect_coords (popover,
-                               &rect_x, &rect_y,
-                               &rect_w, &rect_h);
+  gtk_widget_get_content_size (widget, &width, &height);
 
   graphene_rect_init (&bounds,
-                      clip.x - allocation.x, clip.y - allocation.y,
-                      clip.width, clip.height);
+                      0, 0,
+                      width, height);
   cr = gtk_snapshot_append_cairo (snapshot,
                                   &bounds,
                                   "Popover");
-  /* Render the rect background */
-  gtk_render_background (context, cr,
-                         rect_x, rect_y,
-                         rect_w, rect_h);
-
-  if (popover->priv->widget)
-    {
-      gtk_popover_get_gap_coords (popover,
-                                  &initial_x, &initial_y,
-                                  NULL, NULL,
-                                  &final_x, &final_y,
-                                  &gap_side);
-
-      if (POS_IS_VERTICAL (gap_side))
-        {
-          gap_start = initial_x - rect_x;
-          gap_end = final_x - rect_x;
-        }
-      else
-        {
-          gap_start = initial_y - rect_y;
-          gap_end = final_y - rect_y;
-        }
-
-      /* Now render the frame, without the gap for the arrow tip */
-      gtk_render_frame_gap (context, cr,
-                            rect_x, rect_y,
-                            rect_w, rect_h,
-                            gap_side,
-                            gap_start, gap_end);
-    }
-  else
-    {
-      gtk_render_frame (context, cr,
-                        rect_x, rect_y,
-                        rect_w, rect_h);
-    }
 
   /* Clip to the arrow shape */
   cairo_save (cr);
 
   gtk_popover_apply_tail_path (popover, cr);
+
   cairo_clip (cr);
 
+  context = gtk_widget_get_style_context (widget);
+  gtk_style_context_save_to_node (context, priv->arrow_node);
+  gtk_style_context_get_border (context, &border);
+
   /* Render the arrow background */
   gtk_render_background (context, cr,
                          0, 0,
-                         allocation.width, allocation.height);
+                         width, height);
 
   /* Render the border of the arrow tip */
   if (border.bottom > 0)
     {
+      GdkRGBA border_color;
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_style_context_get_border_color (context, &border_color);
 G_GNUC_END_IGNORE_DEPRECATIONS
@@ -1187,30 +1245,11 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       cairo_stroke (cr);
     }
 
+  cairo_restore (cr);
   /* We're done */
   cairo_destroy (cr);
 
-  child = gtk_bin_get_child (GTK_BIN (widget));
-
-  if (child)
-    gtk_widget_snapshot_child (widget, child, snapshot);
-}
-
-static void
-get_padding_and_border (GtkWidget *widget,
-                        GtkBorder *border)
-{
-  GtkStyleContext *context;
-  GtkBorder tmp;
-
-  context = gtk_widget_get_style_context (widget);
-
-  gtk_style_context_get_padding (context, border);
-  gtk_style_context_get_border (context, &tmp);
-  border->top += tmp.top;
-  border->right += tmp.right;
-  border->bottom += tmp.bottom;
-  border->left += tmp.left;
+  gtk_style_context_restore (context);
 }
 
 static gint
@@ -1254,141 +1293,74 @@ gtk_popover_measure (GtkWidget      *widget,
                      int            *natural_baseline)
 {
   GtkPopover *popover = GTK_POPOVER (widget);
+  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
   GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
-  GtkBorder border, margin;
-  int minimal_size, extra;
-
-  get_padding_and_border (widget, &border);
-  get_margin (widget, &margin);
+  int minimal_size;
 
   *minimum = 0;
   *natural = 0;
 
   if (child != NULL)
     {
-      int child_size = -1;
-
       if (for_size >= 0)
-        {
-          GdkRectangle child_rect;
+        for_size -= TAIL_HEIGHT;
 
-          if (orientation == GTK_ORIENTATION_HORIZONTAL)
-            {
-              gtk_popover_get_rect_for_size (popover, for_size, 0, &child_rect);
-              child_size = child_rect.width;
-            }
-          else
-            {
-              gtk_popover_get_rect_for_size (popover, 0, for_size, &child_rect);
-              child_size = child_rect.height;
-            }
-        }
-
-      gtk_widget_measure (child, orientation, child_size, minimum, natural, NULL, NULL);
+      gtk_widget_measure (priv->contents_widget, orientation, for_size, minimum, natural, NULL, NULL);
     }
 
   minimal_size = get_minimal_size (popover, orientation);
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    {
-      *minimum = MAX (*minimum, minimal_size) + border.left + border.right;
-      *natural = MAX (*natural, minimal_size) + border.left + border.right;
-      extra = MAX (TAIL_HEIGHT, margin.left) + MAX (TAIL_HEIGHT, margin.right);
-    }
-  else
-    {
-      *minimum = MAX (*minimum, minimal_size) + border.top + border.bottom;
-      *natural = MAX (*natural, minimal_size) + border.top + border.bottom;
-      extra = MAX (TAIL_HEIGHT, margin.bottom) + MAX (TAIL_HEIGHT, margin.top);
+  *minimum = MAX (*minimum, minimal_size);
+  *natural = MAX (*natural, minimal_size);
 
-    }
-
-  *minimum += extra;
-  *natural += extra;
+  *minimum += TAIL_HEIGHT;
+  *natural += TAIL_HEIGHT;
 }
 
 static void
-gtk_popover_invalidate_borders (GtkPopover *popover)
-{
-  GtkAllocation allocation;
-  GtkBorder border;
-
-  gtk_widget_get_allocation (GTK_WIDGET (popover), &allocation);
-  get_padding_and_border (GTK_WIDGET (popover), &border);
-
-  gtk_widget_queue_draw_area (GTK_WIDGET (popover), 0, 0, border.left + TAIL_HEIGHT, allocation.height);
-  gtk_widget_queue_draw_area (GTK_WIDGET (popover), 0, 0, allocation.width, border.top + TAIL_HEIGHT);
-  gtk_widget_queue_draw_area (GTK_WIDGET (popover), 0, allocation.height - border.bottom - TAIL_HEIGHT,
-                              allocation.width, border.bottom + TAIL_HEIGHT);
-  gtk_widget_queue_draw_area (GTK_WIDGET (popover), allocation.width - border.right - TAIL_HEIGHT,
-                              0, border.right + TAIL_HEIGHT, allocation.height);
-}
-
-static void
-gtk_popover_check_invalidate_borders (GtkPopover *popover)
-{
-  GtkPopoverPrivate *priv = popover->priv;
-  GtkPositionType gap_side;
-  gint tip_x, tip_y;
-
-  if (!priv->widget)
-    return;
-
-  gtk_popover_get_gap_coords (popover, NULL, NULL,
-                              &tip_x, &tip_y, NULL, NULL,
-                              &gap_side);
-
-  if (tip_x != priv->tip_x || tip_y != priv->tip_y)
-    {
-      priv->tip_x = tip_x;
-      priv->tip_y = tip_y;
-      gtk_popover_invalidate_borders (popover);
-    }
-}
-
-static void
-gtk_popover_size_allocate (GtkWidget     *widget,
-                           GtkAllocation *allocation)
+gtk_popover_size_allocate (GtkWidget           *widget,
+                           const GtkAllocation *allocation,
+                           int                  baseline,
+                           GtkAllocation       *out_clip)
 {
   GtkPopover *popover = GTK_POPOVER (widget);
-  GtkWidget *child;
-  GtkAllocation child_clip;
-  GtkAllocation clip = *allocation;
+  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
+  GtkAllocation child_alloc = *allocation;
 
-  gtk_widget_set_allocation (widget, allocation);
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child)
+  /* Note that we in measure() we add TAIL_HEIGHT in both directions, regardless
+   * of the popovers position. This is to ensure that we get enough space
+   * even priv->current_position changes between measure and size-allocate.
+   */
+  child_alloc.height -= TAIL_HEIGHT;
+  child_alloc.width -= TAIL_HEIGHT;
+
+  switch (priv->current_position)
     {
-      GtkAllocation child_alloc;
-      int x, y, w, h;
-      GtkBorder border;
-
-      gtk_popover_get_rect_coords (popover, &x, &y, &w, &h);
-      get_padding_and_border (widget, &border);
-
-      child_alloc.x = x + border.left;
-      child_alloc.y = y + border.top;
-      child_alloc.width = w - border.left - border.right;
-      child_alloc.height = h - border.top - border.bottom;
-      gtk_widget_size_allocate (child, &child_alloc);
+    case GTK_POS_TOP:
+      child_alloc.x += TAIL_HEIGHT / 2;
+      break;
+    case GTK_POS_BOTTOM:
+      child_alloc.x += TAIL_HEIGHT / 2;
+      child_alloc.y += TAIL_HEIGHT;
+      break;
+    case GTK_POS_LEFT:
+      child_alloc.y += TAIL_HEIGHT / 2;
+      break;
+    case GTK_POS_RIGHT:
+      child_alloc.x += TAIL_HEIGHT;
+      child_alloc.y += TAIL_HEIGHT / 2;
+      break;
     }
 
+  gtk_widget_size_allocate (priv->contents_widget, &child_alloc, -1, out_clip);
+
   if (gtk_widget_get_realized (widget))
     {
+      GtkAllocation a;
+      gtk_widget_get_window_allocation (widget, &a);
       gdk_window_move_resize (gtk_widget_get_window (widget),
-                              0, 0, allocation->width, allocation->height);
+                              a.x, a.y, a.width, a.height);
       gtk_popover_update_shape (popover);
     }
-
-  if (gtk_widget_is_drawable (widget))
-    gtk_popover_check_invalidate_borders (popover);
-
-  gtk_container_get_children_clip (GTK_CONTAINER (widget), &child_clip);
-  gdk_rectangle_union (&clip, &child_clip, &clip);
-
-  clip.x += allocation->x;
-  clip.y += allocation->y;
-
-  gtk_widget_set_clip (widget, &clip);
 }
 
 static gboolean
@@ -1524,7 +1496,7 @@ gtk_popover_show (GtkWidget *widget)
   priv->state = STATE_SHOWN;
 
   if (gtk_widget_get_realized (widget))
-    gdk_window_input_shape_combine_region (gtk_widget_get_parent_window (widget),
+    gdk_window_input_shape_combine_region (gtk_widget_get_window (widget),
                                            NULL, 0, 0);
 }
 
@@ -1545,11 +1517,48 @@ gtk_popover_hide (GtkWidget *widget)
   GTK_WIDGET_CLASS (gtk_popover_parent_class)->hide (widget);
 }
 
+static void
+gtk_popover_add (GtkContainer *container,
+                 GtkWidget    *child)
+{
+  GtkPopover *popover = GTK_POPOVER (container);
+  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
+
+  _gtk_bin_set_child (GTK_BIN (popover), child);
+
+  gtk_widget_set_parent (child, priv->contents_widget);
+}
+
+static void
+gtk_popover_remove (GtkContainer *container,
+                    GtkWidget    *child)
+{
+  GtkPopover *popover = GTK_POPOVER (container);
+
+  _gtk_bin_set_child (GTK_BIN (popover), NULL);
+  gtk_widget_unparent (child);
+}
+
+static void
+gtk_popover_state_flags_changed (GtkWidget     *widget,
+                                 GtkStateFlags  previous_state)
+{
+  GtkPopover *popover = GTK_POPOVER (widget);
+  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
+  GtkStateFlags state;
+
+  state = gtk_widget_get_state_flags (widget);
+  gtk_css_node_set_state (priv->arrow_node, state);
+
+  GTK_WIDGET_CLASS (gtk_popover_parent_class)->state_flags_changed (widget, previous_state);
+}
+
 static void
 gtk_popover_class_init (GtkPopoverClass *klass)
 {
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+  GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
 
   object_class->set_property = gtk_popover_set_property;
   object_class->get_property = gtk_popover_get_property;
@@ -1569,6 +1578,10 @@ gtk_popover_class_init (GtkPopoverClass *klass)
   widget_class->focus = gtk_popover_focus;
   widget_class->show = gtk_popover_show;
   widget_class->hide = gtk_popover_hide;
+  widget_class->state_flags_changed = gtk_popover_state_flags_changed;
+
+  container_class->add = gtk_popover_add;
+  container_class->remove = gtk_popover_remove;
 
   /**
    * GtkPopover:relative-to:
@@ -1768,9 +1781,11 @@ _gtk_popover_parent_unmap (GtkWidget *widget,
 }
 
 static void
-_gtk_popover_parent_size_allocate (GtkWidget     *widget,
-                                   GtkAllocation *allocation,
-                                   GtkPopover    *popover)
+_gtk_popover_parent_size_allocate (GtkWidget           *widget,
+                                   const GtkAllocation *allocation,
+                                   int                  baseline,
+                                   GtkAllocation       *out_clip,
+                                   GtkPopover          *popover)
 {
   gtk_popover_update_position (popover);
 }
@@ -2124,17 +2139,12 @@ gtk_popover_get_pointing_to (GtkPopover   *popover,
   GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
 
   g_return_val_if_fail (GTK_IS_POPOVER (popover), FALSE);
+  g_return_val_if_fail (rect != NULL, FALSE);
 
-  if (rect)
-    {
-      if (priv->has_pointing_to)
-        *rect = priv->pointing_to;
-      else if (priv->widget)
-        {
-          gtk_widget_get_allocation (priv->widget, rect);
-          rect->x = rect->y = 0;
-        }
-    }
+  if (priv->has_pointing_to)
+    *rect = priv->pointing_to;
+   else if (priv->widget)
+    gtk_widget_get_own_allocation (priv->widget, rect);
 
   return priv->has_pointing_to;
 }
@@ -2291,7 +2301,7 @@ gtk_popover_bind_model (GtkPopover  *popover,
   if (child)
     gtk_widget_destroy (child);
 
-  style_context = gtk_widget_get_style_context (GTK_WIDGET (popover));
+  style_context = gtk_widget_get_style_context (popover->priv->contents_widget);
 
   if (model)
     {
