@@ -117,7 +117,7 @@ gtk_printer_option_widget_class_init (GtkPrinterOptionWidgetClass *class)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkPrinterOptionWidgetClass, changed),
 		  NULL, NULL,
-		  NULL,
+		  g_cclosure_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
 
   g_object_class_install_property (object_class,
@@ -573,9 +573,7 @@ filesave_choose_cb (GtkWidget              *button,
   g_signal_connect (dialog, "response",
                     G_CALLBACK (dialog_response_callback), widget);
   gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_window_present (GTK_WINDOW (dialog));
-  G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 static gchar *
@@ -739,10 +737,10 @@ alternative_append (GtkWidget              *box,
   button = gtk_radio_button_new_with_label (group, label);
   gtk_widget_show (button);
   gtk_widget_set_valign (button, GTK_ALIGN_BASELINE);
-  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box), button);
 
   g_object_set_data (G_OBJECT (button), "value", (gpointer)value);
-  g_signal_connect (button, "toggled", 
+  g_signal_connect (button, "toggled",
 		    G_CALLBACK (radio_changed_cb), widget);
 
   return gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
@@ -770,7 +768,7 @@ construct_widgets (GtkPrinterOptionWidget *widget)
       gtk_combo_box_set_active (GTK_COMBO_BOX (priv->combo), 0);
       gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
       gtk_widget_show (priv->combo);
-      gtk_box_pack_start (GTK_BOX (widget), priv->combo, TRUE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (widget), priv->combo);
     }
   else switch (source->type)
     {
@@ -778,7 +776,7 @@ construct_widgets (GtkPrinterOptionWidget *widget)
       priv->check = gtk_check_button_new_with_mnemonic (source->display_text);
       g_signal_connect (priv->check, "toggled", G_CALLBACK (check_toggled_cb), widget);
       gtk_widget_show (priv->check);
-      gtk_box_pack_start (GTK_BOX (widget), priv->check, TRUE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (widget), priv->check);
       break;
     case GTK_PRINTER_OPTION_TYPE_PICKONE:
     case GTK_PRINTER_OPTION_TYPE_PICKONE_PASSWORD:
@@ -810,7 +808,7 @@ construct_widgets (GtkPrinterOptionWidget *widget)
                           source->choices_display[i],
                           source->choices[i]);
       gtk_widget_show (priv->combo);
-      gtk_box_pack_start (GTK_BOX (widget), priv->combo, TRUE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (widget), priv->combo);
       g_signal_connect (priv->combo, "changed", G_CALLBACK (combo_changed_cb), widget);
 
       text = g_strdup_printf ("%s:", source->display_text);
@@ -824,7 +822,7 @@ construct_widgets (GtkPrinterOptionWidget *widget)
       priv->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
       gtk_widget_set_valign (priv->box, GTK_ALIGN_BASELINE);
       gtk_widget_show (priv->box);
-      gtk_box_pack_start (GTK_BOX (widget), priv->box, TRUE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (widget), priv->box);
       for (i = 0; i < source->num_choices; i++)
         {
 	  group = alternative_append (priv->box,
@@ -852,7 +850,7 @@ construct_widgets (GtkPrinterOptionWidget *widget)
       gtk_entry_set_activates_default (GTK_ENTRY (priv->entry),
                                        gtk_printer_option_get_activates_default (source));
       gtk_widget_show (priv->entry);
-      gtk_box_pack_start (GTK_BOX (widget), priv->entry, TRUE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (widget), priv->entry);
       g_signal_connect (priv->entry, "changed", G_CALLBACK (entry_changed_cb), widget);
 
       text = g_strdup_printf ("%s:", source->display_text);
@@ -865,7 +863,7 @@ construct_widgets (GtkPrinterOptionWidget *widget)
     case GTK_PRINTER_OPTION_TYPE_FILESAVE:
       priv->button = gtk_button_new ();
       gtk_widget_show (priv->button);
-      gtk_box_pack_start (GTK_BOX (widget), priv->button, TRUE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (widget), priv->button);
       g_signal_connect (priv->button, "clicked", G_CALLBACK (filesave_choose_cb), widget);
 
       text = g_strdup_printf ("%s:", source->display_text);
@@ -878,13 +876,11 @@ construct_widgets (GtkPrinterOptionWidget *widget)
     case GTK_PRINTER_OPTION_TYPE_INFO:
       priv->info_label = gtk_label_new (NULL);
       gtk_label_set_selectable (GTK_LABEL (priv->info_label), TRUE);
-      gtk_widget_show (priv->info_label);
-      gtk_box_pack_start (GTK_BOX (widget), priv->info_label, FALSE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (widget), priv->info_label);
 
       text = g_strdup_printf ("%s:", source->display_text);
       priv->label = gtk_label_new_with_mnemonic (text);
       g_free (text);
-      gtk_widget_show (priv->label);
 
       break;
 
@@ -893,7 +889,7 @@ construct_widgets (GtkPrinterOptionWidget *widget)
     }
 
   priv->image = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_MENU);
-  gtk_box_pack_start (GTK_BOX (widget), priv->image, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (widget), priv->image);
 }
 
 /*
