@@ -121,14 +121,6 @@ void        gtk_render_frame_gap   (GtkStyleContext     *context,
                                     gdouble              xy0_gap,
                                     gdouble              xy1_gap);
 GDK_AVAILABLE_IN_ALL
-void        gtk_render_extension   (GtkStyleContext     *context,
-                                    cairo_t             *cr,
-                                    gdouble              x,
-                                    gdouble              y,
-                                    gdouble              width,
-                                    gdouble              height,
-                                    GtkPositionType      gap_side);
-GDK_AVAILABLE_IN_ALL
 void        gtk_render_handle      (GtkStyleContext     *context,
                                     cairo_t             *cr,
                                     gdouble              x,
@@ -142,10 +134,6 @@ void        gtk_render_activity    (GtkStyleContext     *context,
                                     gdouble              y,
                                     gdouble              width,
                                     gdouble              height);
-GDK_DEPRECATED_IN_3_10_FOR(gtk_icon_theme_load_icon)
-GdkPixbuf * gtk_render_icon_pixbuf (GtkStyleContext     *context,
-                                    const GtkIconSource *source,
-                                    GtkIconSize          size);
 GDK_AVAILABLE_IN_3_2
 void        gtk_render_icon        (GtkStyleContext     *context,
                                     cairo_t             *cr,
