@@ -52,6 +52,9 @@
  * # CSS nodes
  *
  * GtkRevealer has a single CSS node with name revealer.
+ * When styling #GtkRevealer using CSS, remember that it only hides its contents,
+ * not itself. That means applied margin, padding and borders will be
+ * visible even when the #GtkRevealer:reveal-child property is set to %FALSE.
  *
  * The GtkRevealer widget was added in GTK+ 3.10.
  */
@@ -82,8 +85,6 @@ typedef struct {
   GtkRevealerTransitionType transition_type;
   guint transition_duration;
 
-  GdkWindow* bin_window;
-
   gdouble current_pos;
   gdouble source_pos;
   gdouble target_pos;
@@ -94,12 +95,12 @@ typedef struct {
 
 static GParamSpec *props[LAST_PROP] = { NULL, };
 
-static void     gtk_revealer_real_realize                        (GtkWidget     *widget);
-static void     gtk_revealer_real_unrealize                      (GtkWidget     *widget);
 static void     gtk_revealer_real_add                            (GtkContainer  *widget,
                                                                   GtkWidget     *child);
-static void     gtk_revealer_real_size_allocate                  (GtkWidget     *widget,
-                                                                  GtkAllocation *allocation);
+static void     gtk_revealer_real_size_allocate                  (GtkWidget           *widget,
+                                                                  const GtkAllocation *allocation,
+                                                                  int                  baseline,
+                                                                  GtkAllocation       *out_clip);
 static void gtk_revealer_measure (GtkWidget      *widget,
                                   GtkOrientation  orientation,
                                   int             for_size,
@@ -112,18 +113,6 @@ static void     gtk_revealer_snapshot                            (GtkWidget
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkRevealer, gtk_revealer, GTK_TYPE_BIN)
 
-static void
-gtk_revealer_get_padding (GtkRevealer *revealer,
-                          GtkBorder   *padding)
-{
-  GtkWidget *widget = GTK_WIDGET (revealer);
-  GtkStyleContext *context;
-
-  context = gtk_widget_get_style_context (widget);
-
-  gtk_style_context_get_padding (context, padding);
-}
-
 static void
 gtk_revealer_init (GtkRevealer *revealer)
 {
@@ -236,8 +225,6 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
   object_class->set_property = gtk_revealer_set_property;
   object_class->finalize = gtk_revealer_finalize;
 
-  widget_class->realize = gtk_revealer_real_realize;
-  widget_class->unrealize = gtk_revealer_real_unrealize;
   widget_class->unmap = gtk_revealer_unmap;
   widget_class->size_allocate = gtk_revealer_real_size_allocate;
   widget_class->measure = gtk_revealer_measure;
@@ -311,23 +298,17 @@ effective_transition (GtkRevealer *revealer)
 }
 
 static void
-gtk_revealer_get_child_allocation (GtkRevealer   *revealer,
-                                   GtkAllocation *allocation,
-                                   GtkAllocation *child_allocation)
+gtk_revealer_get_child_allocation (GtkRevealer         *revealer,
+                                   const GtkAllocation *allocation,
+                                   GtkAllocation       *child_allocation)
 {
+  GtkRevealerPrivate *priv = gtk_revealer_get_instance_private (revealer);
   GtkWidget *child;
   GtkRevealerTransitionType transition;
-  GtkBorder padding;
-  gint vertical_padding, horizontal_padding;
 
   g_return_if_fail (revealer != NULL);
   g_return_if_fail (allocation != NULL);
 
-  /* See explanation on gtk_revealer_real_size_allocate */
-  gtk_revealer_get_padding (revealer, &padding);
-  vertical_padding = padding.top + padding.bottom;
-  horizontal_padding = padding.left + padding.right;
-
   child_allocation->x = 0;
   child_allocation->y = 0;
   child_allocation->width = 0;
@@ -337,82 +318,33 @@ gtk_revealer_get_child_allocation (GtkRevealer   *revealer,
   if (child != NULL && gtk_widget_get_visible (child))
     {
       transition = effective_transition (revealer);
+
       if (transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_LEFT ||
           transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT)
         gtk_widget_measure (child, GTK_ORIENTATION_HORIZONTAL,
-                            MAX (0, allocation->height - vertical_padding),
+                            MAX (0, allocation->height),
                             NULL, &child_allocation->width, NULL, NULL);
       else
         gtk_widget_measure (child, GTK_ORIENTATION_VERTICAL,
-                            MAX (0, allocation->width - horizontal_padding),
+                            MAX (0, allocation->width),
                             NULL, &child_allocation->height, NULL, NULL);
+
+      child_allocation->width = MAX (child_allocation->width, allocation->width);
+      child_allocation->height = MAX (child_allocation->height, allocation->height);
+
+      switch (transition)
+        {
+          case GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT:
+            child_allocation->x = - child_allocation->width * (1 - priv->current_pos);
+          break;
+          case GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN:
+            child_allocation->y = - child_allocation->height * (1 - priv->current_pos);
+          break;
+
+          default: {}
+        }
     }
 
-  child_allocation->width = MAX (child_allocation->width, allocation->width - horizontal_padding);
-  child_allocation->height = MAX (child_allocation->height, allocation->height - vertical_padding);
-}
-
-static void
-gtk_revealer_real_realize (GtkWidget *widget)
-{
-  GtkRevealer *revealer = GTK_REVEALER (widget);
-  GtkRevealerPrivate *priv = gtk_revealer_get_instance_private (revealer);
-  GtkAllocation allocation;
-  GtkAllocation child_allocation;
-  GtkWidget *child;
-  GtkRevealerTransitionType transition;
-  GtkBorder padding;
-
-  GTK_WIDGET_CLASS (gtk_revealer_parent_class)->realize (widget);
-
-  gtk_widget_get_allocation (widget, &allocation);
-
-  gtk_revealer_get_child_allocation (revealer, &allocation, &child_allocation);
-
-  gtk_revealer_get_padding (revealer, &padding);
-
-  /* See explanation on gtk_revealer_real_size_allocate */
-  transition = effective_transition (revealer);
-  if (transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN)
-    {
-      child_allocation.y = allocation.height - child_allocation.height - padding.bottom;
-      child_allocation.x = padding.left;
-    }
-  else if (transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT)
-    {
-      child_allocation.y = padding.top;
-      child_allocation.x = allocation.width - child_allocation.width - padding.right;
-    }
- else
-   {
-     child_allocation.y = padding.top;
-     child_allocation.x = padding.left;
-   }
-
-  priv->bin_window =
-    gdk_window_new_child (gtk_widget_get_window (widget),
-                          GDK_ALL_EVENTS_MASK,
-                          &child_allocation);
-  gtk_widget_register_window (widget, priv->bin_window);
-
-  child = gtk_bin_get_child (GTK_BIN (revealer));
-  if (child != NULL)
-    gtk_widget_set_parent_window (child, priv->bin_window);
-
-  gdk_window_show (priv->bin_window);
-}
-
-static void
-gtk_revealer_real_unrealize (GtkWidget *widget)
-{
-  GtkRevealer *revealer = GTK_REVEALER (widget);
-  GtkRevealerPrivate *priv = gtk_revealer_get_instance_private (revealer);
-
-  gtk_widget_unregister_window (widget, priv->bin_window);
-  gdk_window_destroy (priv->bin_window);
-  priv->bin_window = NULL;
-
-  GTK_WIDGET_CLASS (gtk_revealer_parent_class)->unrealize (widget);
 }
 
 static void
@@ -424,82 +356,27 @@ gtk_revealer_real_add (GtkContainer *container,
 
   g_return_if_fail (child != NULL);
 
-  gtk_widget_set_parent_window (child, priv->bin_window);
   gtk_widget_set_child_visible (child, priv->current_pos != 0.0);
 
   GTK_CONTAINER_CLASS (gtk_revealer_parent_class)->add (container, child);
 }
 
 static void
-gtk_revealer_real_size_allocate (GtkWidget     *widget,
-                                 GtkAllocation *allocation)
+gtk_revealer_real_size_allocate (GtkWidget           *widget,
+                                 const GtkAllocation *allocation,
+                                 int                  baseline,
+                                 GtkAllocation       *out_clip)
 {
   GtkRevealer *revealer = GTK_REVEALER (widget);
-  GtkRevealerPrivate *priv = gtk_revealer_get_instance_private (revealer);
-  GtkAllocation child_allocation;
   GtkWidget *child;
-  int bin_x, bin_y;
-  GtkRevealerTransitionType transition;
-  GtkBorder padding;
-
-  g_return_if_fail (allocation != NULL);
-
-  gtk_widget_set_allocation (widget, allocation);
-  gtk_revealer_get_child_allocation (revealer, allocation, &child_allocation);
 
   child = gtk_bin_get_child (GTK_BIN (revealer));
   if (child != NULL && gtk_widget_get_visible (child))
-    gtk_widget_size_allocate (child, &child_allocation);
-
-  if (gtk_widget_get_realized (widget))
     {
-      gtk_revealer_get_padding (revealer, &padding);
-      bin_x = 0;
-      bin_y = 0;
+      GtkAllocation child_allocation;
 
-      transition = effective_transition (revealer);
-      /* The child allocation is fixed (it is not modified by the animation),
-       * and it's origin is relative to the bin_window.
-       * The bin_window has the same allocation as the child, and then the bin_window
-       * deals with the relative positioning with respect to the revealer taking
-       * into account the paddings of the revealer.
-       *
-       * For most of transitions, the bin_window moves along with the revealer,
-       * as its allocation changes.
-       * However for GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN
-       * we need to first move the bin_window upwards and then slide it down in
-       * the revealer.
-       * Otherwise the child would appear as static and the revealer will allocate
-       * following the animation, clipping the child.
-       * To calculate the correct y position for this case:
-       * allocation->height - child_allocation.height is the relative position
-       * towards the revealer taking into account the animation progress with
-       * both vertical paddings added, therefore we need to substract the part
-       * that we don't want to take into account for the y position, which
-       * in this case is the bottom padding.
-       *
-       * The same special treatment is needed for GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT.
-       */
-      if (transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN)
-        {
-          bin_y = allocation->height - child_allocation.height - padding.bottom;
-          bin_x = padding.left;
-        }
-      else if (transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT)
-        {
-          bin_y = padding.top;
-          bin_x = allocation->width - child_allocation.width - padding.right;
-        }
-     else
-       {
-         bin_x = padding.left;
-         bin_y = padding.top;
-       }
-
-      gdk_window_move_resize (priv->bin_window,
-                              bin_x + allocation->x,
-                              bin_y + allocation->y,
-                              child_allocation.width, child_allocation.height);
+      gtk_revealer_get_child_allocation (revealer, allocation, &child_allocation);
+      gtk_widget_size_allocate (child, &child_allocation, -1, out_clip);
     }
 }
 
@@ -524,10 +401,17 @@ gtk_revealer_set_position (GtkRevealer *revealer,
   child = gtk_bin_get_child (GTK_BIN (revealer));
   if (child != NULL &&
       new_visible != gtk_widget_get_child_visible (child))
-    gtk_widget_set_child_visible (child, new_visible);
+    {
+      gtk_widget_set_child_visible (child, new_visible);
+      gtk_widget_queue_resize (GTK_WIDGET (revealer));
+    }
 
   transition = effective_transition (revealer);
-  if (transition == GTK_REVEALER_TRANSITION_TYPE_CROSSFADE)
+  if (transition == GTK_REVEALER_TRANSITION_TYPE_NONE)
+    {
+      gtk_widget_queue_draw (GTK_WIDGET (revealer));
+    }
+  else if (transition == GTK_REVEALER_TRANSITION_TYPE_CROSSFADE)
     {
       gtk_widget_set_opacity (GTK_WIDGET (revealer), priv->current_pos);
       gtk_widget_queue_draw (GTK_WIDGET (revealer));
@@ -681,69 +565,37 @@ gtk_revealer_get_child_revealed (GtkRevealer *revealer)
  */
 
 static void
-set_height_with_paddings (GtkRevealer *revealer,
-                          gint         preferred_minimum_height,
-                          gint         preferred_natural_height,
-                          gint        *minimum_height_out,
-                          gint        *natural_height_out)
+set_height (GtkRevealer *revealer,
+            gint        *minimum_height,
+            gint        *natural_height)
 {
   GtkRevealerPrivate *priv = gtk_revealer_get_instance_private (revealer);
-  gint minimum_height;
-  gint natural_height;
   GtkRevealerTransitionType transition;
-  GtkBorder padding;
-  gint vertical_padding;
-
-  gtk_revealer_get_padding (revealer, &padding);
-  vertical_padding = padding.top + padding.bottom;
-  minimum_height = preferred_minimum_height + vertical_padding;
-  natural_height = preferred_natural_height + vertical_padding;
 
   transition = effective_transition (revealer);
-  if (transition == GTK_REVEALER_TRANSITION_TYPE_NONE ||
-      transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_UP ||
+  if (transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_UP ||
       transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN)
     {
-      /* Padding are included in the animation */
-      minimum_height = round (minimum_height * priv->current_pos);
-      natural_height = round (natural_height * priv->current_pos);
+      *minimum_height = round (*minimum_height * priv->current_pos);
+      *natural_height = round (*natural_height * priv->current_pos);
     }
-
-  *minimum_height_out = MIN (minimum_height, natural_height);
-  *natural_height_out = natural_height;
 }
 
 static void
-set_width_with_paddings (GtkRevealer *revealer,
-                         gint         preferred_minimum_width,
-                         gint         preferred_natural_width,
-                         gint        *minimum_width_out,
-                         gint        *natural_width_out)
+set_width (GtkRevealer *revealer,
+           gint        *minimum_width,
+           gint        *natural_width)
 {
   GtkRevealerPrivate *priv = gtk_revealer_get_instance_private (revealer);
-  gint minimum_width;
-  gint natural_width;
   GtkRevealerTransitionType transition;
-  GtkBorder padding;
-  gint horizontal_padding;
-
-  gtk_revealer_get_padding (revealer, &padding);
-  horizontal_padding = padding.left + padding.right;
-  minimum_width = preferred_minimum_width + horizontal_padding;
-  natural_width = preferred_natural_width + horizontal_padding;
 
   transition = effective_transition (revealer);
-  if (transition == GTK_REVEALER_TRANSITION_TYPE_NONE ||
-      transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_LEFT ||
+  if (transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_LEFT ||
       transition == GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT)
     {
-      /* Paddings are included in the animation */
-      minimum_width = round (minimum_width * priv->current_pos);
-      natural_width = round (natural_width * priv->current_pos);
+      *minimum_width = round (*minimum_width * priv->current_pos);
+      *natural_width = round (*natural_width * priv->current_pos);
     }
-
-  *minimum_width_out = MIN (minimum_width, natural_width);
-  *natural_width_out = natural_width;
 }
 
 static void
@@ -755,16 +607,15 @@ gtk_revealer_measure (GtkWidget      *widget,
                       int            *minimum_baseline,
                       int            *natural_baseline)
 {
-  int min_size, nat_size;
   GTK_WIDGET_CLASS (gtk_revealer_parent_class)->measure (widget,
                                                          orientation,
                                                          for_size,
-                                                         &min_size, &nat_size,
+                                                         minimum, natural,
                                                          NULL, NULL);
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    set_width_with_paddings (GTK_REVEALER (widget), min_size, nat_size, minimum, natural);
+    set_width (GTK_REVEALER (widget), minimum, natural);
   else
-    set_height_with_paddings (GTK_REVEALER (widget), min_size, nat_size, minimum, natural);
+    set_height (GTK_REVEALER (widget), minimum, natural);
 }
 
 static void
@@ -780,7 +631,8 @@ gtk_revealer_snapshot (GtkWidget   *widget,
     return;
 
   transition = effective_transition (revealer);
-  if (transition == GTK_REVEALER_TRANSITION_TYPE_NONE)
+  if (transition == GTK_REVEALER_TRANSITION_TYPE_NONE ||
+      transition == GTK_REVEALER_TRANSITION_TYPE_CROSSFADE)
     {
       gtk_widget_snapshot_child (widget, child, snapshot);
     }
