@@ -37,9 +37,28 @@ gboolean            _gtk_settings_parse_convert              (GtkRcPropertyParse
                                                               const GValue           *src_value,
                                                               GParamSpec             *pspec,
                                                               GValue                 *dest_value);
+const cairo_font_options_t *
+                    gtk_settings_get_font_options            (GtkSettings            *settings);
 GdkScreen          *_gtk_settings_get_screen                 (GtkSettings            *settings);
-GtkStyleCascade    *_gtk_settings_get_style_cascade          (GtkSettings            *settings);
+GtkStyleCascade    *_gtk_settings_get_style_cascade          (GtkSettings            *settings,
+                                                              gint                    scale);
 
+typedef enum
+{
+  GTK_SETTINGS_SOURCE_DEFAULT,
+  GTK_SETTINGS_SOURCE_THEME,
+  GTK_SETTINGS_SOURCE_XSETTING,
+  GTK_SETTINGS_SOURCE_APPLICATION
+} GtkSettingsSource;
+
+GtkSettingsSource  _gtk_settings_get_setting_source (GtkSettings *settings,
+                                                     const gchar *name);
+
+gboolean gtk_settings_get_enable_animations  (GtkSettings *settings);
+gint     gtk_settings_get_dnd_drag_threshold (GtkSettings *settings);
+const gchar *gtk_settings_get_font_family    (GtkSettings *settings);
+gint         gtk_settings_get_font_size      (GtkSettings *settings);
+gboolean     gtk_settings_get_font_size_is_absolute (GtkSettings *settings);
 
 G_END_DECLS
 
