@@ -16,8 +16,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include "gtksidebarrowprivate.h"
 /* For section and place type enums */
 #include "gtkplacessidebarprivate.h"
@@ -29,17 +27,11 @@
 #include "gtkrevealer.h"
 #include "gtkselection.h"
 
-#ifdef HAVE_CLOUDPROVIDERS
-#include <cloudproviders/cloudprovideraccount.h>
-#endif
-
 struct _GtkSidebarRow
 {
   GtkListBoxRow parent_instance;
-  GIcon *start_icon;
-  GIcon *end_icon;
-  GtkWidget *start_icon_widget;
-  GtkWidget *end_icon_widget;
+  GIcon *icon;
+  GtkWidget *icon_widget;
   gchar *label;
   gchar *tooltip;
   GtkWidget *label_widget;
@@ -52,10 +44,8 @@ struct _GtkSidebarRow
   GDrive *drive;
   GVolume *volume;
   GMount *mount;
-  GObject *cloud_provider;
   gboolean placeholder;
   GtkPlacesSidebar *sidebar;
-  GtkWidget *event_box;
   GtkWidget *revealer;
 };
 
@@ -64,8 +54,7 @@ G_DEFINE_TYPE (GtkSidebarRow, gtk_sidebar_row, GTK_TYPE_LIST_BOX_ROW)
 enum
 {
   PROP_0,
-  PROP_START_ICON,
-  PROP_END_ICON,
+  PROP_ICON,
   PROP_LABEL,
   PROP_TOOLTIP,
   PROP_EJECTABLE,
@@ -77,7 +66,6 @@ enum
   PROP_DRIVE,
   PROP_VOLUME,
   PROP_MOUNT,
-  PROP_CLOUD_PROVIDER,
   PROP_PLACEHOLDER,
   LAST_PROP
 };
@@ -98,12 +86,8 @@ gtk_sidebar_row_get_property (GObject    *object,
       g_value_set_object (value, self->sidebar);
       break;
 
-    case PROP_START_ICON:
-      g_value_set_object (value, self->start_icon);
-      break;
-
-    case PROP_END_ICON:
-      g_value_set_object (value, self->end_icon);
+    case PROP_ICON:
+      g_value_set_object (value, self->icon);
       break;
 
     case PROP_LABEL:
@@ -146,10 +130,6 @@ gtk_sidebar_row_get_property (GObject    *object,
       g_value_set_object (value, self->mount);
       break;
 
-    case PROP_CLOUD_PROVIDER:
-      g_value_set_object (value, self->cloud_provider);
-      break;
-
     case PROP_PLACEHOLDER:
       g_value_set_boolean (value, self->placeholder);
       break;
@@ -174,43 +154,9 @@ gtk_sidebar_row_set_property (GObject      *object,
       self->sidebar = g_value_get_object (value);
       break;
 
-    case PROP_START_ICON:
-      {
-        g_clear_object (&self->start_icon);
-        object = g_value_get_object (value);
-        if (object != NULL)
-          {
-            self->start_icon = g_object_ref (object);
-            gtk_image_set_from_gicon (GTK_IMAGE (self->start_icon_widget),
-                                      self->start_icon,
-                                      GTK_ICON_SIZE_MENU);
-          }
-        else
-          {
-            gtk_image_clear (GTK_IMAGE (self->start_icon_widget));
-          }
-        break;
-      }
-
-    case PROP_END_ICON:
-      {
-        g_clear_object (&self->end_icon);
-        object = g_value_get_object (value);
-        if (object != NULL)
-          {
-            self->end_icon = g_object_ref (object);
-            gtk_image_set_from_gicon (GTK_IMAGE (self->end_icon_widget),
-                                      self->end_icon,
-                                      GTK_ICON_SIZE_MENU);
-            gtk_widget_show (self->end_icon_widget);
-          }
-        else
-          {
-            gtk_image_clear (GTK_IMAGE (self->end_icon_widget));
-            gtk_widget_hide (self->end_icon_widget);
-          }
-        break;
-      }
+    case PROP_ICON:
+      gtk_sidebar_row_set_icon (self, g_value_get_object (value));
+      break;
 
     case PROP_LABEL:
       g_free (self->label);
@@ -266,44 +212,33 @@ gtk_sidebar_row_set_property (GObject      *object,
       g_set_object (&self->mount, g_value_get_object (value));
       break;
 
-    case PROP_CLOUD_PROVIDER:
-#ifdef HAVE_CLOUDPROVIDERS
-      g_set_object (&self->cloud_provider, g_value_get_object (value));
-#endif
-      break;
-
     case PROP_PLACEHOLDER:
-      {
-        self->placeholder = g_value_get_boolean (value);
-        if (self->placeholder)
-          {
-            g_clear_object (&self->start_icon);
-            g_clear_object (&self->end_icon);
-            g_free (self->label);
-            self->label = NULL;
-            g_free (self->tooltip);
-            self->tooltip = NULL;
-            gtk_widget_set_tooltip_text (GTK_WIDGET (self), NULL);
-            self->ejectable = FALSE;
-            self->section_type = SECTION_BOOKMARKS;
-            self->place_type = PLACES_BOOKMARK_PLACEHOLDER;
-            g_free (self->uri);
-            self->uri = NULL;
-            g_clear_object (&self->drive);
-            g_clear_object (&self->volume);
-            g_clear_object (&self->mount);
-            g_clear_object (&self->cloud_provider);
+      self->placeholder = g_value_get_boolean (value);
+      if (self->placeholder)
+        {
+          g_clear_object (&self->icon);
+          g_free (self->label);
+          self->label = NULL;
+          g_free (self->tooltip);
+          self->tooltip = NULL;
+          gtk_widget_set_tooltip_text (GTK_WIDGET (self), NULL);
+          self->ejectable = FALSE;
+          self->section_type = SECTION_BOOKMARKS;
+          self->place_type = PLACES_BOOKMARK_PLACEHOLDER;
+          g_free (self->uri);
+          self->uri = NULL;
+          g_clear_object (&self->drive);
+          g_clear_object (&self->volume);
+          g_clear_object (&self->mount);
 
-            gtk_container_foreach (GTK_CONTAINER (self),
-                                   (GtkCallback) gtk_widget_destroy,
-                                   NULL);
+          gtk_container_foreach (GTK_CONTAINER (self),
+                                 (GtkCallback) gtk_widget_destroy,
+                                 NULL);
 
-            context = gtk_widget_get_style_context (GTK_WIDGET (self));
-            gtk_style_context_add_class (context, "sidebar-placeholder-row");
-          }
-
-        break;
-      }
+          context = gtk_widget_get_style_context (GTK_WIDGET (self));
+          gtk_style_context_add_class (context, "sidebar-placeholder-row");
+        }
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -325,7 +260,7 @@ on_child_revealed (GObject    *self,
 void
 gtk_sidebar_row_reveal (GtkSidebarRow *self)
 {
-  gtk_widget_show_all (GTK_WIDGET (self));
+  gtk_widget_show (GTK_WIDGET (self));
   gtk_revealer_set_reveal_child (GTK_REVEALER (self->revealer), TRUE);
 }
 
@@ -345,41 +280,21 @@ gtk_sidebar_row_hide (GtkSidebarRow *self,
 }
 
 void
-gtk_sidebar_row_set_start_icon (GtkSidebarRow *self,
-                                GIcon         *icon)
+gtk_sidebar_row_set_icon (GtkSidebarRow *self,
+                          GIcon         *icon)
 {
   g_return_if_fail (GTK_IS_SIDEBAR_ROW (self));
 
-  if (self->start_icon != icon)
+  if (self->icon != icon)
     {
-      g_set_object (&self->start_icon, icon);
-      if (self->start_icon != NULL)
-        gtk_image_set_from_gicon (GTK_IMAGE (self->start_icon_widget), self->start_icon,
+      g_set_object (&self->icon, icon);
+      if (self->icon != NULL)
+        gtk_image_set_from_gicon (GTK_IMAGE (self->icon_widget), self->icon,
                                   GTK_ICON_SIZE_MENU);
       else
-        gtk_image_clear (GTK_IMAGE (self->start_icon_widget));
+        gtk_image_clear (GTK_IMAGE (self->icon_widget));
 
-      g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_START_ICON]);
-    }
-}
-
-void
-gtk_sidebar_row_set_end_icon (GtkSidebarRow *self,
-                              GIcon         *icon)
-{
-  g_return_if_fail (GTK_IS_SIDEBAR_ROW (self));
-
-  if (self->end_icon != icon)
-    {
-      g_set_object (&self->end_icon, icon);
-      if (self->end_icon != NULL)
-        gtk_image_set_from_gicon (GTK_IMAGE (self->end_icon_widget), self->end_icon,
-                                  GTK_ICON_SIZE_MENU);
-      else
-        if (self->end_icon_widget != NULL)
-          gtk_image_clear (GTK_IMAGE (self->end_icon_widget));
-
-      g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_END_ICON]);
+      g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_ICON]);
     }
 }
 
@@ -388,8 +303,7 @@ gtk_sidebar_row_finalize (GObject *object)
 {
   GtkSidebarRow *self = GTK_SIDEBAR_ROW (object);
 
-  g_clear_object (&self->start_icon);
-  g_clear_object (&self->end_icon);
+  g_clear_object (&self->icon);
   g_free (self->label);
   self->label = NULL;
   g_free (self->tooltip);
@@ -399,7 +313,6 @@ gtk_sidebar_row_finalize (GObject *object)
   g_clear_object (&self->drive);
   g_clear_object (&self->volume);
   g_clear_object (&self->mount);
-  g_clear_object (&self->cloud_provider);
 
   G_OBJECT_CLASS (gtk_sidebar_row_parent_class)->finalize (object);
 }
@@ -429,18 +342,10 @@ gtk_sidebar_row_class_init (GtkSidebarRowClass *klass)
                           G_PARAM_CONSTRUCT_ONLY |
                           G_PARAM_STATIC_STRINGS));
 
-  properties [PROP_START_ICON] =
-    g_param_spec_object ("start-icon",
-                         "start-icon",
-                         "The start icon.",
-                         G_TYPE_ICON,
-                         (G_PARAM_READWRITE |
-                          G_PARAM_STATIC_STRINGS));
-
-  properties [PROP_END_ICON] =
-    g_param_spec_object ("end-icon",
-                         "end-icon",
-                         "The end icon.",
+  properties [PROP_ICON] =
+    g_param_spec_object ("icon",
+                         "icon",
+                         "The place icon.",
                          G_TYPE_ICON,
                          (G_PARAM_READWRITE |
                           G_PARAM_STATIC_STRINGS));
@@ -531,14 +436,6 @@ gtk_sidebar_row_class_init (GtkSidebarRowClass *klass)
                           G_PARAM_CONSTRUCT_ONLY |
                           G_PARAM_STATIC_STRINGS));
 
-  properties [PROP_CLOUD_PROVIDER] =
-    g_param_spec_object ("cloud-provider",
-                         "CloudProviderAccount",
-                         "CloudProviderAccount",
-                         G_TYPE_OBJECT,
-                         (G_PARAM_READWRITE |
-                          G_PARAM_STATIC_STRINGS));
-
   properties [PROP_PLACEHOLDER] =
     g_param_spec_boolean ("placeholder",
                           "Placeholder",
@@ -553,24 +450,22 @@ gtk_sidebar_row_class_init (GtkSidebarRowClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gtk/libgtk/ui/gtksidebarrow.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, start_icon_widget);
-  gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, end_icon_widget);
+  gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, icon_widget);
   gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, label_widget);
   gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, eject_button);
-  gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, event_box);
   gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, revealer);
 
   gtk_widget_class_bind_template_callback (widget_class, on_child_revealed);
   gtk_widget_class_set_css_name (widget_class, "row");
 }
 
+
 GtkSidebarRow*
 gtk_sidebar_row_clone (GtkSidebarRow *self)
 {
  return g_object_new (GTK_TYPE_SIDEBAR_ROW,
                       "sidebar", self->sidebar,
-                      "start-icon", self->start_icon,
-                      "end-icon", self->end_icon,
+                      "icon", self->icon,
                       "label", self->label,
                       "tooltip", self->tooltip,
                       "ejectable", self->ejectable,
@@ -581,18 +476,11 @@ gtk_sidebar_row_clone (GtkSidebarRow *self)
                       "drive", self->drive,
                       "volume", self->volume,
                       "mount", self->mount,
-                      "cloud-provider", self->cloud_provider,
                       NULL);
 }
 
-GtkWidget*
+GtkWidget *
 gtk_sidebar_row_get_eject_button (GtkSidebarRow *self)
 {
   return self->eject_button;
 }
-
-GtkWidget*
-gtk_sidebar_row_get_event_box (GtkSidebarRow *self)
-{
-  return self->event_box;
-}
