@@ -29,6 +29,8 @@
 #include "gtksizegroup-private.h"
 #include "gtksizerequestcacheprivate.h"
 #include "gtkwidgetprivate.h"
+#include "gtkcssnodeprivate.h"
+#include "gtkcssnumbervalueprivate.h"
 
 
 #ifdef G_ENABLE_CONSISTENCY_CHECKS
@@ -36,38 +38,25 @@ static GQuark recursion_check_quark = 0;
 
 static void
 push_recursion_check (GtkWidget       *widget,
-                      GtkOrientation   orientation,
-                      gint             for_size)
+                      GtkOrientation   orientation)
 {
-  const char *previous_method;
-  const char *method;
+  gboolean in_measure = FALSE;
 
   if (recursion_check_quark == 0)
     recursion_check_quark = g_quark_from_static_string ("gtk-size-request-in-progress");
 
-  previous_method = g_object_get_qdata (G_OBJECT (widget), recursion_check_quark);
+  in_measure = GPOINTER_TO_INT (g_object_get_qdata (G_OBJECT (widget), recursion_check_quark));
 
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+  if (in_measure)
     {
-      method = for_size < 0 ? "get_width" : "get_width_for_height";
-    }
-  else
-    {
-      method = for_size < 0 ? "get_height" : "get_height_for_width";
+      g_warning ("%s %p: widget tried to gtk_widget_measure inside "
+                 " GtkWidget::measure implementation. "
+                 "Should just invoke GTK_WIDGET_GET_CLASS(widget)->measure "
+                 "directly rather than using gtk_widget_measure",
+                 G_OBJECT_TYPE_NAME (widget), widget);
     }
 
-  if (previous_method != NULL)
-    {
-      g_warning ("%s %p: widget tried to gtk_widget_%s inside "
-                 " GtkWidget     ::%s implementation. "
-                 "Should just invoke GTK_WIDGET_GET_CLASS(widget)->%s "
-                 "directly rather than using gtk_widget_%s",
-                 G_OBJECT_TYPE_NAME (widget), widget,
-                 method, previous_method,
-                 method, method);
-    }
-
-  g_object_set_qdata (G_OBJECT (widget), recursion_check_quark, (char*) method);
+  g_object_set_qdata (G_OBJECT (widget), recursion_check_quark, GINT_TO_POINTER(TRUE));
 }
 
 static void
@@ -77,16 +66,58 @@ pop_recursion_check (GtkWidget       *widget,
   g_object_set_qdata (G_OBJECT (widget), recursion_check_quark, NULL);
 }
 #else
-#define push_recursion_check(widget, orientation, for_size)
+#define push_recursion_check(widget, orientation)
 #define pop_recursion_check(widget, orientation)
 #endif /* G_ENABLE_CONSISTENCY_CHECKS */
 
+static gint
+get_number (GtkCssStyle *style,
+            guint        property)
+{
+  double d = _gtk_css_number_value_get (gtk_css_style_get_value (style, property), 100);
+
+  if (d < 1)
+    return ceil (d);
+  else
+    return floor (d);
+}
+
+static void
+get_box_margin (GtkCssStyle *style,
+                GtkBorder   *margin)
+{
+  margin->top = get_number (style, GTK_CSS_PROPERTY_MARGIN_TOP);
+  margin->left = get_number (style, GTK_CSS_PROPERTY_MARGIN_LEFT);
+  margin->bottom = get_number (style, GTK_CSS_PROPERTY_MARGIN_BOTTOM);
+  margin->right = get_number (style, GTK_CSS_PROPERTY_MARGIN_RIGHT);
+}
+
+static void
+get_box_border (GtkCssStyle *style,
+                GtkBorder   *border)
+{
+  border->top = get_number (style, GTK_CSS_PROPERTY_BORDER_TOP_WIDTH);
+  border->left = get_number (style, GTK_CSS_PROPERTY_BORDER_LEFT_WIDTH);
+  border->bottom = get_number (style, GTK_CSS_PROPERTY_BORDER_BOTTOM_WIDTH);
+  border->right = get_number (style, GTK_CSS_PROPERTY_BORDER_RIGHT_WIDTH);
+}
+
+static void
+get_box_padding (GtkCssStyle *style,
+                 GtkBorder   *border)
+{
+  border->top = get_number (style, GTK_CSS_PROPERTY_PADDING_TOP);
+  border->left = get_number (style, GTK_CSS_PROPERTY_PADDING_LEFT);
+  border->bottom = get_number (style, GTK_CSS_PROPERTY_PADDING_BOTTOM);
+  border->right = get_number (style, GTK_CSS_PROPERTY_PADDING_RIGHT);
+}
+
 static void
 gtk_widget_query_size_for_orientation (GtkWidget        *widget,
                                        GtkOrientation    orientation,
                                        gint              for_size,
-                                       gint             *minimum_size,
-                                       gint             *natural_size,
+                                       gint             *minimum,
+                                       gint             *natural,
                                        gint             *minimum_baseline,
                                        gint             *natural_baseline)
 {
@@ -97,6 +128,12 @@ gtk_widget_query_size_for_orientation (GtkWidget        *widget,
   gint min_baseline = -1;
   gint nat_baseline = -1;
   gboolean found_in_cache;
+  GtkCssStyle *style;
+  GtkBorder margin, border, padding;
+  int css_min_size;
+  int css_min_for_size;
+  int css_extra_for_size;
+  int css_extra_size;
 
   gtk_widget_ensure_resize (widget);
 
@@ -109,95 +146,85 @@ gtk_widget_query_size_for_orientation (GtkWidget        *widget,
                                                    for_size,
                                                    &min_size,
                                                    &nat_size,
-						   &min_baseline,
-						   &nat_baseline);
+                                                   &min_baseline,
+                                                   &nat_baseline);
 
   widget_class = GTK_WIDGET_GET_CLASS (widget);
-  
+
   if (!found_in_cache)
     {
-      gint adjusted_min, adjusted_natural, adjusted_for_size = for_size;
+      int adjusted_min, adjusted_natural;
+      int adjusted_for_size = for_size;
+      int reported_min_size = 0;
+      int reported_nat_size = 0;
+
+      style = gtk_css_node_get_style (gtk_widget_get_css_node (widget));
+      get_box_margin (style, &margin);
+      get_box_border (style, &border);
+      get_box_padding (style, &padding);
 
       if (orientation == GTK_ORIENTATION_HORIZONTAL)
         {
-          if (for_size < 0)
-            {
-	      push_recursion_check (widget, orientation, for_size);
-              widget_class->measure (widget, GTK_ORIENTATION_HORIZONTAL, -1,
-                                     &min_size, &nat_size, NULL, NULL);
-	      pop_recursion_check (widget, orientation);
-            }
-          else
-            {
-              gint ignored_position = 0;
-              gint minimum_height;
-              gint natural_height;
-
-              /* Pull the base natural height from the cache as it's needed to adjust
-               * the proposed 'for_size' */
-              gtk_widget_measure (widget, GTK_ORIENTATION_VERTICAL, -1,
-                                  &minimum_height, &natural_height,
-                                  NULL, NULL);
-
-              /* convert for_size to unadjusted height (for_size is a proposed allocation) */
-              gtk_widget_adjust_size_allocation (widget,
-                                                 GTK_ORIENTATION_VERTICAL,
-                                                 &minimum_height,
-                                                 &natural_height,
-                                                 &ignored_position,
-                                                 &adjusted_for_size);
-
-	      push_recursion_check (widget, orientation, for_size);
-              widget_class->measure (widget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     MAX (adjusted_for_size, minimum_height),
-                                     &min_size, &nat_size,
-                                     NULL, NULL);
-	      pop_recursion_check (widget, orientation);
-            }
+          css_extra_size = margin.left + margin.right + border.left + border.right + padding.left + padding.right;
+          css_extra_for_size = margin.top + margin.bottom + border.top + border.bottom + padding.top + padding.bottom;
+          css_min_size = get_number (style, GTK_CSS_PROPERTY_MIN_WIDTH);
+          css_min_for_size = get_number (style, GTK_CSS_PROPERTY_MIN_HEIGHT);
         }
       else
         {
-          if (for_size < 0)
-            {
-	      push_recursion_check (widget, orientation, for_size);
-              widget_class->measure (widget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     -1,
-                                     &min_size, &nat_size,
-                                     &min_baseline, &nat_baseline);
-	      pop_recursion_check (widget, orientation);
-            }
-          else
-            {
-              gint ignored_position = 0;
-              gint minimum_width;
-              gint natural_width;
-
-              /* Pull the base natural width from the cache as it's needed to adjust
-               * the proposed 'for_size' */
-              gtk_widget_measure (widget, GTK_ORIENTATION_HORIZONTAL, -1,
-                                  &minimum_width, &natural_width,
-                                  NULL, NULL);
-
-              /* convert for_size to unadjusted width (for_size is a proposed allocation) */
-              gtk_widget_adjust_size_allocation (widget,
-                                                 GTK_ORIENTATION_HORIZONTAL,
-                                                 &minimum_width,
-                                                 &natural_width,
-                                                 &ignored_position,
-                                                 &adjusted_for_size);
-
-	      push_recursion_check (widget, orientation, for_size);
-              widget_class->measure (widget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     MAX (adjusted_for_size, minimum_width),
-                                     &min_size, &nat_size,
-                                     &min_baseline, &nat_baseline);
-	      pop_recursion_check (widget, orientation);
-            }
+          css_extra_size = margin.top + margin.bottom + border.top + border.bottom + padding.top + padding.bottom;
+          css_extra_for_size = margin.left + margin.right + border.left + border.right + padding.left + padding.right;
+          css_min_size = get_number (style, GTK_CSS_PROPERTY_MIN_HEIGHT);
+          css_min_for_size = get_number (style, GTK_CSS_PROPERTY_MIN_WIDTH);
         }
 
+      if (for_size < 0)
+        {
+          push_recursion_check (widget, orientation);
+          widget_class->measure (widget, orientation, -1,
+                                 &reported_min_size, &reported_nat_size,
+                                 &min_baseline, &nat_baseline);
+          pop_recursion_check (widget, orientation);
+        }
+      else
+        {
+          int dummy = 0;
+          int minimum_for_size;
+          int natural_for_size;
+
+          /* Pull the base natural size from the cache as it's needed to adjust
+           * the proposed 'for_size' */
+          widget_class->measure (widget, OPPOSITE_ORIENTATION (orientation), -1,
+                                 &minimum_for_size, &natural_for_size, &dummy, &dummy);
+
+          gtk_widget_adjust_size_allocation (widget,
+                                             OPPOSITE_ORIENTATION (orientation),
+                                             &minimum_for_size,
+                                             &natural_for_size,
+                                             &dummy,
+                                             &adjusted_for_size);
+
+          /* adjusted_for_size now without widget margins */
+          adjusted_for_size -= css_extra_for_size;
+
+          /* TODO: Warn if the given for_size is too small? */
+
+          if (adjusted_for_size < MAX (minimum_for_size, css_min_for_size))
+            adjusted_for_size = MAX (minimum_for_size, css_min_for_size);
+
+          push_recursion_check (widget, orientation);
+          widget_class->measure (widget,
+                                 orientation,
+                                 adjusted_for_size,
+                                 &reported_min_size, &reported_nat_size,
+                                 &min_baseline, &nat_baseline);
+          pop_recursion_check (widget, orientation);
+
+        }
+
+      min_size = MAX (0, MAX (reported_min_size, css_min_size)) + css_extra_size;
+      nat_size = MAX (0, MAX (reported_nat_size, css_min_size)) + css_extra_size;
+
       if (G_UNLIKELY (min_size > nat_size))
         {
           if (orientation == GTK_ORIENTATION_HORIZONTAL)
@@ -211,6 +238,17 @@ gtk_widget_query_size_for_orientation (GtkWidget        *widget,
                          G_OBJECT_TYPE_NAME (widget), widget, min_size, nat_size, for_size);
 
             }
+
+          nat_size = min_size;
+        }
+      else if (G_UNLIKELY (min_size < 0))
+        {
+          g_warning ("%s %p reported min %s %d, but sizes must be >= 0",
+                     G_OBJECT_TYPE_NAME (widget), widget,
+                     orientation == GTK_ORIENTATION_HORIZONTAL ? "width" : "height",
+                     min_size);
+          min_size = 0;
+          nat_size = MAX (0, min_size);
         }
 
       adjusted_min     = min_size;
@@ -261,15 +299,32 @@ gtk_widget_query_size_for_orientation (GtkWidget        *widget,
 	      min_baseline = -1;
 	      nat_baseline = -1;
 	    }
-	  else if (gtk_widget_get_valign (widget) != GTK_ALIGN_BASELINE)
-	    {
-	      /* Ignore requested baseline for non-aligned widgets */
-	      min_baseline = -1;
-	      nat_baseline = -1;
-	    }
+          else if (min_baseline > reported_min_size ||
+                   nat_baseline > reported_nat_size ||
+                   min_baseline < 0 ||
+                   nat_baseline < 0)
+            {
+              g_warning ("%s %p reported baselines of minimum %d and natural %d, but sizes of "
+                         "minimum %d and natural %d. Baselines must be inside the widget size.",
+                         G_OBJECT_TYPE_NAME (widget), widget, min_baseline, nat_baseline,
+                         reported_min_size, reported_nat_size);
+              min_baseline = -1;
+              nat_baseline = -1;
+            }
 	  else
-      gtk_widget_adjust_baseline_request (widget, &min_baseline, &nat_baseline);
-	}
+            {
+              if (css_min_size > reported_min_size)
+                {
+                  min_baseline += (css_min_size - reported_min_size) / 2;
+                  nat_baseline += (css_min_size - reported_min_size) / 2;
+                }
+
+              min_baseline += margin.top + border.top + padding.top;
+              nat_baseline += margin.top + border.top + padding.top;
+
+              gtk_widget_adjust_baseline_request (widget, &min_baseline, &nat_baseline);
+            }
+        }
 
       _gtk_size_request_cache_commit (cache,
                                       orientation,
@@ -280,11 +335,11 @@ gtk_widget_query_size_for_orientation (GtkWidget        *widget,
 				      nat_baseline);
     }
 
-  if (minimum_size)
-    *minimum_size = min_size;
+  if (minimum)
+    *minimum = min_size;
 
-  if (natural_size)
-    *natural_size = nat_size;
+  if (natural)
+    *natural = nat_size;
 
   if (minimum_baseline)
     *minimum_baseline = min_baseline;
@@ -316,7 +371,6 @@ gtk_widget_query_size_for_orientation (GtkWidget        *widget,
 	    });
 }
 
-
 /**
  * gtk_widget_measure:
  * @widget: A #GtkWidget instance
@@ -379,7 +433,7 @@ gtk_widget_measure (GtkWidget        *widget,
   if (G_LIKELY (!_gtk_widget_get_sizegroups (widget)))
     {
       gtk_widget_query_size_for_orientation (widget, orientation, for_size, minimum, natural,
-					     minimum_baseline, natural_baseline);
+                                             minimum_baseline, natural_baseline);
       return;
     }
 
