@@ -38,7 +38,8 @@
 #include "gtkstylecontextprivate.h"
 #include "gtkwidgetprivate.h"
 #include "a11y/gtkspinneraccessible.h"
-#include "gtkbuiltiniconprivate.h"
+#include "gtkcssnumbervalueprivate.h"
+#include "gtkrendericonprivate.h"
 
 
 /**
@@ -67,69 +68,54 @@ enum {
 
 struct _GtkSpinnerPrivate
 {
-  GtkCssGadget *gadget;
-  gboolean active;
+  guint active : 1;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSpinner, gtk_spinner, GTK_TYPE_WIDGET)
 
+#define DEFAULT_SIZE 16
+
 static void
-gtk_spinner_finalize (GObject *object)
+gtk_spinner_measure (GtkWidget      *widget,
+                     GtkOrientation  orientation,
+                     int             for_size,
+                     int            *minimum,
+                     int            *natural,
+                     int            *minimum_baseline,
+                     int            *natural_baseline)
 {
-  GtkSpinner *spinner = GTK_SPINNER (object);
+  double min_size;
+  guint property;
+  GtkCssStyle *style;
 
-  g_clear_object (&spinner->priv->gadget);
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    property = GTK_CSS_PROPERTY_MIN_WIDTH;
+  else
+    property = GTK_CSS_PROPERTY_MIN_HEIGHT;
 
-  G_OBJECT_CLASS (gtk_spinner_parent_class)->finalize (object);
+  style = gtk_css_node_get_style (gtk_widget_get_css_node (widget));
+
+  min_size = _gtk_css_number_value_get (gtk_css_style_get_value (style, property), 100);
+
+  if (min_size > 0.0)
+    *minimum = *natural = min_size;
+  else
+    *minimum = *natural = DEFAULT_SIZE;
 }
 
 static void
-gtk_spinner_get_preferred_width (GtkWidget *widget,
-                                 gint      *minimum,
-                                 gint      *natural)
+gtk_spinner_snapshot (GtkWidget   *widget,
+                      GtkSnapshot *snapshot)
 {
-  gtk_css_gadget_get_preferred_size (GTK_SPINNER (widget)->priv->gadget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     -1,
-                                     minimum, natural,
-                                     NULL, NULL);
-}
+  GtkCssStyle *style = gtk_css_node_get_style (gtk_widget_get_css_node (widget));
+  int width, height;
 
-static void
-gtk_spinner_get_preferred_height (GtkWidget *widget,
-                                  gint      *minimum,
-                                  gint      *natural)
-{
-  gtk_css_gadget_get_preferred_size (GTK_SPINNER (widget)->priv->gadget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     -1,
-                                     minimum, natural,
-                                     NULL, NULL);
-}
+  gtk_widget_get_content_size (widget, &width, &height);
 
-static void
-gtk_spinner_size_allocate (GtkWidget     *widget,
-                           GtkAllocation *allocation)
-{
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (GTK_SPINNER (widget)->priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static gboolean
-gtk_spinner_draw (GtkWidget *widget,
-                  cairo_t   *cr)
-{
-  gtk_css_gadget_draw (GTK_SPINNER (widget)->priv->gadget, cr);
-
-  return FALSE;
+  gtk_css_style_snapshot_icon (style,
+                               snapshot,
+                               width, height,
+                               GTK_CSS_IMAGE_BUILTIN_SPINNER);
 }
 
 static void
@@ -198,15 +184,12 @@ gtk_spinner_class_init (GtkSpinnerClass *klass)
   GtkWidgetClass *widget_class;
 
   gobject_class = G_OBJECT_CLASS(klass);
-  gobject_class->finalize = gtk_spinner_finalize;
   gobject_class->get_property = gtk_spinner_get_property;
   gobject_class->set_property = gtk_spinner_set_property;
 
   widget_class = GTK_WIDGET_CLASS(klass);
-  widget_class->size_allocate = gtk_spinner_size_allocate;
-  widget_class->draw = gtk_spinner_draw;
-  widget_class->get_preferred_width = gtk_spinner_get_preferred_width;
-  widget_class->get_preferred_height = gtk_spinner_get_preferred_height;
+  widget_class->snapshot = gtk_spinner_snapshot;
+  widget_class->measure = gtk_spinner_measure;
 
   /* GtkSpinner:active:
    *
@@ -229,16 +212,9 @@ gtk_spinner_class_init (GtkSpinnerClass *klass)
 static void
 gtk_spinner_init (GtkSpinner *spinner)
 {
-  GtkCssNode *widget_node;
-
   spinner->priv = gtk_spinner_get_instance_private (spinner);
 
   gtk_widget_set_has_window (GTK_WIDGET (spinner), FALSE);
-
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (spinner));
-  spinner->priv->gadget = gtk_builtin_icon_new_for_node (widget_node, GTK_WIDGET (spinner));
-  gtk_builtin_icon_set_image (GTK_BUILTIN_ICON (spinner->priv->gadget), GTK_CSS_IMAGE_BUILTIN_SPINNER);
-  gtk_builtin_icon_set_default_size (GTK_BUILTIN_ICON (spinner->priv->gadget), 16);
 }
 
 /**
