@@ -25,7 +25,6 @@
 #include "gtkstack.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkcontainerprivate.h"
 #include "gtkprogresstrackerprivate.h"
 #include "gtksettingsprivate.h"
@@ -132,12 +131,8 @@ struct _GtkStackChildInfo {
 typedef struct {
   GList *children;
 
-  GdkWindow* bin_window;
-
   GtkStackChildInfo *visible_child;
 
-  GtkCssGadget *gadget;
-
   gboolean hhomogeneous;
   gboolean vhomogeneous;
 
@@ -173,11 +168,13 @@ static void     gtk_stack_forall                         (GtkContainer  *contain
 static void     gtk_stack_compute_expand                 (GtkWidget     *widget,
                                                           gboolean      *hexpand,
                                                           gboolean      *vexpand);
-static void     gtk_stack_size_allocate                  (GtkWidget     *widget,
-                                                          GtkAllocation *allocation);
+static void     gtk_stack_size_allocate                  (GtkWidget           *widget,
+                                                          const GtkAllocation *allocation,
+                                                          int                  baseline,
+                                                          GtkAllocation       *out_clip);
 static void     gtk_stack_snapshot                       (GtkWidget     *widget,
                                                           GtkSnapshot   *snapshot);
-static void     gtk_stack_measure_                       (GtkWidget      *widget,
+static void     gtk_stack_measure                        (GtkWidget      *widget,
                                                           GtkOrientation  orientation,
                                                           int             for_size,
                                                           int            *minimum,
@@ -204,10 +201,6 @@ static void     gtk_stack_set_child_property             (GtkContainer  *contain
                                                           const GValue  *value,
                                                           GParamSpec    *pspec);
 static void     gtk_stack_unschedule_ticks               (GtkStack      *stack);
-static gint     get_bin_window_x                         (GtkStack            *stack,
-                                                          const GtkAllocation *allocation);
-static gint     get_bin_window_y                         (GtkStack            *stack,
-                                                          const GtkAllocation *allocation);
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkStack, gtk_stack, GTK_TYPE_CONTAINER)
 
@@ -221,8 +214,6 @@ gtk_stack_finalize (GObject *obj)
 
   g_clear_pointer (&priv->last_visible_node, gsk_render_node_unref);
 
-  g_clear_object (&priv->gadget);
-
   G_OBJECT_CLASS (gtk_stack_parent_class)->finalize (obj);
 }
 
@@ -309,72 +300,6 @@ gtk_stack_set_property (GObject     *object,
     }
 }
 
-static void
-gtk_stack_realize (GtkWidget *widget)
-{
-  GtkStack *stack = GTK_STACK (widget);
-  GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
-  GtkAllocation allocation;
-  GtkStackChildInfo *info;
-  GList *l;
-
-  GTK_WIDGET_CLASS (gtk_stack_parent_class)->realize (widget);
-
-  gtk_css_gadget_get_content_allocation (priv->gadget, &allocation, NULL);
-
-  priv->bin_window =
-    gdk_window_new_child (gtk_widget_get_window (widget),
-                          GDK_ALL_EVENTS_MASK,
-                          &(GdkRectangle) {
-                            get_bin_window_x (stack, &allocation),
-                            get_bin_window_y (stack, &allocation),
-                            allocation.width,
-                            allocation.height});
-  gtk_widget_register_window (widget, priv->bin_window);
-
-  for (l = priv->children; l != NULL; l = l->next)
-    {
-      info = l->data;
-
-      gtk_widget_set_parent_window (info->widget, priv->bin_window);
-    }
-}
-
-static void
-gtk_stack_unrealize (GtkWidget *widget)
-{
-  GtkStack *stack = GTK_STACK (widget);
-  GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
-
-  gtk_widget_unregister_window (widget, priv->bin_window);
-  gdk_window_destroy (priv->bin_window);
-  priv->bin_window = NULL;
-
-  GTK_WIDGET_CLASS (gtk_stack_parent_class)->unrealize (widget);
-}
-
-static void
-gtk_stack_map (GtkWidget *widget)
-{
-  GtkStack *stack = GTK_STACK (widget);
-  GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
-
-  GTK_WIDGET_CLASS (gtk_stack_parent_class)->map (widget);
-
-  gdk_window_show (priv->bin_window);
-}
-
-static void
-gtk_stack_unmap (GtkWidget *widget)
-{
-  GtkStack *stack = GTK_STACK (widget);
-  GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
-
-  gdk_window_hide (priv->bin_window);
-
-  GTK_WIDGET_CLASS (gtk_stack_parent_class)->unmap (widget);
-}
-
 static void
 gtk_stack_class_init (GtkStackClass *klass)
 {
@@ -388,11 +313,7 @@ gtk_stack_class_init (GtkStackClass *klass)
 
   widget_class->size_allocate = gtk_stack_size_allocate;
   widget_class->snapshot = gtk_stack_snapshot;
-  widget_class->realize = gtk_stack_realize;
-  widget_class->unrealize = gtk_stack_unrealize;
-  widget_class->map = gtk_stack_map;
-  widget_class->unmap = gtk_stack_unmap;
-  widget_class->measure = gtk_stack_measure_;
+  widget_class->measure = gtk_stack_measure;
   widget_class->compute_expand = gtk_stack_compute_expand;
 
   container_class->add = gtk_stack_add;
@@ -797,39 +718,43 @@ get_simple_transition_type (gboolean               new_child_first,
 }
 
 static gint
-get_bin_window_x (GtkStack            *stack,
-                  const GtkAllocation *allocation)
+get_bin_window_x (GtkStack *stack)
 {
   GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
+  int width, height;
   int x = 0;
 
+  gtk_widget_get_content_size (GTK_WIDGET (stack), &width, &height);
+
   if (gtk_progress_tracker_get_state (&priv->tracker) != GTK_PROGRESS_STATE_AFTER)
     {
       if (is_left_transition (priv->active_transition_type))
-        x = allocation->width * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
+        x = width * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
       if (is_right_transition (priv->active_transition_type))
-        x = -allocation->width * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
+        x = -width * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
     }
 
-  return x + allocation->x;
+  return x;
 }
 
 static gint
-get_bin_window_y (GtkStack            *stack,
-                  const GtkAllocation *allocation)
+get_bin_window_y (GtkStack *stack)
 {
   GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
+  int width, height;
   int y = 0;
 
+  gtk_widget_get_content_size (GTK_WIDGET (stack), &width, &height);
+
   if (gtk_progress_tracker_get_state (&priv->tracker) != GTK_PROGRESS_STATE_AFTER)
     {
       if (is_up_transition (priv->active_transition_type))
-        y = allocation->height * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
+        y = height * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
       if (is_down_transition(priv->active_transition_type))
-        y = -allocation->height * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
+        y = -height * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
     }
 
-  return y + allocation->y;
+  return y;
 }
 
 static void
@@ -841,15 +766,8 @@ gtk_stack_progress_updated (GtkStack *stack)
 
   if (!priv->vhomogeneous || !priv->hhomogeneous)
     gtk_widget_queue_resize (GTK_WIDGET (stack));
-
-  if (priv->bin_window != NULL &&
-      is_window_moving_transition (priv->active_transition_type))
-    {
-      GtkAllocation allocation;
-      gtk_widget_get_allocation (GTK_WIDGET (stack), &allocation);
-      gdk_window_move (priv->bin_window,
-                       get_bin_window_x (stack, &allocation), get_bin_window_y (stack, &allocation));
-    }
+  else if (is_window_moving_transition (priv->active_transition_type))
+    gtk_widget_queue_allocate (GTK_WIDGET (stack));
 
   if (gtk_progress_tracker_get_state (&priv->tracker) == GTK_PROGRESS_STATE_AFTER)
     {
@@ -1214,13 +1132,8 @@ gtk_stack_add (GtkContainer *container,
   priv->children = g_list_append (priv->children, child_info);
 
   gtk_widget_set_child_visible (child, FALSE);
-  gtk_widget_set_parent_window (child, priv->bin_window);
   gtk_widget_set_parent (child, GTK_WIDGET (stack));
 
-  if (priv->bin_window)
-    gdk_window_set_events (priv->bin_window,
-                           GDK_ALL_EVENTS_MASK);
-
   g_signal_connect (child, "notify::visible",
                     G_CALLBACK (stack_child_visibility_notify_cb), stack);
 
@@ -1923,35 +1836,37 @@ gtk_stack_snapshot_under (GtkWidget   *widget,
 {
   GtkStack *stack = GTK_STACK (widget);
   GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
-  GtkAllocation allocation;
+  int widget_width, widget_height;
   gint x, y, width, height, pos_x, pos_y;
 
-  gtk_widget_get_allocation (widget, &allocation);
+
+  gtk_widget_get_content_size (widget, &widget_width, &widget_height);
   x = y = 0;
-  width = allocation.width;
-  height = allocation.height;
+  width = widget_width;
+  height = widget_height;
+
   pos_x = pos_y = 0;
 
   switch (priv->active_transition_type)
     {
     case GTK_STACK_TRANSITION_TYPE_UNDER_DOWN:
       y = 0;
-      height = allocation.height * (gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
+      height = widget_height * (gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
       pos_y = height;
       break;
     case GTK_STACK_TRANSITION_TYPE_UNDER_UP:
-      y = allocation.height * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
-      height = allocation.height - y;
-      pos_y = y - allocation.height;
+      y = widget_height * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
+      height = widget_height - y;
+      pos_y = y - widget_height;
       break;
     case GTK_STACK_TRANSITION_TYPE_UNDER_LEFT:
-      x = allocation.width * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
-      width = allocation.width - x;
-      pos_x = x - allocation.width;
+      x = widget_width * (1 - gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
+      width = widget_width - x;
+      pos_x = x - widget_width;
       break;
     case GTK_STACK_TRANSITION_TYPE_UNDER_RIGHT:
       x = 0;
-      width = allocation.width * (gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
+      width = widget_width * (gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE));
       pos_x = width;
       break;
     default:
@@ -1989,28 +1904,28 @@ gtk_stack_snapshot_slide (GtkWidget   *widget,
 
   if (priv->last_visible_node)
     {
-      GtkAllocation allocation;
       graphene_matrix_t matrix;
       int x, y;
+      int width, height;
 
-      gtk_widget_get_allocation (widget, &allocation);
+      gtk_widget_get_content_size (widget, &width, &height);
 
-      x = get_bin_window_x (stack, &allocation) - allocation.x;
-      y = get_bin_window_y (stack, &allocation) - allocation.y;
+      x = get_bin_window_x (stack);
+      y = get_bin_window_y (stack);
 
       switch (priv->active_transition_type)
         {
         case GTK_STACK_TRANSITION_TYPE_SLIDE_LEFT:
-          x -= allocation.width;
+          x -= width;
           break;
         case GTK_STACK_TRANSITION_TYPE_SLIDE_RIGHT:
-          x += allocation.width;
+          x += width;
           break;
         case GTK_STACK_TRANSITION_TYPE_SLIDE_UP:
-          y -= allocation.height;
+          y -= height;
           break;
         case GTK_STACK_TRANSITION_TYPE_SLIDE_DOWN:
-          y += allocation.height;
+          y += height;
           break;
         case GTK_STACK_TRANSITION_TYPE_OVER_UP:
         case GTK_STACK_TRANSITION_TYPE_OVER_DOWN:
@@ -2025,14 +1940,11 @@ gtk_stack_snapshot_slide (GtkWidget   *widget,
           break;
         }
 
-      x += priv->last_visible_surface_allocation.x;
-      y += priv->last_visible_surface_allocation.y;
-
       if (gtk_widget_get_valign (priv->last_visible_child->widget) == GTK_ALIGN_END &&
-          priv->last_visible_widget_height > allocation.height)
-        y -= priv->last_visible_widget_height - allocation.height;
+          priv->last_visible_widget_height > height)
+        y -= priv->last_visible_widget_height - height;
       else if (gtk_widget_get_valign (priv->last_visible_child->widget) == GTK_ALIGN_CENTER)
-        y -= (priv->last_visible_widget_height - allocation.height) / 2;
+        y -= (priv->last_visible_widget_height - height) / 2;
 
       graphene_matrix_init_translate (&matrix, &GRAPHENE_POINT3D_INIT (x, y, 0));
       gtk_snapshot_push_transform (snapshot, &matrix, "StackSlide");
@@ -2052,26 +1964,12 @@ gtk_stack_snapshot (GtkWidget   *widget,
   GtkStack *stack = GTK_STACK (widget);
   GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
 
-  gtk_css_gadget_snapshot (priv->gadget, snapshot);
-}
-
-static gboolean
-gtk_stack_render (GtkCssGadget *gadget,
-                  GtkSnapshot  *snapshot,
-                  int           x,
-                  int           y,
-                  int           width,
-                  int           height,
-                  gpointer      data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkStack *stack = GTK_STACK (widget);
-  GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
-
   if (priv->visible_child)
     {
       if (gtk_progress_tracker_get_state (&priv->tracker) != GTK_PROGRESS_STATE_AFTER)
         {
+          int width, height;
+
           if (priv->last_visible_node == NULL &&
               priv->last_visible_child != NULL)
             {
@@ -2088,11 +1986,11 @@ gtk_stack_render (GtkCssGadget *gadget,
               priv->last_visible_node = gtk_snapshot_finish (&last_visible_snapshot);
             }
 
+          gtk_widget_get_content_size (widget, &width, &height);
           gtk_snapshot_push_clip (snapshot,
                                   &GRAPHENE_RECT_INIT(
                                       0, 0,
-                                      gtk_widget_get_allocated_width (widget),
-                                      gtk_widget_get_allocated_height (widget)
+                                      width, height
                                   ),
                                   "StackAnimationClip");
 
@@ -2128,53 +2026,21 @@ gtk_stack_render (GtkCssGadget *gadget,
                                    priv->visible_child->widget,
                                    snapshot);
     }
-
-  return FALSE;
 }
 
 static void
-gtk_stack_size_allocate (GtkWidget     *widget,
-                         GtkAllocation *allocation)
+gtk_stack_size_allocate (GtkWidget           *widget,
+                         const GtkAllocation *allocation,
+                         int                  baseline,
+                         GtkAllocation       *out_clip)
 {
   GtkStack *stack = GTK_STACK (widget);
   GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-
-  gtk_css_gadget_allocate (priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_stack_allocate (GtkCssGadget        *gadget,
-                    const GtkAllocation *allocation,
-                    int                  baseline,
-                    GtkAllocation       *out_clip,
-                    gpointer             data)
-{
-  GtkWidget *widget;
-  GtkStack *stack;
-  GtkStackPrivate *priv;
+  GdkRectangle child_clip;
   GtkAllocation child_allocation;
 
-  widget = gtk_css_gadget_get_owner (gadget);
-  stack = GTK_STACK (widget);
-  priv = gtk_stack_get_instance_private (stack);
-
-  child_allocation.x = 0;
-  child_allocation.y = 0;
-
-  if (gtk_widget_get_realized (widget))
-    {
-      gdk_window_move_resize (priv->bin_window,
-                              get_bin_window_x (stack, allocation), get_bin_window_y (stack, allocation),
-                              allocation->width, allocation->height);
-    }
+  child_allocation.x = get_bin_window_x (stack);
+  child_allocation.y = get_bin_window_y (stack);
 
   if (priv->last_visible_child)
     {
@@ -2189,7 +2055,8 @@ gtk_stack_allocate (GtkCssGadget        *gadget,
                           &min, &nat, NULL, NULL);
       child_allocation.height = MAX (min, allocation->height);
 
-      gtk_widget_size_allocate (priv->last_visible_child->widget, &child_allocation);
+      gtk_widget_size_allocate (priv->last_visible_child->widget, &child_allocation, -1, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
 
       if (!gdk_rectangle_equal (&priv->last_visible_surface_allocation,
                                 &child_allocation))
@@ -2231,44 +2098,24 @@ gtk_stack_allocate (GtkCssGadget        *gadget,
           if (valign == GTK_ALIGN_CENTER || valign == GTK_ALIGN_FILL)
             child_allocation.y = (allocation->height - child_allocation.height) / 2;
           else if (valign == GTK_ALIGN_END)
-            child_allocation.x = (allocation->height - child_allocation.height);
+            child_allocation.y = (allocation->height - child_allocation.height);
         }
 
-      gtk_widget_size_allocate (priv->visible_child->widget, &child_allocation);
+      gtk_widget_size_allocate (priv->visible_child->widget, &child_allocation, -1, &child_clip);
+      gdk_rectangle_union (out_clip, &child_clip, out_clip);
     }
-  gtk_container_get_children_clip (GTK_CONTAINER (widget), out_clip);
-}
-static void
-gtk_stack_measure_ (GtkWidget      *widget,
-                    GtkOrientation  orientation,
-                    int             for_size,
-                    int            *minimum,
-                    int            *natural,
-                    int            *minimum_baseline,
-                    int            *natural_baseline)
-{
-  GtkStackPrivate *priv = gtk_stack_get_instance_private (GTK_STACK (widget));
-
-  gtk_css_gadget_get_preferred_size (priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
 }
 
 #define LERP(a, b, t) ((a) + (((b) - (a)) * (1.0 - (t))))
-
 static void
-gtk_stack_measure (GtkCssGadget   *gadget,
+gtk_stack_measure (GtkWidget      *widget,
                    GtkOrientation  orientation,
                    int             for_size,
                    int            *minimum,
                    int            *natural,
                    int            *minimum_baseline,
-                   int            *natural_baseline,
-                   gpointer        data)
+                   int            *natural_baseline)
 {
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
   GtkStack *stack = GTK_STACK (widget);
   GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
   GtkStackChildInfo *child_info;
@@ -2326,13 +2173,4 @@ gtk_stack_init (GtkStack *stack)
   priv->hhomogeneous = TRUE;
   priv->transition_duration = 200;
   priv->transition_type = GTK_STACK_TRANSITION_TYPE_NONE;
-
-  priv->gadget = gtk_css_custom_gadget_new_for_node (gtk_widget_get_css_node (GTK_WIDGET (stack)),
-                                                     GTK_WIDGET (stack),
-                                                     gtk_stack_measure,
-                                                     gtk_stack_allocate,
-                                                     gtk_stack_render,
-                                                     NULL,
-                                                     NULL);
-
 }
