@@ -125,7 +125,6 @@ struct _GtkStyleContextPrivate
   GtkStyleContext *parent;
   GtkCssNode *cssnode;
   GSList *saved_nodes;
-  GArray *property_cache;
 
   GdkFrameClock *frame_clock;
 
@@ -238,23 +237,6 @@ gtk_style_context_class_init (GtkStyleContextClass *klass)
   g_object_class_install_properties (object_class, LAST_PROP, properties);
 }
 
-void
-gtk_style_context_clear_property_cache (GtkStyleContext *context)
-{
-  GtkStyleContextPrivate *priv = context->priv;
-  guint i;
-
-  for (i = 0; i < priv->property_cache->len; i++)
-    {
-      PropertyValue *node = &g_array_index (priv->property_cache, PropertyValue, i);
-
-      g_param_spec_unref (node->pspec);
-      g_value_unset (&node->value);
-    }
-
-  g_array_set_size (priv->property_cache, 0);
-}
-
 static void
 gtk_style_context_pop_style_node (GtkStyleContext *context)
 {
@@ -321,14 +303,8 @@ gtk_style_context_init (GtkStyleContext *context)
   if (priv->screen == NULL)
     g_error ("Can't create a GtkStyleContext without a display connection");
 
-  priv->property_cache = g_array_new (FALSE, FALSE, sizeof (PropertyValue));
-
   gtk_style_context_set_cascade (context,
                                  _gtk_settings_get_style_cascade (gtk_settings_get_for_screen (priv->screen), 1));
-
-  /* Create default info store */
-  priv->cssnode = gtk_css_path_node_new (context);
-  gtk_css_node_set_state (priv->cssnode, GTK_STATE_FLAG_DIR_LTR);
 }
 
 static void
@@ -355,10 +331,8 @@ gtk_style_context_finalize (GObject *object)
   gtk_style_context_clear_parent (context);
   gtk_style_context_set_cascade (context, NULL);
 
-  g_object_unref (priv->cssnode);
-
-  gtk_style_context_clear_property_cache (context);
-  g_array_free (priv->property_cache, TRUE);
+  if (priv->cssnode)
+    g_object_unref (priv->cssnode);
 
   G_OBJECT_CLASS (gtk_style_context_parent_class)->finalize (object);
 }
@@ -481,7 +455,15 @@ gtk_style_context_get_node (GtkStyleContext *context)
 GtkStyleContext *
 gtk_style_context_new (void)
 {
-  return g_object_new (GTK_TYPE_STYLE_CONTEXT, NULL);
+  GtkStyleContext *context;
+
+  context = g_object_new (GTK_TYPE_STYLE_CONTEXT, NULL);
+
+  /* Create default info store */
+  context->priv->cssnode = gtk_css_path_node_new (context);
+  gtk_css_node_set_state (context->priv->cssnode, GTK_STATE_FLAG_DIR_LTR);
+
+  return context;
 }
 
 GtkStyleContext *
@@ -491,8 +473,8 @@ gtk_style_context_new_for_node (GtkCssNode *node)
 
   g_return_val_if_fail (GTK_IS_CSS_NODE (node), NULL);
 
-  context = gtk_style_context_new ();
-  g_set_object (&context->priv->cssnode, node);
+  context = g_object_new (GTK_TYPE_STYLE_CONTEXT, NULL);
+  context->priv->cssnode = g_object_ref (node);
 
   return context;
 }
@@ -1314,22 +1296,6 @@ gtk_style_context_list_classes (GtkStyleContext *context)
   return classes_list;
 }
 
-static gint
-style_property_values_cmp (gconstpointer bsearch_node1,
-                           gconstpointer bsearch_node2)
-{
-  const PropertyValue *val1 = bsearch_node1;
-  const PropertyValue *val2 = bsearch_node2;
-
-  if (val1->widget_type != val2->widget_type)
-    return val1->widget_type < val2->widget_type ? -1 : 1;
-
-  if (val1->pspec != val2->pspec)
-    return val1->pspec < val2->pspec ? -1 : 1;
-
-  return 0;
-}
-
 GtkCssValue *
 _gtk_style_context_peek_property (GtkStyleContext *context,
                                   guint            property_id)
@@ -1339,267 +1305,6 @@ _gtk_style_context_peek_property (GtkStyleContext *context,
   return gtk_css_style_get_value (values, property_id);
 }
 
-const GValue *
-_gtk_style_context_peek_style_property (GtkStyleContext *context,
-                                        GType            widget_type,
-                                        GParamSpec      *pspec)
-{
-  GtkStyleContextPrivate *priv;
-  GtkWidgetPath *path;
-  PropertyValue *pcache, key = { 0 };
-  guint i;
-
-  priv = context->priv;
-
-  /* ensure the style cache is valid by forcing a validation */
-  gtk_style_context_lookup_style (context);
-
-  key.widget_type = widget_type;
-  key.pspec = pspec;
-
-  /* need value cache array */
-  pcache = bsearch (&key,
-                    priv->property_cache->data, priv->property_cache->len,
-                    sizeof (PropertyValue), style_property_values_cmp);
-  if (pcache)
-    return &pcache->value;
-
-  i = 0;
-  while (i < priv->property_cache->len &&
-         style_property_values_cmp (&key, &g_array_index (priv->property_cache, PropertyValue, i)) >= 0)
-    i++;
-
-  g_array_insert_val (priv->property_cache, i, key);
-  pcache = &g_array_index (priv->property_cache, PropertyValue, i);
-
-  /* cache miss, initialize value type, then set contents */
-  g_param_spec_ref (pcache->pspec);
-  g_value_init (&pcache->value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-
-  path = gtk_css_node_create_widget_path (gtk_style_context_get_root (context));
-  if (path && gtk_widget_path_length (path) > 0)
-    {
-      if (gtk_style_provider_get_style_property (GTK_STYLE_PROVIDER (priv->cascade),
-                                                 path,
-                                                 gtk_widget_path_iter_get_state (path, -1),
-                                                 pspec, &pcache->value))
-        {
-          gtk_widget_path_unref (path);
-
-          return &pcache->value;
-        }
-    }
-
-  gtk_widget_path_unref (path);
-
-  /* not supplied by any provider, revert to default */
-  g_param_value_set_default (pspec, &pcache->value);
-
-  return &pcache->value;
-}
-
-/**
- * gtk_style_context_get_style_property:
- * @context: a #GtkStyleContext
- * @property_name: the name of the widget style property
- * @value: Return location for the property value
- *
- * Gets the value for a widget style property.
- *
- * When @value is no longer needed, g_value_unset() must be called
- * to free any allocated memory.
- **/
-void
-gtk_style_context_get_style_property (GtkStyleContext *context,
-                                      const gchar     *property_name,
-                                      GValue          *value)
-{
-  GtkCssNode *root;
-  GtkWidgetClass *widget_class;
-  GParamSpec *pspec;
-  const GValue *peek_value;
-  GType widget_type;
-
-  g_return_if_fail (GTK_IS_STYLE_CONTEXT (context));
-  g_return_if_fail (property_name != NULL);
-  g_return_if_fail (value != NULL);
-
-  root = gtk_style_context_get_root (context);
-
-  if (GTK_IS_CSS_WIDGET_NODE (root))
-    {
-      GtkWidget *widget;
-
-      widget = gtk_css_widget_node_get_widget (GTK_CSS_WIDGET_NODE (root));
-      if (widget == NULL)
-        return;
-
-      widget_type = G_OBJECT_TYPE (widget);
-    }
-  else if (GTK_IS_CSS_PATH_NODE (root))
-    {
-      GtkWidgetPath *path;
-
-      path = gtk_css_path_node_get_widget_path (GTK_CSS_PATH_NODE (root));
-      if (path == NULL)
-        return;
-
-      widget_type = gtk_widget_path_get_object_type (path);
-
-      if (!g_type_is_a (widget_type, GTK_TYPE_WIDGET))
-        {
-          g_warning ("%s: can't get style properties for non-widget class '%s'",
-                     G_STRLOC,
-                     g_type_name (widget_type));
-          return;
-        }
-    }
-  else
-    {
-      return;
-    }
-
-  widget_class = g_type_class_ref (widget_type);
-  pspec = gtk_widget_class_find_style_property (widget_class, property_name);
-  g_type_class_unref (widget_class);
-
-  if (!pspec)
-    {
-      g_warning ("%s: widget class '%s' has no style property named '%s'",
-                 G_STRLOC,
-                 g_type_name (widget_type),
-                 property_name);
-      return;
-    }
-
-  peek_value = _gtk_style_context_peek_style_property (context, widget_type, pspec);
-
-  if (G_VALUE_TYPE (value) == G_VALUE_TYPE (peek_value))
-    g_value_copy (peek_value, value);
-  else if (g_value_type_transformable (G_VALUE_TYPE (peek_value), G_VALUE_TYPE (value)))
-    g_value_transform (peek_value, value);
-  else
-    g_warning ("can't retrieve style property '%s' of type '%s' as value of type '%s'",
-               pspec->name,
-               G_VALUE_TYPE_NAME (peek_value),
-               G_VALUE_TYPE_NAME (value));
-}
-
-/**
- * gtk_style_context_get_style_valist:
- * @context: a #GtkStyleContext
- * @args: va_list of property name/return location pairs, followed by %NULL
- *
- * Retrieves several widget style properties from @context according to the
- * current style.
- *
- * Since: 3.0
- **/
-void
-gtk_style_context_get_style_valist (GtkStyleContext *context,
-                                    va_list          args)
-{
-  GtkCssNode *root;
-  const gchar *prop_name;
-  GType widget_type;
-
-  g_return_if_fail (GTK_IS_STYLE_CONTEXT (context));
-
-  prop_name = va_arg (args, const gchar *);
-  root = gtk_style_context_get_root (context);
-
-  if (GTK_IS_CSS_WIDGET_NODE (root))
-    {
-      GtkWidget *widget;
-
-      widget = gtk_css_widget_node_get_widget (GTK_CSS_WIDGET_NODE (root));
-      if (widget == NULL)
-        return;
-
-      widget_type = G_OBJECT_TYPE (widget);
-    }
-  else if (GTK_IS_CSS_PATH_NODE (root))
-    {
-      GtkWidgetPath *path;
-
-      path = gtk_css_path_node_get_widget_path (GTK_CSS_PATH_NODE (root));
-      if (path == NULL)
-        return;
-
-      widget_type = gtk_widget_path_get_object_type (path);
-
-      if (!g_type_is_a (widget_type, GTK_TYPE_WIDGET))
-        {
-          g_warning ("%s: can't get style properties for non-widget class '%s'",
-                     G_STRLOC,
-                     g_type_name (widget_type));
-          return;
-        }
-    }
-  else
-    {
-      return;
-    }
-
-  while (prop_name)
-    {
-      GtkWidgetClass *widget_class;
-      GParamSpec *pspec;
-      const GValue *peek_value;
-      gchar *error;
-
-      widget_class = g_type_class_ref (widget_type);
-      pspec = gtk_widget_class_find_style_property (widget_class, prop_name);
-      g_type_class_unref (widget_class);
-
-      if (!pspec)
-        {
-          g_warning ("%s: widget class '%s' has no style property named '%s'",
-                     G_STRLOC,
-                     g_type_name (widget_type),
-                     prop_name);
-          break;
-        }
-
-      peek_value = _gtk_style_context_peek_style_property (context, widget_type, pspec);
-
-      G_VALUE_LCOPY (peek_value, args, 0, &error);
-
-      if (error)
-        {
-          g_warning ("can't retrieve style property '%s' of type '%s': %s",
-                     pspec->name,
-                     G_VALUE_TYPE_NAME (peek_value),
-                     error);
-          g_free (error);
-          break;
-        }
-
-      prop_name = va_arg (args, const gchar *);
-    }
-}
-
-/**
- * gtk_style_context_get_style:
- * @context: a #GtkStyleContext
- * @...: property name /return value pairs, followed by %NULL
- *
- * Retrieves several widget style properties from @context according to the
- * current style.
- *
- * Since: 3.0
- **/
-void
-gtk_style_context_get_style (GtkStyleContext *context,
-                             ...)
-{
-  va_list args;
-
-  va_start (args, context);
-  gtk_style_context_get_style_valist (context, args);
-  va_end (args);
-}
-
 /**
  * gtk_style_context_set_screen:
  * @context: a #GtkStyleContext
