@@ -1048,17 +1048,6 @@ GDK_AVAILABLE_IN_ALL
 gboolean gtk_style_context_has_class    (GtkStyleContext *context,
                                          const gchar     *class_name);
 
-GDK_AVAILABLE_IN_ALL
-void gtk_style_context_get_style_property (GtkStyleContext *context,
-                                           const gchar     *property_name,
-                                           GValue          *value);
-GDK_AVAILABLE_IN_ALL
-void gtk_style_context_get_style_valist   (GtkStyleContext *context,
-                                           va_list          args);
-GDK_AVAILABLE_IN_ALL
-void gtk_style_context_get_style          (GtkStyleContext *context,
-                                           ...);
-
 GDK_AVAILABLE_IN_ALL
 void        gtk_style_context_set_screen (GtkStyleContext *context,
                                           GdkScreen       *screen);
@@ -1071,12 +1060,6 @@ void           gtk_style_context_set_frame_clock (GtkStyleContext *context,
 GDK_AVAILABLE_IN_3_8
 GdkFrameClock *gtk_style_context_get_frame_clock (GtkStyleContext *context);
 
-GDK_AVAILABLE_IN_ALL
-void             gtk_style_context_set_junction_sides (GtkStyleContext  *context,
-                                                       GtkJunctionSides  sides);
-GDK_AVAILABLE_IN_ALL
-GtkJunctionSides gtk_style_context_get_junction_sides (GtkStyleContext  *context);
-
 GDK_AVAILABLE_IN_ALL
 gboolean gtk_style_context_lookup_color (GtkStyleContext *context,
                                          const gchar     *color_name,
