@@ -49,10 +49,6 @@ struct _GtkStylePropertyClass
 {
   GObjectClass  parent_class;
   
-  void              (* assign)                             (GtkStyleProperty       *property,
-                                                            GtkStyleProperties     *props,
-                                                            GtkStateFlags           state,
-                                                            const GValue           *value);
   void              (* query)                              (GtkStyleProperty       *property,
                                                             GValue                 *value,
                                                             GtkStyleQueryFunc       query_func,
@@ -67,6 +63,9 @@ GType               _gtk_style_property_get_type             (void) G_GNUC_CONST
 
 void                _gtk_style_property_init_properties      (void);
 
+void                _gtk_style_property_add_alias (const gchar *name,
+                                                   const gchar *alias);
+
 GtkStyleProperty *       _gtk_style_property_lookup        (const char             *name);
 
 const char *             _gtk_style_property_get_name      (GtkStyleProperty       *property);
@@ -79,10 +78,6 @@ void                     _gtk_style_property_query         (GtkStyleProperty *
                                                             GValue                 *value,
                                                             GtkStyleQueryFunc       query_func,
                                                             gpointer                query_data);
-void                     _gtk_style_property_assign        (GtkStyleProperty       *property,
-                                                            GtkStyleProperties     *props,
-                                                            GtkStateFlags           state,
-                                                            const GValue           *value);
 
 G_END_DECLS
 
