@@ -40,9 +40,9 @@
  *
  * |[<!-- language="plain" -->
  * switch
- * ├── slider
  * ├── label
- * ╰── label
+ * ├── label
+ * ╰── slider
  * ]|
  *
  * GtkSwitch has four css nodes, the main node with the name switch and subnodes
@@ -61,8 +61,6 @@
 #include "gtkactionable.h"
 #include "a11y/gtkswitchaccessible.h"
 #include "gtkactionhelper.h"
-#include "gtkcsscustomgadgetprivate.h"
-#include "gtkcssgadgetprivate.h"
 #include "gtkstylecontextprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtkcssshadowsvalueprivate.h"
@@ -70,6 +68,7 @@
 #include "gtkprogresstrackerprivate.h"
 #include "gtksettingsprivate.h"
 #include "gtkcontainerprivate.h"
+#include "gtkgizmoprivate.h"
 
 #include "fallback-c89.c"
 
@@ -80,15 +79,12 @@ struct _GtkSwitchPrivate
   GtkGesture *pan_gesture;
   GtkGesture *multipress_gesture;
 
-  GtkCssGadget *gadget;
-
   double handle_pos;
   guint tick_id;
   GtkProgressTracker tracker;
 
   guint state                 : 1;
   guint is_active             : 1;
-  guint in_switch             : 1;
 
   GtkWidget *on_label;
   GtkWidget *off_label;
@@ -194,7 +190,7 @@ gtk_switch_multipress_gesture_pressed (GtkGestureMultiPress *gesture,
   GtkSwitchPrivate *priv = sw->priv;
   GtkAllocation allocation;
 
-  gtk_widget_get_allocation (GTK_WIDGET (sw), &allocation);
+  gtk_widget_get_outer_allocation (GTK_WIDGET (sw), &allocation);
   gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
 
   /* If the press didn't happen in the draggable handle,
@@ -212,12 +208,14 @@ gtk_switch_multipress_gesture_released (GtkGestureMultiPress *gesture,
                                         gdouble               y,
                                         GtkSwitch            *sw)
 {
-  GtkSwitchPrivate *priv = sw->priv;
   GdkEventSequence *sequence;
+  GdkRectangle own_alloc;
 
   sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
 
-  if (priv->in_switch &&
+  gtk_widget_get_own_allocation (GTK_WIDGET (sw), &own_alloc);
+
+  if (gdk_rectangle_contains_point (&own_alloc, x, y) &&
       gtk_gesture_handles_sequence (GTK_GESTURE (gesture), sequence))
     gtk_switch_begin_toggle_animation (sw);
 }
@@ -231,13 +229,14 @@ gtk_switch_pan_gesture_pan (GtkGesturePan   *gesture,
   GtkWidget *widget = GTK_WIDGET (sw);
   GtkSwitchPrivate *priv = sw->priv;
   gint width;
+  int height;
 
   if (direction == GTK_PAN_DIRECTION_LEFT)
     offset = -offset;
 
   gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
 
-  width = gtk_widget_get_allocated_width (widget);
+  gtk_widget_get_content_size (widget, &width, &height);
 
   if (priv->is_active)
     offset += width / 2;
@@ -279,30 +278,6 @@ gtk_switch_pan_gesture_drag_end (GtkGestureDrag *gesture,
   gtk_widget_queue_allocate (GTK_WIDGET (sw));
 }
 
-static gboolean
-gtk_switch_enter (GtkWidget        *widget,
-                  GdkEventCrossing *event)
-{
-  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
-
-  priv->in_switch = TRUE;
-  gtk_widget_set_state_flags (widget, GTK_STATE_FLAG_PRELIGHT, FALSE);
-
-  return FALSE;
-}
-
-static gboolean
-gtk_switch_leave (GtkWidget        *widget,
-                  GdkEventCrossing *event)
-{
-  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
-
-  priv->in_switch = FALSE;
-  gtk_widget_unset_state_flags (widget, GTK_STATE_FLAG_PRELIGHT);
-
-  return FALSE;
-}
-
 static void
 gtk_switch_activate (GtkSwitch *sw)
 {
@@ -310,22 +285,19 @@ gtk_switch_activate (GtkSwitch *sw)
 }
 
 static void
-gtk_switch_get_content_size (GtkCssGadget   *gadget,
-                             GtkOrientation  orientation,
-                             gint            for_size,
-                             gint           *minimum,
-                             gint           *natural,
-                             gint           *minimum_baseline,
-                             gint           *natural_baseline,
-                             gpointer        unused)
+gtk_switch_measure (GtkWidget      *widget,
+                    GtkOrientation  orientation,
+                    int             for_size,
+                    int            *minimum,
+                    int            *natural,
+                    int            *minimum_baseline,
+                    int            *natural_baseline)
 {
-  GtkWidget *widget;
   GtkSwitch *self;
   GtkSwitchPrivate *priv;
   gint slider_minimum, slider_natural;
   int on_nat, off_nat;
 
-  widget = gtk_css_gadget_get_owner (gadget);
   self = GTK_SWITCH (widget);
   priv = self->priv;
 
@@ -351,101 +323,47 @@ gtk_switch_get_content_size (GtkCssGadget   *gadget,
 }
 
 static void
-gtk_switch_measure (GtkWidget      *widget,
-                    GtkOrientation  orientation,
-                    int             for_size,
-                    int            *minimum,
-                    int            *natural,
-                    int            *minimum_baseline,
-                    int            *natural_baseline)
+gtk_switch_size_allocate (GtkWidget           *widget,
+                          const GtkAllocation *allocation,
+                          int                  baseline,
+                          GtkAllocation       *out_clip)
 {
-  gtk_css_gadget_get_preferred_size (GTK_SWITCH (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
-}
-
-static void
-gtk_switch_allocate_contents (GtkCssGadget        *gadget,
-                              const GtkAllocation *allocation,
-                              int                  baseline,
-                              GtkAllocation       *out_clip,
-                              gpointer             unused)
-{
-  GtkSwitch *self = GTK_SWITCH (gtk_css_gadget_get_owner (gadget));
-  GtkSwitchPrivate *priv = self->priv;
+  GtkSwitch *self = GTK_SWITCH (widget);
+  GtkSwitchPrivate *priv = gtk_switch_get_instance_private (self);
+  GtkAllocation child_clip;
   GtkAllocation child_alloc;
   GtkAllocation slider_alloc;
   int min;
 
-  slider_alloc.x = allocation->x + round (priv->handle_pos * (allocation->width - allocation->width / 2));
-  slider_alloc.y = allocation->y;
+  slider_alloc.x = round (priv->handle_pos * (allocation->width / 2));
+  slider_alloc.y = 0;
   slider_alloc.width = allocation->width / 2;
   slider_alloc.height = allocation->height;
 
-  gtk_widget_size_allocate (priv->slider, &slider_alloc);
+  gtk_widget_size_allocate (priv->slider, &slider_alloc, -1, &child_clip);
+  gtk_widget_get_clip (priv->slider, &child_clip);
+  gdk_rectangle_union (out_clip, &child_clip, out_clip);
+
 
   /* Center ON label in left half */
   gtk_widget_measure (priv->on_label, GTK_ORIENTATION_HORIZONTAL, -1, &min, NULL, NULL, NULL);
-  child_alloc.x = allocation->x + ((allocation->width / 2) - min) / 2;
+  child_alloc.x = ((allocation->width / 2) - min) / 2;
   child_alloc.width = min;
   gtk_widget_measure (priv->on_label, GTK_ORIENTATION_VERTICAL, min, &min, NULL, NULL, NULL);
-  child_alloc.y = allocation->y + (allocation->height - min) / 2;
+  child_alloc.y = (allocation->height - min) / 2;
   child_alloc.height = min;
-  gtk_widget_size_allocate (priv->on_label, &child_alloc);
-
+  gtk_widget_size_allocate (priv->on_label, &child_alloc, -1, &child_clip);
+  gdk_rectangle_union (out_clip, &child_clip, out_clip);
 
   /* Center OFF label in right half */
   gtk_widget_measure (priv->off_label, GTK_ORIENTATION_HORIZONTAL, -1, &min, NULL, NULL, NULL);
-  child_alloc.x = allocation->x + (allocation->width / 2) + ((allocation->width / 2) - min) / 2;
+  child_alloc.x = (allocation->width / 2) + ((allocation->width / 2) - min) / 2;
   child_alloc.width = min;
   gtk_widget_measure (priv->off_label, GTK_ORIENTATION_VERTICAL, min, &min, NULL, NULL, NULL);
-  child_alloc.y = allocation->y + (allocation->height - min) / 2;
+  child_alloc.y = (allocation->height - min) / 2;
   child_alloc.height = min;
-  gtk_widget_size_allocate (priv->off_label, &child_alloc);
-}
-
-static void
-gtk_switch_size_allocate (GtkWidget     *widget,
-                          GtkAllocation *allocation)
-{
-  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
-  GtkAllocation clip;
-
-  gtk_widget_set_allocation (widget, allocation);
-  gtk_css_gadget_allocate (priv->gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static gboolean
-gtk_switch_snapshot_trough (GtkCssGadget *gadget,
-                            GtkSnapshot  *snapshot,
-                            int           x,
-                            int           y,
-                            int           width,
-                            int           height,
-                            gpointer      data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
-
-  gtk_widget_snapshot_child (widget, priv->on_label, snapshot);
-  gtk_widget_snapshot_child (widget, priv->off_label, snapshot);
-  gtk_widget_snapshot_child (widget, priv->slider, snapshot);
-
-  return FALSE;
-}
-
-static void
-gtk_switch_snapshot (GtkWidget   *widget,
-                     GtkSnapshot *snapshot)
-{
-  gtk_css_gadget_snapshot (GTK_SWITCH (widget)->priv->gadget, snapshot);
+  gtk_widget_size_allocate (priv->off_label, &child_alloc, -1, &child_clip);
+  gdk_rectangle_union (out_clip, &child_clip, out_clip);
 }
 
 static void
@@ -565,7 +483,6 @@ gtk_switch_dispose (GObject *object)
   GtkSwitchPrivate *priv = GTK_SWITCH (object)->priv;
 
   g_clear_object (&priv->action_helper);
-  g_clear_object (&priv->gadget);
 
   g_clear_object (&priv->pan_gesture);
   g_clear_object (&priv->multipress_gesture);
@@ -640,9 +557,6 @@ gtk_switch_class_init (GtkSwitchClass *klass)
 
   widget_class->measure = gtk_switch_measure;
   widget_class->size_allocate = gtk_switch_size_allocate;
-  widget_class->snapshot = gtk_switch_snapshot;
-  widget_class->enter_notify_event = gtk_switch_enter;
-  widget_class->leave_notify_event = gtk_switch_leave;
 
   klass->activate = gtk_switch_activate;
   klass->state_set = state_set;
@@ -713,25 +627,12 @@ gtk_switch_init (GtkSwitch *self)
 {
   GtkSwitchPrivate *priv;
   GtkGesture *gesture;
-  GtkCssNode *widget_node;
 
   priv = self->priv = gtk_switch_get_instance_private (self);
 
   gtk_widget_set_has_window (GTK_WIDGET (self), FALSE);
   gtk_widget_set_can_focus (GTK_WIDGET (self), TRUE);
 
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (self));
-  priv->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                     GTK_WIDGET (self),
-                                                     gtk_switch_get_content_size,
-                                                     gtk_switch_allocate_contents,
-                                                     gtk_switch_snapshot_trough,
-                                                     NULL,
-                                                     NULL);
-
-  priv->slider = g_object_new (GTK_TYPE_BUTTON, "css-name", "slider", NULL);
-  gtk_widget_set_parent (priv->slider, GTK_WIDGET (self));
-
   gesture = gtk_gesture_multi_press_new (GTK_WIDGET (self));
   gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (gesture), FALSE);
   gtk_gesture_single_set_exclusive (GTK_GESTURE_SINGLE (gesture), TRUE);
@@ -767,6 +668,9 @@ gtk_switch_init (GtkSwitch *self)
    */
   priv->off_label = gtk_label_new (C_("switch", "OFF"));
   gtk_widget_set_parent (priv->off_label, GTK_WIDGET (self));
+
+  priv->slider = gtk_gizmo_new ("slider", NULL, NULL, NULL);
+  gtk_widget_set_parent (priv->slider, GTK_WIDGET (self));
 }
 
 /**
