@@ -94,7 +94,7 @@ typedef void                (*GtkTextSegCheckFunc)      (GtkTextLineSegment *seg
                                                          GtkTextLine        *line);
 
 struct _GtkTextLineSegmentClass {
-  char *name;                           /* Name of this kind of segment. */
+  const char *name;                     /* Name of this kind of segment. */
   gboolean leftGravity;                 /* If a segment has zero size (e.g. a
                                          * mark or tag toggle), does it
                                          * attach to character to its left
@@ -164,6 +164,7 @@ GtkTextLineSegment *_gtk_char_segment_new_from_two_strings (const gchar    *text
 GtkTextLineSegment *_gtk_toggle_segment_new                (GtkTextTagInfo *info,
                                                             gboolean        on);
 
+void                _gtk_toggle_segment_free               (GtkTextLineSegment *seg);
 
 G_END_DECLS
 
