@@ -422,7 +422,6 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 	    {
               gfloat align;
 
-              gtk_label_set_angle (GTK_LABEL (label), 0);
               align = gtk_tool_item_get_text_alignment (GTK_TOOL_ITEM (button));
               if (align < 0.4)
                 gtk_widget_set_halign (label, GTK_ALIGN_START);
@@ -436,10 +435,6 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
               gfloat align;
 
               gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_NONE);
-	      if (gtk_widget_get_direction (GTK_WIDGET (tool_item)) == GTK_TEXT_DIR_RTL)
-	        gtk_label_set_angle (GTK_LABEL (label), -90);
-	      else
-	        gtk_label_set_angle (GTK_LABEL (label), 90);
               align = gtk_tool_item_get_text_alignment (GTK_TOOL_ITEM (button));
               if (align < 0.4)
                 gtk_widget_set_valign (label, GTK_ALIGN_END);
@@ -518,8 +513,8 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
       else
         box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,0);
       if (icon)
-          gtk_box_pack_start (GTK_BOX (box), icon, TRUE, TRUE);
-      gtk_box_pack_end (GTK_BOX (box), label, FALSE, TRUE);
+          gtk_box_pack_start (GTK_BOX (box), icon);
+      gtk_box_pack_end (GTK_BOX (box), label);
       gtk_container_add (GTK_CONTAINER (button->priv->button), box);
       gtk_style_context_add_class (gtk_widget_get_style_context (button->priv->button), "image-button");
       gtk_style_context_add_class (gtk_widget_get_style_context (button->priv->button), "text-button");
@@ -530,17 +525,17 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 	{
           box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	  if (icon)
-            gtk_box_pack_start (GTK_BOX (box), icon, label? FALSE : TRUE, TRUE);
+            gtk_box_pack_start (GTK_BOX (box), icon);
 	  if (label)
-            gtk_box_pack_end (GTK_BOX (box), label, TRUE, TRUE);
+            gtk_box_pack_end (GTK_BOX (box), label);
 	}
       else
 	{
           box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	  if (icon)
-            gtk_box_pack_end (GTK_BOX (box), icon, label ? FALSE : TRUE, TRUE);
+            gtk_box_pack_end (GTK_BOX (box), icon);
 	  if (label)
-            gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE);
+            gtk_box_pack_start (GTK_BOX (box), label);
 	}
       gtk_container_add (GTK_CONTAINER (button->priv->button), box);
       gtk_style_context_add_class (gtk_widget_get_style_context (button->priv->button), "image-button");
