@@ -386,7 +386,7 @@ gtk_tool_item_group_init (GtkToolItemGroup *group)
   GtkCssNode *widget_node;
 
   gtk_widget_set_redraw_on_allocate (GTK_WIDGET (group), FALSE);
-  gtk_widget_set_has_window (GTK_WIDGET (group), TRUE);
+  gtk_widget_set_has_window (GTK_WIDGET (group), FALSE);
 
   group->priv = priv = gtk_tool_item_group_get_instance_private (group);
 
@@ -879,8 +879,10 @@ gtk_tool_item_group_real_size_query (GtkWidget      *widget,
 }
 
 static void
-gtk_tool_item_group_real_size_allocate (GtkWidget     *widget,
-                                        GtkAllocation *allocation)
+gtk_tool_item_group_real_size_allocate (GtkWidget           *widget,
+                                        const GtkAllocation *allocation,
+                                        int                  baseline,
+                                        GtkAllocation       *out_clip)
 {
   GtkToolItemGroup *group = GTK_TOOL_ITEM_GROUP (widget);
   GtkToolItemGroupPrivate* priv = group->priv;
@@ -903,7 +905,8 @@ gtk_tool_item_group_real_size_allocate (GtkWidget     *widget,
   orientation = gtk_tool_shell_get_orientation (GTK_TOOL_SHELL (group));
 
   /* chain up */
-  GTK_WIDGET_CLASS (gtk_tool_item_group_parent_class)->size_allocate (widget, allocation);
+  GTK_WIDGET_CLASS (gtk_tool_item_group_parent_class)->size_allocate (widget, allocation,
+                                                                      baseline, out_clip);
 
   child_allocation.x = 0;
   child_allocation.y = 0;
@@ -928,7 +931,7 @@ gtk_tool_item_group_real_size_allocate (GtkWidget     *widget,
             child_allocation.x = allocation->width - child_allocation.width;
         }
 
-      gtk_widget_size_allocate (priv->header, &child_allocation);
+      gtk_widget_size_allocate (priv->header, &child_allocation, -1, out_clip);
 
       if (GTK_ORIENTATION_VERTICAL == orientation)
         child_allocation.y += child_allocation.height;
@@ -1055,7 +1058,7 @@ gtk_tool_item_group_real_size_allocate (GtkWidget     *widget,
 
           child_allocation.height = item_size.height;
 
-          gtk_widget_size_allocate (GTK_WIDGET (child->item), &child_allocation);
+          gtk_widget_size_allocate (GTK_WIDGET (child->item), &child_allocation, -1, out_clip);
           gtk_widget_set_child_visible (GTK_WIDGET (child->item), TRUE);
         }
 
@@ -1076,12 +1079,12 @@ gtk_tool_item_group_real_size_allocate (GtkWidget     *widget,
 }
 
 static void
-gtk_tool_item_group_size_allocate (GtkWidget     *widget,
-                                   GtkAllocation *allocation)
+gtk_tool_item_group_size_allocate (GtkWidget           *widget,
+                                   const GtkAllocation *allocation,
+                                   int                  baseline,
+                                   GtkAllocation       *out_clip)
 {
-  gtk_tool_item_group_real_size_allocate (widget, allocation);
-
-  gtk_widget_queue_draw (widget);
+  gtk_tool_item_group_real_size_allocate (widget, allocation, baseline, out_clip);
 }
 
 static void
@@ -1200,26 +1203,9 @@ gtk_tool_item_group_set_toplevel_window (GtkToolItemGroup *group,
 static void
 gtk_tool_item_group_realize (GtkWidget *widget)
 {
-  GtkAllocation allocation;
   GtkWidget *toplevel_window;
-  GdkWindow *window;
 
-  gtk_widget_set_realized (widget, TRUE);
-
-  gtk_widget_get_allocation (widget, &allocation);
-
-  window = gdk_window_new_child (gtk_widget_get_parent_window (widget),
-                                 GDK_ALL_EVENTS_MASK,
-                                 &allocation);
-  gtk_widget_set_window (widget, window);
-
-  gtk_widget_register_window (widget, window);
-
-  gtk_container_forall (GTK_CONTAINER (widget),
-                        (GtkCallback) gtk_widget_set_parent_window,
-                        window);
-
-  gtk_widget_queue_resize_no_redraw (widget);
+  GTK_WIDGET_CLASS (gtk_tool_item_group_parent_class)->realize (widget);
 
   toplevel_window = gtk_widget_get_ancestor (widget, GTK_TYPE_WINDOW);
   gtk_tool_item_group_set_toplevel_window (GTK_TOOL_ITEM_GROUP (widget),
@@ -1233,18 +1219,6 @@ gtk_tool_item_group_unrealize (GtkWidget *widget)
   GTK_WIDGET_CLASS (gtk_tool_item_group_parent_class)->unrealize (widget);
 }
 
-static gboolean
-gtk_tool_item_group_draw (GtkWidget *widget,
-                          cairo_t   *cr)
-{
-  gtk_render_background (gtk_widget_get_style_context (widget), cr,
-                         0, 0,
-                         gtk_widget_get_allocated_width (widget),
-                         gtk_widget_get_allocated_height (widget));
-
-  return GTK_WIDGET_CLASS (gtk_tool_item_group_parent_class)->draw (widget, cr);
-}
-
 static void
 gtk_tool_item_group_style_updated (GtkWidget *widget)
 {
@@ -1568,7 +1542,6 @@ gtk_tool_item_group_class_init (GtkToolItemGroupClass *cls)
   wclass->unrealize            = gtk_tool_item_group_unrealize;
   wclass->style_updated        = gtk_tool_item_group_style_updated;
   wclass->screen_changed       = gtk_tool_item_group_screen_changed;
-  wclass->draw                 = gtk_tool_item_group_draw;
   wclass->state_flags_changed  = gtk_tool_item_group_state_flags_changed;
 
   cclass->add                = gtk_tool_item_group_add;
