@@ -29,6 +29,7 @@
 #include "gtkprivate.h"
 #include "gtkscrollable.h"
 #include "gtkorientableprivate.h"
+#include "gtkwidgetprivate.h"
 #include "gtkintl.h"
 
 #define DEFAULT_ICON_SIZE       GTK_ICON_SIZE_SMALL_TOOLBAR
@@ -471,8 +472,10 @@ gtk_tool_palette_measure (GtkWidget      *widget,
 
 
 static void
-gtk_tool_palette_size_allocate (GtkWidget     *widget,
-                                GtkAllocation *allocation)
+gtk_tool_palette_size_allocate (GtkWidget           *widget,
+                                const GtkAllocation *allocation,
+                                int                  baseline,
+                                GtkAllocation       *out_clip)
 {
   GtkToolPalette *palette = GTK_TOOL_PALETTE (widget);
   GtkAdjustment *adjustment = NULL;
@@ -495,7 +498,8 @@ gtk_tool_palette_size_allocate (GtkWidget     *widget,
 
   direction = gtk_widget_get_direction (widget);
 
-  GTK_WIDGET_CLASS (gtk_tool_palette_parent_class)->size_allocate (widget, allocation);
+  GTK_WIDGET_CLASS (gtk_tool_palette_parent_class)->size_allocate (widget, allocation,
+                                                                   baseline, out_clip);
 
   if (GTK_ORIENTATION_VERTICAL == palette->priv->orientation)
     {
@@ -639,7 +643,7 @@ gtk_tool_palette_size_allocate (GtkWidget     *widget,
           else
             child_allocation.x = x;
 
-          gtk_widget_size_allocate (GTK_WIDGET (group->widget), &child_allocation);
+          gtk_widget_size_allocate (GTK_WIDGET (group->widget), &child_allocation, -1, out_clip);
           gtk_widget_show (GTK_WIDGET (group->widget));
 
           if (GTK_ORIENTATION_VERTICAL == palette->priv->orientation)
@@ -701,22 +705,11 @@ gtk_tool_palette_adjustment_value_changed (GtkAdjustment *adjustment,
                                            gpointer       data)
 {
   GtkAllocation allocation;
+  GtkAllocation clip;
   GtkWidget *widget = GTK_WIDGET (data);
 
   gtk_widget_get_allocation (widget, &allocation);
-  gtk_tool_palette_size_allocate (widget, &allocation);
-}
-
-static gboolean
-gtk_tool_palette_draw (GtkWidget *widget,
-                       cairo_t   *cr)
-{
-  gtk_render_background (gtk_widget_get_style_context (widget), cr,
-                         0, 0,
-                         gtk_widget_get_allocated_width (widget),
-                         gtk_widget_get_allocated_height (widget));
-
-  return GTK_WIDGET_CLASS (gtk_tool_palette_parent_class)->draw (widget, cr);
+  gtk_tool_palette_size_allocate (widget, &allocation, -1, &clip);
 }
 
 static void
@@ -854,6 +847,21 @@ gtk_tool_palette_screen_changed (GtkWidget *widget,
   gtk_tool_palette_reconfigured (palette);
 }
 
+static void
+gtk_tool_palette_snapshot (GtkWidget   *widget,
+                           GtkSnapshot *snapshot)
+{
+  int width, height;
+
+  gtk_widget_get_content_size (widget, &width, &height);
+
+  gtk_snapshot_push_clip (snapshot,
+                          &GRAPHENE_RECT_INIT(0, 0, width, height), "ToolPalette Clip");
+
+  GTK_WIDGET_CLASS (gtk_tool_palette_parent_class)->snapshot (widget, snapshot);
+
+  gtk_snapshot_pop (snapshot);
+}
 
 static void
 gtk_tool_palette_class_init (GtkToolPaletteClass *cls)
@@ -869,7 +877,7 @@ gtk_tool_palette_class_init (GtkToolPaletteClass *cls)
 
   wclass->measure             = gtk_tool_palette_measure;
   wclass->size_allocate       = gtk_tool_palette_size_allocate;
-  wclass->draw                = gtk_tool_palette_draw;
+  wclass->snapshot            = gtk_tool_palette_snapshot;
 
   cclass->add                 = gtk_tool_palette_add;
   cclass->remove              = gtk_tool_palette_remove;
