@@ -67,10 +67,16 @@ struct _GtkToolPalette
   GtkToolPalettePrivate *priv;
 };
 
+/**
+ * GtkToolPaletteClass:
+ * @parent_class: The parent class.
+ */
 struct _GtkToolPaletteClass
 {
   GtkContainerClass parent_class;
 
+  /*< private >*/
+
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
@@ -78,60 +84,75 @@ struct _GtkToolPaletteClass
   void (*_gtk_reserved4) (void);
 };
 
+GDK_AVAILABLE_IN_ALL
 GType                          gtk_tool_palette_get_type              (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 GtkWidget*                     gtk_tool_palette_new                   (void);
 
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_set_group_position    (GtkToolPalette            *palette,
                                                                        GtkToolItemGroup          *group,
                                                                        gint                       position);
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_set_exclusive         (GtkToolPalette            *palette,
                                                                        GtkToolItemGroup          *group,
                                                                        gboolean                   exclusive);
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_set_expand            (GtkToolPalette            *palette,
                                                                        GtkToolItemGroup          *group,
                                                                        gboolean                   expand);
 
+GDK_AVAILABLE_IN_ALL
 gint                           gtk_tool_palette_get_group_position    (GtkToolPalette            *palette,
                                                                        GtkToolItemGroup          *group);
+GDK_AVAILABLE_IN_ALL
 gboolean                       gtk_tool_palette_get_exclusive         (GtkToolPalette            *palette,
                                                                        GtkToolItemGroup          *group);
+GDK_AVAILABLE_IN_ALL
 gboolean                       gtk_tool_palette_get_expand            (GtkToolPalette            *palette,
                                                                        GtkToolItemGroup          *group);
 
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_set_icon_size         (GtkToolPalette            *palette,
                                                                        GtkIconSize                icon_size);
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_unset_icon_size       (GtkToolPalette            *palette);
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_set_style             (GtkToolPalette            *palette,
                                                                        GtkToolbarStyle            style);
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_unset_style           (GtkToolPalette            *palette);
 
+GDK_AVAILABLE_IN_ALL
 GtkIconSize                    gtk_tool_palette_get_icon_size         (GtkToolPalette            *palette);
+GDK_AVAILABLE_IN_ALL
 GtkToolbarStyle                gtk_tool_palette_get_style             (GtkToolPalette            *palette);
 
+GDK_AVAILABLE_IN_ALL
 GtkToolItem*                   gtk_tool_palette_get_drop_item         (GtkToolPalette            *palette,
                                                                        gint                       x,
                                                                        gint                       y);
+GDK_AVAILABLE_IN_ALL
 GtkToolItemGroup*              gtk_tool_palette_get_drop_group        (GtkToolPalette            *palette,
                                                                        gint                       x,
                                                                        gint                       y);
+GDK_AVAILABLE_IN_ALL
 GtkWidget*                     gtk_tool_palette_get_drag_item         (GtkToolPalette            *palette,
                                                                        const GtkSelectionData    *selection);
 
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_set_drag_source       (GtkToolPalette            *palette,
                                                                        GtkToolPaletteDragTargets  targets);
+GDK_AVAILABLE_IN_ALL
 void                           gtk_tool_palette_add_drag_dest         (GtkToolPalette            *palette,
                                                                        GtkWidget                 *widget,
                                                                        GtkDestDefaults            flags,
                                                                        GtkToolPaletteDragTargets  targets,
                                                                        GdkDragAction              actions);
 
-
-GDK_DEPRECATED_IN_3_0_FOR(gtk_scrollable_get_hadjustment)
-GtkAdjustment*                 gtk_tool_palette_get_hadjustment       (GtkToolPalette            *palette);
-GDK_DEPRECATED_IN_3_0_FOR(gtk_scrollable_get_vadjustment)
-GtkAdjustment*                 gtk_tool_palette_get_vadjustment       (GtkToolPalette            *palette);
-
+GDK_AVAILABLE_IN_ALL
 const GtkTargetEntry*          gtk_tool_palette_get_drag_target_item  (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 const GtkTargetEntry*          gtk_tool_palette_get_drag_target_group (void) G_GNUC_CONST;
 
 
