@@ -93,8 +93,7 @@ gboolean          _gtk_tree_selection_row_is_selectable  (GtkTreeSelection *sele
 
 
 void _gtk_tree_view_column_realize_button   (GtkTreeViewColumn *column);
-void _gtk_tree_view_column_unrealize_button (GtkTreeViewColumn *column);
- 
+
 void _gtk_tree_view_column_set_tree_view    (GtkTreeViewColumn *column,
 					     GtkTreeView       *tree_view);
 gint _gtk_tree_view_column_request_width    (GtkTreeViewColumn *tree_column);
@@ -133,13 +132,15 @@ void		  gtk_tree_view_column_cell_snapshot     (GtkTreeViewColumn  *tree_column,
 void		  _gtk_tree_view_column_cell_set_dirty	 (GtkTreeViewColumn  *tree_column,
 							  gboolean            install_handler);
 gboolean          _gtk_tree_view_column_cell_get_dirty   (GtkTreeViewColumn  *tree_column);
-GdkWindow        *_gtk_tree_view_column_get_window       (GtkTreeViewColumn  *column);
 
 void              _gtk_tree_view_column_push_padding          (GtkTreeViewColumn  *column,
 							       gint                padding);
 gint              _gtk_tree_view_column_get_requested_width   (GtkTreeViewColumn  *column);
 gint              _gtk_tree_view_column_get_drag_x            (GtkTreeViewColumn  *column);
 GtkCellAreaContext *_gtk_tree_view_column_get_context         (GtkTreeViewColumn  *column);
+gboolean         _gtk_tree_view_column_coords_in_resize_rect  (GtkTreeViewColumn *column,
+                                                               double             x,
+                                                               double             y);
 
 
 G_END_DECLS
