@@ -27,7 +27,6 @@
 #include "gtkviewport.h"
 
 #include "gtkadjustment.h"
-#include "gtkcsscustomgadgetprivate.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
@@ -72,10 +71,6 @@ struct _GtkViewportPrivate
   GtkAdjustment  *vadjustment;
   GtkShadowType   shadow_type;
 
-  GdkWindow      *bin_window;
-
-  GtkCssGadget *gadget;
-
   /* GtkScrollablePolicy needs to be checked when
    * driving the scrollable adjustment values */
   guint hscroll_policy : 1;
@@ -100,16 +95,13 @@ static void gtk_viewport_get_property             (GObject         *object,
 						   guint            prop_id,
 						   GValue          *value,
 						   GParamSpec      *pspec);
-static void gtk_viewport_finalize                 (GObject         *object);
 static void gtk_viewport_destroy                  (GtkWidget        *widget);
-static void gtk_viewport_realize                  (GtkWidget        *widget);
-static void gtk_viewport_unrealize                (GtkWidget        *widget);
 static void gtk_viewport_snapshot                 (GtkWidget        *widget,
 						   GtkSnapshot      *snapshot);
-static void gtk_viewport_add                      (GtkContainer     *container,
-						   GtkWidget        *widget);
-static void gtk_viewport_size_allocate            (GtkWidget        *widget,
-						   GtkAllocation    *allocation);
+static void gtk_viewport_size_allocate            (GtkWidget           *widget,
+                                                   const GtkAllocation *allocation,
+                                                   int                  baseline,
+                                                   GtkAllocation       *out_clip);
 static void gtk_viewport_adjustment_value_changed (GtkAdjustment    *adjustment,
 						   gpointer          data);
 static void viewport_set_adjustment               (GtkViewport      *viewport,
@@ -120,30 +112,6 @@ G_DEFINE_TYPE_WITH_CODE (GtkViewport, gtk_viewport, GTK_TYPE_BIN,
                          G_ADD_PRIVATE (GtkViewport)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_SCROLLABLE, NULL))
 
-static void
-gtk_viewport_measure (GtkCssGadget   *gadget,
-                      GtkOrientation  orientation,
-                      int             for_size,
-                      int            *minimum,
-                      int            *natural,
-                      int            *minimum_baseline,
-                      int            *natural_baseline,
-                      gpointer        data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkWidget *child;
-
-  *minimum = *natural = 0;
-
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child && gtk_widget_get_visible (child))
-    gtk_widget_measure (child,
-                        orientation,
-                        for_size,
-                        minimum, natural,
-                        NULL, NULL);
-}
-
 static void
 viewport_set_adjustment_values (GtkViewport    *viewport,
                                 GtkOrientation  orientation)
@@ -154,20 +122,19 @@ viewport_set_adjustment_values (GtkViewport    *viewport,
   GtkScrollablePolicy scroll_policy;
   GtkScrollablePolicy other_scroll_policy;
   GtkOrientation other_orientation;
-  GtkAllocation view_allocation;
   GtkWidget *child;
   gdouble upper, value;
   int viewport_size, other_viewport_size;
+  int view_width, view_height;
 
-  gtk_css_gadget_get_content_allocation (viewport->priv->gadget,
-                                         &view_allocation, NULL);
+  gtk_widget_get_content_size (GTK_WIDGET (viewport), &view_width, &view_height);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       adjustment = priv->hadjustment;
       other_orientation = GTK_ORIENTATION_VERTICAL;
-      viewport_size = view_allocation.width;
-      other_viewport_size = view_allocation.height;
+      viewport_size = view_width;
+      other_viewport_size = view_height;
       scroll_policy = priv->hscroll_policy;
       other_scroll_policy = priv->vscroll_policy;
     }
@@ -175,8 +142,8 @@ viewport_set_adjustment_values (GtkViewport    *viewport,
     {
       adjustment = priv->vadjustment;
       other_orientation = GTK_ORIENTATION_HORIZONTAL;
-      viewport_size = view_allocation.height;
-      other_viewport_size = view_allocation.width;
+      viewport_size = view_height;
+      other_viewport_size = view_width;
       scroll_policy = priv->vscroll_policy;
       other_scroll_policy = priv->hscroll_policy;
     }
@@ -232,73 +199,7 @@ viewport_set_adjustment_values (GtkViewport    *viewport,
 }
 
 static void
-gtk_viewport_allocate (GtkCssGadget        *gadget,
-                       const GtkAllocation *allocation,
-                       int                  baseline,
-                       GtkAllocation       *out_clip,
-                       gpointer             data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-  GtkViewport *viewport = GTK_VIEWPORT (widget);
-  GtkViewportPrivate *priv = viewport->priv;
-  GtkAdjustment *hadjustment = priv->hadjustment;
-  GtkAdjustment *vadjustment = priv->vadjustment;
-  GtkWidget *child;
-
-  g_object_freeze_notify (G_OBJECT (hadjustment));
-  g_object_freeze_notify (G_OBJECT (vadjustment));
-
-  viewport_set_adjustment_values (viewport, GTK_ORIENTATION_HORIZONTAL);
-  viewport_set_adjustment_values (viewport, GTK_ORIENTATION_VERTICAL);
-
-  if (gtk_widget_get_realized (widget))
-    {
-      gdk_window_move_resize (priv->bin_window,
-                              allocation->x - gtk_adjustment_get_value (hadjustment),
-                              allocation->y - gtk_adjustment_get_value (vadjustment),
-                              gtk_adjustment_get_upper (hadjustment),
-                              gtk_adjustment_get_upper (vadjustment));
-    }
-
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child && gtk_widget_get_visible (child))
-    {
-      GtkAllocation child_allocation;
-
-      child_allocation.x = 0;
-      child_allocation.y = 0;
-      child_allocation.width = gtk_adjustment_get_upper (hadjustment);
-      child_allocation.height = gtk_adjustment_get_upper (vadjustment);
-
-      gtk_widget_size_allocate (child, &child_allocation);
-    }
-
-  g_object_thaw_notify (G_OBJECT (hadjustment));
-  g_object_thaw_notify (G_OBJECT (vadjustment));
-}
-
-static gboolean
-gtk_viewport_render (GtkCssGadget *gadget,
-                     GtkSnapshot  *snapshot,
-                     int           x,
-                     int           y,
-                     int           width,
-                     int           height,
-                     gpointer      data)
-{
-  GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
-
-  gtk_snapshot_push_clip (snapshot, &GRAPHENE_RECT_INIT(x, y, width, height), "Viewport");
-
-  GTK_WIDGET_CLASS (gtk_viewport_parent_class)->snapshot (widget, snapshot);
-
-  gtk_snapshot_pop (snapshot);
-
-  return FALSE;
-}
-
-static void
-gtk_viewport_measure_ (GtkWidget     *widget,
+gtk_viewport_measure (GtkWidget      *widget,
                       GtkOrientation  orientation,
                       int             for_size,
                       int            *minimum,
@@ -306,11 +207,17 @@ gtk_viewport_measure_ (GtkWidget     *widget,
                       int            *minimum_baseline,
                       int            *natural_baseline)
 {
-  gtk_css_gadget_get_preferred_size (GTK_VIEWPORT (widget)->priv->gadget,
-                                     orientation,
-                                     for_size,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
+  GtkWidget *child;
+
+  *minimum = *natural = 0;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  if (child && gtk_widget_get_visible (child))
+    gtk_widget_measure (child,
+                        orientation,
+                        for_size,
+                        minimum, natural,
+                        NULL, NULL);
 }
 
 static void
@@ -318,27 +225,20 @@ gtk_viewport_class_init (GtkViewportClass *class)
 {
   GObjectClass   *gobject_class;
   GtkWidgetClass *widget_class;
-  GtkContainerClass *container_class;
 
   gobject_class = G_OBJECT_CLASS (class);
   widget_class = (GtkWidgetClass*) class;
-  container_class = (GtkContainerClass*) class;
 
   gobject_class->set_property = gtk_viewport_set_property;
   gobject_class->get_property = gtk_viewport_get_property;
-  gobject_class->finalize = gtk_viewport_finalize;
 
   widget_class->destroy = gtk_viewport_destroy;
-  widget_class->realize = gtk_viewport_realize;
-  widget_class->unrealize = gtk_viewport_unrealize;
   widget_class->snapshot = gtk_viewport_snapshot;
   widget_class->size_allocate = gtk_viewport_size_allocate;
-  widget_class->measure = gtk_viewport_measure_;
+  widget_class->measure = gtk_viewport_measure;
   
   gtk_widget_class_set_accessible_role (widget_class, ATK_ROLE_VIEWPORT);
 
-  container_class->add = gtk_viewport_add;
-
   /* GtkScrollable implementation */
   g_object_class_override_property (gobject_class, PROP_HADJUSTMENT,    "hadjustment");
   g_object_class_override_property (gobject_class, PROP_VADJUSTMENT,    "vadjustment");
@@ -437,7 +337,6 @@ gtk_viewport_init (GtkViewport *viewport)
 {
   GtkWidget *widget;
   GtkViewportPrivate *priv;
-  GtkCssNode *widget_node;
 
   viewport->priv = gtk_viewport_get_instance_private (viewport);
   priv = viewport->priv;
@@ -447,19 +346,11 @@ gtk_viewport_init (GtkViewport *viewport)
   gtk_widget_set_redraw_on_allocate (widget, FALSE);
 
   priv->shadow_type = GTK_SHADOW_IN;
-  priv->bin_window = NULL;
   priv->hadjustment = NULL;
   priv->vadjustment = NULL;
 
-  widget_node = gtk_widget_get_css_node (widget);
-  priv->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                     widget,
-                                                     gtk_viewport_measure,
-                                                     gtk_viewport_allocate,
-                                                     gtk_viewport_render,
-                                                     NULL, NULL);
-
-  gtk_css_gadget_add_class (priv->gadget, GTK_STYLE_CLASS_FRAME);
+  gtk_style_context_add_class (gtk_widget_get_style_context (widget),
+                               GTK_STYLE_CLASS_FRAME);
   viewport_set_adjustment (viewport, GTK_ORIENTATION_HORIZONTAL, NULL);
   viewport_set_adjustment (viewport, GTK_ORIENTATION_VERTICAL, NULL);
 }
@@ -519,17 +410,6 @@ gtk_viewport_destroy (GtkWidget *widget)
   GTK_WIDGET_CLASS (gtk_viewport_parent_class)->destroy (widget);
 }
 
-static void
-gtk_viewport_finalize (GObject *object)
-{
-  GtkViewport *viewport = GTK_VIEWPORT (object);
-  GtkViewportPrivate *priv = viewport->priv;
-
-  g_clear_object (&priv->gadget);
-
-  G_OBJECT_CLASS (gtk_viewport_parent_class)->finalize (object);
-}
-
 static void
 viewport_set_adjustment (GtkViewport    *viewport,
 			 GtkOrientation  orientation,
@@ -608,130 +488,61 @@ gtk_viewport_get_shadow_type (GtkViewport *viewport)
   return viewport->priv->shadow_type;
 }
 
-static void
-gtk_viewport_realize (GtkWidget *widget)
-{
-  GtkViewport *viewport = GTK_VIEWPORT (widget);
-  GtkViewportPrivate *priv = viewport->priv;
-  GtkBin *bin = GTK_BIN (widget);
-  GtkAdjustment *hadjustment = priv->hadjustment;
-  GtkAdjustment *vadjustment = priv->vadjustment;
-  GtkAllocation view_allocation;
-  GtkWidget *child;
-
-  GTK_WIDGET_CLASS (gtk_viewport_parent_class)->realize (widget);
-
-  gtk_css_gadget_get_content_allocation (priv->gadget,
-                                         &view_allocation, NULL);
-
-  priv->bin_window = gdk_window_new_child (gtk_widget_get_window (widget),
-                                           GDK_ALL_EVENTS_MASK,
-                                           &(GdkRectangle) {
-                                             - gtk_adjustment_get_value (hadjustment),
-                                             - gtk_adjustment_get_value (vadjustment),
-                                             gtk_adjustment_get_upper (hadjustment),
-                                             gtk_adjustment_get_upper (vadjustment)});
-  gtk_widget_register_window (widget, priv->bin_window);
-
-  child = gtk_bin_get_child (bin);
-  if (child)
-    gtk_widget_set_parent_window (child, priv->bin_window);
-
-  gdk_window_show (priv->bin_window);
-}
-
-static void
-gtk_viewport_unrealize (GtkWidget *widget)
-{
-  GtkViewport *viewport = GTK_VIEWPORT (widget);
-  GtkViewportPrivate *priv = viewport->priv;
-
-  gtk_widget_unregister_window (widget, priv->bin_window);
-  gdk_window_destroy (priv->bin_window);
-  priv->bin_window = NULL;
-
-  GTK_WIDGET_CLASS (gtk_viewport_parent_class)->unrealize (widget);
-}
-
 static void
 gtk_viewport_snapshot (GtkWidget   *widget,
                        GtkSnapshot *snapshot)
 {
-  GtkViewport *viewport = GTK_VIEWPORT (widget);
-  GtkViewportPrivate *priv = viewport->priv;
+  int width, height;
 
-  gtk_css_gadget_snapshot (priv->gadget, snapshot);
+  gtk_widget_get_content_size (widget, &width, &height);
+
+  gtk_snapshot_push_clip (snapshot,
+                          &GRAPHENE_RECT_INIT(0, 0, width, height), "Viewport");
+
+  GTK_WIDGET_CLASS (gtk_viewport_parent_class)->snapshot (widget, snapshot);
+
+  gtk_snapshot_pop (snapshot);
 }
 
 static void
-gtk_viewport_add (GtkContainer *container,
-		  GtkWidget    *child)
-{
-  GtkBin *bin = GTK_BIN (container);
-  GtkViewport *viewport = GTK_VIEWPORT (bin);
-  GtkViewportPrivate *priv = viewport->priv;
-
-  g_return_if_fail (gtk_bin_get_child (bin) == NULL);
-
-  gtk_widget_set_parent_window (child, priv->bin_window);
-  
-  GTK_CONTAINER_CLASS (gtk_viewport_parent_class)->add (container, child);
-}
-
-static void
-gtk_viewport_size_allocate (GtkWidget     *widget,
-			    GtkAllocation *allocation)
+gtk_viewport_size_allocate (GtkWidget           *widget,
+                            const GtkAllocation *allocation,
+                            int                  baseline,
+                            GtkAllocation       *out_clip)
 {
   GtkViewport *viewport = GTK_VIEWPORT (widget);
   GtkViewportPrivate *priv = viewport->priv;
-  GtkAllocation clip, content_allocation, widget_allocation;
+  GtkAdjustment *hadjustment = priv->hadjustment;
+  GtkAdjustment *vadjustment = priv->vadjustment;
+  GtkWidget *child;
 
-  /* If our size changed, and we have a shadow, queue a redraw on widget->window to
-   * redraw the shadow correctly.
-   */
-  gtk_widget_get_allocation (widget, &widget_allocation);
-  if (gtk_widget_get_mapped (widget) &&
-      priv->shadow_type != GTK_SHADOW_NONE &&
-      (widget_allocation.width != allocation->width ||
-       widget_allocation.height != allocation->height))
-    gdk_window_invalidate_rect (gtk_widget_get_window (widget), NULL, FALSE);
+  g_object_freeze_notify (G_OBJECT (hadjustment));
+  g_object_freeze_notify (G_OBJECT (vadjustment));
 
-  gtk_widget_set_allocation (widget, allocation);
+  viewport_set_adjustment_values (viewport, GTK_ORIENTATION_HORIZONTAL);
+  viewport_set_adjustment_values (viewport, GTK_ORIENTATION_VERTICAL);
 
-  content_allocation = *allocation;
-  gtk_css_gadget_allocate (priv->gadget,
-                           &content_allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  if (child && gtk_widget_get_visible (child))
+    {
+      GtkAllocation child_allocation, child_clip;
 
-  gtk_widget_set_clip (widget, &clip);
+      child_allocation.x = allocation->x - gtk_adjustment_get_value (hadjustment);
+      child_allocation.y = allocation->y - gtk_adjustment_get_value (vadjustment);
+      child_allocation.width = gtk_adjustment_get_upper (hadjustment);
+      child_allocation.height = gtk_adjustment_get_upper (vadjustment);
+
+      /* Explicitly ignore the child clip here. */
+      gtk_widget_size_allocate (child, &child_allocation, -1, &child_clip);
+    }
+
+  g_object_thaw_notify (G_OBJECT (hadjustment));
+  g_object_thaw_notify (G_OBJECT (vadjustment));
 }
 
 static void
 gtk_viewport_adjustment_value_changed (GtkAdjustment *adjustment,
-				       gpointer       data)
+                                       gpointer       data)
 {
-  GtkViewport *viewport = GTK_VIEWPORT (data);
-  GtkViewportPrivate *priv = viewport->priv;
-  GtkBin *bin = GTK_BIN (data);
-  GtkWidget *child;
-
-  child = gtk_bin_get_child (bin);
-  if (child && gtk_widget_get_visible (child) &&
-      gtk_widget_get_realized (GTK_WIDGET (viewport)))
-    {
-      GtkAdjustment *hadjustment = priv->hadjustment;
-      GtkAdjustment *vadjustment = priv->vadjustment;
-      GtkAllocation allocation;
-      gint old_x, old_y;
-      gint new_x, new_y;
-
-      gtk_widget_get_allocation (GTK_WIDGET (viewport), &allocation);
-      gdk_window_get_position (priv->bin_window, &old_x, &old_y);
-      new_x = - gtk_adjustment_get_value (hadjustment);
-      new_y = - gtk_adjustment_get_value (vadjustment);
-
-      if (new_x != old_x || new_y != old_y)
-	gdk_window_move (priv->bin_window, allocation.x + new_x, allocation.y + new_y);
-    }
+  gtk_widget_queue_allocate (GTK_WIDGET (data));
 }
