@@ -276,10 +276,6 @@ struct _GtkWidget
  * @style_updated: Signal emitted when the GtkStyleContext of a widget
  *   is changed.
  * @touch_event:
- * @queue_draw_region: Invalidates the area of widget defined by
- *   region.
- * @queue_draw_child: Child wants to be redrawn. The region given is in
- *   the child's coordinate system.
  * @snapshot: Signal emitted when a widget is supposed to create a snapshot
  *   of itself.
  */
@@ -304,8 +300,10 @@ struct _GtkWidgetClass
   void (* unmap)	       (GtkWidget        *widget);
   void (* realize)	       (GtkWidget        *widget);
   void (* unrealize)	       (GtkWidget        *widget);
-  void (* size_allocate)       (GtkWidget        *widget,
-				GtkAllocation    *allocation);
+  void (* size_allocate)       (GtkWidget           *widget,
+                                const GtkAllocation *allocation,
+                                int                  baseline,
+                                GtkAllocation       *out_clip);
   void (* state_flags_changed) (GtkWidget        *widget,
 				GtkStateFlags  	  previous_state_flags);
   void (* parent_set)	       (GtkWidget        *widget,
@@ -471,12 +469,6 @@ struct _GtkWidgetClass
   gboolean     (* touch_event)            (GtkWidget     *widget,
                                            GdkEventTouch *event);
 
-  void         (* queue_draw_region)           (GtkWidget            *widget,
-					        const cairo_region_t *region);
-  void         (* queue_draw_child)            (GtkWidget            *widget,
-                                                GtkWidget            *child,
-					        const cairo_region_t *region);
-
   void         (* snapshot)                    (GtkWidget            *widget,
                                                 GtkSnapshot          *snapshot);
 
@@ -548,12 +540,10 @@ GDK_AVAILABLE_IN_3_8
 GdkFrameClock* gtk_widget_get_frame_clock (GtkWidget           *widget);
 
 GDK_AVAILABLE_IN_ALL
-void	   gtk_widget_size_allocate	  (GtkWidget	       *widget,
-					   GtkAllocation       *allocation);
-GDK_AVAILABLE_IN_3_10
-void	   gtk_widget_size_allocate_with_baseline	  (GtkWidget	       *widget,
-							   GtkAllocation       *allocation,
-							   gint                 baseline);
+void	   gtk_widget_size_allocate	  (GtkWidget           *widget,
+                                           const GtkAllocation *allocation,
+                                           int                  baseline,
+                                           GtkAllocation       *out_clip);
 
 GDK_AVAILABLE_IN_ALL
 GtkSizeRequestMode  gtk_widget_get_request_mode               (GtkWidget      *widget);
@@ -759,12 +749,6 @@ void                  gtk_widget_get_allocated_size     (GtkWidget     *widget,
 GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_get_allocation         (GtkWidget     *widget,
                                                          GtkAllocation *allocation);
-GDK_AVAILABLE_IN_ALL
-void                  gtk_widget_set_allocation         (GtkWidget     *widget,
-                                                         const GtkAllocation *allocation);
-GDK_AVAILABLE_IN_3_14
-void                  gtk_widget_set_clip               (GtkWidget     *widget,
-                                                         const GtkAllocation *clip);
 GDK_AVAILABLE_IN_3_14
 void                  gtk_widget_get_clip               (GtkWidget     *widget,
                                                          GtkAllocation *clip);
@@ -932,23 +916,6 @@ GDK_AVAILABLE_IN_ALL
 PangoLayout  *gtk_widget_create_pango_layout  (GtkWidget   *widget,
 					       const gchar *text);
 
-/* widget style properties
- */
-GDK_AVAILABLE_IN_ALL
-void gtk_widget_class_install_style_property        (GtkWidgetClass     *klass,
-						     GParamSpec         *pspec);
-GDK_AVAILABLE_IN_ALL
-GParamSpec*  gtk_widget_class_find_style_property   (GtkWidgetClass     *klass,
-						     const gchar        *property_name);
-GDK_AVAILABLE_IN_ALL
-void gtk_widget_style_get_valist   (GtkWidget	     *widget,
-				    const gchar    *first_property_name,
-				    va_list         var_args);
-GDK_AVAILABLE_IN_ALL
-void gtk_widget_style_get          (GtkWidget	     *widget,
-				    const gchar    *first_property_name,
-				    ...) G_GNUC_NULL_TERMINATED;
-
 /* Functions for setting directionality for widgets */
 
 GDK_AVAILABLE_IN_ALL
