@@ -137,6 +137,7 @@ struct _GtkWidgetPrivate
   gint allocated_size_baseline;
   GtkAllocation allocation;
   GtkAllocation clip;
+  GtkAllocation reported_clip;
   gint allocated_baseline;
 
   /* The widget's requested sizes */
@@ -240,8 +241,6 @@ void              _gtk_widget_buildable_finish_accelerator (GtkWidget *widget,
                                                             GtkWidget *toplevel,
                                                             gpointer   user_data);
 GtkStyleContext * _gtk_widget_peek_style_context           (GtkWidget *widget);
-void              _gtk_widget_set_simple_clip              (GtkWidget *widget,
-                                                            GtkAllocation *content_clip);
 
 typedef gboolean (*GtkCapturedEventHandler) (GtkWidget *widget, GdkEvent *event);
 
@@ -316,6 +315,23 @@ void              gtk_widget_focus_sort                    (GtkWidget        *wi
 gboolean          gtk_widget_focus_move                    (GtkWidget        *widget,
                                                             GtkDirectionType  direction,
                                                             GPtrArray        *focus_order);
+void              gtk_widget_get_content_allocation        (GtkWidget        *widget,
+                                                            GtkAllocation    *allocation);
+void              gtk_widget_get_border_allocation         (GtkWidget        *widget,
+                                                            GtkAllocation    *allocation);
+void              gtk_widget_get_margin_allocation         (GtkWidget        *widget,
+                                                            GtkAllocation    *allocation);
+void              gtk_widget_get_outer_allocation          (GtkWidget        *widget,
+                                                            GtkAllocation    *allocation);
+void              gtk_widget_get_own_allocation            (GtkWidget        *widget,
+                                                            GtkAllocation    *allocation);
+void              gtk_widget_get_content_size              (GtkWidget        *widget,
+                                                            int              *width,
+                                                            int              *height);
+
+void              gtk_widget_get_window_allocation         (GtkWidget *widget,
+                                                            GtkAllocation *allocation);
+
 
 GtkWidget *       gtk_widget_common_ancestor               (GtkWidget *widget_a,
                                                             GtkWidget *widget_b);
@@ -330,12 +346,6 @@ gboolean          gtk_widget_get_pass_through              (GtkWidget *widget);
 
 /* inline getters */
 
-static inline gboolean
-gtk_widget_get_resize_needed (GtkWidget *widget)
-{
-  return widget->priv->resize_needed;
-}
-
 static inline GtkWidget *
 _gtk_widget_get_parent (GtkWidget *widget)
 {
