@@ -227,11 +227,6 @@ void       gtk_window_set_destroy_with_parent  (GtkWindow           *window,
                                                 gboolean             setting);
 GDK_AVAILABLE_IN_ALL
 gboolean   gtk_window_get_destroy_with_parent  (GtkWindow           *window);
-GDK_AVAILABLE_IN_3_4
-void       gtk_window_set_hide_titlebar_when_maximized (GtkWindow   *window,
-                                                        gboolean     setting);
-GDK_AVAILABLE_IN_3_4
-gboolean   gtk_window_get_hide_titlebar_when_maximized (GtkWindow   *window);
 GDK_AVAILABLE_IN_ALL
 void       gtk_window_set_mnemonics_visible    (GtkWindow           *window,
                                                 gboolean             setting);
