@@ -22,6 +22,7 @@
 #include <gdk/gdk.h>
 
 #include "gtkwindow.h"
+#include "gtkpointerfocusprivate.h"
 
 G_BEGIN_DECLS
 
@@ -139,6 +140,37 @@ void          gtk_window_unexport_handle (GtkWindow               *window);
 
 GskRenderer *gtk_window_get_renderer (GtkWindow *window);
 
+GtkWidget *      gtk_window_lookup_pointer_focus_widget (GtkWindow        *window,
+                                                         GdkDevice        *device,
+                                                         GdkEventSequence *sequence);
+GtkWidget *      gtk_window_lookup_effective_pointer_focus_widget (GtkWindow        *window,
+                                                                   GdkDevice        *device,
+                                                                   GdkEventSequence *sequence);
+GtkWidget *      gtk_window_lookup_pointer_focus_implicit_grab (GtkWindow        *window,
+                                                                GdkDevice        *device,
+                                                                GdkEventSequence *sequence);
+
+void             gtk_window_update_pointer_focus (GtkWindow        *window,
+                                                  GdkDevice        *device,
+                                                  GdkEventSequence *sequence,
+                                                  GtkWidget        *target,
+                                                  gdouble           x,
+                                                  gdouble           y);
+void             gtk_window_set_pointer_focus_grab (GtkWindow        *window,
+                                                    GdkDevice        *device,
+                                                    GdkEventSequence *sequence,
+                                                    GtkWidget        *grab_widget);
+
+void             gtk_window_update_pointer_focus_on_state_change (GtkWindow *window,
+                                                                  GtkWidget *widget);
+
+void             gtk_window_maybe_revoke_implicit_grab (GtkWindow *window,
+                                                        GdkDevice *device,
+                                                        GtkWidget *grab_widget);
+void             gtk_window_maybe_update_cursor (GtkWindow *window,
+                                                 GtkWidget *widget,
+                                                 GdkDevice *device);
+
 G_END_DECLS
 
 #endif /* __GTK_WINDOW_PRIVATE_H__ */
