@@ -59,6 +59,9 @@ struct _GtkInspectorMiscInfoPrivate {
   GtkWidget *baseline;
   GtkWidget *clip_area_row;
   GtkWidget *clip_area;
+  GtkWidget *frame_clock_row;
+  GtkWidget *frame_clock;
+  GtkWidget *frame_clock_button;
   GtkWidget *tick_callback_row;
   GtkWidget *tick_callback;
   GtkWidget *framerate_row;
@@ -79,7 +82,6 @@ struct _GtkInspectorMiscInfoPrivate {
   GtkWidget *is_toplevel;
   GtkWidget *child_visible_row;
   GtkWidget *child_visible;
-  GtkWidget *object_title;
 
   guint update_source_id;
   gint64 last_frame;
@@ -133,7 +135,11 @@ state_flags_changed (GtkWidget *w, GtkStateFlags old_flags, GtkInspectorMiscInfo
 }
 
 static void
-allocation_changed (GtkWidget *w, GdkRectangle *allocation, GtkInspectorMiscInfo *sl)
+allocation_changed (GtkWidget    *w,
+                    GdkRectangle *allocation,
+                    int           baseline,
+                    GdkRectangle *out_clip,
+                    GtkInspectorMiscInfo *sl)
 {
   GtkAllocation alloc;
   GtkAllocation clip;
@@ -285,6 +291,37 @@ show_mnemonic_label (GtkWidget *button, GtkInspectorMiscInfo *sl)
     show_object (sl, G_OBJECT (widget), "properties");
 }
 
+static void
+show_frame_clock (GtkWidget *button, GtkInspectorMiscInfo *sl)
+{
+  GObject *clock;
+
+  clock = (GObject *)gtk_widget_get_frame_clock (GTK_WIDGET (sl->priv->object));
+  if (clock)
+    show_object (sl, G_OBJECT (clock), "properties");
+}
+
+static void
+update_frame_clock (GtkInspectorMiscInfo *sl)
+{
+  GObject *clock;
+
+  clock = (GObject *)gtk_widget_get_frame_clock (GTK_WIDGET (sl->priv->object));
+  if (clock)
+    {
+      gchar *tmp;
+      tmp = g_strdup_printf ("%p", clock);
+      gtk_label_set_label (GTK_LABEL (sl->priv->frame_clock), tmp);
+      g_free (tmp);
+      gtk_widget_set_sensitive (sl->priv->frame_clock_button, TRUE);
+    }
+  else
+    {
+      gtk_label_set_label (GTK_LABEL (sl->priv->frame_clock), "NULL");
+      gtk_widget_set_sensitive (sl->priv->frame_clock_button, FALSE);
+    }
+}
+
 static gboolean
 update_info (gpointer data)
 {
@@ -335,6 +372,8 @@ update_info (gpointer data)
       gtk_widget_set_visible (sl->priv->realized, gtk_widget_get_realized (GTK_WIDGET (sl->priv->object)));
       gtk_widget_set_visible (sl->priv->is_toplevel, gtk_widget_is_toplevel (GTK_WIDGET (sl->priv->object)));
       gtk_widget_set_visible (sl->priv->child_visible, gtk_widget_get_child_visible (GTK_WIDGET (sl->priv->object)));
+
+      update_frame_clock (sl);
     }
 
   if (GTK_IS_BUILDABLE (sl->priv->object))
@@ -393,8 +432,6 @@ void
 gtk_inspector_misc_info_set_object (GtkInspectorMiscInfo *sl,
                                     GObject              *object)
 {
-  const gchar *title;
-
   if (sl->priv->object)
     {
       g_signal_handlers_disconnect_by_func (sl->priv->object, state_flags_changed, sl);
@@ -411,9 +448,6 @@ gtk_inspector_misc_info_set_object (GtkInspectorMiscInfo *sl,
   g_object_weak_ref (G_OBJECT (sl), disconnect_each_other, object);
   g_object_weak_ref (object, disconnect_each_other, sl);
 
-  title = (const gchar *)g_object_get_data (object, "gtk-inspector-object-title");
-  gtk_label_set_label (GTK_LABEL (sl->priv->object_title), title);
-
   if (GTK_IS_WIDGET (object))
     {
       gtk_widget_show (sl->priv->refcount_row);
@@ -431,12 +465,13 @@ gtk_inspector_misc_info_set_object (GtkInspectorMiscInfo *sl,
       gtk_widget_show (sl->priv->realized_row);
       gtk_widget_show (sl->priv->is_toplevel_row);
       gtk_widget_show (sl->priv->is_toplevel_row);
+      gtk_widget_show (sl->priv->frame_clock_row);
 
       g_signal_connect_object (object, "state-flags-changed", G_CALLBACK (state_flags_changed), sl, 0);
       state_flags_changed (GTK_WIDGET (sl->priv->object), 0, sl);
 
       g_signal_connect_object (object, "size-allocate", G_CALLBACK (allocation_changed), sl, 0);
-      allocation_changed (GTK_WIDGET (sl->priv->object), NULL, sl);
+      allocation_changed (GTK_WIDGET (sl->priv->object), NULL, -1, NULL, sl);
     }
   else
     {
@@ -454,6 +489,7 @@ gtk_inspector_misc_info_set_object (GtkInspectorMiscInfo *sl,
       gtk_widget_hide (sl->priv->realized_row);
       gtk_widget_hide (sl->priv->is_toplevel_row);
       gtk_widget_hide (sl->priv->child_visible_row);
+      gtk_widget_hide (sl->priv->frame_clock_row);
     }
 
   if (GTK_IS_BUILDABLE (object))
@@ -601,6 +637,9 @@ gtk_inspector_misc_info_class_init (GtkInspectorMiscInfoClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, baseline);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, clip_area_row);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, clip_area);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, frame_clock_row);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, frame_clock);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, frame_clock_button);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, tick_callback_row);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, tick_callback);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, framecount_row);
@@ -621,10 +660,10 @@ gtk_inspector_misc_info_class_init (GtkInspectorMiscInfoClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, is_toplevel);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, child_visible_row);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, child_visible);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorMiscInfo, object_title);
 
   gtk_widget_class_bind_template_callback (widget_class, show_default_widget);
   gtk_widget_class_bind_template_callback (widget_class, show_focus_widget);
+  gtk_widget_class_bind_template_callback (widget_class, show_frame_clock);
 }
 
 // vim: set et sw=2 ts=2:
