@@ -236,7 +236,7 @@ row_activated (GtkTreeView *tv,
 
   g_signal_connect (editor, "show-object", G_CALLBACK (show_object), pl);
 
-  gtk_widget_show (popover);
+  gtk_popover_popup (GTK_POPOVER (popover));
 
   g_signal_connect (popover, "unmap", G_CALLBACK (gtk_widget_destroy), NULL);
 
@@ -528,7 +528,7 @@ gtk_inspector_prop_list_set_object (GtkInspectorPropList *pl,
         }
 
       parent = gtk_widget_get_parent (GTK_WIDGET (object));
-      if (!parent)
+      if (!parent || !GTK_IS_CONTAINER (parent))
         {
           gtk_widget_hide (GTK_WIDGET (pl));
           return TRUE;
