@@ -236,9 +236,9 @@ gtk_render_node_view_snapshot (GtkWidget   *widget,
   height = gtk_widget_get_allocated_height (widget);
 
   graphene_rect_init (&rect, 0, 0, width, height);
-  cr = gtk_snapshot_append_cairo_node (snapshot,
-                                       &rect,
-                                       "RenderNodeView");
+  cr = gtk_snapshot_append_cairo (snapshot,
+                                  &rect,
+                                  "RenderNodeView");
 
   cairo_translate (cr, width / 2.0, height / 2.0);
   if (width < viewport.width || height < viewport.height)
