@@ -25,24 +25,13 @@
 #include "gtk/gtkintl.h"
 #include "gtk/gtkimmodule.h"
 
-#include <AvailabilityMacros.h>
-
-#define GTK_COMPILATION 1 // For gdkquartz-gtk-only.h
-
-#include "gdk/quartz/gdkinternal-quartz.h"
-#include "gdk/quartz/gdkquartz-cocoa-access.h"
+#include "gdk/quartz/gdkquartz.h"
 #include "gdk/quartz/GdkQuartzView.h"
 
 #define GTK_IM_CONTEXT_TYPE_QUARTZ (type_quartz)
 #define GTK_IM_CONTEXT_QUARTZ(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_IM_CONTEXT_TYPE_QUARTZ, GtkIMContextQuartz))
 #define GTK_IM_CONTEXT_QUARTZ_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), GTK_IM_CONTEXT_TYPE_QUARTZ, GtkIMContextQuartzClass))
 
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 101200
-#define NS_EVENT_KEY_DOWN NSKeyDown
-#else
-#define NS_EVENT_KEY_DOWN NSEventTypeKeyDown
-#endif
-
 typedef struct _GtkIMContextQuartz
 {
   GtkIMContext parent;
@@ -140,26 +129,16 @@ output_result (GtkIMContext *context,
 {
   GtkIMContextQuartz *qc = GTK_IM_CONTEXT_QUARTZ (context);
   gboolean retval = FALSE;
-  int fixed_str_replace_len;
   gchar *fixed_str, *marked_str;
 
-  fixed_str_replace_len = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (win),
-      TIC_INSERT_TEXT_REPLACE_LEN));
-  fixed_str = g_strdup (g_object_get_data (G_OBJECT (win), TIC_INSERT_TEXT));
-  marked_str = g_strdup (g_object_get_data (G_OBJECT (win), TIC_MARKED_TEXT));
+  fixed_str = g_object_get_data (G_OBJECT (win), TIC_INSERT_TEXT);
+  marked_str = g_object_get_data (G_OBJECT (win), TIC_MARKED_TEXT);
   if (fixed_str)
     {
       GTK_NOTE (MISC, g_print ("tic-insert-text: %s\n", fixed_str));
       g_free (qc->preedit_str);
       qc->preedit_str = NULL;
       g_object_set_data (G_OBJECT (win), TIC_INSERT_TEXT, NULL);
-      if (fixed_str_replace_len)
-        {
-          gboolean retval;
-          g_object_set_data (G_OBJECT (win), TIC_INSERT_TEXT_REPLACE_LEN, 0);
-          g_signal_emit_by_name (context, "delete-surrounding",
-              -fixed_str_replace_len, fixed_str_replace_len, &retval);
-        }
       g_signal_emit_by_name (context, "commit", fixed_str);
       g_signal_emit_by_name (context, "preedit_changed");
 
@@ -189,16 +168,10 @@ output_result (GtkIMContext *context,
     }
   if (!fixed_str && !marked_str)
     {
-      unsigned int filtered =
-	  GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (win),
-					       GIC_FILTER_KEY));
-      if (filtered)
-        retval = TRUE;
       if (qc->preedit_str && strlen (qc->preedit_str) > 0)
         retval = TRUE;
     }
-  g_free (fixed_str);
-  g_free (marked_str);
+
   return retval;
 }
 
@@ -216,34 +189,30 @@ quartz_filter_keypress (GtkIMContext *context,
   if (!GDK_IS_QUARTZ_WINDOW (qc->client_window))
     return FALSE;
 
+  nsview = gdk_quartz_window_get_nsview (qc->client_window);
+  win = (GdkWindow *)[ (GdkQuartzView *)nsview gdkWindow];
+  GTK_NOTE (MISC, g_print ("client_window: %p, win: %p, nsview: %p\n",
+			   qc->client_window, win, nsview));
+
   NSEvent *nsevent = gdk_quartz_event_get_nsevent ((GdkEvent *)event);
 
   if (!nsevent)
     {
       if (event->hardware_keycode == 0 && event->keyval == 0xffffff)
         /* update text input changes by mouse events */
-        return output_result (context, event->window);
+        return output_result (context, win);
       else
         return gtk_im_context_filter_keypress (qc->slave, event);
     }
 
-  nsview = gdk_quartz_window_get_nsview (qc->client_window);
-
-  win = (GdkWindow *)[(GdkQuartzView *)[[nsevent window] contentView] gdkWindow];
-  GTK_NOTE (MISC, g_print ("client_window: %p, win: %p, nsview: %p\n",
-                           qc->client_window, win, nsview));
-
   if (event->type == GDK_KEY_RELEASE)
     return FALSE;
 
   if (event->hardware_keycode == 55)	/* Command */
     return FALSE;
 
-  if (event->hardware_keycode == 53) /* Escape */
-    return FALSE;
-
   NSEventType etype = [nsevent type];
-  if (etype == NS_EVENT_KEY_DOWN)
+  if (etype == NSKeyDown)
     {
        g_object_set_data (G_OBJECT (win), TIC_IN_KEY_DOWN,
                                           GUINT_TO_POINTER (TRUE));
@@ -278,12 +247,9 @@ discard_preedit (GtkIMContext *context)
 
   /* reset any partial input for this NSView */
   [(GdkQuartzView *)nsview unmarkText];
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
   NSInputManager *currentInputManager = [NSInputManager currentInputManager];
   [currentInputManager markedTextAbandoned:nsview];
-#else
-  [[NSTextInputContext currentInputContext] discardMarkedText];
-#endif
+
   if (qc->preedit_str && strlen (qc->preedit_str) > 0)
     {
       g_signal_emit_by_name (context, "commit", qc->preedit_str);
@@ -302,13 +268,13 @@ quartz_reset (GtkIMContext *context)
 }
 
 static void
-quartz_set_client_window (GtkIMContext *context, GdkWindow *window)
+quartz_set_client_window (GtkIMContext *context, GtkWidget *widget)
 {
   GtkIMContextQuartz *qc = GTK_IM_CONTEXT_QUARTZ (context);
 
-  GTK_NOTE (MISC, g_print ("quartz_set_client_window: %p\n", window));
+  GTK_NOTE (MISC, g_print ("quartz_set_client_window: %p\n", widget));
 
-  qc->client_window = window;
+  qc->client_window = gtk_widget_get_parent_window (widget);
 }
 
 static void
@@ -362,16 +328,7 @@ quartz_set_cursor_location (GtkIMContext *context, GdkRectangle *area)
     return;
 
   nsview = gdk_quartz_window_get_nsview (qc->client_window);
-  if (nsview == NULL)
-    return;
-
-  win = (GdkWindow *)[(GdkQuartzView*)nsview gdkWindow];
-  if (win == NULL)
-    {
-      g_warning ("quartz_set_cursor_location received NULL gdkWindow");
-      return;
-    }
-
+  win = (GdkWindow *)[ (GdkQuartzView*)nsview gdkWindow];
   g_object_set_data (G_OBJECT (win), GIC_CURSOR_RECT, qc->cursor_rect);
 }
 
@@ -415,7 +372,7 @@ gtk_im_context_quartz_class_init (GtkIMContextClass *klass)
   klass->get_preedit_string = quartz_get_preedit_string;
   klass->filter_keypress = quartz_filter_keypress;
   klass->reset = quartz_reset;
-  klass->set_client_window = quartz_set_client_window;
+  klass->set_client_widget = quartz_set_client_window;
   klass->focus_in = quartz_focus_in;
   klass->focus_out = quartz_focus_out;
   klass->set_cursor_location = quartz_set_cursor_location;
