@@ -100,7 +100,6 @@ static void
 gtk_printer_cups_init (GtkPrinterCups *printer)
 {
   printer->device_uri = NULL;
-  printer->original_device_uri = NULL;
   printer->printer_uri = NULL;
   printer->state = 0;
   printer->hostname = NULL;
@@ -115,10 +114,12 @@ gtk_printer_cups_init (GtkPrinterCups *printer)
   printer->remote_cups_connection_test = NULL;
   printer->auth_info_required = NULL;
   printer->default_number_up = 1;
+#ifdef HAVE_CUPS_API_1_6
   printer->avahi_browsed = FALSE;
   printer->avahi_name = NULL;
   printer->avahi_type = NULL;
   printer->avahi_domain = NULL;
+#endif
   printer->ipp_version_major = 1;
   printer->ipp_version_minor = 1;
   printer->supports_copies = FALSE;
@@ -150,7 +151,6 @@ gtk_printer_cups_finalize (GObject *object)
   printer = GTK_PRINTER_CUPS (object);
 
   g_free (printer->device_uri);
-  g_free (printer->original_device_uri);
   g_free (printer->printer_uri);
   g_free (printer->hostname);
   g_free (printer->ppd_name);
@@ -174,19 +174,17 @@ gtk_printer_cups_finalize (GObject *object)
     g_object_unref (printer->colord_profile);
 #endif
 
+#ifdef HAVE_CUPS_API_1_6
   g_free (printer->avahi_name);
   g_free (printer->avahi_type);
   g_free (printer->avahi_domain);
+#endif
 
   g_strfreev (printer->covers);
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-
   if (printer->ppd_file)
     ppdClose (printer->ppd_file);
 
-  G_GNUC_END_IGNORE_DEPRECATIONS
-
   g_free (printer->media_default);
   g_list_free_full (printer->media_supported, g_free);
   g_list_free_full (printer->media_size_supported, g_free);
@@ -539,17 +537,12 @@ colord_client_connect_cb (GObject *source_object,
   gboolean ret;
   GError *error = NULL;
   GtkPrinterCups *printer = GTK_PRINTER_CUPS (user_data);
-  static gboolean colord_warned = FALSE;
 
   ret = cd_client_connect_finish (CD_CLIENT (source_object),
                                   res, &error);
   if (!ret)
     {
-      if (!colord_warned)
-        {
-          g_warning ("failed to contact colord: %s", error->message);
-          colord_warned = TRUE;
-        }
+      g_warning ("failed to contact colord: %s", error->message);
       g_error_free (error);
     }
 
