@@ -167,16 +167,16 @@ create_widget_visible_border (const char *text)
   GtkWidget *test_widget;
   GtkWidget *label;
 
-  outer_box = gtk_event_box_new ();
+  outer_box = gtk_overlay_new ();
   gtk_style_context_add_class (gtk_widget_get_style_context (outer_box), "black-bg");
 
-  inner_box = gtk_event_box_new ();
+  inner_box = gtk_overlay_new ();
   gtk_style_context_add_class (gtk_widget_get_style_context (inner_box), "blue-bg");
 
   gtk_container_add (GTK_CONTAINER (outer_box), inner_box);
 
 
-  test_widget = gtk_event_box_new ();
+  test_widget = gtk_overlay_new ();
   gtk_style_context_add_class (gtk_widget_get_style_context (test_widget), "red-bg");
 
   gtk_container_add (GTK_CONTAINER (inner_box), test_widget);
