@@ -129,7 +129,7 @@ main (int    argc,
   /* GtkHButtonBox */
 
   hbbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-  gtk_box_pack_start (GTK_BOX (vbox), hbbox, TRUE, TRUE);
+  gtk_box_pack_start (GTK_BOX (vbox), hbbox);
 
   for (i = 0; i < N_BUTTONS; i++) {
     buttons[i] = gtk_button_new_with_label (strings[i]);
@@ -142,27 +142,27 @@ main (int    argc,
 
   /* GtkVButtonBox */
   vbbox = gtk_button_box_new (GTK_ORIENTATION_VERTICAL);
-  gtk_box_pack_start (GTK_BOX (vbox), vbbox, TRUE, TRUE);
+  gtk_box_pack_start (GTK_BOX (vbox), vbbox);
 
   /* Options */
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox);
 
   combo_types = gtk_combo_box_text_new ();
   populate_combo_with (GTK_COMBO_BOX_TEXT (combo_types), types);
   g_signal_connect (G_OBJECT (combo_types), "changed", G_CALLBACK (combo_types_changed_cb), buttons);
-  gtk_box_pack_start (GTK_BOX (hbox), combo_types, TRUE, TRUE);
+  gtk_box_pack_start (GTK_BOX (hbox), combo_types);
 
   combo_styles = gtk_combo_box_text_new ();
   populate_combo_with (GTK_COMBO_BOX_TEXT (combo_styles), styles);
   g_signal_connect (G_OBJECT (combo_styles), "changed", G_CALLBACK (combo_changed_cb), NULL);
-  gtk_box_pack_start (GTK_BOX (hbox), combo_styles, TRUE, TRUE);
+  gtk_box_pack_start (GTK_BOX (hbox), combo_styles);
 
   option = gtk_check_button_new_with_label ("Help is secondary");
   g_signal_connect (G_OBJECT (option), "toggled", G_CALLBACK (option_cb), buttons[N_BUTTONS - 1]);
 
-  gtk_box_pack_start (GTK_BOX (hbox), option, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox), option);
 
   gtk_widget_show (window);
   gtk_widget_hide (vbbox);
