@@ -1,4 +1,5 @@
 #include <gtk/gtk.h>
+#include <glib/gstdio.h>
 
 static void
 show_message_dialog1 (GtkWindow *parent)
@@ -316,7 +317,11 @@ main (int argc, char *argv[])
   GtkWidget *box;
   GtkWidget *button;
 
-  gtk_init (NULL, NULL);
+#ifdef GTK_SRCDIR
+  g_chdir (GTK_SRCDIR);
+#endif
+
+  gtk_init ();
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size (GTK_WINDOW (window), 600, 400);
