@@ -17,6 +17,7 @@
  */
 
 #include <gtk/gtk.h>
+#include <glib/gstdio.h>
 
 int
 main (int argc,
@@ -28,6 +29,10 @@ main (int argc,
   GEmblem *emblem;
   gchar *str;
 
+#ifdef GTK_SRCDIR
+  g_chdir (GTK_SRCDIR);
+#endif
+
   gtk_init ();
 
   pixbuf = gdk_pixbuf_new_from_file ("apple-red.png", NULL);
@@ -36,30 +41,30 @@ main (int argc,
   gtk_container_add (GTK_CONTAINER (toplevel), hbox);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox);
 
   image = gtk_image_new_from_gicon (G_ICON (pixbuf), GTK_ICON_SIZE_DIALOG);
-  gtk_box_pack_start (GTK_BOX (vbox), image, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (vbox), image);
 
   label = gtk_label_new (NULL);
   str = g_strdup_printf ("Normal icon, hash %u", g_icon_hash (G_ICON (pixbuf)));
   gtk_label_set_label (GTK_LABEL (label), str);
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (vbox), label);
 
   otherpix = gdk_pixbuf_new_from_file ("gnome-textfile.png", NULL);
   emblem = g_emblem_new (G_ICON (otherpix));
   emblemed = g_emblemed_icon_new (G_ICON (pixbuf), emblem);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox);
 
   image2 = gtk_image_new_from_gicon (emblemed, GTK_ICON_SIZE_DIALOG);
-  gtk_box_pack_start (GTK_BOX (vbox), image2, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (vbox), image2);
 
   label = gtk_label_new (NULL);
   str = g_strdup_printf ("Emblemed icon, hash %u", g_icon_hash (emblemed));
   gtk_label_set_label (GTK_LABEL (label), str);
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (vbox), label);
 
   gtk_widget_show (toplevel);
 
