@@ -235,7 +235,7 @@ main (int argc, char *argv[])
   GtkWidget *vbox;
   Views views;
 
-  gtk_init (&argc, &argv);
+  gtk_init ();
 
   set_styles ();
 
@@ -274,7 +274,7 @@ main (int argc, char *argv[])
   gtk_container_add (GTK_CONTAINER (vbox), views.header2);
   gtk_container_add (GTK_CONTAINER (vbox), views.view2);
 
-  gtk_widget_show_all (window);
+  gtk_widget_show (window);
 
   gtk_main ();
 
