@@ -137,7 +137,7 @@ kinetic_scrolling (void)
 int
 main (int argc, char **argv)
 {
-  gtk_init (NULL, NULL);
+  gtk_init ();
 
   kinetic_scrolling ();
 
