@@ -1,4 +1,5 @@
 #include <gtk/gtk.h>
+#include <glib/gstdio.h>
 
 static void
 split_decorations (GtkSettings *settings,
@@ -42,6 +43,10 @@ main (int argc, char *argv[])
   GtkWidget *check;
   GtkWidget *header;
 
+#ifdef GTK_SRCDIR
+  g_chdir (GTK_SRCDIR);
+#endif
+
   gtk_init ();
 
   builder = gtk_builder_new_from_file ("testsplitheaders.ui");
