@@ -509,19 +509,19 @@ main (gint argc, gchar **argv)
   gtk_grid_attach (GTK_GRID (grid), hbox2, 1, 2, 1, 1);
 
   checkbox = gtk_check_button_new_with_mnemonic("_Vertical");
-  gtk_box_pack_start (GTK_BOX (hbox1), checkbox, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox1), checkbox);
   g_signal_connect (checkbox, "toggled",
 		    G_CALLBACK (change_orientation), toolbar);
 
   checkbox = gtk_check_button_new_with_mnemonic("_Show Arrow");
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), TRUE);
-  gtk_box_pack_start (GTK_BOX (hbox1), checkbox, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox1), checkbox);
   g_signal_connect (checkbox, "toggled",
 		    G_CALLBACK (change_show_arrow), toolbar);
 
   checkbox = gtk_check_button_new_with_mnemonic("_Set Toolbar Style:");
   g_signal_connect (checkbox, "toggled", G_CALLBACK (set_toolbar_style_toggled), toolbar);
-  gtk_box_pack_start (GTK_BOX (hbox1), checkbox, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox1), checkbox);
 
   option_menu = gtk_combo_box_text_new ();
   gtk_widget_set_sensitive (option_menu, FALSE);
@@ -531,13 +531,13 @@ main (gint argc, gchar **argv)
     gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (option_menu), toolbar_styles[i]);
   gtk_combo_box_set_active (GTK_COMBO_BOX (option_menu),
                             gtk_toolbar_get_style (GTK_TOOLBAR (toolbar)));
-  gtk_box_pack_start (GTK_BOX (hbox2), option_menu, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox2), option_menu);
   g_signal_connect (option_menu, "changed",
 		    G_CALLBACK (change_toolbar_style), toolbar);
 
   checkbox = gtk_check_button_new_with_mnemonic("_Set Icon Size:");
   g_signal_connect (checkbox, "toggled", G_CALLBACK (set_icon_size_toggled), toolbar);
-  gtk_box_pack_start (GTK_BOX (hbox2), checkbox, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox2), checkbox);
 
   option_menu = gtk_combo_box_text_new ();
   g_object_set_data (G_OBJECT (checkbox), "option-menu", option_menu);
@@ -545,7 +545,7 @@ main (gint argc, gchar **argv)
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (option_menu), "small toolbar");
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (option_menu), "large toolbar");
 
-  gtk_box_pack_start (GTK_BOX (hbox2), option_menu, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox2), option_menu);
   g_signal_connect (option_menu, "changed",
 		    G_CALLBACK (icon_size_history_changed), toolbar);
 
@@ -695,13 +695,13 @@ main (gint argc, gchar **argv)
   gtk_grid_attach (GTK_GRID (grid), hbox, 1, 4, 1, 1);
 
   button = gtk_button_new_with_label ("Drag me to the toolbar");
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox), button);
 
   label = gtk_label_new ("Drop index:");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox), label);
 
   label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox), label);
 
   checkbox = gtk_check_button_new_with_mnemonic("_Right to left");
   if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL)
@@ -710,7 +710,7 @@ main (gint argc, gchar **argv)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), FALSE);
   g_signal_connect (checkbox, "toggled", G_CALLBACK (rtl_toggled), NULL);
 
-  gtk_box_pack_end (GTK_BOX (hbox), checkbox, FALSE, FALSE);
+  gtk_box_pack_end (GTK_BOX (hbox), checkbox);
 
   gtk_drag_source_set (button, GDK_BUTTON1_MASK,
 		       target_table, G_N_ELEMENTS (target_table),
