@@ -383,7 +383,7 @@ test_phases (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -435,7 +435,7 @@ test_mixed (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -494,7 +494,7 @@ test_early_exit (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -548,7 +548,7 @@ test_claim_capture (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -595,7 +595,7 @@ test_claim_target (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -642,7 +642,7 @@ test_claim_bubble (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -696,7 +696,7 @@ test_early_claim_capture (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -753,7 +753,7 @@ test_late_claim_capture (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -812,7 +812,7 @@ test_group (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -865,7 +865,7 @@ test_gestures_outside_grab (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -925,7 +925,7 @@ test_gestures_inside_grab (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -986,7 +986,7 @@ test_multitouch_on_single (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -1038,7 +1038,7 @@ test_multitouch_activation (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
@@ -1116,7 +1116,7 @@ test_multitouch_interaction (void)
   gtk_widget_set_name (A, "A");
   B = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_name (B, "B");
-  C = gtk_event_box_new ();
+  C = gtk_image_new ();
   gtk_widget_set_hexpand (C, TRUE);
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
