@@ -19,9 +19,6 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkunixprint.h>
-#ifdef GDK_WINDOWING_X11
-#include <gtk/gtkx.h>
-#endif
 #ifdef GDK_WINDOWING_WAYLAND
 #include "gdk/wayland/gdkwayland.h"
 #endif
@@ -365,24 +362,7 @@ test_type (gconstpointer data)
 
   /* Deprecated, not getting fixed */
   if (g_str_equal (g_type_name (type), "GtkColorSelection") ||
-      g_str_equal (g_type_name (type), "GtkHandleBox") ||
-      g_str_equal (g_type_name (type), "GtkHPaned") ||
-      g_str_equal (g_type_name (type), "GtkVPaned") ||
-      g_str_equal (g_type_name (type), "GtkHScale") ||
-      g_str_equal (g_type_name (type), "GtkVScale") ||
-      g_str_equal (g_type_name (type), "GtkHScrollbar") ||
-      g_str_equal (g_type_name (type), "GtkVScrollbar") ||
-      g_str_equal (g_type_name (type), "GtkHSeparator") ||
-      g_str_equal (g_type_name (type), "GtkVSeparator") ||
-      g_str_equal (g_type_name (type), "GtkHBox") ||
-      g_str_equal (g_type_name (type), "GtkVBox") ||
-      g_str_equal (g_type_name (type), "GtkArrow") ||
-      g_str_equal (g_type_name (type), "GtkNumerableIcon") ||
-      g_str_equal (g_type_name (type), "GtkRadioAction") ||
-      g_str_equal (g_type_name (type), "GtkToggleAction") ||
-      g_str_equal (g_type_name (type), "GtkTable") ||
-      g_str_equal (g_type_name (type), "GtkUIManager") ||
-      g_str_equal (g_type_name (type), "GtkImageMenuItem"))
+      g_str_equal (g_type_name (type), "GtkNumerableIcon"))
     return;
 
   /* These can't be freely constructed/destroyed */
@@ -396,7 +376,6 @@ test_type (gconstpointer data)
       g_str_equal (g_type_name (type), "GdkX11DeviceManagerXI2") ||
       g_str_equal (g_type_name (type), "GdkX11DeviceManagerCore") ||
       g_str_equal (g_type_name (type), "GdkX11Display") ||
-      g_str_equal (g_type_name (type), "GdkX11DisplayManager") ||
       g_str_equal (g_type_name (type), "GdkX11Screen") ||
       g_str_equal (g_type_name (type), "GdkX11GLContext"))
     return;
@@ -417,30 +396,15 @@ test_type (gconstpointer data)
   if (g_type_is_a (type, GTK_TYPE_MOUNT_OPERATION))
     return;
 
-  /* Backend-specific */
-#ifdef GDK_WINDOWING_X11
-  if (GDK_IS_X11_DISPLAY (gdk_display_get_default ())) ;
-  else if (g_type_is_a (type, GTK_TYPE_PLUG) ||
-           g_type_is_a (type, GTK_TYPE_SOCKET))
-    return;
-#endif
-
-  if (g_type_is_a (type, GTK_TYPE_STATUS_ICON))
-    return;
-
   klass = g_type_class_ref (type);
 
   if (g_type_is_a (type, GTK_TYPE_SETTINGS))
-    instance = G_OBJECT (g_object_ref (gtk_settings_get_default ()));
+    instance = g_object_ref (gtk_settings_get_default ());
   else if (g_type_is_a (type, GDK_TYPE_WINDOW))
     {
-      GdkWindowAttr attributes;
-      attributes.wclass = GDK_INPUT_OUTPUT;
-      attributes.window_type = GDK_WINDOW_TEMP;
-      attributes.event_mask = 0;
-      attributes.width = 100;
-      attributes.height = 100;
-      instance = G_OBJECT (g_object_ref (gdk_window_new (NULL, &attributes, 0)));
+      instance = g_object_ref (gdk_window_new_popup (gdk_display_get_default (),
+                                                     0,
+                                                     &(GdkRectangle) { 0, 0, 100, 100 }));
     }
   else if (g_str_equal (g_type_name (type), "GdkX11Cursor"))
     instance = g_object_new (type, "display", display, NULL);
@@ -523,11 +487,6 @@ test_type (gconstpointer data)
 	  g_str_equal (pspec->name, "draw-indicator"))
         continue;
 
-      /* Not supported in subclass */
-      if (g_str_equal (g_type_name (type), "GtkRecentAction") &&
-	  g_str_equal (pspec->name, "select-multiple"))
-        continue;
-
       if (g_str_equal (g_type_name (type), "GtkRecentChooserMenu") &&
 	  g_str_equal (pspec->name, "select-multiple"))
         continue;
@@ -539,10 +498,6 @@ test_type (gconstpointer data)
       if (type == GTK_TYPE_COLOR_BUTTON && pspec->owner_type == GTK_TYPE_BUTTON)
         continue;
 
-      /* GdkOffscreenWindow is missing many implementations */
-      if (type == GTK_TYPE_OFFSCREEN_WINDOW)
-        continue;
-
       /* Too many special cases involving -set properties */
       if (g_str_equal (g_type_name (pspec->owner_type), "GtkCellRendererText") ||
           g_str_equal (g_type_name (pspec->owner_type), "GtkTextTag"))
@@ -553,9 +508,7 @@ test_type (gconstpointer data)
         continue;
 
       /* Deprecated, not getting fixed */
-      if (g_str_equal (g_type_name (pspec->owner_type), "GtkActivatable") ||
-          g_str_equal (g_type_name (pspec->owner_type), "GtkActionGroup") ||
-          g_str_equal (g_type_name (pspec->owner_type), "GtkAction"))
+      if (g_str_equal (g_type_name (pspec->owner_type), "GtkAction"))
         continue;
 
       if (g_type_is_a (pspec->owner_type, GTK_TYPE_CONTAINER) &&
@@ -573,11 +526,6 @@ test_type (gconstpointer data)
            g_str_equal (pspec->name, "stock-detail")))
         continue;
 
-       if (g_str_equal (g_type_name (pspec->owner_type), "GtkArrow") ||
-          g_str_equal (g_type_name (pspec->owner_type), "GtkAlignment") ||
-          g_str_equal (g_type_name (pspec->owner_type), "GtkMisc"))
-        continue;
-
       if (g_type_is_a (pspec->owner_type, GTK_TYPE_MENU) &&
 	  g_str_equal (pspec->name, "tearoff-state"))
         continue;
