@@ -37,9 +37,6 @@ test_dialog_basic (void)
 
   dialog = gtk_dialog_new();
   g_assert (GTK_IS_DIALOG (dialog));
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  g_assert (gtk_dialog_get_action_area (GTK_DIALOG (dialog)) != NULL);
-G_GNUC_END_IGNORE_DEPRECATIONS
   g_assert (gtk_dialog_get_content_area (GTK_DIALOG (dialog)) != NULL);
 
   gtk_widget_destroy (dialog);
@@ -353,8 +350,7 @@ main (int argc, char **argv)
 
   /* g_test_build_filename must be called after gtk_test_init */
   schema_dir = g_test_build_filename (G_TEST_BUILT, "", NULL);
-  if (g_getenv ("GTK_TEST_MESON") == NULL)
-    g_setenv ("GSETTINGS_SCHEMA_DIR", schema_dir, TRUE);
+  g_setenv ("GSETTINGS_SCHEMA_DIR", schema_dir, TRUE);
 
   /* This environment variable cooperates with gtk_widget_destroy()
    * to assert that all automated compoenents are properly finalized
